////////////////////////////////////////////////////////////////////////////////////////////////////
// Histograms
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <sstream>
#include <sys/stat.h>
#include "TChain.h"
#include <TApplication.h>
#include <TCanvas.h>
#include <TMultiGraph.h>
#include <TGraph.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLegend.h>
#include <TPaveText.h>
#include <TLorentzVector.h>
#include <TStyle.h>
#include <TTree.h>
#include <TTreeReader.h>
#include <TColor.h>
#define SP12 setprecision(12) << setw(30)
#define SW5 setw(15)

using namespace std;

const double PI = 3.1415926535897932385;

void set_style();

double Observable(int iObs, double *px, double *py, double *pz, double *E);

int main()
{
// Create output folder
    mkdir("Results/histograms", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
    set_style();
    TCanvas c("c", "c", 3000, 2000);
    const string htitle("");
    c.SetLeftMargin(0.14);
    c.SetRightMargin(0.1);
    c.SetTopMargin(0.12);
    c.SetBottomMargin(0.15);
//--------------------------------------------------------------------------------------------------
// User defined colors
    TColor kColor0(10000, 0, 0, 0);
    TColor kColor1(10001, 1, 0, 0);
    TColor kColor2(10002, 0, 1, 0);
    TColor kColor3(10003, 0, 0, 1);
//--------------------------------------------------------------------------------------------------
    const int LNAME = 100;
    const int NOBS = 14;
    const int NBINSMAX = 300;

    double loop_xs = 0.;
    double weightloop[1], pxloop[4], pyloop[4], pzloop[4], Eloop[4];

    double ObsLoopMin[NOBS],ObsLoopMax[NOBS];

    ofstream fObs[NOBS];
    char obsName[NOBS][LNAME];
    char xTitle[NOBS][LNAME];
    char yTitle[NOBS][LNAME];
    char nameRoot[NOBS][LNAME];
    char namePdf[NOBS][LNAME];
    char nameDat[NOBS][LNAME];

    const int ivarbin = 0;

    const int binN[NOBS]    = {100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100};
    const double binL[NOBS] = {0.,   0., -15., 0., 0., 0.,  0., -15., 0., 0., 0., 0., 0., -10.};
    const double binU[NOBS] = {180., 1., 15., 100., 360., 180., 1., 15., 100., 360., 100., 100., 500, 10.};

    double bins[NOBS][NBINSMAX+1] = {
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 180.},
	{-1., -0.9, -0.8, -0.7, -0.6, 0., 0.6, 0.7, 0.8, 0.9, 1.},
	{-15., -0.9, -0.8, -0.7, -0.6, 0., 0.6, 0.7, 0.8, 0.9, 15.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 250.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 360.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 180.},
	{-1., -0.9, -0.8, -0.7, -0.6, 0., 0.6, 0.7, 0.8, 0.9, 1.},
	{-15., -0.9, -0.8, -0.7, -0.6, 0., 0.6, 0.7, 0.8, 0.9, 15.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 250.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 360.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 500.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 500.},
	{0., 10., 20., 30., 40., 50., 60., 70., 80., 90., 500.},
	{-10., -0.9, -0.8, -0.7, -0.6, 0., 0.6, 0.7, 0.8, 0.9, 10.},
    };

    if (ivarbin == 0) {
	for (int i = 0; i < NOBS; i ++) {
	    for (int j = 0; j < binN[i]+1; j ++) {
		bins[i][j] = binL[i]+(binU[i]-binL[i])*j/binN[i];
	    }
	}
    }

    double binW[NOBS][NBINSMAX];
    for (int i = 0; i < NOBS; i++) {
	for (int j = 0; j < NBINSMAX; j++) {
	    binW[i][j] = (bins[i][j+1]-bins[i][j]);
	}
    }

    sprintf(obsName[0], "Theta3");
    sprintf(xTitle[0], "#theta(3) [deg.]");
    sprintf(yTitle[0], "d#sigma/d#theta(3) [fb/deg.]");
    sprintf(obsName[1], "Cost3");
    sprintf(xTitle[1], "cos#theta(3)");
    sprintf(yTitle[1], "d#sigma/dcos#theta(3) [fb]");
    sprintf(obsName[2], "eta3");
    sprintf(xTitle[2], "#eta(3)");
    sprintf(yTitle[2], "d#sigma/d#eta(3) [pb]");
    sprintf(obsName[3], "pt3");
    sprintf(xTitle[3], "P_{T}(3) [GeV]");
    sprintf(yTitle[3], "d#sigma/dP_{T}(3) [pb/GeV]");
    sprintf(obsName[4], "Phi3");
    sprintf(xTitle[4], "#phi(3) [deg.]");
    sprintf(yTitle[4], "d#sigma/d#phi(3) [fb/deg.]");
    sprintf(obsName[5], "Theta4");
    sprintf(xTitle[5], "#theta(4) [deg.]");
    sprintf(yTitle[5], "d#sigma/d#theta(4) [fb/deg.]");
    sprintf(obsName[6], "Cost4");
    sprintf(xTitle[6], "cos#theta(4)");
    sprintf(yTitle[6], "d#sigma/dcos#theta(4) [fb]");
    sprintf(obsName[7], "eta4");
    sprintf(xTitle[7], "#eta(4)");
    sprintf(yTitle[7], "d#sigma/d#eta(4) [pb]");
    sprintf(obsName[8], "pt4");
    sprintf(xTitle[8], "P_{T}(4) [GeV]");
    sprintf(yTitle[8], "d#sigma/dP_{T}(4) [pb/GeV]");
    sprintf(obsName[9], "Phi4");
    sprintf(xTitle[9], "#phi(4) [deg.]");
    sprintf(yTitle[9], "d#sigma/d#phi(4) [fb/deg.]");
    sprintf(obsName[10], "E1");
    sprintf(xTitle[10], "E(1) [GeV]");
    sprintf(yTitle[10], "d#sigma/dE(1) [pb/GeV]");
    sprintf(obsName[11], "E2");
    sprintf(xTitle[11], "E(2) [GeV]");
    sprintf(yTitle[11], "d#sigma/dE(2) [pb/GeV]");
    sprintf(obsName[12], "m34");
    sprintf(xTitle[12], "m(34)");
    sprintf(yTitle[12], "d#sigma/dm(34) [fb/GeV]");
    sprintf(obsName[13], "Y34");
    sprintf(xTitle[13], "Y(34)");
    sprintf(yTitle[13], "d#sigma/dY(34) [fb]");

    for (int i = 0; i < NOBS; i++) {
	sprintf(nameRoot[i], "Results/histograms/%s.root", obsName[i]);
	sprintf(namePdf[i], "Results/histograms/%s.pdf", obsName[i]);
	sprintf(nameDat[i], "Results/histograms/%s.dat", obsName[i]);
    }
//--------------------------------------------------------------------------------------------------
    TFile *floop = new TFile("Results/EventsLO.root");
    TTree *tloop = (TTree*)floop->Get("events_loop");
    tloop->SetBranchAddress("weight_loop",&weightloop);
    tloop->SetBranchAddress("px_loop",&pxloop);
    tloop->SetBranchAddress("py_loop",&pyloop);
    tloop->SetBranchAddress("pz_loop",&pzloop);
    tloop->SetBranchAddress("E_loop",&Eloop);
//--------------------------------------------------------------------------------------------------
    ifstream f_loop_xs;
    f_loop_xs.open("Results/EventsLO.dat");
    f_loop_xs >> loop_xs;
    f_loop_xs.close();
    loop_xs = 1.E0; // to get cross section in fb multiply by 1.E3
//--------------------------------------------------------------------------------------------------
    int nentriesloop = tloop->GetEntries();
//--------------------------------------------------------------------------------------------------
    TH1D *hLoop[NOBS], *hLoopMin[NOBS], *hLoopMax[NOBS];
//--------------------------------------------------------------------------------------------------
    gStyle->SetHistLineColor(10000);
    gStyle->SetHistFillColor(10000);
    gStyle->SetMarkerColor(10000);
    gStyle->SetHistLineWidth(1);
    for (int i = 0; i < NOBS; i++) {
	hLoop[i] = new TH1D("","",binN[i],bins[i]);
	hLoop[i]->Sumw2();
	hLoopMin[i] = new TH1D("","",binN[i],bins[i]);
	hLoopMin[i]->Sumw2();
	hLoopMax[i] = new TH1D("","",binN[i],bins[i]);
	hLoopMax[i]->Sumw2();
    }
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < nentriesloop; i++) {
	tloop->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hLoop[j]->Fill(Observable(j, pxloop, pyloop, pzloop, Eloop), weightloop[0]*loop_xs/nentriesloop);
	}
    }

    for (int i = 0; i < NOBS; i++) {
	fObs[i].open(nameDat[i]);
        fObs[i] << "B  C  E" << endl;
	for (int j = 1; j <= binN[i]; j++) {
	    hLoop[i]->SetBinContent(j, (hLoop[i]->GetBinContent(j))/binW[i][j-1]);
	    hLoop[i]->SetBinError(j, (hLoop[i]->GetBinError(j))/binW[i][j-1]);
	    fObs[i] << SW5 << bins[i][j-1]+0.5*binW[i][j-1] << SP12 << hLoop[i]->GetBinContent(j) << SP12 << hLoop[i]->GetBinError(j) << endl;
	}
	fObs[i].close();
    }

    for (int i = 0; i < NOBS; i++) {
	for (int j = 1; j <= binN[i]; j++) {
	    hLoopMin[i]->SetBinContent(j, hLoop[i]->GetBinContent(j)-hLoop[i]->GetBinError(j));
	    hLoopMax[i]->SetBinContent(j, hLoop[i]->GetBinContent(j)+hLoop[i]->GetBinError(j));
	}
	ObsLoopMin[i] = hLoopMin[i]->GetBinContent(hLoopMin[i]->GetMinimumBin());
	ObsLoopMax[i] = hLoopMax[i]->GetBinContent(hLoopMax[i]->GetMaximumBin());
    }
//--------------------------------------------------------------------------------------------------
    TLegend *legend = new TLegend(0.15, 0.9, 0.45, 1.0);
    legend->SetFillColor(0);
    legend->SetFillStyle(4000);
    legend->SetBorderSize(0);
    legend->AddEntry(hLoop[0], "1-loop", "L");
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < NOBS; i++) {
	c.SetLogy(0);
//	hLoop[i]->SetMinimum(ObsLoopMin[i]);
	hLoop[i]->SetMinimum(0.);
	hLoop[i]->SetMaximum(ObsLoopMax[i]);
	hLoop[i]->GetXaxis()->SetTitle(xTitle[i]);
	hLoop[i]->GetYaxis()->SetTitle(yTitle[i]);
	hLoop[i]->DrawCopy("");
	hLoop[i]->DrawCopy("hist same");
	legend->Draw();
	c.Update();
	c.SaveAs(nameRoot[i]);
	c.Print(namePdf[i]);
	delete hLoop[i];
	delete hLoopMin[i];
	delete hLoopMax[i];
    }

    return 0;
}

void set_style()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineColor(kBlack);
    gStyle->SetHistFillStyle(0);
    gStyle->SetFillStyle(4050);
    gStyle->SetMarkerColor(kBlack);
    gStyle->SetMarkerStyle(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetHistLineWidth(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetPalette(1);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelSize(0.06,"X");
    gStyle->SetLabelOffset(0.006,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.06,"X");
    gStyle->SetTitleOffset(1.1,"X");
    gStyle->SetTitleColor(kBlack,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelSize(0.06,"Y");
    gStyle->SetLabelOffset(0.006,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.06,"Y");
    gStyle->SetTitleOffset(1.1,"Y");
    gStyle->SetTitleColor(kBlack,"Y");
    gStyle->SetTextFont(42);
}

double Observable(int iObs, double *px, double *py, double *pz, double *E) {
    double Obs;
    switch (iObs) {
    case 0: // Angle theta for 3rd particle
	Obs = 180/PI*acos(pz[2]/sqrt(px[2]*px[2]+py[2]*py[2]+pz[2]*pz[2]));
	break;
    case 1: // Cosine of theta for 3rd particle
	Obs = pz[2]/sqrt(px[2]*px[2]+py[2]*py[2]+pz[2]*pz[2]);
	break;
    case 2: // Pseudorapidity of 3rd particle
	Obs = 0.5*log((sqrt(px[2]*px[2]+py[2]*py[2]+pz[2]*pz[2])+pz[2])/(sqrt(px[2]*px[2]+py[2]*py[2]+pz[2]*pz[2])-pz[2]));
	break;
    case 3: // Transverse momentum of 3rd particle
	Obs = sqrt(px[2]*px[2]+py[2]*py[2]);
	break;
    case 4: // Angle phi for 3rd particle
	Obs = 180./PI*atan2(py[2],px[2]);
	if (Obs < 0.) Obs += 360.;
	break;
    case 5: // Angle theta for 4th particle
	Obs = 180/PI*acos(pz[3]/sqrt(px[3]*px[3]+py[3]*py[3]+pz[3]*pz[3]));
	break;
    case 6: // Cosine of theta for 4th particle
	Obs = fabs(pz[3]/sqrt(px[3]*px[3]+py[3]*py[3]+pz[3]*pz[3]));
	break;
    case 7: // Pseudorapidity of 4th particle
	Obs = 0.5*log((sqrt(px[3]*px[3]+py[3]*py[3]+pz[3]*pz[3])+pz[3])/(sqrt(px[3]*px[3]+py[3]*py[3]+pz[3]*pz[3])-pz[3]));
	break;
    case 8: // Transverse momentum of 4th particle
	Obs = sqrt(px[3]*px[3]+py[3]*py[3]);
	break;
    case 9: // Angle phi for 3rd particle
	Obs = 180./PI*atan2(py[3],px[3]);
	if (Obs < 0.) Obs += 360.;
	break;
    case 10: // Energy of 1st particle
	Obs = E[0];
	break;
    case 11: // Energy of 2nd particle
	Obs = E[1];
	break;
    case 12: // Invariant mass of 3rd and 4th particle
	Obs = sqrt((E[2]+E[3])*(E[2]+E[3])-(px[2]+px[3])*(px[2]+px[3])-(py[2]+py[3])*(py[2]+py[3])-(pz[2]+pz[3])*(pz[2]+pz[3]));
	break;
    case 13: // Rapidity of 3rd and 4th particle
	Obs = 0.5*log((E[2]+E[3]+pz[2]+pz[3])/(E[2]+E[3]-pz[2]-pz[3]));
	break;
    default:
	Obs = 0.;
    }
    return Obs;
};
