////////////////////////////////////////////////////////////////////////////////////////////////////
// Histograms
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <sstream>
#include <sys/stat.h>
#include "TChain.h"
#include <TApplication.h>
#include <TCanvas.h>
#include <TMultiGraph.h>
#include <TGraph.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLegend.h>
#include <TPaveText.h>
#include <TLorentzVector.h>
#include <TStyle.h>
#include <TTree.h>
#include <TTreeReader.h>
#include <TColor.h>
#define SP12 setprecision(12) << setw(21)
#define SW5 setw(9)

using namespace std;

const double PI = 3.1415926535897932385;

void set_style();

double Observable(int iObs, double *px, double *py, double *pz, double *E);

int main()
{
// Create output folder
    mkdir("Results/histograms", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
    set_style();
    TCanvas c("c", "c", 3000, 2000);
    const string htitle("");
    c.SetLeftMargin(0.14);
    c.SetRightMargin(0.1);
    c.SetTopMargin(0.12);
    c.SetBottomMargin(0.15);
//--------------------------------------------------------------------------------------------------
// User defined colors
    TColor kColor0(10000, 0, 0, 0);
    TColor kColor1(10001, 1, 0, 0);
    TColor kColor2(10002, 0, 1, 0);
    TColor kColor3(10003, 0, 0, 1);
//--------------------------------------------------------------------------------------------------
    const int LNAME = 100;
    const int NOBS = 3;
    const int NBINSMAX = 300;
    const int NPAIR = NOBS*(NOBS-1)/2;

    double loop_xs = 0.;
    double weightloop[1], pxloop[4], pyloop[4], pzloop[4], Eloop[4];

    ofstream fObs[NPAIR];
    char obsName[NOBS][LNAME];
    char xTitle[NPAIR][LNAME];
    char yTitle[NPAIR][LNAME];
    char zTitle[NPAIR][LNAME];
    char nameRoot[NPAIR][3*LNAME];
    char namePdf[NPAIR][3*LNAME];
    char nameDat[NPAIR][3*LNAME];

    const int    ivarbin = 1;

    const int    binN[NOBS] = {6, 6, 5};
    const double binL[NOBS] = {0., 0., 0.};
    const double binU[NOBS] = {2.4, 2.4, 500.};

    double bins[NOBS][NBINSMAX+1] = {
	{0., 0.4, 0.8, 1.2, 1.6, 2., 2.4},
	{0., 0.4, 0.8, 1.2, 1.6, 2., 2.4},
	{0., 100., 200., 300., 400., 500.},
    };

    if (ivarbin == 0) {
	for (int i = 0; i < NOBS; i ++) {
	    for (int j = 0; j < binN[i]+1; j ++) {
		bins[i][j] = binL[i]+(binU[i]-binL[i])*j/binN[i];
	    }
	}
    }

    double binW[NOBS][NBINSMAX];
    for (int i = 0; i < NOBS; i++) {
	for (int j = 0; j < NBINSMAX; j++) {
	    binW[i][j] = (bins[i][j+1]-bins[i][j]);
	}
    }

    sprintf(obsName[0], "Eta3");
    sprintf(obsName[1], "Eta4");
    sprintf(obsName[2], "M34");
    sprintf(xTitle[0], "#eta(3)");
    sprintf(yTitle[0], "#eta(4)");
    sprintf(zTitle[0], "d#sigma/d#eta(3)d#eta(4) [fb]");
    sprintf(xTitle[1], "#eta(3)");
    sprintf(yTitle[1], "M(34) [GeV]");
    sprintf(zTitle[1], "d#sigma/d#eta(3)dM(34) [fb/GeV]");
    sprintf(xTitle[2], "#eta(4)");
    sprintf(yTitle[2], "M(34) [GeV]");
    sprintf(zTitle[2], "d#sigma/d#eta(4)dM(34) [fb/GeV]");

    int k2d = 0;
    for (int i = 0; i < NOBS-1; i++) {
	for (int j = i+1; j < NOBS; j++) {
	    sprintf(nameRoot[k2d], "Results/histograms/%s%s.root", obsName[i], obsName[j]);
	    sprintf(namePdf[k2d], "Results/histograms/%s%s.pdf", obsName[i], obsName[j]);
	    sprintf(nameDat[k2d], "Results/histograms/%s%s.dat", obsName[i], obsName[j]);
	    k2d++;
	}
    }
//--------------------------------------------------------------------------------------------------
    TFile *floop = new TFile("Results/EventsLO.root");
    TTree *tloop = (TTree*)floop->Get("events_loop");
    tloop->SetBranchAddress("weight_loop",&weightloop);
    tloop->SetBranchAddress("px_loop",&pxloop);
    tloop->SetBranchAddress("py_loop",&pyloop);
    tloop->SetBranchAddress("pz_loop",&pzloop);
    tloop->SetBranchAddress("E_loop",&Eloop);
//--------------------------------------------------------------------------------------------------
    ifstream f_loop_xs;
    f_loop_xs.open("Results/EventsLO.dat");
    f_loop_xs >> loop_xs;
    f_loop_xs.close();
    loop_xs *= 1.E0; // to get cross section in pb multiply by 1.E-3
//--------------------------------------------------------------------------------------------------
    int nentriesloop = tloop->GetEntries();
//--------------------------------------------------------------------------------------------------
    TH2D *hLoop[NPAIR];
//--------------------------------------------------------------------------------------------------
    gStyle->SetHistLineColor(10000);
    gStyle->SetHistFillColor(10000);
    gStyle->SetMarkerColor(10000);
    gStyle->SetHistLineWidth(1);
    k2d = 0;
    for (int i = 0; i < NOBS-1; i++) {
	for (int j = i+1; j < NOBS; j++) {
	    hLoop[k2d] = new TH2D("","",binN[i],bins[i],binN[j],bins[j]);
	    hLoop[k2d]->Sumw2();
	    k2d++;
	}
    }
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < nentriesloop; i++) {
	tloop->GetEntry(i);
	k2d = 0;
	for (int j = 0; j < NOBS-1; j++) {
	    for (int k = j+1; k < NOBS; k++) {
		hLoop[k2d]->Fill(Observable(j, pxloop, pyloop, pzloop, Eloop),Observable(k, pxloop, pyloop, pzloop, Eloop), weightloop[0]*loop_xs/nentriesloop);
		k2d++;
	    }
	}
    }

    k2d = 0;
    for (int i = 0; i < NOBS-1; i++) {
	for (int j = i+1; j < NOBS; j++) {
	    fObs[k2d].open(nameDat[k2d]);
	    for (int k = 1; k <= binN[i]; k++) {
		for (int l = 1; l <= binN[j]; l++) {
		    hLoop[k2d]->SetBinContent(k, l, (hLoop[k2d]->GetBinContent(k, l))/binW[i][k-1]/binW[j][l-1]);
		    hLoop[k2d]->SetBinError(k, l, (hLoop[k2d]->GetBinError(k, l))/binW[i][k-1]/binW[j][l-1]);
		    fObs[k2d] << SW5 << bins[i][k-1] << SW5 << bins[i][k] << SW5 << bins[j][l-1] << SW5 << bins[j][l] << SP12 << hLoop[k2d]->GetBinContent(k,l) << SP12 << hLoop[k2d]->GetBinError(k,l) << endl;
		}
	    }
	    fObs[k2d].close();
	    k2d++;
	}
    }
//--------------------------------------------------------------------------------------------------
    c.SetLogx(0);
    c.SetLogy(0);
    c.SetLogz(0);
    for (int i = 0; i < NPAIR; i++) {
	hLoop[i]->SetMinimum(0.);
	hLoop[i]->GetXaxis()->SetTitle(xTitle[i]);
	hLoop[i]->GetYaxis()->SetTitle(yTitle[i]);
	hLoop[i]->GetZaxis()->SetTitle(zTitle[i]);
	hLoop[i]->DrawCopy("lego2");
	c.Update();
	c.SaveAs(nameRoot[i]);
	c.Print(namePdf[i]);
	delete hLoop[i];
    }

    return 0;
}

void set_style()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineColor(kBlack);
    gStyle->SetHistFillStyle(0);
    gStyle->SetFillStyle(4050);
    gStyle->SetMarkerColor(kBlack);
    gStyle->SetMarkerStyle(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetHistLineWidth(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetPalette(1);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelSize(0.06,"X");
    gStyle->SetLabelOffset(0.006,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.06,"X");
    gStyle->SetTitleOffset(1.1,"X");
    gStyle->SetTitleColor(kBlack,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelSize(0.06,"Y");
    gStyle->SetLabelOffset(0.006,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.06,"Y");
    gStyle->SetTitleOffset(1.1,"Y");
    gStyle->SetTitleColor(kBlack,"Y");
    gStyle->SetTextFont(42);
}

double Observable(int iObs, double *px, double *py, double *pz, double *E) {
    double Obs;
    switch (iObs) {
    case 0: // Pseudorapidity of 3rd particle
	Obs = fabs(0.5*log((sqrt(px[2]*px[2]+py[2]*py[2]+pz[2]*pz[2])+pz[2])/(sqrt(px[2]*px[2]+py[2]*py[2]+pz[2]*pz[2])-pz[2])));
	break;
    case 1: // Pseudorapidity of 4th particle
	Obs = fabs(0.5*log((sqrt(px[3]*px[3]+py[3]*py[3]+pz[3]*pz[3])+pz[3])/(sqrt(px[3]*px[3]+py[3]*py[3]+pz[3]*pz[3])-pz[3])));
	break;
    case 2: // Invariant mass of 3rd and 4th particle
	Obs = sqrt((E[2]+E[3])*(E[2]+E[3])-(px[2]+px[3])*(px[2]+px[3])-(py[2]+py[3])*(py[2]+py[3])-(pz[2]+pz[3])*(pz[2]+pz[3]));
	break;
    default:
	Obs = 0.;
    }
    return Obs;
};
