////////////////////////////////////////////////////////////////////////////////////////////////////
// Histograms
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <sstream>
#include <sys/stat.h>
#include "TChain.h"
#include <TApplication.h>
#include <TCanvas.h>
#include <TMultiGraph.h>
#include <TGraph.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLegend.h>
#include <TPaveText.h>
#include <TLorentzVector.h>
#include <TStyle.h>
#include <TTree.h>
#include <TTreeReader.h>
#include <TColor.h>
#include "CLI11.hpp"

using namespace std;

void set_style();

int main(int argc, char* argv[])
{
    const int LNAME = 100;
    CLI::App options{"ReneSANCe"};
    CLI11_PARSE(options, argc, argv)

// Create output folder
    mkdir("Results/histograms", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
    set_style();
    TCanvas c("c", "c", 3000, 2000);
    const string htitle("");
    c.SetLeftMargin(0.14);
    c.SetRightMargin(0.1);
    c.SetTopMargin(0.12);
    c.SetBottomMargin(0.15);
//--------------------------------------------------------------------------------------------------
// User defined colors
    TColor kColor0(10000, 0, 0, 0);
    TColor kColor1(10001, 1, 0, 0);
    TColor kColor2(10002, 0, 1, 0);
    TColor kColor3(10003, 0, 0, 1);
//--------------------------------------------------------------------------------------------------
    TFile *floopvar = new TFile("Results/loopvar.root");
    TTree *tloopvar = (TTree*)floopvar->Get("events_var");
    Double_t weightvar[1], var[6];
    tloopvar->SetBranchAddress("weight_var",&weightvar);
    tloopvar->SetBranchAddress("var",&var);
//--------------------------------------------------------------------------------------------------
    int nentriesloopvar = tloopvar->GetEntries();
//--------------------------------------------------------------------------------------------------
    const int    binN[6] = { 100, 100, 100, 100, 100, 100};
    const double binL[6] = {  0.,  0.,  0.,  0.,  0.,  0.};
    const double binU[6] = {  1.,  1.,  1.,  1.,  1.,  1.};

    char xTitle[6][LNAME];

    char nameRoot1d[6][LNAME];
    char namePdf1d[6][LNAME];

    char nameRoot2d[21][LNAME];
    char namePdf2d[21][LNAME];

    for (int i = 0; i < 6; i++) {
	sprintf(xTitle[i], "X%i", i+1);
	sprintf(nameRoot1d[i], "Results/histograms/X%i.root", i+1);
	sprintf(namePdf1d[i], "Results/histograms/X%i.pdf", i+1);
    }

    int k2d = 0;
    for (int i = 0; i < 5; i++) {
	for (int j = i+1; j < 6; j++) {
	    sprintf(nameRoot2d[k2d], "Results/histograms/X%iX%i.root", i+1, j+1);
	    sprintf(namePdf2d[k2d], "Results/histograms/X%iX%i.pdf", i+1, j+1);
	    k2d++;
	}
    }
//--------------------------------------------------------------------------------------------------
    TH1D *hX1d[7];
    TH2D *hX2d[7][7];

    gStyle->SetHistLineColor(10000);
    gStyle->SetHistFillColor(10000);
    gStyle->SetMarkerColor(10000);
    gStyle->SetHistLineWidth(1);

    for (int i = 0; i < 6; i++) {
	hX1d[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hX1d[i]->Sumw2();
    }

    for (int i = 0; i < 5; i++) {
	for (int j = i+1; j < 6; j++) {
	    hX2d[i][j] = new TH2D("","",binN[i],binL[i],binU[i],binN[j],binL[j],binU[j]);
	    hX2d[i][j]->Sumw2();
	}
    }
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < nentriesloopvar; i++) {
	tloopvar->GetEntry(i);
	if (weightvar[0] < 0.) weightvar[0] *= -1;
	for (int j = 0; j < 6; j++) {
	    hX1d[j]->Fill(var[j],weightvar[0]);
	}
	for (int j = 0; j < 5; j++) {
	    for (int k = j+1; k < 6; k++) {
		hX2d[j][k]->Fill(var[j],var[k],weightvar[0]);
	    }
	}
    }
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < 6; i++) {
	c.SetLogx(0);
	c.SetLogy(0);
	hX1d[i]->SetMinimum(0.0);
	hX1d[i]->GetXaxis()->SetTitle(xTitle[i]);
	hX1d[i]->GetYaxis()->SetTitle("N");
	hX1d[i]->DrawCopy("");
	hX1d[i]->DrawCopy("hist same");
	c.Update();
	c.SaveAs(nameRoot1d[i]);
	c.Print(namePdf1d[i]);
	delete hX1d[i];
    }
//--------------------------------------------------------------------------------------------------
    k2d = 0;
    for (int i = 0; i < 5; i++) {
	for (int j = i+1; j < 6; j++) {
	    c.SetLogx(0);
	    c.SetLogy(0);
	    c.SetLogz(0);
	    hX2d[i][j]->SetMinimum(0.0);
	    hX2d[i][j]->GetXaxis()->SetTitle(xTitle[i]);
	    hX2d[i][j]->GetYaxis()->SetTitle(xTitle[j]);
	    hX2d[i][j]->GetZaxis()->SetTitle("N");
	    hX2d[i][j]->DrawCopy("lego2");
	    c.Update();
	    c.SaveAs(nameRoot2d[k2d]);
	    c.Print(namePdf2d[k2d]);
	    delete hX2d[i][j];
	    k2d++;
	}
    }
//--------------------------------------------------------------------------------------------------
    return 0;
}

void set_style()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineColor(kBlack);
    gStyle->SetHistFillStyle(0);
    gStyle->SetFillStyle(4050);
    gStyle->SetMarkerColor(kBlack);
    gStyle->SetMarkerStyle(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetHistLineWidth(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelSize(0.06,"X");
    gStyle->SetLabelOffset(0.006,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.06,"X");
    gStyle->SetTitleOffset(1.1,"X");
    gStyle->SetTitleColor(kBlack,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelSize(0.06,"Y");
    gStyle->SetLabelOffset(0.006,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.06,"Y");
    gStyle->SetTitleOffset(1.1,"Y");
    gStyle->SetTitleColor(kBlack,"Y");
    gStyle->SetTextFont(42);
}
