#ifndef INFO_H
#define INFO_H

#include "General.h"

namespace SANC {

class Settings;

class Info
{
public:
    Info(Settings & _cfg);
    std::string version();
/*
    Process ID:
    301: A A --> A A
    302: A A --> A Z
    303: A A --> Z Z
*/
    const int Pid;
    //--------------------------------------------------------------------------------------------------
    const int Scheme; // Scheme: 0 - alpha(0), 1 - Gmu, 2 - Gmu'
    const double Ecm; // Total energy of colliding particles in CM frame
    const double Alphai;
    const double Alphas;
    const double AlphaMZi;
    //--------------------------------------------------------------------------------------------------
    const double Polcirc1;
    const double Pollin1;
    const double Polphi1;
    const double Polcirc2;
    const double Pollin2;
    const double Polphi2;

    const double Pe1;
    const double Pg1;
    const double Pt1;
    const double Phi1;
    const double Pe2;
    const double Pg2;
    const double Pt2;
    const double Phi2;
    const double x0;
    const double A;
    const double Rho;
    const double Eamax;
    const double Lumee;
    //--------------------------------------------------------------------------------------------------
    const bool theta3cut;
    const double theta3min;
    const double theta3max;
    const bool theta4cut;
    const double theta4min;
    const double theta4max;
    const bool cost3cut;
    const double cost3min;
    const double cost3max;
    const bool cost4cut;
    const double cost4min;
    const double cost4max;
    const bool eta3cut;
    const double eta3min;
    const double eta3max;
    const bool eta4cut;
    const double eta4min;
    const double eta4max;
    const bool pt3cut;
    const double pt3min;
    const double pt3max;
    const bool pt4cut;
    const double pt4min;
    const double pt4max;
    const bool m34cut;
    const double m34min;
    const double m34max;
    //--------------------------------------------------------------------------------------------------
    const int Ivar;
    const int Ipdf;
    const int Iqed;
    const int Iew;
    //--------------------------------------------------------------------------------------------------
    const double Men;
    const double Mel;
    const double Mup;
    const double Mdn;
    const double Mmn;
    const double Mmo;
    const double Mch;
    const double Mst;
    const double Mtn;
    const double Mta;
    const double Mtp;
    const double Mbt;
    const double MW;
    const double MZ;
    const double MH;
    const double Wz;
    const double Ww;
    const double Wh;
    const double Wtp;
    const double Vud;
    const double Vus;
    const double Vcd;
    const double Vcs;
    //--------------------------------------------------------------------------
    const long RanSeed; // Seed for RNG initialization (0 <= RanSeed <= 9E8)
    const long NevLoop; // MC statistics for Loop
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Loop
    const int  kDimLoop; // HYP-CUBICAL subspace
    const int  nCellsLoop; // Number of Cells
    const int  nSamplLoop; // Number of MC events per cell in build-up
    const int  nBinLoop; // Number of bins in build-up
    const int  EvPerBinLoop; // Maximum events (equiv.) per bin in buid-up
    const double MaxWtRejLoop; // Maximum wt for rejection, for OptRej=1
    //--------------------------------------------------------------------------------------------------
    //  General FOAM parameters
    const int  OptRej; // =0, weighted events;  =1, wt=1 events
    const int  OptDrive; // (D=2) Option, type of Drive =0,1,2 for TrueVol,Sigma,WtMax
    const int  OptEdge; // (D=0) Option, vertices are included in the sampling (0) or not (1)
    const int  OptOrd; // (D=0) Option, single simplex or nDim! simlices
    const int  OptPeek; // (D=0) Option, choice of cell in buid-up maximum(0), random(1)
    const int  OptMCell; // (D=1) Option, Mega-Cell = slim memory
    const int  Chat; // Chat level
    double Sigma;
    double SigmaErr;
    //--------------------------------------------------------------------------------------------------
    //  Parameters for branching
    double SigmaLoop;
    //--------------------------------------------------------------------------------------------------
    const bool IEXPLLOOP;
    //--------------------------------------------------------------------------------------------------
    const bool PrintLHE;
    const bool PrintHEPEVT;
    const bool PrintHepMC;
    const std::string HepMCFormat;
    const bool PrintROOT;
    const bool PrintDEBUG;
    const int verMajor, verMinor, verPatch;
    std::string outputDir;
private:
    Settings & cfg;
};

}
#endif // INFO_H
