#include <fstream>
#include <iostream>
#include <cmath>
#include "Integrand.h"
#include "Info.h"
#include "LHEF.h"
#include "TMath.h"
#include "ModulesInterface.h"
#include "Process.h"
#include "Vec4.h"
using namespace std;
using namespace SANC;
using namespace SANCmodules_aa;
using namespace SANCmodulesVar;

Integrand_aa::Integrand_aa(SANC::Info &_info, SANC::Process & _proc, PDF * _pdf, int type):info(_info),proc(_proc),pdf(_pdf){
    m_Type=type;
    Mf1 = proc.Mf1;
    Mf2 = proc.Mf2;
    Mf3 = proc.Mf3;
    Mf4 = proc.Mf4;
    Mf1_2 = proc.Mf1_2;
    Mf2_2 = proc.Mf2_2;
    Mf3_2 = proc.Mf3_2;
    Mf4_2 = proc.Mf4_2;
    sm = proc.sm;
    dm = proc.dm;
}

double Integrand_aa::Density(int nDim, double *Xarg){
    double rho;
    rho = 1e-100;
    if (m_Type == 20) rho = LoopAA(nDim, Xarg);
    else{
	cout << " TFDISTR::Density: !!!!!! WRONG m_Type= " << m_Type << endl;
    }
    return rho;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_aa::LoopAA(int nDim, double *Xarg) {
    double LoopXS(0.);
    double Z[nDim];
    double Jac(1.);
    double AA(0.);
    double Loop(0.);
    Vec4   P[4];
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesLOOP(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelim(Z)) return LoopXS;
    proc.GetFourMomenta(Z,P);
    if (proc.Cuts(P)) return LoopXS;
    if (info.Ipdf != 0) proc.PdfConv(Z,&AA);
    if (info.Ipdf >= 0) proc.Loop(Z,&Loop);
    if (info.Ipdf == 0) {
	LoopXS = Loop*Jac;
    } else if (info.Ipdf >= 1) {
	LoopXS = AA*Loop*Jac;
    } else if (info.Ipdf <= -1) {
	LoopXS = AA*Jac/2.;
    }
    if (LoopXS < 0) proc.event.weightProc = -1.;
    LoopXS = abs(LoopXS);
//--------------------------------------------------------------------------------------------------
    if (LoopXS != LoopXS) {
	LoopXS = 0.;
	cout << "Loop NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    proc.event.P_lab(1)=P[0];
    proc.event.P_lab(2)=P[1];
    proc.event.P_lab(3)=P[2];
    proc.event.P_lab(4)=P[3];
//--------------------------------------------------------------------------------------------------
    return LoopXS;
}
