#include "General.h"
#include "Process.h"
#include "Info.h"
#include "TMath.h"
#include "ModulesInterface.h"
#include <fstream>
#include <sstream>
#include <cmath>

using namespace SANC;
using namespace SANCmodulesVar;
using namespace SANCmodules_aa;

Process::Process(Info & _info):info(_info)
{
    nup = 4;
    idup.resize(4);
    idup[1] = 22;
    idup[2] = 22;
    switch (info.Pid) {
    case 301:
	procName = "A A -> A A";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = 0.;
	Mf4 = 0.;
	idup[3] = 22;
	idup[4] = 22;
	break;
    case 302:
	procName = "A A -> A Z";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = 0.;
	Mf4 = info.MZ;
	idup[3] = 22;
	idup[4] = 23;
	break;
    case 303:
	procName = "A A -> Z Z";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.MZ;
	Mf4 = info.MZ;
	idup[3] = 23;
	idup[4] = 23;
	break;
    case 304:
	procName = "A A -> e- e+";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mel;
	Mf4 = info.Mel;
	idup[3] = 11;
	idup[4] = -11;
	break;
    default:
	std::cerr<<"Error: Unknown process "<<info.Pid<<std::endl;
	exit(1);
    }
    std::ifstream fpol_x1,fpol_x2;
    std::ifstream f_x1x2;
    fpol_x1.open("fpol1.dat");
    fpol_x2.open("fpol2.dat");
    f_x1x2.open("lumx1x2.dat");
    for (int i = 0; i < 100; i++) {
	fpol_x1 >> fp1[i];
	fpol_x2 >> fp2[i];
	xl1[i] = ((double)i)*2.*info.Eamax/info.Ecm/100.;
	xu1[i] = ((double)i+1.)*2.*info.Eamax/info.Ecm/100.;
	xl2[i] = ((double)i)*2.*info.Eamax/info.Ecm/100.;
	xu2[i] = ((double)i+1.)*2.*info.Eamax/info.Ecm/100.;
    }
    for (int i = 0; i < 10000; i++) {
	f_x1x2 >> fa12[i];
    }
    fpol_x1.close();
    fpol_x2.close();
    f_x1x2.close();
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    dm = (Mf1-Mf2)*(Mf1+Mf2)*(Mf4-Mf3)*(Mf4+Mf3);
    sm = Mf1_2+Mf2_2+Mf3_2+Mf4_2;
    event.particles.clear();
    for (auto const & id:  idup) {
	event.particles.push_back(Particle(id));
    }
    event.particles.push_back(Particle(22));
    event.particles[0].motherList = {};
    event.particles[1].motherList = {};
    event.particles[2].motherList = {};
    event.particles[3].motherList = {1,2};
    event.particles[4].motherList = {1,2};
    event.particles[5].motherList = {1,2};
    event.particles[0].statusHepMC = 0;
    event.particles[1].statusHepMC = 4;
    event.particles[2].statusHepMC = 4;
    event.particles[3].statusHepMC = 1;
    event.particles[4].statusHepMC = 1;
    event.particles[5].statusHepMC = 1;
}

double Process::fpol1(double x1) {
    double fres(0.);
    for (int i = 0; i < 100; i++) {
	if ((x1 >= xl1[i]) && (x1 < xu1[i])) {
	     fres = fp1[i];
	     break;
	}
    }
    return fres;
}

double Process::fpol2(double x2) {
    double fres(0.);
    for (int i = 0; i < 100; i++) {
	if ((x2 >= xl2[i]) && (x2 < xu2[i])) {
	     fres = fp2[i];
	     break;
	}
    }
    return fres;
}

double Process::Lx1x2(double x1, double x2) {
    double fres(0.);
    double xmax;
    xmax = 2.*info.Eamax/info.Ecm;
    int i,j;
    for (int k = 0; k < 10000; k++) {
	i = k%100, j = k/100;
	 if ((x1 >= i*xmax/100. && x1 < (i+1)*xmax/100.) && (x2 >= j*xmax/100. && x2 < (j+1)*xmax/100.)) {
		fres = fa12[k];
		break;
	}
    }
    return fres;
}

double Process::fEPA1(double x) {
    double nIWW;
    double mp,Q02,Qmin2,Qmax2;
    double phi1,phi2;
    double y,c1,c2,c3;
    double qmin,qmax;
    mp = 0.938;
    Q02 = 0.71;
    Qmin2 = x*x*mp*mp/(1.-x);
    Qmax2 = 2.;
    y = x*x/(1.-x);
    c1 = 8.78/4.+ 4.*mp*mp/Q02;
    c2 = 1.-4.*mp*mp/Q02;
    c3 = 6.78/pow(c2,4);
    qmin = Qmin2/Q02;
    qmax = Qmax2/Q02;
    phi1 = (1.+c1*y)*(-log((1.+qmin)/qmin)+1./(1+qmin)+1./pow2(1.+qmin)/2.+1./pow(1.+qmin,3)/3.)
    +(1.-c2)*y/4./qmin/pow(1.+qmin,3)
    +c3*(1.+y/4.)*(log((1+qmin-c2)/(1+qmin))+c2/(1+qmin)+pow2(c2)/pow2(1.+qmin)/2.+pow(c2,3)/pow(1.+qmin,3)/3.);
    phi2 = (1.+c1*y)*(-log((1.+qmax)/qmax)+1./(1+qmax)+1./pow2(1.+qmax)/2.+1./pow(1.+qmax,3)/3.)
    +(1.-c2)*y/4./qmax/pow(1.+qmax,3)
    +c3*(1.+y/4.)*(log((1+qmax-c2)/(1+qmax))+c2/(1+qmax)+pow2(c2)/pow2(1.+qmax)/2.+pow(c2,3)/pow(1.+qmax,3)/3.);
    nIWW = 1./info.Alphai/consts_.pi*(1.-x)*(phi1-phi2);
    return nIWW;
}

double Process::fEPA2(double x) {
    double nEPA;
    double mp(0.938),ra(4.41),gammal;
    double Z(1.),A(1.);
    double chi;
    double K0,K1;
    chi = x*mp*ra*A;
    gammal = info.Ecm/2./(mp);
    K0 = TMath::BesselK0(chi);
    K1 = TMath::BesselK1(chi);
    nEPA = 2.*Z*Z/info.Alphai/consts_.pi*(chi*K0*K1-(1.-1/gammal/gammal)*chi*chi/2.*(K1*K1-K0*K0));
    return nEPA;
}

void Process::VariablesLOOP(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double x0(info.x0),x1,x2;
    double Eamax(info.Eamax);
    double Factor(1.);
//--------------------------------------------------------------------------------------------------
    if (info.Ipdf == -1 || info.Ipdf == 1) Eamax = x0/(1.+x0)*info.Ecm/2.;
    if (info.Ipdf == -2 || info.Ipdf == 2) x0 = Eamax/(info.Ecm/2.-Eamax);
    if (info.Ipdf == -3 || info.Ipdf == 3) Eamax = info.Ecm/2.;
    if (info.Ipdf == 0) {
	Vl[0] = 0.;
	Vu[0] = 1.;
	Vl[1] = 0.;
	Vu[1] = 1.;
    } else {
	if (info.Ivar == 0) {
	    Vl[0] = 0.;                // x1 min value
	    Vu[0] = 2.*Eamax/info.Ecm; // x1 max value
	    Vl[1] = 0.;                // x2 min value
	    Vu[1] = 2.*Eamax/info.Ecm; // x2 max value
	} else {
	    Vl[0] = 0.;                    // tau min value
	    Vu[0] = x0*x0/(1.+x0)/(1.+x0); // tau max value
	    Vl[1] = x0/(1.+x0)*Xarg[0];    // x1 min value
	    Vu[1] = x0/(1.+x0);            // x1 max value
	    if (info.Ipdf == -3 || info.Ipdf == 3) {
		Vl[0] = 0.;         // tau min value
		Vu[0] = 1.;         // tau max value
		Vl[1] = Xarg[0];    // x1 min value
		Vu[1] = 1.;         // x1 max value
	    }
	}
    }
    Vl[2] = -1.;             // cos(theta_3) min value in cms frame
    Vu[2] =  1.;             // cos(theta_3) max value in cms frame
    Vl[3] = 0.;              // phi_3 min value in lab frame
    Vu[3] = 2.*consts_.pi;   // phi_3 max value in lab frame
    Vl[4] = 0.;
    Vu[4] = 1.;
    Vl[5] = 0.;
    Vu[5] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
    if (info.Ipdf == 0) {
	*Jac = 1./2./consts_.pi;
	x1 = 1.;
	x2 = 1.;
    } else {
	if (info.Ivar == 0) {
	    *Jac = 1./2./consts_.pi;
	    x1 = V[0];
	    x2 = V[1];
	} else {
	    *Jac = 1./2./consts_.pi/V[1];
	    x1 = V[1];
	    x2 = V[0]/V[1];
	}
    }
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = x1;
    Z[1] = x2;
    Z[2] = V[2];
    Z[3] = V[3];
    Z[4] = V[4];
    Z[5] = V[5];
}

bool Process::CutsPrelim(double *Z) {
    bool cuts = false;
    double x0(info.x0),x1,x2,sqS;
    double Eamax(info.Eamax);
    double xmax;
    if (info.Ipdf == -1 || info.Ipdf == 1) Eamax = x0/(1.+x0)*info.Ecm/2.;
    if (info.Ipdf == -2 || info.Ipdf == 2) x0 = Eamax/(info.Ecm/2.-Eamax);
    if (info.Ipdf == -3 || info.Ipdf == 3) Eamax = info.Ecm/2.;
    xmax = 2.*Eamax/info.Ecm;
    x1 = Z[0];
    x2 = Z[1];
    sqS = info.Ecm*sqrt(x1*x2);
    if (info.m34cut && ((sqS < info.m34min) || (sqS > info.m34max))) cuts = true;
    if ( (info.Ipdf != 0) && ((x1 >= xmax) || (x2 >= xmax)) ) cuts = true;
    return cuts;
}

void Process::GetFourMomenta(double *Z, Vec4 *P) {
    double x1,x2,Cth3,Phi3;
    double Sth3,Cph3,Sph3;
    double beta;
    double kappa3,kappa4,beta3,beta4;
    double sqS,S;
//--------------------------------------------------------------------------------------------------
    x1 = Z[0];
    x2 = Z[1];
    Cth3 = Z[2];
    Phi3 = Z[3];
//--------------------------------------------------------------------------------------------------
    Sth3 = sqrt(1.-Cth3*Cth3);
    Cph3 = cos(Phi3);
    Sph3 = sin(Phi3);
    sqS = info.Ecm*sqrt(x1*x2);
    S = sqS*sqS;
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    beta  = (x1-x2)/(x1+x2);
//--------------------------------------------------------------------------------------------------
    P[0] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
    P[1] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
    P[2] = Vec4(0., 0., sqS/2.*kappa3*beta3, sqS/2.*kappa3).rot(Sth3,Cth3,Sph3,Cph3).bstZ(beta);
    P[3] = Vec4(0., 0., -sqS/2.*kappa4*beta4, sqS/2.*kappa4).rot(Sth3,Cth3,Sph3,Cph3).bstZ(beta);
}

bool Process::Cuts(Vec4 *P) {
    bool cuts = false;
    Vec4 v3 = P[2];
    Vec4 v4 = P[3];
    double Theta3 = 180./consts_.pi*acos(v3.pz()/sqrt(v3.px()*v3.px()+v3.py()*v3.py()+v3.pz()*v3.pz()));
    double Theta4 = 180./consts_.pi*acos(v4.pz()/sqrt(v4.px()*v4.px()+v4.py()*v4.py()+v4.pz()*v4.pz()));
    double Cost3 = v3.pz()/sqrt(v3.px()*v3.px()+v3.py()*v3.py()+v3.pz()*v3.pz());
    double Cost4 = v4.pz()/sqrt(v4.px()*v4.px()+v4.py()*v4.py()+v4.pz()*v4.pz());
    double Eta3L = v3.eta();
    double Eta4L = v4.eta();
    double Pt3 = v3.pT();
    double Pt4 = v4.pT();
    double M34 = m(v3,v4);
    if (info.theta3cut && ((Theta3 < info.theta3min) || (Theta3 > info.theta3max))) cuts = true;
    if (info.theta4cut && ((Theta4 < info.theta4min) || (Theta4 > info.theta4max))) cuts = true;
    if (info.cost3cut && ((Cost3 < info.cost3min) || (Cost3 > info.cost3max))) cuts = true;
    if (info.cost4cut && ((Cost4 < info.cost4min) || (Cost4 > info.cost4max))) cuts = true;
    if (info.eta3cut && ((Eta3L < info.eta3min) || (Eta3L > info.eta3max))) cuts = true;
    if (info.eta4cut && ((Eta4L < info.eta4min) || (Eta4L > info.eta4max))) cuts = true;
    if (info.pt3cut && ((Pt3 < info.pt3min) || (Pt3 > info.pt3max))) cuts = true;
    if (info.pt4cut && ((Pt4 < info.pt4min) || (Pt4 > info.pt4max))) cuts = true;
    if (info.m34cut && ((M34 < info.m34min) || (M34 > info.m34max))) cuts = true;
    return cuts;
}

void Process::PdfConv(double *Z, double *AA) {
    double x1,x2,z1,z2;
    double x0(info.x0),r1,r2,f01,f11,f02,f12,d1,d2;
    double c1,c2;
    double Pe1(info.Pe1),Pg1(info.Pg1),Pe2(info.Pe2),Pg2(info.Pg2);
    double Rho(info.Rho),A(info.A);
    double g1,g2,Psi,lumfactor;
    double xmax;
    xmax = 2.*info.Eamax/info.Ecm;
    x1 = Z[0];
    x2 = Z[1];
    z1 = Z[4];
    z2 = Z[5];
    g1 = sqrt(fabs(x0/x1-x1-1.));
    g2 = sqrt(fabs(x0/x2-x2-1.));
    Psi = A*A*(g1*cos(2.*consts_.pi*z1)+g2*cos(2.*consts_.pi*z2))*(g1*cos(2.*consts_.pi*z1)+g2*cos(2.*consts_.pi*z2))
             +(g1*sin(2.*consts_.pi*z1)+g2*sin(2.*consts_.pi*z2))*(g1*sin(2.*consts_.pi*z1)+g2*sin(2.*consts_.pi*z2));
    lumfactor = exp(-Rho*Rho*Psi/4./(1.+A*A));
    r1 = x1/x0/(1.-x1);
    r2 = x2/x0/(1.-x2);
    f01 = 1./(1.-x1)+(1.-x1)-4.*r1*(1-r1);
    f11 = x1/(1.-x1)*(1.-2.*r1)*(2.-x1);
    f02 = 1./(1.-x2)+(1.-x2)-4.*r2*(1-r2);
    f12 = x2/(1.-x2)*(1.-2.*r2)*(2.-x2);
    d1 =      (1.-4./x0-8./x0/x0)*log(1.+x0)+1./2.+8./x0-1./2./(1.+x0)/(1.+x0)
    +Pe1*Pg1*((1.+2./x0)*log(1.+x0)-5./2.+1./(1.+x0)-1./2./(1.+x0)/(1.+x0));
    d2 =      (1.-4./x0-8./x0/x0)*log(1.+x0)+1./2.+8./x0-1./2./(1.+x0)/(1.+x0)
    +Pe2*Pg2*((1.+2./x0)*log(1.+x0)-5./2.+1./(1.+x0)-1./2./(1.+x0)/(1.+x0));
    c1 = f01+Pe1*Pg1*f11;
    c2 = f02+Pe2*Pg2*f12;
    *AA = c1*c2/d1/d2*lumfactor;
    if (info.Ipdf == -2 || info.Ipdf == 2) {
	*AA = Lx1x2(x1,x2)*100.*100./xmax/xmax/info.Lumee;
    }
    if (info.Ipdf == -3 || info.Ipdf == 3) {
	*AA = fEPA1(x1)*fEPA1(x2);
    }
}

void Process::Loop(double *Z, double *Loop) {
    double x1,x2,Cth3,Phi3;
    double kappa3,kappa4,beta3,beta4;
    double sqS,S,T,U;
    double Nsig;
    double x0(info.x0),r1,r2,f01,f11,f21,f31,f02,f12,f22,f32;
    double c1,c2,ksi21,ksi22,ksi31,ksi32;
    double Pe1(info.Pe1),Pg1(info.Pg1),Pt1(info.Pt1),Pe2(info.Pe2),Pg2(info.Pg2),Pt2(info.Pt2);
    std::complex<double> AmpAA[2][2][2][2]; // indecies in reverse order compared to fortran array: A[i][j][k][l] = A(l+1,k+1,j+1,i+1);
    std::complex<double> AmpAZ[3][2][2][2];
    std::complex<double> AmpZZ[3][3][2][2];
    std::complex<double> AmpEE[2][2][2][2];
    std::complex<double> Amp2mix[2][2][2][2];
    std::complex<double> Rho1[2][2], Rho2[2][2];
    const std::complex<double> I_(0.,1.);
    x1 = Z[0];
    x2 = Z[1];
    Cth3 = Z[2];
    Phi3 = Z[3];
    r1 = x1/x0/(1.-x1);
    r2 = x2/x0/(1.-x2);
    f01 = 1./(1.-x1)+(1.-x1)-4.*r1*(1-r1);
    f11 = x1/(1.-x1)*(1.-2.*r1)*(2.-x1);
    f21 = x0*r1*(1.+(1.-x1)*(1.-2.*r1)*(1.-2.*r1));
    f31 = (1.-2.*r1)*(1./(1.-x1)+(1.-x1));
    f02 = 1./(1.-x2)+(1.-x2)-4.*r2*(1-r2);
    f12 = x2/(1.-x2)*(1.-2.*r2)*(2.-x2);
    f22 = x0*r2*(1.+(1.-x2)*(1.-2.*r2)*(1.-2.*r2));
    f32 = (1.-2.*r2)*(1./(1.-x2)+(1.-x2));
    c1 = f01+Pe1*Pg1*f11;
    c2 = f02+Pe2*Pg2*f12;
    ksi21 = (Pe1*f21+Pg1*f31)/c1;
    ksi22 = (Pe2*f22+Pg2*f32)/c2;
    ksi31 = 2.*r1*r1*Pt1/c1;
    ksi32 = 2.*r2*r2*Pt2/c2;
    if (info.Ipdf == 0) {
	Rho1[0][0] = 0.5*(1.+info.Polcirc1);
	Rho1[1][0] = -0.5*info.Pollin1*exp(-2.*I_*(info.Polphi1*consts_.pi/180.0-Phi3));
	Rho1[0][1] = std::conj(Rho1[1][0]);
	Rho1[1][1] = 0.5*(1.-info.Polcirc1);

	Rho2[0][0] = 0.5*(1.+info.Polcirc2);
	Rho2[1][0] = -0.5*info.Pollin2*exp(-2.*I_*(info.Polphi2*consts_.pi/180.0-Phi3));
	Rho2[0][1] = std::conj(Rho2[1][0]);
	Rho2[1][1] = 0.5*(1.-info.Polcirc2);
    } else if (info.Ipdf == 1) {
	Rho1[0][0] = 0.5*(1.+ksi21);
	Rho1[1][0] = -0.5*ksi31*exp(-2.*I_*(info.Phi1*consts_.pi/180.0-Phi3));
	Rho1[0][1] = std::conj(Rho1[1][0]);
	Rho1[1][1] = 0.5*(1.-ksi21);

	Rho2[0][0] = 0.5*(1.+ksi22);
	Rho2[1][0] = -0.5*ksi32*exp(-2.*I_*(info.Phi2*consts_.pi/180.0-Phi3));
	Rho2[0][1] = std::conj(Rho2[1][0]);
	Rho2[1][1] = 0.5*(1.-ksi22);
    } else if (info.Ipdf == 2) {
	Rho1[0][0] = 0.5*(1.+fpol1(x1));
	Rho1[1][0] = 0.;
	Rho1[0][1] = 0.;
	Rho1[1][1] = 0.5*(1.-fpol1(x1));

	Rho2[0][0] = 0.5*(1.+fpol2(x2));
	Rho2[1][0] = 0.;
	Rho2[0][1] = 0.;
	Rho2[1][1] = 0.5*(1.-fpol2(x2));
    } else if (info.Ipdf == 3) {
	Rho1[0][0] = 0.5;
	Rho1[1][0] = 0.;
	Rho1[0][1] = 0.;
	Rho1[1][1] = 0.5;

	Rho2[0][0] = 0.5;
	Rho2[1][0] = 0.;
	Rho2[0][1] = 0.;
	Rho2[1][1] = 0.5;
    }
    sqS = info.Ecm*sqrt(x1*x2);
    S = sqS*sqS;
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth3);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth3);
    if (info.Pid == 301) {
	Nsig = consts_.conhc/64./consts_.pi/S;
	bbbb_ha_11_11_amp_(&S,&T,&U,AmpAA);
	for (int k = 0; k <= 1; ++k) {
	    for (int l = 0; l <= 1; ++l) {
		for (int i = 0; i <= 1; ++i) {
		    for (int j = 0; j <= 1; ++j) {
			Amp2mix[l][j][k][i] = 0.;
			for (int m = 0; m <= 1; ++m) {
			    for (int n = 0; n <= 1; ++n) {
				if (info.Iqed == 1) {
				    Amp2mix[l][j][k][i] = Amp2mix[l][j][k][i]+AmpAA[n][m][k][i]*std::conj(AmpAA[n][m][l][j])*Rho1[j][i]*Rho2[l][k];
				} else {
				    Amp2mix[l][j][k][i] = Amp2mix[l][j][k][i]+AmpAA[n][m][k][i]*std::conj(AmpAA[n][m][l][j])*Rho1[j][i]*Rho2[l][k];
				}
			    }
			}
			*Loop = *Loop + std::real(Amp2mix[l][j][k][i]);
		    }
		}
	    }
	}
    } else if (info.Pid == 302) {
	Nsig = consts_.conhc/32./consts_.pi/S*(1.-info.MZ*info.MZ/S);
	bbbb_ha_11_12_amp_(&S,&T,&U,AmpAZ);
	for (int k = 0; k <= 1; ++k) {
	    for (int l = 0; l <= 1; ++l) {
		for (int i = 0; i <= 1; ++i) {
		    for (int j = 0; j <= 1; ++j) {
			Amp2mix[l][j][k][i] = 0.;
			for (int m = 0; m <= 1; ++m) {
			    for (int n = 0; n <= 2; ++n) {
				Amp2mix[l][j][k][i] = Amp2mix[l][j][k][i]+AmpAZ[n][m][k][i]*std::conj(AmpAZ[n][m][l][j])*Rho1[j][i]*Rho2[l][k];
			    }
			}
			*Loop = *Loop + std::real(Amp2mix[l][j][k][i]);
		    }
		}
	    }
	}
    } else if (info.Pid == 303) {
	Nsig = 1000.*consts_.conhc/64./consts_.pi/S*sqrt(1.-4.*info.MZ*info.MZ/S);
	bbbb_ha_11_22_amp_(&S,&T,&U,AmpZZ);
	for (int k = 0; k <= 1; ++k) {
	    for (int l = 0; l <= 1; ++l) {
		for (int i = 0; i <= 1; ++i) {
		    for (int j = 0; j <= 1; ++j) {
			Amp2mix[l][j][k][i] = 0.;
			for (int m = 0; m <= 2; ++m) {
			    for (int n = 0; n <= 2; ++n) {
				Amp2mix[l][j][k][i] = Amp2mix[l][j][k][i]+AmpZZ[n][m][k][i]*std::conj(AmpZZ[n][m][l][j])*Rho1[j][i]*Rho2[l][k];
			    }
			}
			*Loop = *Loop + std::real(Amp2mix[l][j][k][i]);
		    }
		}
	    }
	}
    } else if (info.Pid == 304) {
	Nsig = consts_.conhc/32./consts_.pi/S*sqrt(1.-4.*info.Mel*info.Mel/S)*pow(consts_.e,4);
	bbff_ha_11_1212_amp_(&S,&T,&U,AmpEE);
	for (int k = 0; k <= 1; ++k) {
	    for (int l = 0; l <= 1; ++l) {
		for (int i = 0; i <= 1; ++i) {
		    for (int j = 0; j <= 1; ++j) {
			Amp2mix[l][j][k][i] = 0.;
			for (int m = 0; m <= 1; ++m) {
			    for (int n = 0; n <= 1; ++n) {
				Amp2mix[l][j][k][i] = Amp2mix[l][j][k][i]+AmpEE[n][m][k][i]*std::conj(AmpEE[n][m][l][j])*Rho1[j][i]*Rho2[l][k];
			    }
			}
			*Loop = *Loop + std::real(Amp2mix[l][j][k][i]);
		    }
		}
	    }
	}
    }
    *Loop = (*Loop)*Nsig;
}
