#include "ReneSANCe_aa.h"
#include "TRandom3.h"
#include <fstream>
#include <iostream>
#include <iomanip>
#include <sys/stat.h>
#include <complex>
using namespace std;
typedef complex<double> double_complex;
#include "TFoam.h"
#include "Integrand.h"
#include "TFile.h"
#include "TTree.h"
#include "Info.h"
#include "Process.h"
#include "ModulesInterface.h"
#include "Time.h"
#include "DataSaver.h"
#include "clooptools.h"

extern void LHEInit(void);

using namespace SANC;

#define SP12 setprecision(12) << setw(21)
#define SW18 setw(18)
#define SW3  setw(3)
#define SW6  setw(6)

ReneSANCe_aa::ReneSANCe_aa(Settings & _cfg, Info & _info, Process & _proc, ModulesInterface & _mod, Time & _time, DataSaver & _ds)
:cfg(_cfg),timing(_time),info(_info),proc(_proc),modules(_mod),ds(_ds)
{
    PseRan=new TRandom3();
    PseRan->SetSeed(info.RanSeed);
    ffini();
}

void ReneSANCe_aa::generateGrid()
{

}

void ReneSANCe_aa::generateEvents()
{
    TFoam *FoamLoop;
    TFoamIntegrand *Density0;
    TFile *fileLoop;
    if (info.IEXPLLOOP) {
        FoamLoop = new TFoam("FoamLoop");   // Create Simulator
        Density0 = new Integrand_aa(info, proc, pdf, 20);   // Create integrand function
        FoamLoop->SetkDim(info.kDimLoop);
        FoamLoop->SetnCells(info.nCellsLoop);
        FoamLoop->SetnSampl(info.nSamplLoop);
        FoamLoop->SetnBin(info.nBinLoop);
        FoamLoop->SetEvPerBin(info.EvPerBinLoop);
        FoamLoop->SetOptRej(info.OptRej);
        FoamLoop->SetOptDrive(info.OptDrive);
        FoamLoop->SetMaxWtRej(info.MaxWtRejLoop);
        FoamLoop->SetChat(info.Chat);
        FoamLoop->SetRho(Density0);
        FoamLoop->SetPseRan(PseRan);
        FoamLoop->Initialize();
        TFile RootLoopFile("FoamLoop.root","RECREATE","Loop");
        FoamLoop->Write("FoamLoop");    // Writing Foam on the disk
    }
    else {
        fileLoop = new TFile("FoamLoop.root"); // connect disk file
        FoamLoop = (TFoam*)fileLoop->Get("FoamLoop"); // find object
        Density0 = new Integrand_aa(info, proc, pdf, 20);   // Create integrand function
        FoamLoop->SetPseRan(PseRan);
        FoamLoop->SetRho(Density0);
        FoamLoop->SetMaxWtRej(info.MaxWtRejLoop);
        FoamLoop->LinkCells(); // restore pointers of the binary tree of cells
        FoamLoop->GetTotDim();
    }
    cout << "====== Initialization done, entering MC loop" << '\n';
//--------------------------------------------------------------------------------------------------
    double *MCvectLoop = new double[info.kDimLoop];   // vector generated in the MC run
//--------------------------------------------------------------------------------------------------
    int iLoopPWt1  = 0, iLoopNWt1  = 0;
    int iLoopPWtN1 = 0, iLoopNWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    mkdir("Results", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
    string bornName="EventsLO";
//--------------------------------------------------------------------------------------------------
    ds.start(info.outputDir, bornName);
//--------------------------------------------------------------------------------------------------
    TFile *floop = new TFile("Results/EventsLO.root","recreate");
    TTree *tloop = new TTree("events_loop","loop events");
    Double_t weight_loop[1], px_loop[4], py_loop[4], pz_loop[4], E_loop[4];
    tloop->Branch("weight_loop", weight_loop, "weight_loop[1]/D");
    tloop->Branch("px_loop", px_loop, "px_loop[4]/D");
    tloop->Branch("py_loop", py_loop, "py_loop[4]/D");
    tloop->Branch("pz_loop", pz_loop, "pz_loop[4]/D");
    tloop->Branch("E_loop", E_loop, "E_loop[4]/D");
//--------------------------------------------------------------------------------------------------
    Double_t weight_var[1], var[6];
    TFile *floopvar = new TFile("Results/loopvar.root","recreate");
    TTree *tloopvar = new TTree("events_var","tot events");
    tloopvar->Branch("weight_var", weight_var, "weight_var[1]/D");
    tloopvar->Branch("var", var, "var[6]/D");
//--------------------------------------------------------------------------------------------------
	for (loop = 0; loop < info.NevLoop; loop++) {
	    FoamLoop->MakeEvent();   // generate Loop MC event
	    FoamLoop->GetMCvect(MCvectLoop);
	    MCwt = FoamLoop->GetMCwt();
	    if (MCwt == 1.0) {
		if (ds.isReady()) {
		    ds.writeEvent();
		}
		xwgtup = proc.event.weight();
		weight_loop[0] = proc.event.weight();
		px_loop[0] = proc.event.P_lab(1).px();
		px_loop[1] = proc.event.P_lab(2).px();
		px_loop[2] = proc.event.P_lab(3).px();
		px_loop[3] = proc.event.P_lab(4).px();
		py_loop[0] = proc.event.P_lab(1).py();
		py_loop[1] = proc.event.P_lab(2).py();
		py_loop[2] = proc.event.P_lab(3).py();
		py_loop[3] = proc.event.P_lab(4).py();
		pz_loop[0] = proc.event.P_lab(1).pz();
		pz_loop[1] = proc.event.P_lab(2).pz();
		pz_loop[2] = proc.event.P_lab(3).pz();
		pz_loop[3] = proc.event.P_lab(4).pz();
		E_loop[0]  = proc.event.P_lab(1).e();
		E_loop[1]  = proc.event.P_lab(2).e();
		E_loop[2]  = proc.event.P_lab(3).e();
		E_loop[3]  = proc.event.P_lab(4).e();
		if (info.PrintROOT) {
		    tloop->Fill();
		}
		weight_var[0] = proc.event.weight();
		for (int i = 0; i < 6; i++) {
		    var[i] = MCvectLoop[i];
		}
		if (info.PrintDEBUG) {
		    tloopvar->Fill();
		}
		if (xwgtup > 0.0) iLoopPWt1++;
		if (xwgtup < 0.0) iLoopNWt1++;
		if ((info.NevLoop >= 100 && ((loop+1)%(info.NevLoop/100) == 0 || loop == info.NevLoop-1)) || info.NevLoop < 100) {
		    cout << "Generating Loop event: " << setw(12) << loop+1 << "; progress: " << setw(5) << 100*(loop+1)/(info.NevLoop) << "\%" << flush << "\r";
		}
	    } else if (MCwt > 1.0) {
		xwgtup = proc.event.weight();
		if (xwgtup > 0.0) iLoopPWtN1++;
		if (xwgtup < 0.0) { iLoopNWtN1++; MCwt = -MCwt; }
	    }
	}
	cout << endl;
	floop->Write();
	floopvar->Write();
//--------------------------------------------------------------------------------------------------
    ds.finish();
//--------------------------------------------------------------------------------------------------
    double LoopF = 1.0;
    if ((iLoopPWt1+iLoopNWt1) > 0) LoopF = (double) (iLoopPWt1-iLoopNWt1)/(iLoopPWt1+iLoopNWt1);
//--------------------------------------------------------------------------------------------------
    cout << "====== Events generated, entering Finalize" << '\n';
    Double_t IntNorm, Errel;
    FoamLoop->Finalize(IntNorm, Errel);
//--------------------------------------------------------------------------------------------------
    double ErrLoop = 0.;
    double SigmaLoopAbs = 0.;
    FoamLoop->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaLoop = LoopF*MCresult;
    ErrLoop = MCerror;
    SigmaLoopAbs = MCresult;
    cout << "================================================================" << '\n';
    cout << "        MCresult (Loop) = " << info.SigmaLoop << " +- " << ErrLoop << " RelErr = " << ErrLoop/info.SigmaLoop << '\n';
    delete [] MCvectLoop;
    cout << "================================================================" << '\n';
    ofstream sigma_loop;
    sigma_loop.open("Results/EventsLO.dat");
    sigma_loop << setprecision(10) << SigmaLoopAbs << '\n';
    sigma_loop.close();
//--------------------------------------------------------------------------------------------------
    cout << "Total number of Loop events: " << iLoopPWt1 + iLoopPWtN1 << " pos. " << iLoopNWt1 + iLoopNWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Number of Loop events with Wt > 1: " << iLoopPWtN1 << " pos. " << iLoopNWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Relative number of Loop events with Wt > 1: " << (double) (iLoopPWtN1+iLoopNWtN1)/(iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1) << '\n';
    cout << "***** End of Program *****" << '\n';
}
