#include "SettingsUCL.h"
using namespace SANC;

SettingsUCL::SettingsUCL()
{}

void SettingsUCL::init()
{
    const ucl_object_t *elt, *tmp;
    ucl_object_t *cur;
    ucl_object_iter_t iter = nullptr;
    struct ucl_parser *parser = ucl_parser_new(0);
    ucl_parser_add_file(parser, input_file.c_str());
    user_obj = ucl_parser_get_object(parser);
    ucl_parser_free(parser);
    parser = ucl_parser_new(0);
    ucl_parser_add_file(parser,schema_file.c_str());
    schema_obj = ucl_parser_get_object(parser);
    ucl_parser_free(parser);
    merged_obj = ucl_object_new();
    while ((elt = ucl_object_iterate(ucl_object_lookup_path(schema_obj,"schema.properties"), &iter, true)) != nullptr) {
        tmp = ucl_object_lookup(elt,"default");
        if (tmp) {
            cur = ucl_object_copy(tmp);
            ucl_object_insert_key(merged_obj, cur, ucl_object_key(elt), 0, false);
        }
    }
    ucl_object_merge(merged_obj,user_obj,true);
}

void SettingsUCL::print_diff()
{
    const ucl_object_t *elt,*tmp;
    ucl_object_iter_t iter = nullptr;
    char buffer[50];
    std::cout << "Changed parameters:\n";
    std::cout << "|-----------------|------------------------|------------------------|\n";
    std::cout << "| setting         | value                  | default                |\n";
    std::cout << "|-----------------|------------------------|------------------------|\n";
    while ((elt = ucl_object_iterate (user_obj, &iter, true)) != nullptr) {
        sprintf (buffer, "schema.properties.%s.default", ucl_object_key(elt));
        tmp = ucl_object_lookup_path(schema_obj, buffer);
        if (!tmp) {
            fprintf (stdout, "| %-15s | %-22s | %-22s |\n",
                    ucl_object_key(elt),
                    ucl_object_tostring_forced(elt),
                    " "
            );
        } else if (ucl_object_compare(elt,tmp)!=0) {
            fprintf (stdout, "| %-15s | %-22s | %-22s |\n",
                    ucl_object_key(elt),
                    ucl_object_tostring_forced(elt),
                    ucl_object_tostring_forced(tmp)
            );
        }
    }
        std::cout << "|-----------------|------------------------|------------------------|\n";
}

bool SettingsUCL::get_b(string key)
{
    bool result;
    const ucl_object_t *elt = ucl_object_lookup_len(merged_obj, key.data(), key.size());
    if (ucl_object_toboolean_safe(elt, &result)) {
        return result;
    }
    else {
        std::cerr<<"Error in configuration! Wrong value of key:\n";
        std::cerr<<key;
        if (elt) std::cerr<<'='<<ucl_object_tostring_forced(elt);
        std::cerr<<'\n';
        std::cerr<<"Booleans can be specified as [true, yes, on] and [false, no, off].\n";
        exit(10);
    }
}

long SettingsUCL::get_l(string key)
{
    long result;
    const ucl_object_t *elt = ucl_object_lookup_len(merged_obj, key.data(), key.size());
    if (ucl_object_toint_safe(elt, &result)) {
        return result;
    }
    else {
        std::cerr<<"Error in configuration! Wrong value of key:\n";
        std::cerr<<key;
        if (elt) std::cerr<<'='<<ucl_object_tostring_forced(elt);
        std::cerr<<'\n';
        std::cerr<<"Integer value expected.\n";
        exit(10);
    }
}

double SettingsUCL::get_d(string key)
{
    double result;
    const ucl_object_t *elt = ucl_object_lookup_len(merged_obj, key.data(), key.size());
    if (ucl_object_todouble_safe(elt, &result)) {
        return result;
    }
    else {
        std::cerr<<"Error in configuration! Wrong value of key:\n";
        std::cerr<<key;
        if (elt) std::cerr<<'='<<ucl_object_tostring_forced(elt);
        std::cerr<<'\n';
        std::cerr<<"Float value expected.\n";
        exit(10);
    }
}

string SettingsUCL::get_s(string key)
{
    const char *result = nullptr;
    const ucl_object_t *elt = ucl_object_lookup_len(merged_obj, key.data(), key.size());
    if (ucl_object_tostring_safe(elt, &result)) {
        return result;
    }
    else {
        std::cerr<<"Error in configuration! Wrong value of key:\n";
        std::cerr<<key;
        if (elt) std::cerr<<'='<<ucl_object_tostring_forced(elt);
        std::cerr<<'\n';
        std::cerr<<"String value expected.\n";
        exit(10);
    }
}

void SettingsUCL::set(string key, bool val)
{
    ucl_object_t *obj2, *cur;
    obj2 = ucl_object_typed_new(UCL_OBJECT);
    cur  = ucl_object_frombool(val);
    ucl_object_insert_key(obj2, cur, key.c_str(), 0, true);
    ucl_object_merge(user_obj, obj2, true);
    ucl_object_merge(merged_obj,obj2,true);
    ucl_object_unref(cur);
    ucl_object_unref(obj2);
}

void SettingsUCL::set(string key, long val)
{
    ucl_object_t *obj2, *cur;
    obj2 = ucl_object_typed_new(UCL_OBJECT);
    cur  = ucl_object_fromint(val);
    ucl_object_insert_key(obj2, cur, key.c_str(), 0, true);
    ucl_object_merge(user_obj, obj2, true);
    ucl_object_merge(merged_obj,obj2,true);
    ucl_object_unref(cur);
    ucl_object_unref(obj2);
}

void SettingsUCL::set(string key, double val)
{
    ucl_object_t *obj2, *cur;
    obj2 = ucl_object_typed_new(UCL_OBJECT);
    cur  = ucl_object_fromdouble(val);
    ucl_object_insert_key(obj2, cur, key.c_str(), 0, true);
    ucl_object_merge(user_obj, obj2, true);
    ucl_object_merge(merged_obj,obj2,true);
    ucl_object_unref(cur);
    ucl_object_unref(obj2);
}

void SettingsUCL::set(string pair)
{
    ucl_object_t *obj2;
    struct ucl_parser *parser = ucl_parser_new(0);
    ucl_parser_add_string(parser, pair.c_str(), 0);
    obj2 = ucl_parser_get_object(parser);
    ucl_parser_free(parser);
    ucl_object_merge(user_obj, obj2, true);
    ucl_object_merge(merged_obj,obj2,true);
    ucl_object_unref(obj2);
}

void SettingsUCL::print()
{
    std::cout << ucl_object_emit(user_obj, UCL_EMIT_JSON_COMPACT) << '\n';
}

void SettingsUCL::validate() {
    struct ucl_schema_error err;
    bool match;
    memset(&err, 0, sizeof(err));
    match = ucl_object_validate(ucl_object_lookup(schema_obj, "schema"), user_obj, &err);
    if (!match) {
        fprintf(stderr, "Validation of '%s'='%s' failed:\n'%s'\nRequirements:\n",
                ucl_object_key(err.obj),
                ucl_object_tostring_forced(err.obj),
                err.msg
               );
        std::string key_name=std::string("schema.properties.")+std::string(ucl_object_key(err.obj));
        std::cerr << ucl_object_emit(ucl_object_lookup_path(schema_obj,key_name.data()), UCL_EMIT_CONFIG)<<'\n';
        exit(10);
    }
}
