#include "ReneSANCe_aa.h"
#include "SettingsUCL.h"
#include "CLI11.hpp"
#include "ModulesInterface.h"
#include "Info.h"
#include "Time.h"
#include "Process.h"
#include "DataSaver.h"

using namespace SANC;


int main(int argc, char* argv[]) {
    std::vector<std::string> other;
    std::string path;
    long seed, pid;
    double ecm;

    CLI::App options{"ReneSANCe_aa is a Monte-Carlo event generator for,"
                      "photon-photon collisions"};
    options.add_option("-f,--file", path, "Set path to index.conf file")
        ->check(CLI::ExistingFile);
    options.add_option("-s,--seed", seed, "Set seed");
    options.add_option("-p,--pid", pid, "Set process")
        ->check(CLI::Range(201,207));
    options.add_option("-e,--ecm", ecm, "Set energy of collider");
    options.add_option("-D,--define", other, "Set other settings as list of key:value");
    CLI11_PARSE(options, argc, argv);

    SettingsUCL cfg;

    if (options.count("-f")>0) {
        cfg.setPath(path);
    }
    cfg.init();
    if (options.count("-s")>0) {
        cfg.set("RanSeed", seed);
    }
    if (options.count("-p")>0) {
        cfg.set("pid", pid);
    }
    if (options.count("-e")>0) {
        cfg.set("ecm", ecm);
    }
    for (auto i:other) {
        cfg.set(i);
    }
    // cfg.print();
    cfg.validate();
    cfg.print_diff();
    Info info(cfg);
    Time timing("run1");
    Process proc(info);
    ModulesInterface modules(info);
    DataSaver ds(info, proc);
    ReneSANCe_aa * generator( new ReneSANCe_aa(cfg, info, proc, modules, timing, ds) );
    generator->generateEvents();

    return 0;
}
