# ReneSANCe

ReneSANCe MC event generator for photon-photon collisions


## Compilation
We highly recommend to build ReneSANCe in the separate directory from the source.
The ``ROOT`` framework must be available at the stage of compilation.

ReneSANCe_pp binary is for hadron-hadron collision mode only.
To compile with support for the hadron collision mode,
the ``LHAPDF`` library must be installed on the system.

If not required, then its compilation can be disabled using the cmake option:

```shell
cmake <path_to_source>
```

Run in shell to compile:

```shell
cmake <path_to_source> <other options>
cmake --build .
cmake --build . --target install
```
Using CMake variable CMAKE_INSTALL_PREFIX you can set installation directory. By default -DCMAKE_INSTALL_PREFIX=<inst_prefix> is set to <path_to_source> directory.

## Run
```shell
source <inst_prefix>/bin/renesance-init.sh
ReneSANCe -f <inst_prefix>/share/renesance/examples/301
```

The ``ReneSANCe`` event generator has a multilayer configuration system: default parameters are defined in the program, parameters are provided by a user through configuration files (higher priority)
and command line parameters (the highest priority).

Many parameters would be initialized to a default value, which the user can find in the schema files installed in system.

However, some parameters like `pid`, `ecm` do not have default value, so the user must provide it manually using a configuration file or command line interface.

The list of command line options:

```shell
   -h,--help                   Print this help message and exit
   -f,--file FILE              Set path to configuretion file
   -o,--out DIR                Set output directory
   -s,--seed INT               Set seed
   -p,--pid INT in [301 - 304] Set process
   -e,--ecm FLOAT              Set energy of collider
   --pol1 FLOAT in [-1 - 1]    Set first beam polarization
   --pol2 FLOAT in [-1 - 1]    Set second beam polarization
   -D,--define TEXT ...        Set other settings as list of key:value
```

By default the program searches for a configuration file that has the name `index.conf` in the `./input/` directory.
However, it can be provided by the ``ReneSANCe`` using the command line parameter `-f`.
Argument of `-f` can be directory containing index.conf or full path including arbitrary name of configuration file.

The input file language was built on JSON syntax, so configuration in JSON format is valid.
Our input format is less restrictive. The line comments are allowed with `#` as a separator.
Also, opening and closing brackets { } can be omitted.
The parameters are initialized using the key:value or key=value syntax.
The type of each key is defined in the schema files.



## Output

By default ``ReneSANCe`` produces only console output with cross-sections. To save this output please use console redirection to
file, for example

```shell
   ReneSANCe -f <folder with input files> | tee file_name.txt
```

Also in the run directory there are FOAM\*.root files with grid created for each branch. They can be used for further calculations with
the same input parameters when you want to produce more events on the same grid. The option `explore[branch name]`=false switches on the search
of existing FOAM*.root files with the existing grid.

Using option printROOT=true you can turn on saving of events in ROOT format. These events are saved to the Results directory by default,
but you can change the pass using option `outputDir`=/your/desired/path.
Born events are saved into the file named events_LO.root and NLO events are saved into the file named events_NLO.root. Also the .dat files
are generated with technical information which is used by plot1d and plot2d utilities.
You can produce LHE events files using the option `printROOT`=true.
Born events are saved into the file named events_LO.lhe and NLO events are saved into the file named events_NLO.lhe.

## Plots

There is a simple utility to produce plots from ROOT output files. It is compiled and installed by default.
Change directory to the folder with ROOT output files and run plot1d to create 1d histograms or plot2d to create 2d histograms. Generally
this utility is sample and can be simply modified for your needs.

## Documentation

See renesance.hepforge.org for details.

### Processes

pid=301 -- "A A -> A A"

pid=302 -- "A A -> A Z"

pid=303 -- "A A -> Z Z"

pid=304 -- "A A -> e- e+"

### Photon luminosity used

ipdf=1 -- calculated for Compton backward scattering

ipdf=2 -- use results from CAIN 

ipdf=3 -- for ion-ion collisions

If ipdf<0 then only Photon luminosity spectra calculated.
