#---Check for installed packages depending on the build options/components eamnbled -
include(ExternalProject)

#---Check for Zlib ------------------------------------------------------------------
#if(NOT builtin_zlib)
#  message(STATUS "Looking for ZLib")
#  find_Package(ZLIB)
#  if(NOT ZLIB_FOUND)
#    message(STATUS "Zlib not found. Switching on builtin_zlib option")
#    set(builtin_zlib ON CACHE BOOL "" FORCE)
#   endif()
#endif()
#if(builtin_zlib)
#  set(ZLIB_LIBRARY "")
#endif()

#---Check for GSL library---------------------------------------------------------------
#if(mathmore)
#  message(STATUS "Looking for GSL")
#  if(NOT builtin_gsl)
#    find_package(GSL)
#    if(NOT GSL_FOUND)
#      message(STATUS "GSL not found. Set variable GSL_DIR to point to your GSL installation")
#      message(STATUS "               Alternatively, you can also enable the option 'builtin_gsl' to build the GSL libraries internally'") 
#      message(STATUS "               For the time being switching OFF 'mathmore' option")
#      set(mathmore OFF CACHE BOOL "" FORCE)
#    endif()
#  else()
#    set(gsl_version 1.15)
#    message(STATUS "Downloading and building GSL version ${gsl_version}")
#    ExternalProject_Add(
#      GSL
#      URL http://mirror.switch.ch/ftp/mirror/gnu/gsl/gsl-${gsl_version}.tar.gz
#      INSTALL_DIR ${CMAKE_BINARY_DIR}
#      CONFIGURE_COMMAND <SOURCE_DIR>/configure --prefix <INSTALL_DIR>
#    )
#    set(GSL_INCLUDE_DIRS ${CMAKE_BINARY_DIR}/include)
#    set(GSL_LIBRARIES -L${CMAKE_BINARY_DIR}/lib -lgsl -lgslcblas -lm)
#  endif()
#endif()

#---Check for Python installation-------------------------------------------------------
if(python)
  message(STATUS "Looking for Python")
#  ---First look for the python interpreter and fix the version of it for the libraries--
  find_package(PythonInterp 3)
  if(NOT PYTHONINTERP_FOUND)
    find_package(PythonInterp 2)
  endif()
  if(PYTHONINTERP_FOUND)
    execute_process(COMMAND ${PYTHON_EXECUTABLE} -c "import sys;sys.stdout.write(sys.version[:3])"
                    OUTPUT_VARIABLE PYTHON_VERSION)
    message(STATUS "Found Python interpreter version ${PYTHON_VERSION} at ${PYTHON_EXECUTABLE}")
    execute_process(COMMAND ${PYTHON_EXECUTABLE} -c "import sys;sys.stdout.write(sys.prefix)"
                    OUTPUT_VARIABLE PYTHON_PREFIX)
    set(CMAKE_PREFIX_PATH ${CMAKE_PREFIX_PATH} ${PYTHON_PREFIX})
  endif()
  if(NOT PYTHONINTERP_FOUND)
    message(STATUS "Python interpreter not found. Switching off Python option.")
    set(python OFF CACHE BOOL "" FORCE)
  endif()
#  find_package(PythonLibs ${PYTHON_VERSION_STRING} REQUIRED)
#  find_package(PythonLibs 2)
#  if(NOT PYTHONLIBS_FOUND)
#    if(fail-on-missing)
#      message(FATAL_ERROR "PythonLibs package not found and python component required")
#    else()
#      set(python OFF CACHE BOOL "" FORCE)
#      message(STATUS "Python not found. Switching off python option")
#    endif()
#  else()
#  endif()
endif()

#---Check for GCCXML installation-------------------------------------------------------
#if(cintex OR reflex)
#  message(STATUS "Looking for GCCXML")
#  find_package(GCCXML)
#  if(GCCXML_FOUND)
#    set(gccxml ${GCCXML_EXECUTABLE})
#  else()
#    if(fail-on-missing)
#      message(FATAL_ERROR "GCCXML not found and cintex or reflex option required")
#    endif()
#  endif()
#endif()

#---Check for XML Parser Support-----------------------------------------------------------
#if(xml)
#  message(STATUS "Looking for LibXml2")
#  find_package(LibXml2)
#  if(NOT LIBXML2_FOUND)
#    if(fail-on-missing)
#      message(FATAL_ERROR "LibXml2 libraries not found and they are required (xml option enabled)")
#    else()
#      message(STATUS "LibXml2 not found. Switching off xml option")
#      set(xml OFF CACHE BOOL "" FORCE)
#    endif()
#  endif()
#endif()

#---Check for LHAPDF------------------------------------------------------------------
if(lhapdf)
    find_package(LHAPDF)
    if(NOT LHAPDF_FOUND)
        message(STATUS "LHAPDF not found. Switching off LHAPDF option.")
        set(lhapdf OFF CACHE BOOL "" FORCE)
        message(STATUS "LHAPDF=" ${lhapdf})
    endif()
endif()

#---Check for HepMC------------------------------------------------------------------
if(hepmc3)
    find_package(HepMC3 CONFIG)
  if(HepMC3_FOUND)
    add_definitions(-DHEPMC)
    MESSAGE(STATUS "Looking for HepMC3... - found ${HEPMC3_LIB}")
  else()
    message(STATUS "HepMC3 not found. Switching off HepMC3 option.")
    set(hepmc3 OFF CACHE BOOL "" FORCE)
    message(STATUS "HepMC3=" ${hepmc})
  endif()
endif()

#---Check for Pythia8-------------------------------------------------------------------
if(pythia8)
    find_package(Pythia8)
    if(NOT PYTHIA8_FOUND)
#    if(fail-on-missing)
        message(STATUS "Pythia8 libraries not found. Switching off Pythia8 option.")
#    else()
        set(pythia8 OFF CACHE BOOL "" FORCE)
#    endif()
    endif()
endif()

#---Check for Boost------------------------------------------------------------------
if(boost)
#  set(Boost_USE_STATIC_LIBS OFF)
#  set(Boost_USE_MULTITHREADED ON)
  set(Boost_USE_STATIC_RUNTIME OFF)
  find_package(Boost 1.46.0)
  if(NOT Boost_FOUND)
    message(STATUS "Boost not found. Switching off Boost option.")
    set(boost OFF CACHE BOOL "" FORCE)
  endif()
endif()

#---Check for ROOT-------------------------------------------------------------------
if(root)
  message(STATUS "Looking for ROOT framework")
  if (NOT DEFINED ROOT_DIR)
    if(DEFINED ROOTSYS)
      set (ROOT_DIR ${ROOTSYS} CACHE STRING "ROOT build directory")
    elseif(DEFINED ENV{ROOTSYS})
      set (ROOT_DIR $ENV{ROOTSYS} CACHE STRING "ROOT build directory")
    else()
      set (ROOT_DIR /opt/root CACHE STRING "ROOT build directory")
    endif()
  endif()

  find_package(ROOT 5.0 CONFIG REQUIRED COMPONENTS Foam)

  if(ROOT_FOUND)
    MESSAGE(STATUS "Looking for Root... - found ${ROOT_root_CMD}")
  else()
    message(FATAL_ERROR
    "ROOT framework not found and it is required.
    Install it from your distributive repo or from oficial site
    https://root.cern/install/
    To setup ROOT using CVMFS run
    source /cvmfs/sft.cern.ch/lcg/app/releases/ROOT/<version>/<platform>/bin/thisroot.sh
    ")
  endif()
endif()

#---Build LoopTools------------------------------------------------------------------
if(looptools)
  message(STATUS "Building LoopTools-2.1 included in ReneSANCe")
  ExternalProject_Add(
    LoopTools
    URL ${CMAKE_SOURCE_DIR}/external/LoopTools.tar.gz
    URL_HASH MD5=a3a72de3613ddfa97816654e82c3b99f
    INSTALL_DIR ${CMAKE_BINARY_DIR}
    CONFIGURE_COMMAND <SOURCE_DIR>/configure --prefix=<INSTALL_DIR>  FFLAGS=-std=legacy
    BUILD_COMMAND make libooptools.a
    LOG_DOWNLOAD 1 LOG_CONFIGURE 1 LOG_BUILD 1 LOG_INSTALL 1 BUILD_IN_SOURCE 1
  )
  set(LOOPTOOLS_INCLUDE_DIR ${CMAKE_BINARY_DIR}/include)
  set(LOOPTOOLS_LIBRARY_DIR ${CMAKE_BINARY_DIR}/lib)
  set(LOOPTOOLS_LIBRARIES ${LOOPTOOLS_LIBRARY_DIR}/libooptools.a)
endif()

#---Build libucl------------------------------------------------------------------
if(libucl)
  message(STATUS "Building libucl included in ReneSANCe")
  ExternalProject_Add(
    libucl
    URL ${CMAKE_SOURCE_DIR}/external/libucl.tar.gz
    URL_HASH MD5=4b9d0bb15aecda025d03cdc080d65cd6
    CMAKE_ARGS
        -DCMAKE_C_FLAGS=-std=c11\ -Wno-error=implicit-function-declaration
    INSTALL_DIR ${CMAKE_BINARY_DIR}
    INSTALL_COMMAND ${CMAKE_COMMAND} -E copy_if_different libucl.a <INSTALL_DIR>/lib/libucl.a
    COMMAND ${CMAKE_COMMAND} -E copy_directory ${CMAKE_BINARY_DIR}/libucl-prefix/src/libucl/include <INSTALL_DIR>/include
    LOG_DOWNLOAD 1 LOG_CONFIGURE 1 LOG_BUILD 1 LOG_INSTALL 1 BUILD_IN_SOURCE 0
  )
  set(LIBUCL_INCLUDE_DIR ${CMAKE_BINARY_DIR}/include)
  set(LIBUCL_LIBRARY_DIR ${CMAKE_BINARY_DIR}/lib)
  set(LIBUCL_LIBRARIES ${LIBUCL_LIBRARY_DIR}/libucl.a)
endif()

#---Build CLI11------------------------------------------------------------------
if(cli11)
  message(STATUS "Building cli11 included in ReneSANCe")
  ExternalProject_Add(
    cli11
    SOURCE_DIR ${CMAKE_SOURCE_DIR}/external/CLI11
    CONFIGURE_COMMAND ""
    BUILD_COMMAND ""
    INSTALL_DIR ${CMAKE_BINARY_DIR}
    INSTALL_COMMAND ${CMAKE_COMMAND} -E copy_if_different CLI11.hpp <INSTALL_DIR>/include/CLI11.hpp
    LOG_DOWNLOAD 1 LOG_CONFIGURE 1 LOG_BUILD 1 LOG_INSTALL 1 BUILD_IN_SOURCE 1
  )
  set(CLI11_INCLUDE_DIR ${CMAKE_BINARY_DIR}/include)
endif()

#if(builtin_looptools)
#  add_subdirectory(external/LoopTools)
#  set(LOOPTOOLS_INCLUDE_DIR ${CMAKE_SOURCE_DIR}/external/LoopTools/include)
#  set(LOOPTOOLS_LIBRARY_DIR ${CMAKE_SOURCE_DIR}/external/LoopTools)
#  set(LOOPTOOLS_LIBRARIES ${LOOPTOOLS_LIBRARY_DIR}/libooptools.a)
#  set(LOOPTOOLS_FOUND 1)
#endif()

if(openmp)
  find_package(OpenMP)
endif()

#---Report non implemented options---------------------------------------------------
#foreach(opt afs chirp clarens cling dcache gfal glite globus hdfs lzma pch peac pgsql sapdb srp)
#  if(${opt})
#    message(STATUS ">>> Option '${opt}' not implemented yet! Signal your urgency to pere.mato@cern.ch")
#    set(${opt} OFF CACHE BOOL "" FORCE)
#  endif()
#endforeach()

