************************************************************************
* File (bbbb_ha_11_22.f)
************************************************************************
* This is the FORTRAN module (bbbb_ha_11_22) to calculate EW
* cross-section (width) by using the helicity amplitudes
* for the a + a -> z + z process.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2015.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine bbbb_ha_11_22_amp (s,t,u,amp)
      implicit none!
#include "looptools.h"
#include "s2n_declare.h"

      integer i,j,k,l,m,n,lu,lm,ii,jj,ihaa2zz,iaa2zz
      integer iph1,iph2,iz1,iz2
      character ipm*(2),itl*(1)
      real*8 sum,nsig,ewcoeff,qedcoeff

      complex*16 b0fmzff,b0fmzww,b0fqsff,b0fqsww,b0ftsff,b0fusff,b0ftsww
      complex*16 b0fusww
      complex*16 c0ccqsfff,c0ccqswww,c0cmztsfff,c0cmztswww,c0cmzusfff
      complex*16 c0cmzuswww,c0mzmzqsfff,c0mzmzqswww
      complex*16 d0ccmzmzqstsffff
      complex*16 d0ccmzmzqstswwww,d0ccmzmzqsusffff,d0ccmzmzqsuswwww
      complex*16 d0cmzcmztsusffff,d0cmzcmztsuswwww
      complex*16 denh12p,sigmac
      real*8 betaZfun,Yfun,s2fun,s4fun,t1fun,u1fun,af,vf,qff,deltafun
      
      complex*16 ewamp(2,2,3,3),qedamp(2,2,3,3),qedsum(2,2,3,3),amp(2,2,3,3)
      real*8 mgen2,cqfactor,sigmat
      complex*16 mgen2i,rfm2(12),cfm2(12),cfa(12)
      common /aa2zz/ihaa2zz,iaa2zz

c!!!      nsig=1000d0*conhc/32/pi/s*(1-rzm2/s) ! pb goes to fb
      nsig=1000d0*conhc/32/pi/s*sqrt(1d0-4d0*mz**2/s)/4d0 ! pb goes to fb
c      ewcoeff = 1d0*alpha**2*ctw/stw
c      qedcoeff = 1d0*alpha**2
      ewcoeff = 1d0/(4d0*pi)**2
      qedcoeff = 1d0

***************************
*  EW helicity amplitudes *
***************************
* {{{

      do iph1=1,2
         do iph2=1,2
            do iz1=1,3
               do iz2 = 1,3
                  ewamp(iph1,iph2,iz1,iz2) = dcmplx(0d0,0d0)
                  qedamp(iph1,iph2,iz1,iz2) = dcmplx(0d0,0d0)
                  qedsum(iph1,iph2,iz1,iz2) = dcmplx(0d0,0d0)
               enddo
            enddo
         enddo
      enddo

      betaZfun = sqrt(1 - 4*mz**2/s)
      Yfun = t*u-mz**4
      s2fun = s-2*mz**2
      s4fun = s-4*mz**2
      t1fun = t-mz**2
      u1fun = u-mz**2
      deltafun = sqrt(s*mz**2/2/Yfun)
      denh12p = 1d0/(mh**2-s)

c$$$      print *, s,t,u
c$$$      print *, Yfun
c$$$      print *, s2fun
c$$$      print *, s4fun
c$$$      print *, t1fun
c$$$      print *, u1fun
c$$$      print *, deltafun
c$$$      print *, denh12p
c$$$      stop
            
      if (iew.eq.1) then

      b0fmzww          = b0f(-rzm2,thmu2,wm2,wm2)
      b0fqsww          = b0f(-s,thmu2,wm2,wm2)
      b0ftsww          = b0f(-t,thmu2,wm2,wm2)
      b0fusww          = b0f(-u,thmu2,wm2,wm2)
      c0ccqswww        = -c0(0d0,0d0,s,rwm2,rwm2,rwm2)
      c0cmztswww       = -c0(0d0,rzm2,t,rwm2,rwm2,rwm2)
      c0cmzuswww       = -c0(0d0,rzm2,u,rwm2,rwm2,rwm2)
      c0mzmzqswww      = -c0(rzm2,rzm2,s,rwm2,rwm2,rwm2)
      d0ccmzmzqstswwww = d0(0d0,0d0,rzm2,rzm2,s,t,rwm2,rwm2,rwm2,rwm2)
      d0ccmzmzqsuswwww = d0(0d0,0d0,rzm2,rzm2,s,u,rwm2,rwm2,rwm2,rwm2)
      d0cmzcmztsuswwww = d0(0d0,rzm2,0d0,rzm2,t,u,rwm2,rwm2,rwm2,rwm2)

      ewamp(1,1,1,1) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s*t1fun**(-1)*u1fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 4.D0
     &    *mz**2*s*t1fun**(-1)*u1fun**(-1) + 24.D0*ctw**2*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2
     &     *s*t1fun**(-1)*u1fun**(-1) )
c$$$      print *, g,stw,ctw,mz
c$$$      print *, ewamp(1,1,1,1)
c$$$      stop
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*denh12p * ( 6.D0
     &    *ctw**(-2)*rwm2 + ctw**(-2)*mh**2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 1.D0/2.D0*ctw**(-2)*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 - 8.D0*s**(-1)*s2fun*s4fun**(-1)*
     &    Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 - 4.D0*mz**2*s**(-1)*Yfun + 6.D0*ctw**2*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 - 8.D0*ctw**2*s**(-1)*s2fun*Yfun + 24.D0
     &    *ctw**2*rwm2**2 - 16.D0*ctw**2*s2fun*rwm2 + 4.D0*ctw**2*s*
     &    s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D
     &    0*ctw**2*mz**4 )
c$$$      print *, ewamp(1,1,1,1)
c$$$      stop
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*ctw**(-2)*
     &    s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s2fun*
     &    s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0
     &    *s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 4.D0*s**(-2)*u*s2fun*
     &    s4fun**(-1)*Yfun - 8.D0*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*rwm2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 8.D0*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2
     &    *rzm2 + 8.D0*mz**2*s**(-1)*rzm2 + 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 - 8.
     &    D0*mz**2*s**(-1)*u - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*
     &    Yfun*rzm2 + 12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 
     &    24.D0*ctw**2*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2
     &     + 16.D0*ctw**2*s**(-1)*s2fun*rzm2 + 48.D0*ctw**2*s**(-1)*u*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*ctw**2*s**(-1)*u*s2fun + 24.D0
     &    *ctw**2*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*
     &    rwm2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*c0cmztswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - ctw**(-2)*s**(-1)*
     &    s2fun*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 + 2.D
     &    0*ctw**(-2)*mz**2*s**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*rwm2 + 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2
     &     + 4.D0*s**(-2)*u*s2fun*s4fun**(-1)*Yfun - 8.D0*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 8.
     &    D0*s2fun*s4fun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2
     &     + 8.D0*mz**2*s**(-1)*rzm2 - 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 8.
     &    D0*mz**2*s**(-1)*u - 16.D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*
     &    mz**2 + 32.D0*mz**4*s**(-1)*s4fun**(-1)*rwm2 - 16.D0*mz**4*
     &    s**(-1) - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 
     &    12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    s**(-1)*t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - 12.D0*
     &    ctw**2*s**(-1)*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s**(-1)*
     &    s2fun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 
     &    16.D0*ctw**2*s**(-1)*u*s2fun - 24.D0*ctw**2*s2fun*s4fun**(-1)
     &    *rwm2 + 16.D0*ctw**2*s2fun + 24.D0*ctw**2*mz**2*s**(-2)*s2fun
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2
     &     - 32.D0*ctw**2*mz**2*s**(-1)*s2fun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s4fun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*rwm2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 16.D0*mz**2*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*c0ccqswww*
     & denh12p * (  - 12.D0*ctw**(-2)*rwm2**2 + 8.D0*ctw**(-2)*s*rwm2
     &     - 2.D0*ctw**(-2)*mh**2*rwm2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) + 3.D0/2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s2fun*s4fun**(-1) + 2.D0
     &    *u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1) + 2.D0*s*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 4.D0*mz**2
     &    *s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0
     &    *ctw**2*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1) - 6.
     &    D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*
     &    ctw**2*mz**2*s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 48.D
     &    0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) + 18.D0*ctw**2*mz**2
     &    *t1fun**(-1)*s2fun*s4fun**(-1) + 6.D0*ctw**2*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) + 4.D0*s**(-1)*u*s2fun*s4fun**(-1) + 2.D0*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*mz**2*s**(-1)*s2fun
     &    *s4fun**(-1) + 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D
     &    0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0*ctw**2*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*ctw**2*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) + ctw**(-2)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*t1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 3.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*s**(-1)*u*s2fun*s4fun**(-1) - 4.D0*s2fun*
     &    s4fun**(-1) - 2.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 6.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     + 12.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) - 
     &    12.D0*ctw**2*mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun
     &     - 36.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) - 18.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun * ( 
     &    ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 4.D0*s
     &    *t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 12.D0*ctw**2*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*
     &    Yfun**2 + 2.D0*ctw**(-2)*s4fun**(-1)*Yfun*rwm2 - 2.D0*s**(-1)
     &    *s4fun**(-1)*Yfun**2 - 8.D0*s4fun**(-1)*Yfun*rwm2 - 4.D0*
     &    mz**2*s4fun**(-1)*Yfun + 6.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    Yfun**2 + 24.D0*ctw**2*s4fun**(-1)*Yfun*rwm2 - 8.D0*ctw**2*
     &    Yfun - 16.D0*ctw**2*s*rwm2 + 4.D0*ctw**2*s**2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*mz**2*s*s2fun*s4fun**(-1) - 16.D0*
     &    ctw**2*s*rwm2 + 4.D0*ctw**2*s**2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*mz**2*s*s2fun*s4fun**(-1) - 16.D0*
     &    ctw**2*s*rwm2 + 4.D0*ctw**2*s**2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 4.D0*ctw**(-2)*u*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*rwm2 - 4.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 4.D0*s**(-1)*s4fun**(-1)*
     &    Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*Yfun - 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 16.D0*u*s4fun**(-1)*rwm2
     &     - 8.D0*s*s4fun**(-1)*rwm2 + 8.D0*mz**2*s4fun**(-1)*rzm2 + 16.
     &    D0*mz**2*s4fun**(-1)*rwm2 - 8.D0*mz**2*u*s4fun**(-1) - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun + 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 16.D0*ctw**2*rzm2 + 48.D0*ctw**2*u*s4fun**(-1)*rwm2
     &     - 16.D0*ctw**2*u + 24.D0*ctw**2*s*s4fun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - ctw**(-2)*s4fun**(-1)*
     &    Yfun - 4.D0*ctw**(-2)*u*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 4.D0*s**(-1)*
     &    s4fun**(-1)*Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*Yfun - 8.D0
     &    *t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 4.D0*s4fun**(-1)*Yfun + 
     &    16.D0*u*s4fun**(-1)*rwm2 + 8.D0*s*s4fun**(-1)*rwm2 - 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2*s4fun**(-1)*rzm2
     &     - 16.D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*mz**2*u*s4fun**(-1) + 
     &    8.D0*mz**2*s*s4fun**(-1) - 16.D0*mz**4*s4fun**(-1) - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 - 12.D0*ctw**2*s4fun**(-1)*Yfun + 16.D0*ctw**2*rzm2 - 
     &    48.D0*ctw**2*u*s4fun**(-1)*rwm2 + 16.D0*ctw**2*u - 24.D0*
     &    ctw**2*s*s4fun**(-1)*rwm2 + 16.D0*ctw**2*s + 24.D0*ctw**2*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*rwm2 - 32.D0*ctw**2*mz**2 )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 8.D0*mz**2*s*s4fun**(-1) )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - ctw**(-2)*s*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*u
     &    *t1fun**(-1)*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*u*
     &    u1fun**(-1)*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1) + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s4fun**(-1) + 3.D0/2.D0*ctw**(-2)
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*mz**2*
     &    s*u1fun**(-1)*s4fun**(-1) + 4.D0*s*s4fun**(-1) + 2.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1) - 2.D0*s*u*u1fun**(-1)*s4fun**(-1) + 
     &    2.D0*s**2*t1fun**(-1)*s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 
     &    16.D0*mz**2*s4fun**(-1) - 6.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 2.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) - 12.D0*
     &    ctw**2*s*s4fun**(-1) - 6.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 6.D0*ctw**2*s*u*u1fun**(-1)*s4fun**(-1) - 6.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s4fun**(-1) + 18.D0*
     &    ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1) + 6.D0*ctw**2*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1) )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & b0fusww * (  - ctw**(-2)*u*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*u
     &    *u1fun**(-1)*s4fun**(-1) - ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s4fun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*mz**2*s*u1fun**(-1)*s4fun**(-1) + 4.D0*u*
     &    s4fun**(-1) + 2.D0*s*u*u1fun**(-1)*s4fun**(-1) + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*mz**2*s4fun**(-1) + 2.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) - 12.D0*ctw**2*u*s4fun**(-1)
     &     - 6.D0*ctw**2*s*u*u1fun**(-1)*s4fun**(-1) - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) - 6.D0*ctw**2*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      ewamp(1,1,1,1) = ewamp(1,1,1,1) + g**4*stw**2*betaZfun*
     & b0ftsww * ( ctw**(-2)*u*s4fun**(-1) + ctw**(-2)*s*s4fun**(-1) + 
     &    1.D0/2.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1) - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun - 3.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*u*s4fun**(-1) - 4.D0*s*s4fun**(-1) - 2.D0*
     &    s*u*t1fun**(-1)*s4fun**(-1) - 2.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 12.D0
     &    *mz**2*s4fun**(-1) + 6.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 
     &    12.D0*ctw**2*u*s4fun**(-1) + 12.D0*ctw**2*s*s4fun**(-1) + 6.D0
     &    *ctw**2*s*u*t1fun**(-1)*s4fun**(-1) + 6.D0*ctw**2*s**2*
     &    t1fun**(-1)*s4fun**(-1) - 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun - 36.D0*ctw**2*mz**2*s4fun**(-1) - 18.D0*
     &    ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1) )
c$$$      print *, "1", ewamp(1,1,1,1)
c$$$      stop

      ewamp(1,1,1,3) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun + 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 - 
     &    u**2*deltafun - 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 + mz**4*
     &    deltafun - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u
     &    *s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + u**2*deltafun - 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 + 2.D0*s*u*deltafun - 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 + s**2*deltafun + 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**2*u*deltafun + 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s*deltafun - 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*mz**4*deltafun + 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*ctw**(-2)*
     &    u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)
     &    *mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*
     &    mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*deltafun*
     &    rzm2 - 2.D0*u*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 + 4.D0*mz**2*s**(-1)*u*deltafun
     &     + 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0
     &    *s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun + 2.D0*deltafun*rzm2 + 6.D0*u*deltafun + 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 12.D0*
     &    mz**2*s**(-1)*u*deltafun - 8.D0*mz**2*deltafun - 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**4*s**(-1)*deltafun + 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*u*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*u*deltafun + 8.D0*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    s*deltafun - 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*
     &    mz**2*deltafun - 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 
     &    24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*mz**2
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*
     &    ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.
     &    D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*b0fusww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D
     &    0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun * ( 2.D
     &    0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun - s*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun + mz**4*s*s4fun**(-1)*deltafun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u*s4fun**(-1)*deltafun + 
     &    s**3*s4fun**(-1)*deltafun - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*s**2*s4fun**(-1)*deltafun + 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *u**2*s4fun**(-1)*deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 2.D0*s*u*s4fun**(-1)*deltafun - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun + 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*s4fun**(-1)*deltafun*Yfun + 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**2*s4fun**(-1)
     &    *deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 + 6.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s**2*s4fun**(-1)
     &    *deltafun - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 + 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     - 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 4.D0*s*u*s4fun**(-1)*deltafun - 2.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & b0fusww * ( 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,1,1,3) = ewamp(1,1,1,3) + g**4*stw**2*betaZfun*
     & b0ftsww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun - 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(1,1,1,2) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*rwm2**2 - ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*
     &    Yfun**2 - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2
     &     - 4.D0*s**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 4.D0*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 + 12.D0*ctw**2*s**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2
     &     - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 - 24.D0*
     &    ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 8.D0
     &    *rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2
     &    *Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 - 8.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 16.D0*mz**2*s*u*Yfun**(-1)*rwm2 + 16.D0*
     &    mz**2*s**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.
     &    D0*mz**4*s*Yfun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*Yfun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 48.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*rwm2 - 2.D0*ctw**(-2)*u*Yfun**(-1)*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*
     &    rzm2 - 8.D0*s**(-1)*u*rwm2 - 8.D0*rwm2 + 8.D0*u*Yfun**(-1)*
     &    rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*
     &    ctw**2*s**(-1)*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0
     &    *ctw**2*rwm2 - 24.D0*ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*
     &    ctw**2*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 24.D0*ctw**2*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*
     &    rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 4.D
     &    0*ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*Yfun + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*rzm2 + 8.D0*s**(-1)
     &    *u*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*
     &    rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 16.D0*s*u*Yfun**(-1)*
     &    rwm2 - 8.D0*s**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*
     &    rwm2 - 8.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 
     &    32.D0*mz**2*s*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*Yfun**(-1)*rwm2 - 24.D0*ctw**2
     &    *s**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u
     &    *Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 24.D0*ctw**2*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*
     &    Yfun**(-1)*rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 48.D0
     &    *ctw**2*mz**4*s**(-2)*s4fun**(-1)*Yfun + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*c0ccqswww * ( 2.
     &    D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 - ctw**(-2)*s*s2fun*
     &    Yfun**(-1)*rwm2 - 8.D0*s2fun*s4fun**(-1)*rwm2 + 4.D0*s*s2fun*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 12.D0
     &    *ctw**2*s*s2fun*Yfun**(-1)*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*s4fun*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*
     &    Yfun - 3.D0*ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)
     &    *mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*u
     &    *u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 
     &    16.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 36.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**4*
     &    s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun
     &     + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*Yfun )
      ewamp(1,1,1,2) = ewamp(1,1,1,2) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 3.D0
     &    *ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 36.D0*ctw**2*mz**2*t1fun**(-2)
     &    *s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )

      ewamp(1,1,3,1) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun - 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 + 
     &    u**2*deltafun + 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 - mz**4*
     &    deltafun + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*
     &    s*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - u**2*deltafun + 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*s*u*deltafun + 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 - s**2*deltafun - 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**2*u*deltafun - 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s*deltafun + 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*mz**4*deltafun - 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     + 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 + 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - ctw**(-2)*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*ctw**(-2)*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*deltafun*
     &    rzm2 + 2.D0*u*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 - 4.D0*mz**2*s**(-1)*u*deltafun
     &     - 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 3.D0*ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun - 2.D0*deltafun*rzm2 - 6.D0*u*deltafun - 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun
     &     - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 12.D0*
     &    mz**2*s**(-1)*u*deltafun + 8.D0*mz**2*deltafun + 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*mz**4*s**(-1)*deltafun - 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*c0ccqswww * ( 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*u*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*
     &    deltafun + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*b0ftsww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2
     &    *s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun * ( 
     &     - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun + s*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun - mz**4*s*s4fun**(-1)*deltafun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u*s4fun**(-1)*deltafun - 
     &    s**3*s4fun**(-1)*deltafun + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*s**2*s4fun**(-1)*deltafun - 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *u**2*s4fun**(-1)*deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 2.D0*s*u*s4fun**(-1)*deltafun + 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun - 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**2*s4fun**(-1)
     &    *deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 - 6.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s**2*s4fun**(-1)
     &    *deltafun + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 - 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     + 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 4.D0*s*u*s4fun**(-1)*deltafun + 2.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & b0fmzww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & b0fusww * (  - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,1,3,1) = ewamp(1,1,3,1) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(1,1,3,3) =
     &  + g**4*stw**2 * ( 4.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)
     &    *s4fun**(-1)*Yfun )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*denh12p * (  - 
     &    3.D0/2.D0*ctw**(-2)*s*rwm2*rzm2**(-1) - 1.D0/4.D0*ctw**(-2)*
     &    mh**2*s*rzm2**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 2.D0*ctw**(-2)*mz**2*s**(-2)*u**2*
     &    s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)
     &    *Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 - 
     &    1.D0/2.D0*ctw**(-2)*mz**2*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)*
     &    mz**2*u*s4fun**(-1)*rwm2 - ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2
     &     + 4.D0*ctw**(-2)*mz**4*s**(-2)*u*s4fun**(-1)*Yfun + 2.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*Yfun + 8.D0*ctw**(-2)*
     &    mz**4*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**4*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**6*s**(-2)*s4fun**(-1)*
     &    Yfun - 4.D0*ctw**(-2)*mz**6*s**(-1)*s4fun**(-1)*rwm2 + 16.D0*
     &    s**(-1)*Yfun*rwm2 + 32.D0*rwm2**2 - 4.D0*s*rwm2 + 8.D0*mz**2*
     &    s**(-2)*u**2*s4fun**(-1)*Yfun - 4.D0*mz**2*s**(-1)*Yfun + 8.D0
     &    *mz**2*s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*mz**2*s**(-1)*u**2*
     &    s4fun**(-1)*rwm2 + 2.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*
     &    rwm2 + 16.D0*mz**2*u*s4fun**(-1)*rwm2 + 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**4*s**(-2)*u*s4fun**(-1)*Yfun - 8.
     &    D0*mz**4*s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**4*s4fun**(-1)*rwm2 + 8.D0*mz**6*
     &    s**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**6*s**(-1)*s4fun**(-1)*
     &    rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*u**2*s4fun**(-1)*Yfun - 24.D
     &    0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*Yfun - 48.D0*ctw**2*
     &    mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 - 6.D0*ctw**2*mz**2*
     &    s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**2*u*s4fun**(-1)*rwm2 - 12.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**4*
     &    s**(-2)*u*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**4*s**(-1)*
     &    s4fun**(-1)*Yfun + 96.D0*ctw**2*mz**4*s**(-1)*u*s4fun**(-1)*
     &    rwm2 + 48.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 - 24.D0*ctw**2*
     &    mz**6*s**(-2)*s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**6*s**(-1)*
     &    s4fun**(-1)*rwm2 )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 32.D0*
     &    rwm2**2 - 4.D0*s*rwm2 - 8.D0*mz**2*rwm2 - 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 32.D0*
     &    rwm2**2 - 4.D0*s*rwm2 - 8.D0*mz**2*rwm2 - 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*c0cmzuswww * ( 
     &    4.D0*ctw**(-2)*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 - 4.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u**3*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**2*s4fun**(-1) + ctw**(-2)*mz**2*s4fun**(-1)*rzm2
     &     - ctw**(-2)*mz**2*u*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*
     &    s**(-2)*u*s4fun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**4*s**(-2)*
     &    u**2*s4fun**(-1) - 4.D0*ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s**(-1)*u*s4fun**(-1) - 8.D0*
     &    ctw**(-2)*mz**4*u1fun**(-1)*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)
     &    *mz**6*s**(-2)*s4fun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**6*
     &    s**(-2)*u*s4fun**(-1) - 32.D0*s**(-1)*rwm2*rzm2 + 32.D0*
     &    s**(-1)*u*rwm2 - 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 
     &    16.D0*mz**2*s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rzm2 - 8.D0*mz**2*
     &    s**(-1)*u + 16.D0*mz**2*s**(-1)*u**2*s4fun**(-1) - 4.D0*mz**2
     &    *s4fun**(-1)*rzm2 + 4.D0*mz**2*u*s4fun**(-1) + 32.D0*mz**4*
     &    s**(-2)*u*s4fun**(-1)*rzm2 - 32.D0*mz**4*s**(-2)*u**2*
     &    s4fun**(-1) + 16.D0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 16.D0*
     &    mz**4*s**(-1)*u*s4fun**(-1) + 32.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**6*s**(-2)*s4fun**(-1)*rzm2 + 16.D
     &    0*mz**6*s**(-2)*u*s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-2)*
     &    u**2*s4fun**(-1)*rzm2 - 48.D0*ctw**2*mz**2*s**(-2)*u**3*
     &    s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*rzm2
     &     - 48.D0*ctw**2*mz**2*s**(-1)*u**2*s4fun**(-1) + 12.D0*ctw**2
     &    *mz**2*s4fun**(-1)*rzm2 - 12.D0*ctw**2*mz**2*u*s4fun**(-1) - 
     &    96.D0*ctw**2*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 + 96.D0*ctw**2*
     &    mz**4*s**(-2)*u**2*s4fun**(-1) - 48.D0*ctw**2*mz**4*s**(-1)*
     &    s4fun**(-1)*rzm2 + 48.D0*ctw**2*mz**4*s**(-1)*u*s4fun**(-1)
     &     - 96.D0*ctw**2*mz**4*u1fun**(-1)*s4fun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**6*s**(-2)*s4fun**(-1)*rzm2 - 48.D0*ctw**2*mz**6*
     &    s**(-2)*u*s4fun**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*c0cmztswww * ( 
     &    4.D0*ctw**(-2)*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u**3*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**2*s4fun**(-1) + ctw**(-2)*mz**2*s4fun**(-1)*rzm2
     &     + 5.D0*ctw**(-2)*mz**2*u*s4fun**(-1) + ctw**(-2)*mz**2*s*
     &    s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*s**(-2)*u*s4fun**(-1)*rzm2
     &     - 16.D0*ctw**(-2)*mz**4*s**(-2)*u**2*s4fun**(-1) - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 20.D0*ctw**(-2)*
     &    mz**4*s**(-1)*u*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**4*
     &    s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*s**(-2)*s4fun**(-1)*rzm2
     &     + 20.D0*ctw**(-2)*mz**6*s**(-2)*u*s4fun**(-1) + 12.D0*
     &    ctw**(-2)*mz**6*s**(-1)*s4fun**(-1) - 8.D0*ctw**(-2)*mz**8*
     &    s**(-2)*s4fun**(-1) - 32.D0*s**(-1)*rwm2*rzm2 - 32.D0*s**(-1)
     &    *u*rwm2 - 32.D0*rwm2 - 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*
     &    s**(-1)*rzm2 + 64.D0*mz**2*s**(-1)*rwm2 - 16.D0*mz**2*s**(-1)
     &    *u*s4fun**(-1)*rzm2 + 8.D0*mz**2*s**(-1)*u - 32.D0*mz**2*
     &    s**(-1)*u**2*s4fun**(-1) - 4.D0*mz**2*s4fun**(-1)*rzm2 + 8.D0
     &    *mz**2 - 20.D0*mz**2*u*s4fun**(-1) - 4.D0*mz**2*s*s4fun**(-1)
     &     + 32.D0*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 + 64.D0*mz**4*
     &    s**(-2)*u**2*s4fun**(-1) + 16.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    rzm2 - 16.D0*mz**4*s**(-1) + 80.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1) + 32.D0*mz**4*t1fun**(-1)*s4fun**(-1)*rwm2 + 24.D0
     &    *mz**4*s4fun**(-1) - 16.D0*mz**6*s**(-2)*s4fun**(-1)*rzm2 - 
     &    80.D0*mz**6*s**(-2)*u*s4fun**(-1) - 48.D0*mz**6*s**(-1)*
     &    s4fun**(-1) + 32.D0*mz**8*s**(-2)*s4fun**(-1) + 48.D0*ctw**2*
     &    mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 48.D0*ctw**2*mz**2*
     &    s**(-2)*u**3*s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*rzm2 + 96.D0*ctw**2*mz**2*s**(-1)*u**2*
     &    s4fun**(-1) + 12.D0*ctw**2*mz**2*s4fun**(-1)*rzm2 + 60.D0*
     &    ctw**2*mz**2*u*s4fun**(-1) + 12.D0*ctw**2*mz**2*s*s4fun**(-1)
     &     - 96.D0*ctw**2*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 - 192.D0*
     &    ctw**2*mz**4*s**(-2)*u**2*s4fun**(-1) - 48.D0*ctw**2*mz**4*
     &    s**(-1)*s4fun**(-1)*rzm2 - 240.D0*ctw**2*mz**4*s**(-1)*u*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*rwm2
     &     - 72.D0*ctw**2*mz**4*s4fun**(-1) + 48.D0*ctw**2*mz**6*
     &    s**(-2)*s4fun**(-1)*rzm2 + 240.D0*ctw**2*mz**6*s**(-2)*u*
     &    s4fun**(-1) + 144.D0*ctw**2*mz**6*s**(-1)*s4fun**(-1) - 96.D0
     &    *ctw**2*mz**8*s**(-2)*s4fun**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*c0ccqswww * ( 
     &     - 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 - 4.D0*s + 32.D0*
     &    mz**2*s4fun**(-1)*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*rwm2
     &     )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*c0ccqswww*
     & denh12p * ( 3.D0*ctw**(-2)*s*rwm2**2*rzm2**(-1) - 2.D0*ctw**(-2)
     &    *s**2*rwm2*rzm2**(-1) + 1.D0/2.D0*ctw**(-2)*mh**2*s*rwm2*
     &    rzm2**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*b0fmzww * ( 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) + 6.D0*
     &    ctw**(-2)*mz**2*u*t1fun**(-1)*s4fun**(-1) + 2.D0*ctw**(-2)*
     &    mz**2*u*u1fun**(-1)*s4fun**(-1) + 2.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*ctw**(-2)*mz**4*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1) - 10.D0*ctw**(-2)*mz**4*t1fun**(-1)
     &    *s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u1fun**(-1)*s4fun**(-1)
     &     + 12.D0*ctw**(-2)*mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) - 4.D
     &    0*ctw**(-2)*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) - 16.D0*
     &    mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 16.D0*mz**2*
     &    s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 24.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1) - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 16.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*
     &    s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun + 64.D0*mz**4*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1) + 40.D0*mz**4*t1fun**(-1)*s4fun**(-1)
     &     - 8.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 48.D0*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) + 16.D0*mz**6*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u**2*t1fun**(-1)*
     &    s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u**2*u1fun**(-1)*
     &    s4fun**(-1) + 72.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1) + 
     &    24.D0*ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1) + 24.D0*ctw**2*
     &    mz**2*s*t1fun**(-1)*s4fun**(-1) + 48.D0*ctw**2*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 192.D0*ctw**2*mz**4*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1) - 120.D0*ctw**2*mz**4*t1fun**(-1)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*u1fun**(-1)*s4fun**(-1) + 
     &    144.D0*ctw**2*mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) - 48.D0*
     &    ctw**2*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*b0fusww * (  - 
     &    4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 2.
     &    D0*ctw**(-2)*mz**2*u*u1fun**(-1)*s4fun**(-1) - 4.D0*ctw**(-2)
     &    *mz**4*s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*
     &    mz**4*u1fun**(-1)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s**(-1)*u**2*
     &    u1fun**(-1)*s4fun**(-1) + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) + 16.D0*mz**4*s**(-1)*u1fun**(-2)*s4fun**(-1)*
     &    Yfun + 8.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 16.D0*mz**6*
     &    s**(-1)*u1fun**(-1)*s4fun**(-1) - 48.D0*ctw**2*mz**2*s**(-1)*
     &    u**2*u1fun**(-1)*s4fun**(-1) - 24.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1) - 48.D0*ctw**2*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*mz**4*u1fun**(-1)
     &    *s4fun**(-1) + 48.D0*ctw**2*mz**6*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*b0ftsww * (  - 
     &    4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 6.
     &    D0*ctw**(-2)*mz**2*u*t1fun**(-1)*s4fun**(-1) - 2.D0*ctw**(-2)
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1) - 4.D0*ctw**(-2)*mz**4*
     &    s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u*t1fun**(-1)*s4fun**(-1) + 10.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1) - 12.D0*ctw**(-2)*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s**(-1)*u**2*
     &    t1fun**(-1)*s4fun**(-1) + 24.D0*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 64.D0*mz**4*
     &    s**(-1)*u*t1fun**(-1)*s4fun**(-1) - 40.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1) + 48.D0*mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 72.D
     &    0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1) - 24.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1) - 48.D0*ctw**2*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 192.D0*ctw**2*mz**4*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1) + 120.D0*ctw**2*mz**4*t1fun**(-1)*
     &    s4fun**(-1) - 144.D0*ctw**2*mz**6*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*betaZfun**2*
     & denh12p * (  - 3.D0/2.D0*ctw**(-2)*s*rwm2*rzm2**(-1) - 1.D0/4.D0
     &    *ctw**(-2)*mh**2*s*rzm2**(-1) )
      ewamp(1,1,3,3) = ewamp(1,1,3,3) + g**4*stw**2*betaZfun**2*
     & c0ccqswww*denh12p * ( 3.D0*ctw**(-2)*s*rwm2**2*rzm2**(-1) - 2.D0
     &    *ctw**(-2)*s**2*rwm2*rzm2**(-1) + 1.D0/2.D0*ctw**(-2)*mh**2*s
     &    *rwm2*rzm2**(-1) )

      ewamp(1,1,3,2) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun + 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 - 
     &    u**2*deltafun - 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 + mz**4*
     &    deltafun - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u
     &    *s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + u**2*deltafun - 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 + 2.D0*s*u*deltafun - 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 + s**2*deltafun + 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**2*u*deltafun + 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s*deltafun - 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*mz**4*deltafun + 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*ctw**(-2)*
     &    u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)
     &    *mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*
     &    mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*deltafun*
     &    rzm2 - 2.D0*u*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 + 4.D0*mz**2*s**(-1)*u*deltafun
     &     + 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0
     &    *s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun + 2.D0*deltafun*rzm2 + 6.D0*u*deltafun + 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 12.D0*
     &    mz**2*s**(-1)*u*deltafun - 8.D0*mz**2*deltafun - 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**4*s**(-1)*deltafun + 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*u*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*u*deltafun + 8.D0*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    s*deltafun - 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*
     &    mz**2*deltafun - 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 
     &    24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*mz**2
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*
     &    ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.
     &    D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*b0fusww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D
     &    0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun * ( 
     &     - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun + s*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun - mz**4*s*s4fun**(-1)*deltafun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u*s4fun**(-1)*deltafun - 
     &    s**3*s4fun**(-1)*deltafun + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*s**2*s4fun**(-1)*deltafun - 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *u**2*s4fun**(-1)*deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 2.D0*s*u*s4fun**(-1)*deltafun + 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun - 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**2*s4fun**(-1)
     &    *deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 - 6.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s**2*s4fun**(-1)
     &    *deltafun + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 - 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     + 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 4.D0*s*u*s4fun**(-1)*deltafun + 2.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & b0fmzww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & b0fusww * (  - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,1,3,2) = ewamp(1,1,3,2) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(1,1,2,1) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*rwm2**2 - ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*
     &    Yfun**2 - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2
     &     - 4.D0*s**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 4.D0*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 + 12.D0*ctw**2*s**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2
     &     - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 - 24.D0*
     &    ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 8.D0
     &    *rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2
     &    *Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 - 8.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 16.D0*mz**2*s*u*Yfun**(-1)*rwm2 + 16.D0*
     &    mz**2*s**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.
     &    D0*mz**4*s*Yfun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*Yfun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 48.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*rwm2 - 2.D0*ctw**(-2)*u*Yfun**(-1)*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*
     &    rzm2 - 8.D0*s**(-1)*u*rwm2 - 8.D0*rwm2 + 8.D0*u*Yfun**(-1)*
     &    rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*
     &    ctw**2*s**(-1)*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0
     &    *ctw**2*rwm2 - 24.D0*ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*
     &    ctw**2*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 24.D0*ctw**2*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*
     &    rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 4.D
     &    0*ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*Yfun + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*rzm2 + 8.D0*s**(-1)
     &    *u*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*
     &    rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 16.D0*s*u*Yfun**(-1)*
     &    rwm2 - 8.D0*s**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*
     &    rwm2 - 8.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 
     &    32.D0*mz**2*s*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*Yfun**(-1)*rwm2 - 24.D0*ctw**2
     &    *s**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u
     &    *Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 24.D0*ctw**2*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*
     &    Yfun**(-1)*rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 48.D0
     &    *ctw**2*mz**4*s**(-2)*s4fun**(-1)*Yfun + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*c0ccqswww * ( 2.
     &    D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 - ctw**(-2)*s*s2fun*
     &    Yfun**(-1)*rwm2 - 8.D0*s2fun*s4fun**(-1)*rwm2 + 4.D0*s*s2fun*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 12.D0
     &    *ctw**2*s*s2fun*Yfun**(-1)*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*s4fun*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*
     &    Yfun - 3.D0*ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)
     &    *mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*u
     &    *u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 
     &    16.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 36.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**4*
     &    s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun
     &     + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*Yfun )
      ewamp(1,1,2,1) = ewamp(1,1,2,1) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 3.D0
     &    *ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 36.D0*ctw**2*mz**2*t1fun**(-2)
     &    *s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )

      ewamp(1,1,2,3) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun - 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 + 
     &    u**2*deltafun + 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 - mz**4*
     &    deltafun + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*
     &    s*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - u**2*deltafun + 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*s*u*deltafun + 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 - s**2*deltafun - 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**2*u*deltafun - 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s*deltafun + 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*mz**4*deltafun - 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     + 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 + 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - ctw**(-2)*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*ctw**(-2)*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*deltafun*
     &    rzm2 + 2.D0*u*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 - 4.D0*mz**2*s**(-1)*u*deltafun
     &     - 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 3.D0*ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun - 2.D0*deltafun*rzm2 - 6.D0*u*deltafun - 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun
     &     - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 12.D0*
     &    mz**2*s**(-1)*u*deltafun + 8.D0*mz**2*deltafun + 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*mz**4*s**(-1)*deltafun - 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*c0ccqswww * ( 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*u*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*
     &    deltafun + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*b0ftsww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2
     &    *s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun * ( 2.D
     &    0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun - s*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun + mz**4*s*s4fun**(-1)*deltafun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u*s4fun**(-1)*deltafun + 
     &    s**3*s4fun**(-1)*deltafun - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*s**2*s4fun**(-1)*deltafun + 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *u**2*s4fun**(-1)*deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 2.D0*s*u*s4fun**(-1)*deltafun - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun + 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*s4fun**(-1)*deltafun*Yfun + 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**2*s4fun**(-1)
     &    *deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 + 6.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s**2*s4fun**(-1)
     &    *deltafun - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 + 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     - 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 4.D0*s*u*s4fun**(-1)*deltafun - 2.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & b0fusww * ( 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,1,2,3) = ewamp(1,1,2,3) + g**4*stw**2*betaZfun*
     & b0ftsww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun - 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(1,1,2,2) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s*t1fun**(-1)*u1fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 4.D0
     &    *mz**2*s*t1fun**(-1)*u1fun**(-1) + 24.D0*ctw**2*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*denh12p * ( 6.D0
     &    *ctw**(-2)*rwm2 + ctw**(-2)*mh**2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 1.D0/2.D0*ctw**(-2)*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 - 8.D0*s**(-1)*s2fun*s4fun**(-1)*
     &    Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 - 4.D0*mz**2*s**(-1)*Yfun + 6.D0*ctw**2*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 - 8.D0*ctw**2*s**(-1)*s2fun*Yfun + 24.D0
     &    *ctw**2*rwm2**2 - 16.D0*ctw**2*s2fun*rwm2 + 4.D0*ctw**2*s*
     &    s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D
     &    0*ctw**2*mz**4 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*ctw**(-2)*
     &    s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s2fun*
     &    s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0
     &    *s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 4.D0*s**(-2)*u*s2fun*
     &    s4fun**(-1)*Yfun - 8.D0*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*rwm2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 8.D0*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2
     &    *rzm2 + 8.D0*mz**2*s**(-1)*rzm2 + 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 - 8.
     &    D0*mz**2*s**(-1)*u - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*
     &    Yfun*rzm2 + 12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 
     &    24.D0*ctw**2*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2
     &     + 16.D0*ctw**2*s**(-1)*s2fun*rzm2 + 48.D0*ctw**2*s**(-1)*u*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*ctw**2*s**(-1)*u*s2fun + 24.D0
     &    *ctw**2*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*
     &    rwm2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*c0cmztswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - ctw**(-2)*s**(-1)*
     &    s2fun*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 + 2.D
     &    0*ctw**(-2)*mz**2*s**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*rwm2 + 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2
     &     + 4.D0*s**(-2)*u*s2fun*s4fun**(-1)*Yfun - 8.D0*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 8.
     &    D0*s2fun*s4fun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2
     &     + 8.D0*mz**2*s**(-1)*rzm2 - 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 8.
     &    D0*mz**2*s**(-1)*u - 16.D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*
     &    mz**2 + 32.D0*mz**4*s**(-1)*s4fun**(-1)*rwm2 - 16.D0*mz**4*
     &    s**(-1) - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 
     &    12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    s**(-1)*t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - 12.D0*
     &    ctw**2*s**(-1)*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s**(-1)*
     &    s2fun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 
     &    16.D0*ctw**2*s**(-1)*u*s2fun - 24.D0*ctw**2*s2fun*s4fun**(-1)
     &    *rwm2 + 16.D0*ctw**2*s2fun + 24.D0*ctw**2*mz**2*s**(-2)*s2fun
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2
     &     - 32.D0*ctw**2*mz**2*s**(-1)*s2fun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s4fun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*rwm2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 16.D0*mz**2*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*c0ccqswww*
     & denh12p * (  - 12.D0*ctw**(-2)*rwm2**2 + 8.D0*ctw**(-2)*s*rwm2
     &     - 2.D0*ctw**(-2)*mh**2*rwm2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) + 3.D0/2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s2fun*s4fun**(-1) + 2.D0
     &    *u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1) + 2.D0*s*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 4.D0*mz**2
     &    *s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0
     &    *ctw**2*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1) - 6.
     &    D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*
     &    ctw**2*mz**2*s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 48.D
     &    0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) + 18.D0*ctw**2*mz**2
     &    *t1fun**(-1)*s2fun*s4fun**(-1) + 6.D0*ctw**2*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) + 4.D0*s**(-1)*u*s2fun*s4fun**(-1) + 2.D0*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*mz**2*s**(-1)*s2fun
     &    *s4fun**(-1) + 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D
     &    0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0*ctw**2*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*ctw**2*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) + ctw**(-2)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*t1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 3.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*s**(-1)*u*s2fun*s4fun**(-1) - 4.D0*s2fun*
     &    s4fun**(-1) - 2.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 6.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     + 12.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) - 
     &    12.D0*ctw**2*mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun
     &     - 36.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) - 18.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun * ( 
     &     - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 4.D0
     &    *s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 12.D0*ctw**2*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*
     &    Yfun**2 - 2.D0*ctw**(-2)*s4fun**(-1)*Yfun*rwm2 + 2.D0*s**(-1)
     &    *s4fun**(-1)*Yfun**2 + 8.D0*s4fun**(-1)*Yfun*rwm2 + 4.D0*
     &    mz**2*s4fun**(-1)*Yfun - 6.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    Yfun**2 - 24.D0*ctw**2*s4fun**(-1)*Yfun*rwm2 + 8.D0*ctw**2*
     &    Yfun + 16.D0*ctw**2*s*rwm2 - 4.D0*ctw**2*s**2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*mz**2*s*s2fun*s4fun**(-1) + 16.D0*
     &    ctw**2*s*rwm2 - 4.D0*ctw**2*s**2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*mz**2*s*s2fun*s4fun**(-1) + 16.D0*
     &    ctw**2*s*rwm2 - 4.D0*ctw**2*s**2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 4.D0*ctw**(-2)*u*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*rwm2 - 4.D0*s**(-1)*s4fun**(-1)*
     &    Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*Yfun + 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 16.D0*u*s4fun**(-1)*rwm2
     &     + 8.D0*s*s4fun**(-1)*rwm2 - 8.D0*mz**2*s4fun**(-1)*rzm2 - 16.
     &    D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*mz**2*u*s4fun**(-1) + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 - 16.D0*ctw**2*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*rwm2
     &     + 16.D0*ctw**2*u - 24.D0*ctw**2*s*s4fun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + ctw**(-2)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*u*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 - 4.D0*s**(-1)*
     &    s4fun**(-1)*Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*Yfun + 8.D0
     &    *t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 4.D0*s4fun**(-1)*Yfun - 
     &    16.D0*u*s4fun**(-1)*rwm2 - 8.D0*s*s4fun**(-1)*rwm2 + 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun - 8.D0*mz**2*s4fun**(-1)*rzm2
     &     + 16.D0*mz**2*s4fun**(-1)*rwm2 - 8.D0*mz**2*u*s4fun**(-1) - 
     &    8.D0*mz**2*s*s4fun**(-1) + 16.D0*mz**4*s4fun**(-1) + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 12.D0*ctw**2*s4fun**(-1)*Yfun - 16.D0*ctw**2*rzm2 + 
     &    48.D0*ctw**2*u*s4fun**(-1)*rwm2 - 16.D0*ctw**2*u + 24.D0*
     &    ctw**2*s*s4fun**(-1)*rwm2 - 16.D0*ctw**2*s - 24.D0*ctw**2*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*rwm2 + 32.D0*ctw**2*mz**2 )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 8.D0*mz**2*s*s4fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & b0fmzww * ( ctw**(-2)*s*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*u*u1fun**(-1)
     &    *s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1) - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)
     &    *mz**2*s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*s*u1fun**(-1)*
     &    s4fun**(-1) - 4.D0*s*s4fun**(-1) - 2.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 2.D0*s*u*u1fun**(-1)*s4fun**(-1) - 2.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**2*
     &    s4fun**(-1) + 6.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 2.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*s*s4fun**(-1)
     &     + 6.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1) - 6.D0*ctw**2*s*u*
     &    u1fun**(-1)*s4fun**(-1) + 6.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun
     &     - 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 48.D0*
     &    ctw**2*mz**2*s4fun**(-1) - 18.D0*ctw**2*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 6.D0*ctw**2*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & b0fusww * ( ctw**(-2)*u*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*u*
     &    u1fun**(-1)*s4fun**(-1) + ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun + ctw**(-2)*mz**2*s4fun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*mz**2*s*u1fun**(-1)*s4fun**(-1) - 4.D0*u*
     &    s4fun**(-1) - 2.D0*s*u*u1fun**(-1)*s4fun**(-1) - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*mz**2*s4fun**(-1) - 2.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*u*s4fun**(-1)
     &     + 6.D0*ctw**2*s*u*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) + 6.D0*ctw**2*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      ewamp(1,1,2,2) = ewamp(1,1,2,2) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - ctw**(-2)*u*s4fun**(-1) - ctw**(-2)*s*s4fun**(-1)
     &     - 1.D0/2.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1) - 1.D0/2.D0
     &    *ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1) + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 3.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) + 3.D0/2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*u*s4fun**(-1) + 4.D0*s*s4fun**(-1) + 2.D0*
     &    s*u*t1fun**(-1)*s4fun**(-1) + 2.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0
     &    *mz**2*s4fun**(-1) - 6.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 
     &    12.D0*ctw**2*u*s4fun**(-1) - 12.D0*ctw**2*s*s4fun**(-1) - 6.D0
     &    *ctw**2*s*u*t1fun**(-1)*s4fun**(-1) - 6.D0*ctw**2*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + 36.D0*ctw**2*mz**2*s4fun**(-1) + 18.D0*
     &    ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      ewamp(1,2,1,1) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 8.D0*mz**2*rwm2 + 12.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 - 32.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*
     &    Yfun**(-1) - 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 12.D0
     &    *ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 4.D
     &    0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*mz**8*s*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 + 
     &    48.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    16.D0*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*
     &    s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 + 16.D0*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 128.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 256.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 8.D0*mz**2*s**2*u*Yfun**(-1) - 128.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s**3*Yfun**(-1) - 8.D
     &    0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*Yfun**(-1)*rwm2 + 
     &    336.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s*
     &    u*Yfun**(-1) - 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 336.
     &    D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**2*
     &    Yfun**(-1) - 8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 288.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*mz**6*s*Yfun**(-1) + 16.D0
     &    *mz**6*s*u*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2
     &     + 24.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*
     &    Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.
     &    D0*ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**4*s*Yfun**(-1)*rwm2 + 12.D
     &    0*ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*
     &    Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 
     &    2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D0*
     &    ctw**(-2)*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 8.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 8.D0*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2
     &     + 8.D0*u*Yfun**(-1)*rwm2*rzm2 + 32.D0*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 32.D0*u**3
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 64.D0*mz**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 + 8.D0*mz**2*u*Yfun**(-1)*rzm2 - 64.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*u**2*
     &    Yfun**(-1) + 32.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 8.
     &    D0*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2
     &     + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*ctw**2*u*
     &    Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 24.
     &    D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 24.D0*ctw**2*
     &    mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D
     &    0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*
     &    Yfun**(-1)*rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2
     &     - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 8.D
     &    0*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 + 8.D0*s2fun*s4fun**(-1)*rwm2 + 8.D0*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*
     &    rzm2 + 32.D0*u**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 - 8.D0*
     &    u**2*Yfun**(-1)*rwm2 + 32.D0*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 8.D0*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*s*
     &    Yfun**(-1)*rwm2*rzm2 + 64.D0*s*u*s4fun**(-1)*Yfun**(-1)*rwm2*
     &    rzm2 - 16.D0*s*u*Yfun**(-1)*rwm2 + 96.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*s**2*Yfun**(-1)*rwm2 + 96.D0*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.
     &    D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 - 192.D0*mz**2*u*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*mz**2*u*Yfun**(-1)*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*
     &    rwm2 - 256.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*
     &    mz**2*u**2*Yfun**(-1) - 192.D0*mz**2*s*s4fun**(-1)*Yfun**(-1)
     &    *rwm2*rzm2 - 8.D0*mz**2*s*Yfun**(-1)*rzm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 - 512.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 16.D0*mz**2*s*u*Yfun**(-1) - 256.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**2*Yfun**(-1) + 
     &    288.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 16.D0*mz**4*
     &    Yfun**(-1)*rzm2 - 32.D0*mz**4*Yfun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 672.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*mz**4*u*Yfun**(-1) - 8.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 672.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*
     &    mz**4*s*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 576.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 32.D0*mz**6*Yfun**(-1) + 32.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 24.D0
     &    *ctw**2*u*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*ctw**2*u
     &    *Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 
     &    24.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 48.D0*ctw**2*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*rwm2
     &     - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*u*s4fun**(-1)*Yfun**(-1)
     &     - 96.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2
     &    *mz**8*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s*s4fun**(-1)*rwm2 - 12.D
     &    0*s*s2fun*Yfun**(-1)*rwm2 + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 4.
     &    D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s*
     &    s2fun*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*c0mzmzqswww
     &  * ( 2.D0*ctw**(-2)*rwm2 - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*mz**4*s4fun**(-1) - ctw**(-2)*mz**4*s*Yfun**(-1)
     &     + 24.D0*rwm2 - 12.D0*s*s4fun*Yfun**(-1)*rwm2 - 8.D0*mz**2 + 
     &    4.D0*mz**2*s*s4fun*Yfun**(-1) - 8.D0*mz**4*s4fun**(-1) + 4.D0
     &    *mz**4*s*Yfun**(-1) + 24.D0*ctw**2*rwm2 - 12.D0*ctw**2*s*
     &    s4fun*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1) - 12.D0
     &    *ctw**2*mz**4*s*Yfun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1)
     &     - 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 12.D
     &    0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,1,1) = ewamp(1,2,1,1) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 48.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      ewamp(1,2,1,3) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + deltafun*Yfun + 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**2*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u**2*s4fun**(-1)*deltafun + 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**4*u*s4fun**(-1)*deltafun - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**6*s4fun**(-1)*deltafun + 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun + 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.D0*
     &    ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 5.D0*s**2*u*s4fun**(-1)*deltafun - 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**3*
     &    s4fun**(-1)*deltafun - 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun + 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*s4fun**(-1)*deltafun + 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 - 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun - 22.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 20.D0*mz**6*s4fun**(-1)*deltafun + 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun + 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun + 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 4.D0*s**(-1)*u**2*deltafun + 8.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D
     &    0*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*mz**2*
     &    s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s**(-1)*u*deltafun + 16.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)
     &    *deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 48.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 - 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 4.D0*s**(-1)*u*deltafun*rzm2 - 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*deltafun
     &     - 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*deltafun*rzm2 - 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 6.D0*u*
     &    deltafun - 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun - 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*s**(-1)*u*deltafun - 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*deltafun
     &     + 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s**(-1)*
     &    deltafun - 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 - 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 1.
     &    D0/2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*u*deltafun + 2.D0*s*deltafun + 4.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*s**2*s2fun**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*deltafun - 4.D0
     &    *mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**6*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*s*u*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2
     &    *s*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) + 32.D0*ctw**2*u*deltafun + 16.D0
     &    *ctw**2*s*deltafun - 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) - 32.D
     &    0*ctw**2*mz**2*deltafun + 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - ctw**(-2)*s*s4fun**(-1)*deltafun - 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*deltafun + 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*s4fun**(-1)*deltafun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun - 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*b0fqsww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*
     &    deltafun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 8.D0*u
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun + 8.D0*
     &    mz**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 24.D0*ctw**2
     &    *mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun * ( 
     &    ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - deltafun*Yfun - 64.D0*ctw**2*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun + s**2*u*s4fun**(-1)*deltafun + 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 + 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + s**3*s4fun**(-1)*deltafun - 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun - 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 10.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     + 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun + 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u
     &    *u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 16.D0*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*u*deltafun + 8.D0*s*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**2*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*
     &    mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*mz**2*u*
     &    s4fun**(-1)*deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun
     &    *rwm2*rzm2 + 2.D0*deltafun*rzm2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*u*deltafun - 16.D0*s*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*s*deltafun - 8.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0
     &    *mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*deltafun + 16.D0
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*mz**2*s*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun + 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*deltafun
     &     - 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * ( ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun - 1.D0/2.D
     &    0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 
     &    16.D0*ctw**2*s*deltafun + 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & b0fmzww * ( ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & b0fusww * ( ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 4.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun - 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,1,3) = ewamp(1,2,1,3) + g**4*stw**2*betaZfun*
     & b0fqsww * ( ctw**(-2)*s*s4fun**(-1)*deltafun - 4.D0*s*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(1,2,1,2) =
     &  + g**4*stw**2 * ( ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 - 
     &    8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D
     &    0*ctw**2*s*s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**4*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*rwm2**2
     &     - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 
     &    4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 - 8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun
     &     + 16.D0*ctw**2*s*rwm2 + 4.D0*ctw**2*s*s2fun - 8.D0*ctw**2*s*
     &    u*s2fun*s4fun**(-1) + 24.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2 + 6.
     &    D0*ctw**2*s**2*u**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s**2*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*
     &    s**3*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**3*u**3*Yfun**(-2) + 
     &    3.D0*ctw**(-2)*s**4*u**2*Yfun**(-2) + 2.D0*ctw**(-2)*s**5*u*
     &    Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**6*Yfun**(-2) - 8.D0*
     &    ctw**(-2)*mz**2*s*u*Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s**2*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*u**3*
     &    Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*s**3*u**2*Yfun**(-2) - 12.D
     &    0*ctw**(-2)*mz**2*s**4*u*Yfun**(-2) - 4.D0*ctw**(-2)*mz**2*
     &    s**5*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 8.D0
     &    *ctw**(-2)*mz**4*s*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    Yfun**(-2) - ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 24.
     &    D0*ctw**(-2)*mz**4*s**3*u*Yfun**(-2) + 12.D0*ctw**(-2)*mz**4*
     &    s**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*ctw**(-2)*mz**6*s**2*u*Yfun**(-2) - 16.D0*ctw**(-2)*
     &    mz**6*s**3*Yfun**(-2) - 4.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s**2*Yfun**(-2) - 8.D0*
     &    rwm2**2 - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 16.D0*s**2*u*
     &    Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 8.D0*s**3*
     &    Yfun**(-1)*rwm2 - 8.D0*s**3*u**3*Yfun**(-2) - 12.D0*s**4*u**2
     &    *Yfun**(-2) - 8.D0*s**5*u*Yfun**(-2) - 2.D0*s**6*Yfun**(-2)
     &     - 4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 32.D0*mz**2*s
     &    *u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s**2*Yfun**(-1)*rwm2 + 16.D0
     &    *mz**2*s**2*u**3*Yfun**(-2) + 48.D0*mz**2*s**3*u**2*
     &    Yfun**(-2) + 48.D0*mz**2*s**4*u*Yfun**(-2) + 16.D0*mz**2*s**5
     &    *Yfun**(-2) + 8.D0*mz**4*s4fun**(-1)*rwm2 - 32.D0*mz**4*s*
     &    Yfun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 
     &    8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    u**2*Yfun**(-2) + 4.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 96.
     &    D0*mz**4*s**3*u*Yfun**(-2) - 48.D0*mz**4*s**4*Yfun**(-2) - 16.
     &    D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s**2*u*Yfun**(-2) + 64.D0
     &    *mz**6*s**3*Yfun**(-2) + 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s**2*Yfun**(-2) + 24.D0*ctw**2*rwm2**2 - 8.D0*
     &    ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D0*
     &    ctw**2*s*s2fun + 8.D0*ctw**2*s*u*s2fun*s4fun**(-1) + 24.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 8.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1) + 48.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 6.D0*
     &    ctw**2*s**2*u**4*Yfun**(-2) + 24.D0*ctw**2*s**3*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**3*u**3*Yfun**(-2) + 36.D0*ctw**2*s**4*
     &    u**2*Yfun**(-2) + 24.D0*ctw**2*s**5*u*Yfun**(-2) + 6.D0*
     &    ctw**2*s**6*Yfun**(-2) - 16.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 96.D0*
     &    ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    u**3*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*u**2*Yfun**(-2) - 
     &    144.D0*ctw**2*mz**2*s**4*u*Yfun**(-2) - 48.D0*ctw**2*mz**2*
     &    s**5*Yfun**(-2) - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 96.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*u**2*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*Yfun**(-1)
     &     + 288.D0*ctw**2*mz**4*s**3*u*Yfun**(-2) + 144.D0*ctw**2*
     &    mz**4*s**4*Yfun**(-2) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 192.D0*ctw**2*mz**6*s**2*u*Yfun**(-2) - 192.D0*ctw**2*
     &    mz**6*s**3*Yfun**(-2) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**8*s**2*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    u**2*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**3*Yfun**(-2)*rzm2 + 
     &    ctw**(-2)*s*u**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*
     &    Yfun**(-1)*rwm2 + 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0*s*u**4*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*u1fun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*ctw**2*s2fun*s4fun**(-1)*rzm2 - 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 - 16.D0*ctw**2*u*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0*ctw**2*s*u**4*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*u1fun**(-1)*rwm2
     &     + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*s*u*Yfun**(-1)*rwm2 + ctw**(-2)*s*u**3*Yfun**(-2)*
     &    rzm2 + ctw**(-2)*s*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*s**2*
     &    Yfun**(-1)*rwm2 + 3.D0*ctw**(-2)*s**2*u**2*Yfun**(-2)*rzm2 + 
     &    4.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) + 3.D0*ctw**(-2)*s**3*u*
     &    Yfun**(-2)*rzm2 + 6.D0*ctw**(-2)*s**3*u**2*Yfun**(-2) + 
     &    ctw**(-2)*s**4*Yfun**(-2)*rzm2 + 4.D0*ctw**(-2)*s**4*u*
     &    Yfun**(-2) + ctw**(-2)*s**5*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*Yfun**(-1)*rwm2
     &    *rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s*
     &    u**2*Yfun**(-2)*rzm2 - 8.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2)
     &     - 12.D0*ctw**(-2)*mz**2*s**2*u*Yfun**(-2)*rzm2 - 24.D0*
     &    ctw**(-2)*mz**2*s**2*u**2*Yfun**(-2) - 6.D0*ctw**(-2)*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) + 8.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s*u*Yfun**(-2)*rzm2 + 24.D0
     &    *ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*
     &    Yfun**(-2)*rzm2 + 48.D0*ctw**(-2)*mz**4*s**2*u*Yfun**(-2) + 
     &    24.D0*ctw**(-2)*mz**4*s**3*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*Yfun**(-2)*rzm2 - 32.D0*
     &    ctw**(-2)*mz**6*s*u*Yfun**(-2) - 32.D0*ctw**(-2)*mz**6*s**2*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 16.
     &    D0*ctw**(-2)*mz**8*s*Yfun**(-2) - 8.D0*u*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 
     &    16.D0*s*u*Yfun**(-1)*rwm2 - 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0
     &    *s*u**4*Yfun**(-2) - 8.D0*s**2*Yfun**(-1)*rwm2 - 12.D0*s**2*
     &    u**2*Yfun**(-2)*rzm2 - 16.D0*s**2*u**3*Yfun**(-2) - 12.D0*
     &    s**3*u*Yfun**(-2)*rzm2 - 24.D0*s**3*u**2*Yfun**(-2) - 4.D0*
     &    s**4*Yfun**(-2)*rzm2 - 16.D0*s**4*u*Yfun**(-2) - 4.D0*s**5*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*t1fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 + 24.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 32.D0*
     &    mz**2*s*u**3*Yfun**(-2) + 48.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 96.D0*mz**2*s**2*u**2*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 96.D0*mz**2*s**3*u*Yfun**(-2) + 32.D0*mz**2
     &    *s**4*Yfun**(-2) - 32.D0*mz**4*Yfun**(-1)*rwm2 + 8.D0*mz**4*u
     &    *s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0
     &    *mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s*u*
     &    Yfun**(-2)*rzm2 - 96.D0*mz**4*s*u**2*Yfun**(-2) + 8.D0*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*Yfun**(-2)*
     &    rzm2 - 192.D0*mz**4*s**2*u*Yfun**(-2) - 96.D0*mz**4*s**3*
     &    Yfun**(-2) - 16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*
     &    mz**6*u*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*mz**6*s*Yfun**(-2)*rzm2 + 128.D0*mz**6*s*u
     &    *Yfun**(-2) + 128.D0*mz**6*s**2*Yfun**(-2) + 32.D0*mz**8*
     &    s4fun**(-1)*Yfun**(-1) - 64.D0*mz**8*s*Yfun**(-2) + 16.D0*
     &    ctw**2*s2fun*s4fun**(-1)*rzm2 + 24.D0*ctw**2*u*Yfun**(-1)*
     &    rwm2*rzm2 + 16.D0*ctw**2*u*s2fun*s4fun**(-1) + 24.D0*ctw**2*
     &    u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 
     &    16.D0*ctw**2*s*s2fun*s4fun**(-1) + 48.D0*ctw**2*s*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0
     &    *ctw**2*s*u**4*Yfun**(-2) + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2
     &     + 36.D0*ctw**2*s**2*u**2*Yfun**(-2)*rzm2 + 48.D0*ctw**2*s**2
     &    *u**3*Yfun**(-2) + 36.D0*ctw**2*s**3*u*Yfun**(-2)*rzm2 + 72.D0
     &    *ctw**2*s**3*u**2*Yfun**(-2) + 12.D0*ctw**2*s**4*Yfun**(-2)*
     &    rzm2 + 48.D0*ctw**2*s**4*u*Yfun**(-2) + 12.D0*ctw**2*s**5*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*t1fun**(-1)*rwm2
     &     - 48.D0*ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 32.D0*ctw**2*
     &    mz**2*s2fun*s4fun**(-1) - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 72.D0*ctw**2*
     &    mz**2*s*u**2*Yfun**(-2)*rzm2 - 96.D0*ctw**2*mz**2*s*u**3*
     &    Yfun**(-2) - 144.D0*ctw**2*mz**2*s**2*u*Yfun**(-2)*rzm2 - 288.
     &    D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) - 72.D0*ctw**2*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 288.D0*ctw**2*mz**2*s**3*u*Yfun**(-2)
     &     - 96.D0*ctw**2*mz**2*s**4*Yfun**(-2) + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s*u*
     &    Yfun**(-2)*rzm2 + 288.D0*ctw**2*mz**4*s*u**2*Yfun**(-2) - 24.D
     &    0*ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*
     &    mz**4*s**2*Yfun**(-2)*rzm2 + 576.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) + 288.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 48.D0*
     &    ctw**2*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6
     &    *u*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**6*s*Yfun**(-2)*rzm2 - 384.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) - 384.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 96.D0*ctw**2*mz**8*s4fun**(-1)*Yfun**(-1) + 192.D
     &    0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 + 3.D0/2.D0*ctw**(-2)*
     &    s**2*s2fun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**3*s2fun*s4fun*
     &    Yfun**(-2) + ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - 1.D0/2.D0*ctw**(-2)*mz**4*s**2*s2fun*Yfun**(-2) + 4.D0*s*
     &    s2fun*Yfun**(-1)*rwm2 - 6.D0*s**2*s2fun*Yfun**(-1) + 2.D0*
     &    s**3*s2fun*s4fun*Yfun**(-2) - 4.D0*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*mz**4*s**2*s2fun*Yfun**(-2) - 16.D0*ctw**2*
     &    s*s2fun*s4fun**(-1) - 12.D0*ctw**2*s*s2fun*Yfun**(-1)*rwm2 + 
     &    18.D0*ctw**2*s**2*s2fun*Yfun**(-1) - 6.D0*ctw**2*s**3*s2fun*
     &    s4fun*Yfun**(-2) + 12.D0*ctw**2*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 6.D0*ctw**2*mz**4*s**2*s2fun*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**2*
     &    Yfun**(-1) - ctw**(-2)*s*u**4*Yfun**(-2) - ctw**(-2)*s**2*u*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**3*Yfun**(-1) - 3.D0*ctw**(-2)*s**3*u**2*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*s**4*u*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**5*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*Yfun**(-1) + 12.D0*ctw**(-2)*mz**2*s**2*
     &    u**2*Yfun**(-2) + 12.D0*ctw**(-2)*mz**2*s**3*u*Yfun**(-2) + 4.
     &    D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s*
     &    Yfun**(-1) - 11.D0*ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 23.D0*
     &    ctw**(-2)*mz**4*s**2*u*Yfun**(-2) - 23.D0/2.D0*ctw**(-2)*
     &    mz**4*s**3*Yfun**(-2) + 14.D0*ctw**(-2)*mz**6*s*u*Yfun**(-2)
     &     + 14.D0*ctw**(-2)*mz**6*s**2*Yfun**(-2) - 6.D0*ctw**(-2)*
     &    mz**8*s*Yfun**(-2) + 4.D0*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*
     &    u**2*Yfun**(-1) + 4.D0*s*u**4*Yfun**(-2) + 4.D0*s**2*u*
     &    Yfun**(-1) + 8.D0*s**2*u**3*Yfun**(-2) + 2.D0*s**3*Yfun**(-1)
     &     + 12.D0*s**3*u**2*Yfun**(-2) + 8.D0*s**4*u*Yfun**(-2) + 2.D0
     &    *s**5*Yfun**(-2) - 8.D0*mz**2*s*u*Yfun**(-1) - 16.D0*mz**2*s*
     &    u**3*Yfun**(-2) - 8.D0*mz**2*s**2*Yfun**(-1) - 48.D0*mz**2*
     &    s**2*u**2*Yfun**(-2) - 48.D0*mz**2*s**3*u*Yfun**(-2) - 16.D0*
     &    mz**2*s**4*Yfun**(-2) + 8.D0*mz**4*s*Yfun**(-1) + 44.D0*mz**4
     &    *s*u**2*Yfun**(-2) + 92.D0*mz**4*s**2*u*Yfun**(-2) + 46.D0*
     &    mz**4*s**3*Yfun**(-2) - 56.D0*mz**6*s*u*Yfun**(-2) - 56.D0*
     &    mz**6*s**2*Yfun**(-2) + 24.D0*mz**8*s*Yfun**(-2) - 16.D0*
     &    ctw**2*s - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1) - 12.D0*ctw**2*s*u**4*Yfun**(-2) - 
     &    12.D0*ctw**2*s**2*u*Yfun**(-1) - 24.D0*ctw**2*s**2*u**3*
     &    Yfun**(-2) - 6.D0*ctw**2*s**3*Yfun**(-1) - 36.D0*ctw**2*s**3*
     &    u**2*Yfun**(-2) - 24.D0*ctw**2*s**4*u*Yfun**(-2) - 6.D0*
     &    ctw**2*s**5*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*u*Yfun**(-1) + 
     &    48.D0*ctw**2*mz**2*s*u**3*Yfun**(-2) + 24.D0*ctw**2*mz**2*
     &    s**2*Yfun**(-1) + 144.D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) + 
     &    144.D0*ctw**2*mz**2*s**3*u*Yfun**(-2) + 48.D0*ctw**2*mz**2*
     &    s**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*s*Yfun**(-1) - 132.D0*
     &    ctw**2*mz**4*s*u**2*Yfun**(-2) - 276.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) - 138.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 168.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) + 168.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 72.D0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*b0fmzww * (  - 
     &    4.D0 + ctw**(-2) + ctw**(-2)*s2fun*s4fun**(-1) + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1) - ctw**(-2)*s*s2fun*Yfun**(-1) + 2.D
     &    0*ctw**(-2)*s*u*Yfun**(-1) + ctw**(-2)*s**2*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s4fun**(-1) - 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    ctw**(-2)*mz**4*s*u1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**6*u1fun**(-2)*s4fun**(-1) - 4.D0*s2fun*s4fun**(-1) - 8.D0
     &    *u**2*Yfun**(-1) + 4.D0*s*s2fun*Yfun**(-1) - 8.D0*s*u*
     &    Yfun**(-1) - 4.D0*s**2*Yfun**(-1) + 8.D0*mz**2*s4fun**(-1) + 
     &    16.D0*mz**2*u*Yfun**(-1) + 16.D0*mz**2*s*Yfun**(-1) - 16.D0*
     &    mz**4*Yfun**(-1) - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2 + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1) - 12.D0*
     &    ctw**2*s*s2fun*Yfun**(-1) + 24.D0*ctw**2*s*u*Yfun**(-1) + 12.D
     &    0*ctw**2*s**2*Yfun**(-1) - 24.D0*ctw**2*mz**2*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**2*u*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**4*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) + 
     &    48.D0*ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*b0fusww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 4.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 
     &    4.D0*u**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 8.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 6.D0*
     &    ctw**2 - 12.D0*ctw**2*u**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) - 48.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*b0ftsww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*s*u*Yfun**(-1) - ctw**(-2)*s**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*u**2*Yfun**(-1) + 8.D0*s*u*Yfun**(-1) + 4.D
     &    0*s**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 16.D0*mz**2*u*
     &    Yfun**(-1) - 16.D0*mz**2*s*Yfun**(-1) + 16.D0*mz**4*
     &    Yfun**(-1) + 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 4.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) - 6.D0*ctw**2 - 12.D0*ctw**2*
     &    u**2*Yfun**(-1) - 24.D0*ctw**2*s*u*Yfun**(-1) - 12.D0*ctw**2*
     &    s**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*s4fun**(-1) + 48.D0*
     &    ctw**2*mz**2*u*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*Yfun**(-1)
     &     - 48.D0*ctw**2*mz**4*Yfun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*b0fqsww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) + ctw**(-2)*s*s2fun*Yfun**(-1) + 
     &    4.D0*s2fun*s4fun**(-1) - 4.D0*s*s2fun*Yfun**(-1) - 12.D0*
     &    ctw**2*s2fun*s4fun**(-1) + 12.D0*ctw**2*s*s2fun*Yfun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun * ( 2.D
     &    0*ctw**(-2)*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 
     &    ctw**(-2)*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 2.D
     &    0*ctw**(-2)*mz**4*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 8.D0
     &    *mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 4.D0*mz**2*
     &    s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 8.D0*mz**4*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 24.D0*ctw**2*mz**2*s*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*mz**2*s**2
     &    *t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - 32.D0*ctw**2*s*u*s4fun**(-1)*rwm2 - 8.D0
     &    *ctw**2*s*u*s2fun*s4fun**(-1) - 16.D0*ctw**2*s**2*s4fun**(-1)
     &    *rwm2 - 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) + 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1)*rwm2 + 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**3*s4fun**(-1)*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 1.D0/2.D0*ctw**(-2)*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*s4fun**(-1)
     &    *rwm2 - 8.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*mz**2*s*u*s4fun**(-1) - 2.D0*mz**2*s**2*
     &    s4fun**(-1) + 4.D0*mz**4*s*s4fun**(-1) + 8.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*ctw**2*s*u*s4fun**(-1)*rwm2 - 8.
     &    D0*ctw**2*s*u*s2fun*s4fun**(-1) + 24.D0*ctw**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*ctw**2*s**2*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) + 8.D0*ctw**2*s**2*
     &    u*s4fun**(-1) + 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 32.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) - 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 6.D0*ctw**2*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)
     &    *s**2*s4fun**(-1)*rwm2 + 6.D0*ctw**(-2)*s**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 1.D0/2.D0*ctw**(-2)*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 6.D0*ctw**(-2)*s**3*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 3.D0/2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 5.D0/2.D0*ctw**(-2)*s**3*u**4*s4fun**(-1)*Yfun**(-2) + 2.D0
     &    *ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 3.D0/2.D0*
     &    ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-1) + 5.D0*ctw**(-2)*s**4
     &    *u**3*s4fun**(-1)*Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**5*
     &    s4fun**(-1)*Yfun**(-1) + 5.D0*ctw**(-2)*s**5*u**2*s4fun**(-1)
     &    *Yfun**(-2) + 5.D0/2.D0*ctw**(-2)*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**7*s4fun**(-1)*Yfun**(-2)
     &     - 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 - 12.D0*ctw**(-2)*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*ctw**(-2)*
     &    mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 5.D0*ctw**(-2)*mz**2
     &    *s**2*u**4*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*
     &    s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) - 20.D0*ctw**(-2)*mz**2*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 3.D0*ctw**(-2)*mz**2*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 30.D0*ctw**(-2)*mz**2*s**4*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 20.D0*ctw**(-2)*mz**2*s**5*u*
     &    s4fun**(-1)*Yfun**(-2) - 5.D0*ctw**(-2)*mz**2*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 22.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 22.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 39.D0/2.D0*ctw**(-2)*mz**4*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**(-2)*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 117.D0/2.D0*ctw**(-2)*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 117.D0/2.D0*ctw**(-2)*mz**4*
     &    s**4*u*s4fun**(-1)*Yfun**(-2) + 39.D0/2.D0*ctw**(-2)*mz**4*
     &    s**5*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 37.D0*ctw**(-2)*mz**6*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 74.D0*ctw**(-2)*mz**6*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) - 37.D0*ctw**(-2)*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 34.D0*ctw**(-2)*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 34.D0*ctw**(-2)*mz**8*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**10*s**2*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*s4fun**(-1)*rwm2 - 8.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*s**2*s4fun**(-1)*rwm2
     &     - 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*
     &    s**3*u**2*s4fun**(-1)*Yfun**(-1) - 10.D0*s**3*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*s**4*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 6.D0*s**4*u*s4fun**(-1)*Yfun**(-1) - 20.D0*s**4*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*s**5*s4fun**(-1)*Yfun**(-1) - 
     &    20.D0*s**5*u**2*s4fun**(-1)*Yfun**(-2) - 10.D0*s**6*u*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*s**7*s4fun**(-1)*Yfun**(-2) + 
     &    16.D0*mz**2*s*s4fun**(-1)*rwm2 - 4.D0*mz**2*s*u*s4fun**(-1)
     &     + 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*
     &    mz**2*s**2*s4fun**(-1) + 96.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 20.D0*mz**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 
     &    48.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*mz**2*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) + 80.D0*mz**2*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 120.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    80.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 20.D0*mz**2*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*mz**4*s*s4fun**(-1) - 88.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 88.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 78.D0*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 
     &    24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 234.D0*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 234.D0*mz**4*s**4*u*s4fun**(-1)
     &    *Yfun**(-2) - 78.D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) + 48.D0
     &    *mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 148.D0*mz**6*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 296.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) + 148.
     &    D0*mz**6*s**4*s4fun**(-1)*Yfun**(-2) - 136.D0*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 136.D0*mz**8*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 48.D0*mz**10*s**2*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    ctw**2*s*u*s4fun**(-1)*rwm2 - 8.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 8.D0*ctw**2*s**2*s4fun**(-1)*rwm2 - 4.D0*ctw**2*s**2*s2fun
     &    *s4fun**(-1) + 8.D0*ctw**2*s**2*u*s4fun**(-1) + 72.D0*ctw**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*ctw**2*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*ctw**2*s**3*s4fun**(-1) + 72.D0*ctw**2*s**3
     &    *u*s4fun**(-1)*Yfun**(-1)*rwm2 + 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 30.D0*ctw**2*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    18.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-1) + 60.D0*ctw**2*
     &    s**4*u**3*s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-1) + 60.D0*ctw**2*s**5*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 30.D0*ctw**2*s**6*u*s4fun**(-1)*Yfun**(-2) + 6.D0
     &    *ctw**2*s**7*s4fun**(-1)*Yfun**(-2) - 16.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*rwm2 + 8.D0*ctw**2*mz**2*s*s2fun*s4fun**(-1) - 
     &    144.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0
     &    *ctw**2*mz**2*s**2*s4fun**(-1) - 288.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 36.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 60.D0*ctw**2*mz**2*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*
     &    Yfun**(-1) - 240.D0*ctw**2*mz**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*Yfun**(-1)
     &     - 360.D0*ctw**2*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 240.
     &    D0*ctw**2*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 60.D0*ctw**2*
     &    mz**2*s**6*s4fun**(-1)*Yfun**(-2) + 264.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 264.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 234.D0*ctw**2*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 72.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 702.D0*ctw**2*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 702.D0*ctw**2*mz**4*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) + 234.D0*ctw**2*mz**4*s**5*s4fun**(-1)*Yfun**(-2)
     &     - 144.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1) - 444.D0*ctw**2*
     &    mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 888.D0*ctw**2*mz**6*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) - 444.D0*ctw**2*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 408.D0*ctw**2*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 408.D0*ctw**2*mz**8*s**3*s4fun**(-1)
     &    *Yfun**(-2) - 144.D0*ctw**2*mz**10*s**2*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2
     &     - 2.D0*ctw**(-2)*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s*
     &    u**4*s4fun**(-1)*Yfun**(-2)*rzm2 + ctw**(-2)*s*u**5*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*u1fun**(-1)*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - ctw**(-2)*mz**4*s*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 8.D0*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    4.D0*s*u**3*s4fun**(-1)*Yfun**(-1) + 4.D0*s*u**4*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1)*rwm2 - 8.D0*mz**4*s*u*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 4.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*ctw**2*s*s4fun**(-1)*rzm2 + 16.D0*ctw**2*s
     &    *u*s4fun**(-1) - 12.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 24.D0*ctw**2*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)
     &    *rwm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1) - 12.D0*
     &    ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 12.D0*ctw**2*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s*u*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*mz**4*
     &    s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 12.D0*ctw**2*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 2.D
     &    0*ctw**(-2)*s*u**3*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s*u**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + ctw**(-2)*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 6.D0*ctw**(-2)*s**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 3.D0*ctw**(-2)*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 5.D0*ctw**(-2)*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 
     &    ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 3.D0*
     &    ctw**(-2)*s**3*u*s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**(-2)*s**3
     &    *u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 10.D0*ctw**(-2)*s**3*u**3
     &    *s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*s**4*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + ctw**(-2)*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 10.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    ctw**(-2)*s**5*s4fun**(-1)*Yfun**(-2)*rzm2 + 5.D0*ctw**(-2)*
     &    s**5*u*s4fun**(-1)*Yfun**(-2) + ctw**(-2)*s**6*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 12.D0*ctw**(-2)*mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 24.D0*ctw**(-2)*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 40.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 12.D0*ctw**(-2)*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 60.D0*ctw**(-2)*mz**2*s**3*u**2
     &    *s4fun**(-1)*Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*ctw**(-2)*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 10.D0*ctw**(-2)*mz**2*s**5*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 22.D0*ctw**(-2)*mz**4*s*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 23.D0*ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 39.D0*ctw**(-2)*mz**4*s*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 22.D0*ctw**(-2)*mz**4*s**2*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**(-2)*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 46.D0*ctw**(-2)*mz**4*
     &    s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 117.D0*ctw**(-2)*mz**4*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-2) + 23.D0*ctw**(-2)*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 + 117.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) + 39.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 28.D0*ctw**(-2)*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 74.D0*ctw**(-2)*mz**6*s*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 28.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 148.D0*ctw**(-2)*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 74.D0*ctw**(-2)*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 68.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*
     &    Yfun**(-2) + 68.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2)
     &     - 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 24.D0*s**2*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 - 20.
     &    D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 24.D0*s**3*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*s**3*u*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *s**3*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 4.D0*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**4*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 4.D0*s**5*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 20.D0*s**5*u*s4fun**(-1)*Yfun**(-2) - 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)
     &    *rwm2 + 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 
     &    96.D0*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    48.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 96.D0*mz**2*s**2*
     &    u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 160.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 48.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     + 240.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 32.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 160.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 40.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 88.D0
     &    *mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 48.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 92.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 156.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 88.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 
     &    184.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 468.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 92.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 468.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) - 156.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 48.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) + 112.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 296.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     + 112.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 592.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 48.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 - 272.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 272.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*ctw**2*s*s4fun**(-1)*rzm2 + 16.
     &    D0*ctw**2*s*u*s4fun**(-1) + 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 24.D0*ctw**2*s*u**3*t1fun**(-1)*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 12.D0*ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 12.D0*
     &    ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) + 16.D0*ctw**2*s**2*
     &    s4fun**(-1) + 24.D0*ctw**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     - 72.D0*ctw**2*s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 48.D0*
     &    ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 + 60.D0*ctw**2*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 72.D0*ctw**2*s**3*u*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 36.D0*ctw**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) + 72.D0*ctw**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 120.D0*ctw**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*ctw**2*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1) + 
     &    48.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 120.D0*
     &    ctw**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 60.D0*ctw**2*s**5*u*s4fun**(-1)
     &    *Yfun**(-2) + 12.D0*ctw**2*s**6*s4fun**(-1)*Yfun**(-2) - 24.D0
     &    *ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*rwm2 - 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1) - 48.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**2*s*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*ctw**2*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 120.D0*ctw**2*mz**2*s*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 48.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 288.D0*ctw**2*mz**2*s**2*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 144.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 288.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 480.D0*ctw**2*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*mz**2*s**3*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 288.D0*ctw**2*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 720.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*ctw**2*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 480.D0*ctw**2*mz**2*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) - 120.D0*ctw**2*mz**2*s**5*s4fun**(-1)*Yfun**(-2)
     &     + 48.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 264.D0*
     &    ctw**2*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    144.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 276.D0*
     &    ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 468.D0*
     &    ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) - 264.D0*ctw**2*
     &    mz**4*s**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 144.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 552.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 1404.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 276.D0*ctw**2*mz**4*
     &    s**3*s4fun**(-1)*Yfun**(-2)*rzm2 + 1404.D0*ctw**2*mz**4*s**3*
     &    u*s4fun**(-1)*Yfun**(-2) + 468.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 336.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 888.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 336.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 1776.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 
     &    888.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2) + 144.D0*
     &    ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-2)*rzm2 + 816.D0*ctw**2*
     &    mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 816.D0*ctw**2*mz**8*s**2*
     &    s4fun**(-1)*Yfun**(-2) - 288.D0*ctw**2*mz**10*s*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &    *rwm2 - ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**2*u*s2fun*s4fun**(-1)*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**3*s2fun*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s**3*
     &    u*s2fun*Yfun**(-2) - 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*mz**2*s**3*s2fun*Yfun**(-2) - 
     &    ctw**(-2)*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2) - 1.D0/2.D
     &    0*ctw**(-2)*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 
     &    ctw**(-2)*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 2.D0*s**3*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*
     &    s**3*u*s2fun*Yfun**(-2) + 2.D0*s**4*s2fun*Yfun**(-2) - 8.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**2*s**3*s2fun*
     &    Yfun**(-2) + 4.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2)
     &     + 2.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 4.D0*mz**6*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**2*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*ctw**2*s**3*u*s2fun*Yfun**(-2) - 6.D0*
     &    ctw**2*s**4*s2fun*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**2*s**3*s2fun*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 6.D0*ctw**2*mz**4*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 12.D0*ctw**2*mz**6*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * (  - 2.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 - 2.D0*
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) - ctw**(-2)*s**2*Yfun**(-1)*rwm2
     &     - ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) - 3.D0*
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 5.D0*ctw**(-2)*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*ctw**(-2)*s**3*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 1.D0/2.D0*ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)
     &     - 7.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 3.D0*
     &    ctw**(-2)*s**5*u*s4fun**(-1)*Yfun**(-2) - 1.D0/2.D0*ctw**(-2)
     &    *s**6*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)
     &    *Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 3.D0*ctw**(-2)*mz**2*s**3*s4fun**(-1)*Yfun**(-1)
     &     + 42.D0*ctw**(-2)*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    24.D0*ctw**(-2)*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) + 5.D0*
     &    ctw**(-2)*mz**2*s**5*s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 30.D0*ctw**(-2)*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) - 5.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 81.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 70.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) - 39.D0/2.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 50.D0*ctw**(-2)*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 88.D0*ctw**(-2)*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) + 37.D0*ctw**(-2)*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 40.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*Yfun**(-2)
     &     - 34.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 12.D0*
     &    ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*
     &    Yfun**(-1)*rwm2 + 8.D0*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) + 
     &    8.D0*s*u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**2*Yfun**(-1)*rwm2 + 4.D0*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 20.D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) + 32.D0*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*s**4*s4fun**(-1)*Yfun**(-1) + 28.D0*s**4*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 12.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*s**6*s4fun**(-1)*Yfun**(-2) - 8.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 - 8.D0*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 40.D0
     &    *mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) - 32.D0*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 128.D0*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 12.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 168.D0
     &    *mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 20.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 120.D0*
     &    mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 20.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 324.D0*mz**4*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 280.D0*mz**4*s**3*u*s4fun**(-1)*Yfun**(-2) + 78.D
     &    0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 8.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 200.D0*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 352.D0*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 148.
     &    D0*mz**6*s**3*s4fun**(-1)*Yfun**(-2) + 160.D0*mz**8*s*u*
     &    s4fun**(-1)*Yfun**(-2) + 136.D0*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) - 48.D0*mz**10*s*s4fun**(-1)*Yfun**(-2) - 24.D0*
     &    ctw**2*s*u*Yfun**(-1)*rwm2 - 24.D0*ctw**2*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) - 12.D
     &    0*ctw**2*s**2*Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) - 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 60.D0*ctw**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 
     &    24.D0*ctw**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 96.D0*ctw**2*
     &    s**3*u**3*s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**2*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 84.D0*ctw**2*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 36.D0*ctw**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 6.D0
     &    *ctw**2*s**6*s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 120.D0*ctw**2*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) + 96.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 384.D0*ctw**2*
     &    mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 36.D0*ctw**2*mz**2*
     &    s**3*s4fun**(-1)*Yfun**(-1) + 504.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 288.D0*ctw**2*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 60.D0*ctw**2*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 72.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 
     &    360.D0*ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) - 60.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 972.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 840.D0*ctw**2*mz**4*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) - 234.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 600.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 1056.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) + 444.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2)
     &     - 480.D0*ctw**2*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 408.D0*
     &    ctw**2*mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*
     &    mz**10*s*s4fun**(-1)*Yfun**(-2) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & b0fmzww * ( 4.D0*ctw**(-2)*u*s4fun**(-1) + 4.D0*ctw**(-2)*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*s*s4fun**(-1) - 2.D0*
     &    ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**(-2)*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1) - 12.D0*ctw**(-2)*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*ctw**(-2)*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*s4fun**(-1)
     &     + 16.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1) + ctw**(-2)
     &    *mz**4*s*t1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*
     &    u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) - 
     &    2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) - 8.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1) - 16.D0*u*s4fun**(-1) - 16.D0*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 8.D0*s*s4fun**(-1) + 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 24.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 48.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 64.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**6*u1fun**(-2)*s4fun**(-1) + 32.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*u*s4fun**(-1) + 
     &    48.D0*ctw**2*u**3*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*s*
     &    s4fun**(-1) - 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &     + 72.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**2*s4fun**(-1) - 144.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**2*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) + 
     &    24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 192.D0*ctw**2*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 144.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1) - 
     &    24.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) - 96.D0*ctw**2*mz**6*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & b0fusww * (  - 2.D0 + 1.D0/2.D0*ctw**(-2) - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) - 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 
     &    8.D0*u*s4fun**(-1) + 8.D0*u**3*s4fun**(-1)*Yfun**(-1) + 4.D0*
     &    s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 4.D0*
     &    mz**4*s*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**6*u1fun**(-2)*
     &    s4fun**(-1) + 6.D0*ctw**2 - 24.D0*ctw**2*u*s4fun**(-1) - 24.D0
     &    *ctw**2*u**3*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) - 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & b0ftsww * ( 2.D0 - 1.D0/2.D0*ctw**(-2) - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) - 2.D
     &    0*ctw**(-2)*s*s4fun**(-1) - 5.D0*ctw**(-2)*s*u**2*s4fun**(-1)
     &    *Yfun**(-1) - 4.D0*ctw**(-2)*s**2*u*s4fun**(-1)*Yfun**(-1) - 
     &    ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) + 10.D0*ctw**(-2)*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)
     &     + 6.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*u*t1fun**(-2)*s4fun**(-1) - 16.D0*ctw**(-2)*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) + 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 
     &    8.D0*ctw**(-2)*mz**6*s4fun**(-1)*Yfun**(-1) + 8.D0*u*
     &    s4fun**(-1) + 8.D0*u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*s*
     &    s4fun**(-1) + 20.D0*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s4fun**(-1) - 40.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1) + 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*mz**4*s*t1fun**(-2)*s4fun**(-1) + 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1) - 6.D0*
     &    ctw**2 - 24.D0*ctw**2*u*s4fun**(-1) - 24.D0*ctw**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s*s4fun**(-1) - 60.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**2*s4fun**(-1) + 120.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) - 
     &    192.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) - 144.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 96.D0*ctw**2*mz**6*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,1,2) = ewamp(1,2,1,2) + g**4*stw**2*betaZfun*
     & b0fqsww * ( 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      ewamp(1,2,3,1) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - deltafun*Yfun - 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     - ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**2*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*u**2*s4fun**(-1)*deltafun - 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**4*u*s4fun**(-1)*deltafun + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 4.D0*mz**6*s4fun**(-1)*deltafun - 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun - 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.D0
     &    *ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    /2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 5.D0*s**2*u*s4fun**(-1)*deltafun + 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**3*
     &    s4fun**(-1)*deltafun + 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun - 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun + 22.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 20.D0*mz**6*s4fun**(-1)*deltafun - 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun - 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D
     &    0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0
     &    *ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*
     &    rzm2 + 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0
     &    *s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*
     &    deltafun - 8.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D
     &    0*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun - 4.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*mz**2*s**(-1)*deltafun*
     &    rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *mz**2*s**(-1)*u*deltafun - 16.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0
     &    *ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D
     &    0*ctw**2*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 
     &    32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*
     &    mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s**(-1)*
     &    u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 + 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*rzm2 + 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*s**(-1)*u**2*deltafun
     &     + 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 6.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun + 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*s**(-1)*u*deltafun + 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*deltafun
     &     - 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s**(-1)*
     &    deltafun + 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun - 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*c0ccqswww * ( 
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    deltafun - 2.D0*s*deltafun - 4.D0*s*u*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**2*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**2*deltafun + 4.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*ctw**2*s*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*mz**2*s*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*c0mzmzqswww
     &  * ( ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) - 32.D0*ctw**2*u*deltafun - 16.D0
     &    *ctw**2*s*deltafun + 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) + 32.D
     &    0*ctw**2*mz**2*deltafun - 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*deltafun + 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u*s4fun**(-1)*deltafun - 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun - 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*s4fun**(-1)*deltafun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun + 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*b0ftsww * (  - 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*b0fqsww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    24.D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun * ( 
     &    ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - deltafun*Yfun - 64.D0*ctw**2*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun + s**2*u*s4fun**(-1)*deltafun + 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 + 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + s**3*s4fun**(-1)*deltafun - 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun - 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 10.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     + 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun + 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u
     &    *u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 16.D0*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*u*deltafun + 8.D0*s*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**2*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*
     &    mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*mz**2*u*
     &    s4fun**(-1)*deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun
     &    *rwm2*rzm2 + 2.D0*deltafun*rzm2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*u*deltafun - 16.D0*s*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*s*deltafun - 8.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0
     &    *mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*deltafun + 16.D0
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*mz**2*s*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun + 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*deltafun
     &     - 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * ( ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun - 1.D0/2.D
     &    0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 
     &    16.D0*ctw**2*s*deltafun + 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & b0fmzww * ( ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & b0fusww * ( ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 4.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun - 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,3,1) = ewamp(1,2,3,1) + g**4*stw**2*betaZfun*
     & b0fqsww * ( ctw**(-2)*s*s4fun**(-1)*deltafun - 4.D0*s*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(1,2,3,3) =
     &  + g**4*stw**2 * (  - 4.D0*ctw**(-2)*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**2*t1fun**(-1)
     &    *u1fun**(-1)*s4fun**(-1)*Yfun )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*
     &    s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*rwm2 - 
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 8.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**4*s4fun**(-1)
     &    *rwm2 - 4.D0*ctw**(-2)*mz**6*s**(-1)*s4fun**(-1)*rwm2 + 32.D0
     &    *s4fun**(-1)*Yfun*rwm2 + 32.D0*rwm2**2 + 2.D0*Yfun + 4.D0*
     &    s2fun*rwm2 + 16.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 + 16.D0
     &    *mz**2*u*s4fun**(-1)*rwm2 + 4.D0*mz**2*s*s4fun**(-1)*rwm2 - 
     &    32.D0*mz**4*s**(-1)*u*s4fun**(-1)*rwm2 - 16.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 16.D0*mz**6*s**(-1)*s4fun**(-1)*rwm2 - 48.D
     &    0*ctw**2*mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 - 48.D0*ctw**2*
     &    mz**2*u*s4fun**(-1)*rwm2 - 12.D0*ctw**2*mz**2*s*s4fun**(-1)*
     &    rwm2 + 96.D0*ctw**2*mz**4*s**(-1)*u*s4fun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**4*s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**6*s**(-1)*
     &    s4fun**(-1)*rwm2 )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*
     &    rwm2**2 + 4.D0*s2fun*rwm2 - 32.D0*u1fun**2*s4fun**(-1)*rwm2
     &     + 16.D0*s*u**2*Yfun**(-1)*rwm2 + 2.D0*s*u**4*s4fun**(-1)*
     &    Yfun**(-1) - 2.D0*s**2*u*s4fun**(-1) - 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) - 2.D0*mz**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 6.D0*ctw**2*mz**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*
     &    s**3*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*rwm2**2 + 4.D0*s2fun*rwm2 - 32.
     &    D0*t1fun**2*s4fun**(-1)*rwm2 + 16.D0*s*u**2*Yfun**(-1)*rwm2
     &     + 2.D0*s*u**4*s4fun**(-1)*Yfun**(-1) + 2.D0*s**2*u*
     &    s4fun**(-1) + 32.D0*s**2*u*Yfun**(-1)*rwm2 + 8.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*s**3*s4fun**(-1) + 16.D0*s**3*
     &    Yfun**(-1)*rwm2 + 12.D0*s**3*u**2*s4fun**(-1)*Yfun**(-1) + 8.D
     &    0*s**4*u*s4fun**(-1)*Yfun**(-1) + 2.D0*s**5*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*mz**2*s*s4fun**(-1)*rwm2 - 64.D0*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) - 16.D0*mz**2*
     &    s*u**3*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**2*s**2*s4fun**(-1)
     &     - 64.D0*mz**2*s**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**2*u*
     &    Yfun**(-1) - 50.D0*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 4.
     &    D0*mz**2*s**3*Yfun**(-1) - 52.D0*mz**2*s**3*u*s4fun**(-1)*
     &    Yfun**(-1) - 18.D0*mz**2*s**4*s4fun**(-1)*Yfun**(-1) + 64.D0*
     &    mz**4*s*Yfun**(-1)*rwm2 + 16.D0*mz**4*s*u*Yfun**(-1) + 44.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 52.D0
     &    *mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s*Yfun**(-1)
     &     - 48.D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 56.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 18.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 6.D0*ctw**2*mz**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**2*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*ctw**2*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*c0cmzuswww * ( 
     &     - 8.D0*ctw**(-2)*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 - ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    s4fun**(-1)*rwm2*rzm2 - 4.D0*rzm2 - 32.D0*u1fun*s4fun**(-1)*
     &    rwm2 + 64.D0*u*s4fun**(-1)*rwm2 + 4.D0*u + 4.D0*u*u1fun*s2fun
     &    *Yfun**(-1) - 32.D0*u**2*u1fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 64.D0
     &    *mz**2*u*u1fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 4.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 32.D0*mz**4*u1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    96.D0*ctw**2*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*rwm2
     &     - 12.D0*ctw**2*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 12.D0
     &    *ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*c0cmztswww * ( 
     &     - 8.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 + ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*mz**4*s**2*s4fun**(-1)*Yfun**(-1)
     &     + 4.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    s4fun**(-1)*rwm2*rzm2 - 4.D0*rzm2 - 32.D0*t1fun*s4fun**(-1)*
     &    rwm2 - 64.D0*u*s4fun**(-1)*rwm2 - 4.D0*u - 4.D0*u*t1fun*s2fun
     &    *Yfun**(-1) - 32.D0*u**2*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    64.D0*s*s4fun**(-1)*rwm2 - 4.D0*s - 4.D0*s*t1fun*s2fun*
     &    Yfun**(-1) - 64.D0*s*u*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 32.
     &    D0*s**2*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 128.D0*mz**2*
     &    s4fun**(-1)*rwm2 + 8.D0*mz**2 + 8.D0*mz**2*t1fun*s2fun*
     &    Yfun**(-1) + 192.D0*mz**2*u*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 192.D0*mz**2*s*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*
     &    mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 4.D0*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 8.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 
     &    4.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 288.D0*mz**4*t1fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 8.D0*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 16.
     &    D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*ctw**2*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 12.D0*ctw**2*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 24.D0*ctw**2*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**2*mz**2*s**3*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*c0ccqswww * ( 
     &     - 1.D0/2.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 64.D0*s*s4fun**(-1)*rwm2 + 4.D0*s - 16.D0*s*
     &    s2fun*Yfun**(-1)*rwm2 - 2.D0*s*s2fun**2*Yfun**(-1) + 2.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**2*mz**2*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*c0mzmzqswww
     &  * ( ctw**(-2)*mz**2*s*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*
     &    s**2*Yfun**(-1) + 32.D0*rwm2 + 4.D0*s2fun - 16.D0*s*s4fun*
     &    Yfun**(-1)*rwm2 - 2.D0*s*s2fun*s4fun*Yfun**(-1) - 4.D0*mz**2*
     &    s*s4fun**(-1) + 2.D0*mz**2*s**2*Yfun**(-1) + 12.D0*ctw**2*
     &    mz**2*s*s4fun**(-1) - 6.D0*ctw**2*mz**2*s**2*Yfun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*u**2*u1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)
     &    *mz**2*s*u*t1fun**(-2)*s4fun**(-1) + 2.D0*ctw**(-2)*mz**2*
     &    s**2*t1fun**(-2)*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 10.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1)
     &     + 2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**2*
     &    u**2*t1fun**(-2)*s4fun**(-1) - 8.D0*mz**2*u**2*u1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**2*s*u*t1fun**(-2)*s4fun**(-1) - 8.D0*
     &    mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 32.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 40.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 8.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*mz**2*u**2*
     &    t1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*mz**2*u**2*u1fun**(-2)
     &    *s4fun**(-1) + 48.D0*ctw**2*mz**2*s*u*t1fun**(-2)*s4fun**(-1)
     &     + 24.D0*ctw**2*mz**2*s**2*t1fun**(-2)*s4fun**(-1) - 96.D0*
     &    ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) - 96.D0*ctw**2*mz**4*s
     &    *t1fun**(-2)*s4fun**(-1) + 120.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*mz**2*u**2*u1fun**(-2)*s4fun**(-1) - 2.D0*
     &    ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 8.D0*mz**2*u**2*
     &    u1fun**(-2)*s4fun**(-1) + 8.D0*mz**6*u1fun**(-2)*s4fun**(-1)
     &     - 24.D0*ctw**2*mz**2*u**2*u1fun**(-2)*s4fun**(-1) - 24.D0*
     &    ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,3,3) = ewamp(1,2,3,3) + g**4*stw**2*b0ftsww * (  - 
     &    2.D0*ctw**(-2)*mz**2*u**2*t1fun**(-2)*s4fun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*u*t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 8.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 8.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 10.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1)
     &     + 8.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 16.D0*mz**2*s*u*
     &    t1fun**(-2)*s4fun**(-1) + 8.D0*mz**2*s**2*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 40.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 24.D0*ctw**2*mz**2*u**2*t1fun**(-2)*s4fun**(-1)
     &     - 48.D0*ctw**2*mz**2*s*u*t1fun**(-2)*s4fun**(-1) - 24.D0*
     &    ctw**2*mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 96.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 96.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) - 120.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      ewamp(1,2,3,2) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + deltafun*Yfun + 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**2*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u**2*s4fun**(-1)*deltafun + 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**4*u*s4fun**(-1)*deltafun - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**6*s4fun**(-1)*deltafun + 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun + 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.D0*
     &    ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 5.D0*s**2*u*s4fun**(-1)*deltafun - 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**3*
     &    s4fun**(-1)*deltafun - 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun + 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*s4fun**(-1)*deltafun + 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 - 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun - 22.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 20.D0*mz**6*s4fun**(-1)*deltafun + 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun + 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun + 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 4.D0*s**(-1)*u**2*deltafun + 8.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D
     &    0*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*mz**2*
     &    s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s**(-1)*u*deltafun + 16.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)
     &    *deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 48.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 - 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 4.D0*s**(-1)*u*deltafun*rzm2 - 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*deltafun
     &     - 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*deltafun*rzm2 - 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 6.D0*u*
     &    deltafun - 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun - 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*s**(-1)*u*deltafun - 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*deltafun
     &     + 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s**(-1)*
     &    deltafun - 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 - 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 1.
     &    D0/2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*u*deltafun + 2.D0*s*deltafun + 4.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*s**2*s2fun**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*deltafun - 4.D0
     &    *mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**6*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*s*u*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2
     &    *s*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) + 32.D0*ctw**2*u*deltafun + 16.D0
     &    *ctw**2*s*deltafun - 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) - 32.D
     &    0*ctw**2*mz**2*deltafun + 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - ctw**(-2)*s*s4fun**(-1)*deltafun - 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*deltafun + 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*s4fun**(-1)*deltafun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun - 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*b0fqsww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*
     &    deltafun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 8.D0*u
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun + 8.D0*
     &    mz**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 24.D0*ctw**2
     &    *mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun * ( 
     &    ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - deltafun*Yfun - 64.D0*ctw**2*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun + s**2*u*s4fun**(-1)*deltafun + 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 + 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + s**3*s4fun**(-1)*deltafun - 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun - 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 10.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     + 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun + 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u
     &    *u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 16.D0*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*u*deltafun + 8.D0*s*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**2*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*
     &    mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*mz**2*u*
     &    s4fun**(-1)*deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun
     &    *rwm2*rzm2 + 2.D0*deltafun*rzm2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*u*deltafun - 16.D0*s*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*s*deltafun - 8.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0
     &    *mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*deltafun + 16.D0
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*mz**2*s*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun + 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*deltafun
     &     - 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * ( ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun - 1.D0/2.D
     &    0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 
     &    16.D0*ctw**2*s*deltafun + 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & b0fmzww * ( ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & b0fusww * ( ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 4.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun - 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,3,2) = ewamp(1,2,3,2) + g**4*stw**2*betaZfun*
     & b0fqsww * ( ctw**(-2)*s*s4fun**(-1)*deltafun - 4.D0*s*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(1,2,2,1) =
     &  + g**4*stw**2 * ( ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 - 
     &    8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D
     &    0*ctw**2*s*s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**4*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*rwm2**2
     &     - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 
     &    4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 - 8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun
     &     + 16.D0*ctw**2*s*rwm2 + 4.D0*ctw**2*s*s2fun - 8.D0*ctw**2*s*
     &    u*s2fun*s4fun**(-1) + 24.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2 + 6.
     &    D0*ctw**2*s**2*u**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s**2*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*
     &    s**3*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**3*u**3*Yfun**(-2) + 
     &    3.D0*ctw**(-2)*s**4*u**2*Yfun**(-2) + 2.D0*ctw**(-2)*s**5*u*
     &    Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**6*Yfun**(-2) - 8.D0*
     &    ctw**(-2)*mz**2*s*u*Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s**2*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*u**3*
     &    Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*s**3*u**2*Yfun**(-2) - 12.D
     &    0*ctw**(-2)*mz**2*s**4*u*Yfun**(-2) - 4.D0*ctw**(-2)*mz**2*
     &    s**5*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 8.D0
     &    *ctw**(-2)*mz**4*s*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    Yfun**(-2) - ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 24.
     &    D0*ctw**(-2)*mz**4*s**3*u*Yfun**(-2) + 12.D0*ctw**(-2)*mz**4*
     &    s**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*ctw**(-2)*mz**6*s**2*u*Yfun**(-2) - 16.D0*ctw**(-2)*
     &    mz**6*s**3*Yfun**(-2) - 4.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s**2*Yfun**(-2) - 8.D0*
     &    rwm2**2 - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 16.D0*s**2*u*
     &    Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 8.D0*s**3*
     &    Yfun**(-1)*rwm2 - 8.D0*s**3*u**3*Yfun**(-2) - 12.D0*s**4*u**2
     &    *Yfun**(-2) - 8.D0*s**5*u*Yfun**(-2) - 2.D0*s**6*Yfun**(-2)
     &     - 4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 32.D0*mz**2*s
     &    *u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s**2*Yfun**(-1)*rwm2 + 16.D0
     &    *mz**2*s**2*u**3*Yfun**(-2) + 48.D0*mz**2*s**3*u**2*
     &    Yfun**(-2) + 48.D0*mz**2*s**4*u*Yfun**(-2) + 16.D0*mz**2*s**5
     &    *Yfun**(-2) + 8.D0*mz**4*s4fun**(-1)*rwm2 - 32.D0*mz**4*s*
     &    Yfun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 
     &    8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    u**2*Yfun**(-2) + 4.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 96.
     &    D0*mz**4*s**3*u*Yfun**(-2) - 48.D0*mz**4*s**4*Yfun**(-2) - 16.
     &    D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s**2*u*Yfun**(-2) + 64.D0
     &    *mz**6*s**3*Yfun**(-2) + 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s**2*Yfun**(-2) + 24.D0*ctw**2*rwm2**2 - 8.D0*
     &    ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D0*
     &    ctw**2*s*s2fun + 8.D0*ctw**2*s*u*s2fun*s4fun**(-1) + 24.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 8.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1) + 48.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 6.D0*
     &    ctw**2*s**2*u**4*Yfun**(-2) + 24.D0*ctw**2*s**3*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**3*u**3*Yfun**(-2) + 36.D0*ctw**2*s**4*
     &    u**2*Yfun**(-2) + 24.D0*ctw**2*s**5*u*Yfun**(-2) + 6.D0*
     &    ctw**2*s**6*Yfun**(-2) - 16.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 96.D0*
     &    ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    u**3*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*u**2*Yfun**(-2) - 
     &    144.D0*ctw**2*mz**2*s**4*u*Yfun**(-2) - 48.D0*ctw**2*mz**2*
     &    s**5*Yfun**(-2) - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 96.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*u**2*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*Yfun**(-1)
     &     + 288.D0*ctw**2*mz**4*s**3*u*Yfun**(-2) + 144.D0*ctw**2*
     &    mz**4*s**4*Yfun**(-2) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 192.D0*ctw**2*mz**6*s**2*u*Yfun**(-2) - 192.D0*ctw**2*
     &    mz**6*s**3*Yfun**(-2) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**8*s**2*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    u**2*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**3*Yfun**(-2)*rzm2 + 
     &    ctw**(-2)*s*u**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*
     &    Yfun**(-1)*rwm2 + 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0*s*u**4*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*u1fun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*ctw**2*s2fun*s4fun**(-1)*rzm2 - 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 - 16.D0*ctw**2*u*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0*ctw**2*s*u**4*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*u1fun**(-1)*rwm2
     &     + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*s*u*Yfun**(-1)*rwm2 + ctw**(-2)*s*u**3*Yfun**(-2)*
     &    rzm2 + ctw**(-2)*s*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*s**2*
     &    Yfun**(-1)*rwm2 + 3.D0*ctw**(-2)*s**2*u**2*Yfun**(-2)*rzm2 + 
     &    4.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) + 3.D0*ctw**(-2)*s**3*u*
     &    Yfun**(-2)*rzm2 + 6.D0*ctw**(-2)*s**3*u**2*Yfun**(-2) + 
     &    ctw**(-2)*s**4*Yfun**(-2)*rzm2 + 4.D0*ctw**(-2)*s**4*u*
     &    Yfun**(-2) + ctw**(-2)*s**5*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*Yfun**(-1)*rwm2
     &    *rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s*
     &    u**2*Yfun**(-2)*rzm2 - 8.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2)
     &     - 12.D0*ctw**(-2)*mz**2*s**2*u*Yfun**(-2)*rzm2 - 24.D0*
     &    ctw**(-2)*mz**2*s**2*u**2*Yfun**(-2) - 6.D0*ctw**(-2)*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) + 8.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s*u*Yfun**(-2)*rzm2 + 24.D0
     &    *ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*
     &    Yfun**(-2)*rzm2 + 48.D0*ctw**(-2)*mz**4*s**2*u*Yfun**(-2) + 
     &    24.D0*ctw**(-2)*mz**4*s**3*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*Yfun**(-2)*rzm2 - 32.D0*
     &    ctw**(-2)*mz**6*s*u*Yfun**(-2) - 32.D0*ctw**(-2)*mz**6*s**2*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 16.
     &    D0*ctw**(-2)*mz**8*s*Yfun**(-2) - 8.D0*u*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 
     &    16.D0*s*u*Yfun**(-1)*rwm2 - 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0
     &    *s*u**4*Yfun**(-2) - 8.D0*s**2*Yfun**(-1)*rwm2 - 12.D0*s**2*
     &    u**2*Yfun**(-2)*rzm2 - 16.D0*s**2*u**3*Yfun**(-2) - 12.D0*
     &    s**3*u*Yfun**(-2)*rzm2 - 24.D0*s**3*u**2*Yfun**(-2) - 4.D0*
     &    s**4*Yfun**(-2)*rzm2 - 16.D0*s**4*u*Yfun**(-2) - 4.D0*s**5*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*t1fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 + 24.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 32.D0*
     &    mz**2*s*u**3*Yfun**(-2) + 48.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 96.D0*mz**2*s**2*u**2*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 96.D0*mz**2*s**3*u*Yfun**(-2) + 32.D0*mz**2
     &    *s**4*Yfun**(-2) - 32.D0*mz**4*Yfun**(-1)*rwm2 + 8.D0*mz**4*u
     &    *s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0
     &    *mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s*u*
     &    Yfun**(-2)*rzm2 - 96.D0*mz**4*s*u**2*Yfun**(-2) + 8.D0*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*Yfun**(-2)*
     &    rzm2 - 192.D0*mz**4*s**2*u*Yfun**(-2) - 96.D0*mz**4*s**3*
     &    Yfun**(-2) - 16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*
     &    mz**6*u*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*mz**6*s*Yfun**(-2)*rzm2 + 128.D0*mz**6*s*u
     &    *Yfun**(-2) + 128.D0*mz**6*s**2*Yfun**(-2) + 32.D0*mz**8*
     &    s4fun**(-1)*Yfun**(-1) - 64.D0*mz**8*s*Yfun**(-2) + 16.D0*
     &    ctw**2*s2fun*s4fun**(-1)*rzm2 + 24.D0*ctw**2*u*Yfun**(-1)*
     &    rwm2*rzm2 + 16.D0*ctw**2*u*s2fun*s4fun**(-1) + 24.D0*ctw**2*
     &    u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 
     &    16.D0*ctw**2*s*s2fun*s4fun**(-1) + 48.D0*ctw**2*s*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0
     &    *ctw**2*s*u**4*Yfun**(-2) + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2
     &     + 36.D0*ctw**2*s**2*u**2*Yfun**(-2)*rzm2 + 48.D0*ctw**2*s**2
     &    *u**3*Yfun**(-2) + 36.D0*ctw**2*s**3*u*Yfun**(-2)*rzm2 + 72.D0
     &    *ctw**2*s**3*u**2*Yfun**(-2) + 12.D0*ctw**2*s**4*Yfun**(-2)*
     &    rzm2 + 48.D0*ctw**2*s**4*u*Yfun**(-2) + 12.D0*ctw**2*s**5*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*t1fun**(-1)*rwm2
     &     - 48.D0*ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 32.D0*ctw**2*
     &    mz**2*s2fun*s4fun**(-1) - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 72.D0*ctw**2*
     &    mz**2*s*u**2*Yfun**(-2)*rzm2 - 96.D0*ctw**2*mz**2*s*u**3*
     &    Yfun**(-2) - 144.D0*ctw**2*mz**2*s**2*u*Yfun**(-2)*rzm2 - 288.
     &    D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) - 72.D0*ctw**2*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 288.D0*ctw**2*mz**2*s**3*u*Yfun**(-2)
     &     - 96.D0*ctw**2*mz**2*s**4*Yfun**(-2) + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s*u*
     &    Yfun**(-2)*rzm2 + 288.D0*ctw**2*mz**4*s*u**2*Yfun**(-2) - 24.D
     &    0*ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*
     &    mz**4*s**2*Yfun**(-2)*rzm2 + 576.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) + 288.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 48.D0*
     &    ctw**2*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6
     &    *u*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**6*s*Yfun**(-2)*rzm2 - 384.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) - 384.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 96.D0*ctw**2*mz**8*s4fun**(-1)*Yfun**(-1) + 192.D
     &    0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 + 3.D0/2.D0*ctw**(-2)*
     &    s**2*s2fun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**3*s2fun*s4fun*
     &    Yfun**(-2) + ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - 1.D0/2.D0*ctw**(-2)*mz**4*s**2*s2fun*Yfun**(-2) + 4.D0*s*
     &    s2fun*Yfun**(-1)*rwm2 - 6.D0*s**2*s2fun*Yfun**(-1) + 2.D0*
     &    s**3*s2fun*s4fun*Yfun**(-2) - 4.D0*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*mz**4*s**2*s2fun*Yfun**(-2) - 16.D0*ctw**2*
     &    s*s2fun*s4fun**(-1) - 12.D0*ctw**2*s*s2fun*Yfun**(-1)*rwm2 + 
     &    18.D0*ctw**2*s**2*s2fun*Yfun**(-1) - 6.D0*ctw**2*s**3*s2fun*
     &    s4fun*Yfun**(-2) + 12.D0*ctw**2*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 6.D0*ctw**2*mz**4*s**2*s2fun*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**2*
     &    Yfun**(-1) - ctw**(-2)*s*u**4*Yfun**(-2) - ctw**(-2)*s**2*u*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**3*Yfun**(-1) - 3.D0*ctw**(-2)*s**3*u**2*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*s**4*u*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**5*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*Yfun**(-1) + 12.D0*ctw**(-2)*mz**2*s**2*
     &    u**2*Yfun**(-2) + 12.D0*ctw**(-2)*mz**2*s**3*u*Yfun**(-2) + 4.
     &    D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s*
     &    Yfun**(-1) - 11.D0*ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 23.D0*
     &    ctw**(-2)*mz**4*s**2*u*Yfun**(-2) - 23.D0/2.D0*ctw**(-2)*
     &    mz**4*s**3*Yfun**(-2) + 14.D0*ctw**(-2)*mz**6*s*u*Yfun**(-2)
     &     + 14.D0*ctw**(-2)*mz**6*s**2*Yfun**(-2) - 6.D0*ctw**(-2)*
     &    mz**8*s*Yfun**(-2) + 4.D0*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*
     &    u**2*Yfun**(-1) + 4.D0*s*u**4*Yfun**(-2) + 4.D0*s**2*u*
     &    Yfun**(-1) + 8.D0*s**2*u**3*Yfun**(-2) + 2.D0*s**3*Yfun**(-1)
     &     + 12.D0*s**3*u**2*Yfun**(-2) + 8.D0*s**4*u*Yfun**(-2) + 2.D0
     &    *s**5*Yfun**(-2) - 8.D0*mz**2*s*u*Yfun**(-1) - 16.D0*mz**2*s*
     &    u**3*Yfun**(-2) - 8.D0*mz**2*s**2*Yfun**(-1) - 48.D0*mz**2*
     &    s**2*u**2*Yfun**(-2) - 48.D0*mz**2*s**3*u*Yfun**(-2) - 16.D0*
     &    mz**2*s**4*Yfun**(-2) + 8.D0*mz**4*s*Yfun**(-1) + 44.D0*mz**4
     &    *s*u**2*Yfun**(-2) + 92.D0*mz**4*s**2*u*Yfun**(-2) + 46.D0*
     &    mz**4*s**3*Yfun**(-2) - 56.D0*mz**6*s*u*Yfun**(-2) - 56.D0*
     &    mz**6*s**2*Yfun**(-2) + 24.D0*mz**8*s*Yfun**(-2) - 16.D0*
     &    ctw**2*s - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1) - 12.D0*ctw**2*s*u**4*Yfun**(-2) - 
     &    12.D0*ctw**2*s**2*u*Yfun**(-1) - 24.D0*ctw**2*s**2*u**3*
     &    Yfun**(-2) - 6.D0*ctw**2*s**3*Yfun**(-1) - 36.D0*ctw**2*s**3*
     &    u**2*Yfun**(-2) - 24.D0*ctw**2*s**4*u*Yfun**(-2) - 6.D0*
     &    ctw**2*s**5*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*u*Yfun**(-1) + 
     &    48.D0*ctw**2*mz**2*s*u**3*Yfun**(-2) + 24.D0*ctw**2*mz**2*
     &    s**2*Yfun**(-1) + 144.D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) + 
     &    144.D0*ctw**2*mz**2*s**3*u*Yfun**(-2) + 48.D0*ctw**2*mz**2*
     &    s**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*s*Yfun**(-1) - 132.D0*
     &    ctw**2*mz**4*s*u**2*Yfun**(-2) - 276.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) - 138.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 168.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) + 168.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 72.D0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*b0fmzww * (  - 
     &    4.D0 + ctw**(-2) + ctw**(-2)*s2fun*s4fun**(-1) + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1) - ctw**(-2)*s*s2fun*Yfun**(-1) + 2.D
     &    0*ctw**(-2)*s*u*Yfun**(-1) + ctw**(-2)*s**2*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s4fun**(-1) - 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    ctw**(-2)*mz**4*s*u1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**6*u1fun**(-2)*s4fun**(-1) - 4.D0*s2fun*s4fun**(-1) - 8.D0
     &    *u**2*Yfun**(-1) + 4.D0*s*s2fun*Yfun**(-1) - 8.D0*s*u*
     &    Yfun**(-1) - 4.D0*s**2*Yfun**(-1) + 8.D0*mz**2*s4fun**(-1) + 
     &    16.D0*mz**2*u*Yfun**(-1) + 16.D0*mz**2*s*Yfun**(-1) - 16.D0*
     &    mz**4*Yfun**(-1) - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2 + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1) - 12.D0*
     &    ctw**2*s*s2fun*Yfun**(-1) + 24.D0*ctw**2*s*u*Yfun**(-1) + 12.D
     &    0*ctw**2*s**2*Yfun**(-1) - 24.D0*ctw**2*mz**2*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**2*u*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**4*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) + 
     &    48.D0*ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*b0fusww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 4.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 
     &    4.D0*u**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 8.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 6.D0*
     &    ctw**2 - 12.D0*ctw**2*u**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) - 48.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*b0ftsww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*s*u*Yfun**(-1) - ctw**(-2)*s**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*u**2*Yfun**(-1) + 8.D0*s*u*Yfun**(-1) + 4.D
     &    0*s**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 16.D0*mz**2*u*
     &    Yfun**(-1) - 16.D0*mz**2*s*Yfun**(-1) + 16.D0*mz**4*
     &    Yfun**(-1) + 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 4.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) - 6.D0*ctw**2 - 12.D0*ctw**2*
     &    u**2*Yfun**(-1) - 24.D0*ctw**2*s*u*Yfun**(-1) - 12.D0*ctw**2*
     &    s**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*s4fun**(-1) + 48.D0*
     &    ctw**2*mz**2*u*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*Yfun**(-1)
     &     - 48.D0*ctw**2*mz**4*Yfun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*b0fqsww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) + ctw**(-2)*s*s2fun*Yfun**(-1) + 
     &    4.D0*s2fun*s4fun**(-1) - 4.D0*s*s2fun*Yfun**(-1) - 12.D0*
     &    ctw**2*s2fun*s4fun**(-1) + 12.D0*ctw**2*s*s2fun*Yfun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun * ( 
     &     - 2.D0*ctw**(-2)*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - ctw**(-2)*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 4.D0*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - 24.D0*ctw**2*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**2*s**2*t1fun**(-1)*u1fun**(-1)
     &    *s4fun**(-1) + 24.D0*ctw**2*mz**4*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( 32.D0*ctw**2*s*u*s4fun**(-1)*rwm2 + 8.D0*
     &    ctw**2*s*u*s2fun*s4fun**(-1) + 16.D0*ctw**2*s**2*s4fun**(-1)*
     &    rwm2 + 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) - 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 - 2.D
     &    0*ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**3*s4fun**(-1)*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*s4fun**(-1)
     &    *rwm2 + 8.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) + 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*mz**2*s*u*s4fun**(-1) + 2.D0*mz**2*s**2*
     &    s4fun**(-1) - 4.D0*mz**4*s*s4fun**(-1) - 8.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*ctw**2*s*u*s4fun**(-1)*rwm2 + 8.
     &    D0*ctw**2*s*u*s2fun*s4fun**(-1) - 24.D0*ctw**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*ctw**2*s**2*s4fun**(-1)*
     &    rwm2 + 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) - 8.D0*ctw**2*s**2*
     &    u*s4fun**(-1) - 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 32.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 6.D0*ctw**2*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 - 2.D
     &    0*ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*
     &    ctw**(-2)*s**2*s4fun**(-1)*rwm2 - 6.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 1.D0/2.D0*ctw**(-2)*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**2*u**5*
     &    s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**(-2)*s**3*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 3.D0/2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 5.D0/2.D0*ctw**(-2)*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)*rwm2
     &     - 3.D0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-1) - 5.D0*
     &    ctw**(-2)*s**4*u**3*s4fun**(-1)*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**5*s4fun**(-1)*Yfun**(-1) - 5.D0*ctw**(-2)*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 5.D0/2.D0*ctw**(-2)*s**6*u*
     &    s4fun**(-1)*Yfun**(-2) - 1.D0/2.D0*ctw**(-2)*s**7*s4fun**(-1)
     &    *Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 12.D0
     &    *ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 3.D0*
     &    ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 5.D0*
     &    ctw**(-2)*mz**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 12.D0*
     &    ctw**(-2)*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 20.D0*
     &    ctw**(-2)*mz**2*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 3.D0*
     &    ctw**(-2)*mz**2*s**4*s4fun**(-1)*Yfun**(-1) + 30.D0*ctw**(-2)
     &    *mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 20.D0*ctw**(-2)*
     &    mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 5.D0*ctw**(-2)*mz**2*
     &    s**6*s4fun**(-1)*Yfun**(-2) - 22.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 22.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 39.D0/2.D0*ctw**(-2)*mz**4*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**(-2)*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 117.D0/2.D0*ctw**(-2)*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 117.D0/2.D0*ctw**(-2)*mz**4*
     &    s**4*u*s4fun**(-1)*Yfun**(-2) - 39.D0/2.D0*ctw**(-2)*mz**4*
     &    s**5*s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 37.D0*ctw**(-2)*mz**6*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 74.D0*ctw**(-2)*mz**6*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) + 37.D0*ctw**(-2)*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 34.D0*ctw**(-2)*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 34.D0*ctw**(-2)*mz**8*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**10*s**2*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*s4fun**(-1)*rwm2 + 8.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 8.D0*s**2*s4fun**(-1)*rwm2
     &     + 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) + 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*
     &    s**3*u**2*s4fun**(-1)*Yfun**(-1) + 10.D0*s**3*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*s**4*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 6.D0*s**4*u*s4fun**(-1)*Yfun**(-1) + 20.D0*s**4*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*s**5*s4fun**(-1)*Yfun**(-1) + 
     &    20.D0*s**5*u**2*s4fun**(-1)*Yfun**(-2) + 10.D0*s**6*u*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*s**7*s4fun**(-1)*Yfun**(-2) - 
     &    16.D0*mz**2*s*s4fun**(-1)*rwm2 + 4.D0*mz**2*s*u*s4fun**(-1)
     &     - 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*
     &    mz**2*s**2*s4fun**(-1) - 96.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 20.D0*mz**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 
     &    48.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*mz**2*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) - 80.D0*mz**2*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 120.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    80.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 20.D0*mz**2*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 4.D0*mz**4*s*s4fun**(-1) + 88.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 88.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 78.D0*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 
     &    24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 234.D0*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 234.D0*mz**4*s**4*u*s4fun**(-1)
     &    *Yfun**(-2) + 78.D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) - 48.D0
     &    *mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 148.D0*mz**6*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 296.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) - 148.
     &    D0*mz**6*s**4*s4fun**(-1)*Yfun**(-2) + 136.D0*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 136.D0*mz**8*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 48.D0*mz**10*s**2*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    ctw**2*s*u*s4fun**(-1)*rwm2 + 8.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1) - 24.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     - 8.D0*ctw**2*s**2*s4fun**(-1)*rwm2 + 4.D0*ctw**2*s**2*s2fun
     &    *s4fun**(-1) - 8.D0*ctw**2*s**2*u*s4fun**(-1) - 72.D0*ctw**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**2*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*ctw**2*s**3*s4fun**(-1) - 72.D0*ctw**2*s**3
     &    *u*s4fun**(-1)*Yfun**(-1)*rwm2 - 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 30.D0*ctw**2*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    18.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-1) - 60.D0*ctw**2*
     &    s**4*u**3*s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-1) - 60.D0*ctw**2*s**5*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 30.D0*ctw**2*s**6*u*s4fun**(-1)*Yfun**(-2) - 6.D0
     &    *ctw**2*s**7*s4fun**(-1)*Yfun**(-2) + 16.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**2*s*s2fun*s4fun**(-1) + 
     &    144.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0
     &    *ctw**2*mz**2*s**2*s4fun**(-1) + 288.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 36.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 60.D0*ctw**2*mz**2*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*mz**2*s**3*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*
     &    Yfun**(-1) + 240.D0*ctw**2*mz**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*Yfun**(-1)
     &     + 360.D0*ctw**2*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 240.
     &    D0*ctw**2*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 60.D0*ctw**2*
     &    mz**2*s**6*s4fun**(-1)*Yfun**(-2) - 264.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 264.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 234.D0*ctw**2*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 72.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 702.D0*ctw**2*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 702.D0*ctw**2*mz**4*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) - 234.D0*ctw**2*mz**4*s**5*s4fun**(-1)*Yfun**(-2)
     &     + 144.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 444.D0*ctw**2*
     &    mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 888.D0*ctw**2*mz**6*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) + 444.D0*ctw**2*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 408.D0*ctw**2*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 408.D0*ctw**2*mz**8*s**3*s4fun**(-1)
     &    *Yfun**(-2) + 144.D0*ctw**2*mz**10*s**2*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D
     &    0*ctw**(-2)*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s*u**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - ctw**(-2)*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*u1fun**(-1)*s4fun**(-1)*
     &    rwm2 - 2.D0*ctw**(-2)*mz**4*s*u*u1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + ctw**(-2)*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*s
     &    *u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*rwm2 + 8.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 4.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 
     &    16.D0*ctw**2*s*s4fun**(-1)*rzm2 - 16.D0*ctw**2*s*u*
     &    s4fun**(-1) + 12.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 24.D0*ctw**2*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 12.D0*
     &    ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 12.D0*ctw**2*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s*u*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*
     &    s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 12.D0*ctw**2*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 2.D0*ctw**(-2)*s*u**3*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s*
     &    u**4*s4fun**(-1)*Yfun**(-2)*rzm2 - ctw**(-2)*s*u**5*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*s**2*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 3.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 5.D0*ctw**(-2)*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 + 6.D0
     &    *ctw**(-2)*s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 3.
     &    D0*ctw**(-2)*s**3*u*s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**(-2)*
     &    s**3*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 10.D0*ctw**(-2)*s**3*
     &    u**3*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*s**4*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 - ctw**(-2)*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 10.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    ctw**(-2)*s**5*s4fun**(-1)*Yfun**(-2)*rzm2 - 5.D0*ctw**(-2)*
     &    s**5*u*s4fun**(-1)*Yfun**(-2) - ctw**(-2)*s**6*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    rwm2 + 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     - 12.D0*ctw**(-2)*mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 24.D0*ctw**(-2)*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 40.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 60.D0*ctw**(-2)*mz**2*s**3*u**2
     &    *s4fun**(-1)*Yfun**(-2) + 8.D0*ctw**(-2)*mz**2*s**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*ctw**(-2)*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 10.D0*ctw**(-2)*mz**2*s**5*
     &    s4fun**(-1)*Yfun**(-2) - 4.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 22.D0*ctw**(-2)*mz**4*s*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 23.D0*ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 39.D0*ctw**(-2)*mz**4*s*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 22.D0*ctw**(-2)*mz**4*s**2*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**(-2)*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 46.D0*ctw**(-2)*mz**4*
     &    s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 117.D0*ctw**(-2)*mz**4*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-2) - 23.D0*ctw**(-2)*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 - 117.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) - 39.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 8.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 28.D0*ctw**(-2)*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 74.D0*ctw**(-2)*mz**6*s*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 28.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 148.D0*ctw**(-2)*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 74.D0*ctw**(-2)*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 68.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*
     &    Yfun**(-2) - 68.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2)
     &     + 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 4.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*s**2*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 + 20.
     &    D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 24.D0*s**3*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*s**3*u*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *s**3*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 4.D0*s**4*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**4*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*s**5*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 20.D0*s**5*u*s4fun**(-1)*Yfun**(-2) + 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)
     &    *rwm2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*
     &    mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 
     &    96.D0*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    48.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**2*s**2*
     &    u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 160.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 48.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 96.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     - 240.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 32.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 160.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 40.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 88.D0
     &    *mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 48.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 92.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 156.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 88.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 
     &    184.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 468.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 92.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 468.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) + 156.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 48.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 32.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) - 112.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 296.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     - 112.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 592.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 48.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 + 272.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 272.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*ctw**2*s*s4fun**(-1)*rzm2 - 16.
     &    D0*ctw**2*s*u*s4fun**(-1) - 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 24.D0*ctw**2*s*u**3*t1fun**(-1)*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 12.D0*ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 12.D0*
     &    ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) - 16.D0*ctw**2*s**2*
     &    s4fun**(-1) - 24.D0*ctw**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 72.D0*ctw**2*s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 48.D0*
     &    ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 - 60.D0*ctw**2*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 72.D0*ctw**2*s**3*u*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 - 36.D0*ctw**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) - 72.D0*ctw**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 120.D0*ctw**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*ctw**2*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1) - 
     &    48.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 120.D0*
     &    ctw**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 60.D0*ctw**2*s**5*u*s4fun**(-1)
     &    *Yfun**(-2) - 12.D0*ctw**2*s**6*s4fun**(-1)*Yfun**(-2) + 24.D0
     &    *ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*rwm2 + 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1) + 48.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**2*s*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 120.D0*ctw**2*mz**2*s*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 48.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 288.D0*ctw**2*mz**2*s**2*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 144.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 288.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 480.D0*ctw**2*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 288.D0*ctw**2*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 720.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*ctw**2*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 480.D0*ctw**2*mz**2*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) + 120.D0*ctw**2*mz**2*s**5*s4fun**(-1)*Yfun**(-2)
     &     - 48.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 264.D0*
     &    ctw**2*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    144.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 276.D0*
     &    ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 468.D0*
     &    ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 264.D0*ctw**2*
     &    mz**4*s**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 144.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 552.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 1404.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 276.D0*ctw**2*mz**4*
     &    s**3*s4fun**(-1)*Yfun**(-2)*rzm2 - 1404.D0*ctw**2*mz**4*s**3*
     &    u*s4fun**(-1)*Yfun**(-2) - 468.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 336.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 888.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 336.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 1776.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 
     &    888.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2) - 144.D0*
     &    ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-2)*rzm2 - 816.D0*ctw**2*
     &    mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 816.D0*ctw**2*mz**8*s**2*
     &    s4fun**(-1)*Yfun**(-2) + 288.D0*ctw**2*mz**10*s*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    ctw**(-2)*s**2*u*s2fun*s4fun**(-1)*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**3*s2fun*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s**3*
     &    u*s2fun*Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - ctw**(-2)*mz**2*s**3*s2fun*Yfun**(-2) + 
     &    ctw**(-2)*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2) + 1.D0/2.D
     &    0*ctw**(-2)*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 
     &    ctw**(-2)*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*s**3*s2fun*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    s**3*u*s2fun*Yfun**(-2) - 2.D0*s**4*s2fun*Yfun**(-2) + 8.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*mz**2*s**3*s2fun*
     &    Yfun**(-2) - 4.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2)
     &     - 2.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 4.D0*mz**6*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**2*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**2*s**3*u*s2fun*Yfun**(-2) + 6.D0*
     &    ctw**2*s**4*s2fun*Yfun**(-2) - 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*mz**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*mz**2*s**3*s2fun*
     &    Yfun**(-2) + 12.D0*ctw**2*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 6.D0*ctw**2*mz**4*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**6*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * ( 2.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) + ctw**(-2)*s**2*Yfun**(-1)*rwm2
     &     + ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) + 3.D0*
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 5.D0*ctw**(-2)*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*ctw**(-2)*s**3*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)
     &     + 7.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 3.D0*
     &    ctw**(-2)*s**5*u*s4fun**(-1)*Yfun**(-2) + 1.D0/2.D0*ctw**(-2)
     &    *s**6*s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*
     &    Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)
     &    *Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 32.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 3.D0*ctw**(-2)*mz**2*s**3*s4fun**(-1)*Yfun**(-1)
     &     - 42.D0*ctw**(-2)*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    24.D0*ctw**(-2)*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) - 5.D0*
     &    ctw**(-2)*mz**2*s**5*s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 30.D0*ctw**(-2)*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) + 5.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 81.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 70.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) + 39.D0/2.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 50.D0*ctw**(-2)*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 88.D0*ctw**(-2)*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) - 37.D0*ctw**(-2)*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 40.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*Yfun**(-2)
     &     + 34.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 12.D0*
     &    ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*
     &    Yfun**(-1)*rwm2 - 8.D0*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) - 
     &    8.D0*s*u**3*s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**2*Yfun**(-1)*rwm2 - 4.D0*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 20.D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) - 32.D0*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*s**4*s4fun**(-1)*Yfun**(-1) - 28.D0*s**4*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 12.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*s**6*s4fun**(-1)*Yfun**(-2) + 8.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 + 8.D0*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 40.D0
     &    *mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) + 32.D0*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 128.D0*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 12.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) + 168.D0
     &    *mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 20.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 120.D0*
     &    mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) - 20.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 324.D0*mz**4*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 280.D0*mz**4*s**3*u*s4fun**(-1)*Yfun**(-2) - 78.D
     &    0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 8.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 200.D0*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 352.D0*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 148.
     &    D0*mz**6*s**3*s4fun**(-1)*Yfun**(-2) - 160.D0*mz**8*s*u*
     &    s4fun**(-1)*Yfun**(-2) - 136.D0*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) + 48.D0*mz**10*s*s4fun**(-1)*Yfun**(-2) + 24.D0*
     &    ctw**2*s*u*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) + 12.D
     &    0*ctw**2*s**2*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 60.D0*ctw**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 
     &    24.D0*ctw**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*
     &    s**3*u**3*s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**2*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 84.D0*ctw**2*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 36.D0*ctw**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 6.D0
     &    *ctw**2*s**6*s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*mz**2*s*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 120.D0*ctw**2*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) - 96.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 384.D0*ctw**2*
     &    mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 36.D0*ctw**2*mz**2*
     &    s**3*s4fun**(-1)*Yfun**(-1) - 504.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 288.D0*ctw**2*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 60.D0*ctw**2*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 72.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 
     &    360.D0*ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 60.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 972.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 840.D0*ctw**2*mz**4*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) + 234.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 600.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 1056.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) - 444.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2)
     &     + 480.D0*ctw**2*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 408.D0*
     &    ctw**2*mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*
     &    mz**10*s*s4fun**(-1)*Yfun**(-2) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - 4.D0*ctw**(-2)*u*s4fun**(-1) - 4.D0*ctw**(-2)*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*s*s4fun**(-1) + 
     &    2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s4fun**(-1) + 12.D0*
     &    ctw**(-2)*mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*ctw**(-2)*mz**2*
     &    s*u*s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*s4fun**(-1)
     &     - 16.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1) - ctw**(-2)
     &    *mz**4*s*t1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*
     &    u1fun**(-2)*s4fun**(-1) - 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) + 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 
     &    2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 8.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1) + 16.D0*u*s4fun**(-1) + 16.D0*
     &    u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*s*s4fun**(-1) - 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 24.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s4fun**(-1) - 48.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 64.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) - 24.D0*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 64.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*u*s4fun**(-1) - 
     &    48.D0*ctw**2*u**3*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s*
     &    s4fun**(-1) + 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - 72.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**2*s4fun**(-1) + 144.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**2*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) - 
     &    24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) - 192.D0*ctw**2*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**2*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 144.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1) + 
     &    24.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 96.D0*ctw**2*mz**6*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & b0fusww * ( 2.D0 - 1.D0/2.D0*ctw**(-2) + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) - 
     &    8.D0*u*s4fun**(-1) - 8.D0*u**3*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    s*u**2*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*
     &    mz**4*s*u1fun**(-2)*s4fun**(-1) + 8.D0*mz**6*u1fun**(-2)*
     &    s4fun**(-1) - 6.D0*ctw**2 + 24.D0*ctw**2*u*s4fun**(-1) + 24.D0
     &    *ctw**2*u**3*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - 2.D0 + 1.D0/2.D0*ctw**(-2) + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) + 2.D
     &    0*ctw**(-2)*s*s4fun**(-1) + 5.D0*ctw**(-2)*s*u**2*s4fun**(-1)
     &    *Yfun**(-1) + 4.D0*ctw**(-2)*s**2*u*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) - 10.D0*ctw**(-2)*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)
     &     - 6.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*ctw**(-2)*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) - 
     &    8.D0*ctw**(-2)*mz**6*s4fun**(-1)*Yfun**(-1) - 8.D0*u*
     &    s4fun**(-1) - 8.D0*u**3*s4fun**(-1)*Yfun**(-1) - 8.D0*s*
     &    s4fun**(-1) - 20.D0*s*u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 40.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1) - 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 64.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*s4fun**(-1) - 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*s4fun**(-1)*Yfun**(-1) + 6.D0*
     &    ctw**2 + 24.D0*ctw**2*u*s4fun**(-1) + 24.D0*ctw**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*s*s4fun**(-1) + 60.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**2*s4fun**(-1) - 120.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) + 
     &    192.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 144.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**6*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,1) = ewamp(1,2,2,1) + g**4*stw**2*betaZfun*
     & b0fqsww * (  - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      ewamp(1,2,2,3) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - deltafun*Yfun - 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     - ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**2*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*u**2*s4fun**(-1)*deltafun - 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**4*u*s4fun**(-1)*deltafun + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 4.D0*mz**6*s4fun**(-1)*deltafun - 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun - 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.D0
     &    *ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    /2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 5.D0*s**2*u*s4fun**(-1)*deltafun + 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**3*
     &    s4fun**(-1)*deltafun + 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun - 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun + 22.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 20.D0*mz**6*s4fun**(-1)*deltafun - 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun - 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D
     &    0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0
     &    *ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*
     &    rzm2 + 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0
     &    *s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*
     &    deltafun - 8.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D
     &    0*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun - 4.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*mz**2*s**(-1)*deltafun*
     &    rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *mz**2*s**(-1)*u*deltafun - 16.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0
     &    *ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D
     &    0*ctw**2*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 
     &    32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*
     &    mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s**(-1)*
     &    u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 + 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*rzm2 + 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*s**(-1)*u**2*deltafun
     &     + 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 6.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun + 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*s**(-1)*u*deltafun + 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*deltafun
     &     - 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s**(-1)*
     &    deltafun + 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun - 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*c0ccqswww * ( 
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    deltafun - 2.D0*s*deltafun - 4.D0*s*u*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**2*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**2*deltafun + 4.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*ctw**2*s*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*mz**2*s*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*c0mzmzqswww
     &  * ( ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) - 32.D0*ctw**2*u*deltafun - 16.D0
     &    *ctw**2*s*deltafun + 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) + 32.D
     &    0*ctw**2*mz**2*deltafun - 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*deltafun + 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u*s4fun**(-1)*deltafun - 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun - 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*s4fun**(-1)*deltafun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun + 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*b0ftsww * (  - 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*b0fqsww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    24.D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun * ( 
     &    ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - deltafun*Yfun - 64.D0*ctw**2*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun + s**2*u*s4fun**(-1)*deltafun + 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 + 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + s**3*s4fun**(-1)*deltafun - 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun - 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 10.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     + 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun + 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u
     &    *u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 16.D0*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*u*deltafun + 8.D0*s*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**2*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*
     &    mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*mz**2*u*
     &    s4fun**(-1)*deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s4fun**(-1)*deltafun
     &    *rwm2*rzm2 + 2.D0*deltafun*rzm2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*u*deltafun - 16.D0*s*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*s*deltafun - 8.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 4.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 12.D0*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0
     &    *mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*deltafun + 16.D0
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*mz**2*s*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun - 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 + 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun + 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*deltafun
     &     - 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * ( ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun - 1.D0/2.D
     &    0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 
     &    16.D0*ctw**2*s*deltafun + 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & b0fmzww * ( ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & b0fusww * ( ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 4.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun - 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(1,2,2,3) = ewamp(1,2,2,3) + g**4*stw**2*betaZfun*
     & b0fqsww * ( ctw**(-2)*s*s4fun**(-1)*deltafun - 4.D0*s*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(1,2,2,2) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 8.D0*mz**2*rwm2 + 12.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 - 32.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*
     &    Yfun**(-1) - 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 12.D0
     &    *ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 4.D
     &    0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*mz**8*s*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 + 
     &    48.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    16.D0*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*
     &    s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 + 16.D0*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 128.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 256.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 8.D0*mz**2*s**2*u*Yfun**(-1) - 128.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s**3*Yfun**(-1) - 8.D
     &    0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*Yfun**(-1)*rwm2 + 
     &    336.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s*
     &    u*Yfun**(-1) - 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 336.
     &    D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**2*
     &    Yfun**(-1) - 8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 288.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*mz**6*s*Yfun**(-1) + 16.D0
     &    *mz**6*s*u*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2
     &     + 24.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*
     &    Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.
     &    D0*ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**4*s*Yfun**(-1)*rwm2 + 12.D
     &    0*ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*
     &    Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 
     &    2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D0*
     &    ctw**(-2)*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 8.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 8.D0*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2
     &     + 8.D0*u*Yfun**(-1)*rwm2*rzm2 + 32.D0*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 32.D0*u**3
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 64.D0*mz**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 + 8.D0*mz**2*u*Yfun**(-1)*rzm2 - 64.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*u**2*
     &    Yfun**(-1) + 32.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 8.
     &    D0*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2
     &     + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*ctw**2*u*
     &    Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 24.
     &    D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 24.D0*ctw**2*
     &    mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D
     &    0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*
     &    Yfun**(-1)*rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2
     &     - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 8.D
     &    0*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 + 8.D0*s2fun*s4fun**(-1)*rwm2 + 8.D0*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*
     &    rzm2 + 32.D0*u**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 - 8.D0*
     &    u**2*Yfun**(-1)*rwm2 + 32.D0*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 8.D0*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*s*
     &    Yfun**(-1)*rwm2*rzm2 + 64.D0*s*u*s4fun**(-1)*Yfun**(-1)*rwm2*
     &    rzm2 - 16.D0*s*u*Yfun**(-1)*rwm2 + 96.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*s**2*Yfun**(-1)*rwm2 + 96.D0*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.
     &    D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 - 192.D0*mz**2*u*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*mz**2*u*Yfun**(-1)*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*
     &    rwm2 - 256.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*
     &    mz**2*u**2*Yfun**(-1) - 192.D0*mz**2*s*s4fun**(-1)*Yfun**(-1)
     &    *rwm2*rzm2 - 8.D0*mz**2*s*Yfun**(-1)*rzm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 - 512.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 16.D0*mz**2*s*u*Yfun**(-1) - 256.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**2*Yfun**(-1) + 
     &    288.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 16.D0*mz**4*
     &    Yfun**(-1)*rzm2 - 32.D0*mz**4*Yfun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 672.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*mz**4*u*Yfun**(-1) - 8.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 672.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*
     &    mz**4*s*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 576.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 32.D0*mz**6*Yfun**(-1) + 32.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 24.D0
     &    *ctw**2*u*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*ctw**2*u
     &    *Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 
     &    24.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 48.D0*ctw**2*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*rwm2
     &     - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*u*s4fun**(-1)*Yfun**(-1)
     &     - 96.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2
     &    *mz**8*s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s*s4fun**(-1)*rwm2 - 12.D
     &    0*s*s2fun*Yfun**(-1)*rwm2 + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 4.
     &    D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s*
     &    s2fun*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*c0mzmzqswww
     &  * ( 2.D0*ctw**(-2)*rwm2 - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*mz**4*s4fun**(-1) - ctw**(-2)*mz**4*s*Yfun**(-1)
     &     + 24.D0*rwm2 - 12.D0*s*s4fun*Yfun**(-1)*rwm2 - 8.D0*mz**2 + 
     &    4.D0*mz**2*s*s4fun*Yfun**(-1) - 8.D0*mz**4*s4fun**(-1) + 4.D0
     &    *mz**4*s*Yfun**(-1) + 24.D0*ctw**2*rwm2 - 12.D0*ctw**2*s*
     &    s4fun*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1) - 12.D0
     &    *ctw**2*mz**4*s*Yfun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1)
     &     - 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 12.D
     &    0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) )
      ewamp(1,2,2,2) = ewamp(1,2,2,2) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 48.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      ewamp(2,1,1,1) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 8.D0*mz**2*rwm2 + 12.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 - 32.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*
     &    Yfun**(-1) - 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 12.D0
     &    *ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 4.D
     &    0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*mz**8*s*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 + 
     &    48.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    16.D0*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*
     &    s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 + 16.D0*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 128.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 256.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 8.D0*mz**2*s**2*u*Yfun**(-1) - 128.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s**3*Yfun**(-1) - 8.D
     &    0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*Yfun**(-1)*rwm2 + 
     &    336.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s*
     &    u*Yfun**(-1) - 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 336.
     &    D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**2*
     &    Yfun**(-1) - 8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 288.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*mz**6*s*Yfun**(-1) + 16.D0
     &    *mz**6*s*u*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2
     &     + 24.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*
     &    Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.
     &    D0*ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**4*s*Yfun**(-1)*rwm2 + 12.D
     &    0*ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*
     &    Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 
     &    2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D0*
     &    ctw**(-2)*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 8.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 8.D0*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2
     &     + 8.D0*u*Yfun**(-1)*rwm2*rzm2 + 32.D0*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 32.D0*u**3
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 64.D0*mz**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 + 8.D0*mz**2*u*Yfun**(-1)*rzm2 - 64.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*u**2*
     &    Yfun**(-1) + 32.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 8.
     &    D0*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2
     &     + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*ctw**2*u*
     &    Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 24.
     &    D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 24.D0*ctw**2*
     &    mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D
     &    0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*
     &    Yfun**(-1)*rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2
     &     - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 8.D
     &    0*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 + 8.D0*s2fun*s4fun**(-1)*rwm2 + 8.D0*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*
     &    rzm2 + 32.D0*u**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 - 8.D0*
     &    u**2*Yfun**(-1)*rwm2 + 32.D0*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 8.D0*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*s*
     &    Yfun**(-1)*rwm2*rzm2 + 64.D0*s*u*s4fun**(-1)*Yfun**(-1)*rwm2*
     &    rzm2 - 16.D0*s*u*Yfun**(-1)*rwm2 + 96.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*s**2*Yfun**(-1)*rwm2 + 96.D0*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.
     &    D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 - 192.D0*mz**2*u*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*mz**2*u*Yfun**(-1)*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*
     &    rwm2 - 256.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*
     &    mz**2*u**2*Yfun**(-1) - 192.D0*mz**2*s*s4fun**(-1)*Yfun**(-1)
     &    *rwm2*rzm2 - 8.D0*mz**2*s*Yfun**(-1)*rzm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 - 512.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 16.D0*mz**2*s*u*Yfun**(-1) - 256.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**2*Yfun**(-1) + 
     &    288.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 16.D0*mz**4*
     &    Yfun**(-1)*rzm2 - 32.D0*mz**4*Yfun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 672.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*mz**4*u*Yfun**(-1) - 8.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 672.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*
     &    mz**4*s*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 576.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 32.D0*mz**6*Yfun**(-1) + 32.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 24.D0
     &    *ctw**2*u*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*ctw**2*u
     &    *Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 
     &    24.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 48.D0*ctw**2*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*rwm2
     &     - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*u*s4fun**(-1)*Yfun**(-1)
     &     - 96.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2
     &    *mz**8*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s*s4fun**(-1)*rwm2 - 12.D
     &    0*s*s2fun*Yfun**(-1)*rwm2 + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 4.
     &    D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s*
     &    s2fun*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*c0mzmzqswww
     &  * ( 2.D0*ctw**(-2)*rwm2 - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*mz**4*s4fun**(-1) - ctw**(-2)*mz**4*s*Yfun**(-1)
     &     + 24.D0*rwm2 - 12.D0*s*s4fun*Yfun**(-1)*rwm2 - 8.D0*mz**2 + 
     &    4.D0*mz**2*s*s4fun*Yfun**(-1) - 8.D0*mz**4*s4fun**(-1) + 4.D0
     &    *mz**4*s*Yfun**(-1) + 24.D0*ctw**2*rwm2 - 12.D0*ctw**2*s*
     &    s4fun*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1) - 12.D0
     &    *ctw**2*mz**4*s*Yfun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1)
     &     - 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 12.D
     &    0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,1,1) = ewamp(2,1,1,1) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 48.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      ewamp(2,1,1,3) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + deltafun*Yfun + 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**2*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u**2*s4fun**(-1)*deltafun + 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**4*u*s4fun**(-1)*deltafun - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**6*s4fun**(-1)*deltafun + 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun + 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.D0*
     &    ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 5.D0*s**2*u*s4fun**(-1)*deltafun - 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**3*
     &    s4fun**(-1)*deltafun - 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun + 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*s4fun**(-1)*deltafun + 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 - 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun - 22.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 20.D0*mz**6*s4fun**(-1)*deltafun + 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun + 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun + 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 4.D0*s**(-1)*u**2*deltafun + 8.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D
     &    0*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*mz**2*
     &    s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s**(-1)*u*deltafun + 16.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)
     &    *deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 48.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 - 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 4.D0*s**(-1)*u*deltafun*rzm2 - 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*deltafun
     &     - 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*deltafun*rzm2 - 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 6.D0*u*
     &    deltafun - 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun - 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*s**(-1)*u*deltafun - 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*deltafun
     &     + 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s**(-1)*
     &    deltafun - 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 - 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 1.
     &    D0/2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*u*deltafun + 2.D0*s*deltafun + 4.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*s**2*s2fun**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*deltafun - 4.D0
     &    *mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**6*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*s*u*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2
     &    *s*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) + 32.D0*ctw**2*u*deltafun + 16.D0
     &    *ctw**2*s*deltafun - 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) - 32.D
     &    0*ctw**2*mz**2*deltafun + 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - ctw**(-2)*s*s4fun**(-1)*deltafun - 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*deltafun + 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*s4fun**(-1)*deltafun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun - 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*b0fqsww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*
     &    deltafun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 8.D0*u
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun + 8.D0*
     &    mz**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 24.D0*ctw**2
     &    *mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun * ( 
     &     - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( deltafun*Yfun + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun - s**2*u*s4fun**(-1)*deltafun - 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 - 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - s**3*s4fun**(-1)*deltafun + 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun + 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 10.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     - 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun - 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 - 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*u*deltafun - 8.D0*s*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2
     &     + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*
     &    u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun
     &    *rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*mz**2*s
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 + 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun + 16.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 8.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*deltafun - 16.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun - 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*deltafun
     &     + 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * (  - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 
     &    16.D0*ctw**2*s*deltafun - 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & b0fusww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*u*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & b0ftsww * ( ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun - 4.D0*u*s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun + 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,1,3) = ewamp(2,1,1,3) + g**4*stw**2*betaZfun*
     & b0fqsww * (  - ctw**(-2)*s*s4fun**(-1)*deltafun + 4.D0*s*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(2,1,1,2) =
     &  + g**4*stw**2 * ( ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 - 
     &    8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D
     &    0*ctw**2*s*s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**4*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*rwm2**2
     &     - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 
     &    4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 - 8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun
     &     + 16.D0*ctw**2*s*rwm2 + 4.D0*ctw**2*s*s2fun - 8.D0*ctw**2*s*
     &    u*s2fun*s4fun**(-1) + 24.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2 + 6.
     &    D0*ctw**2*s**2*u**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s**2*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*
     &    s**3*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**3*u**3*Yfun**(-2) + 
     &    3.D0*ctw**(-2)*s**4*u**2*Yfun**(-2) + 2.D0*ctw**(-2)*s**5*u*
     &    Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**6*Yfun**(-2) - 8.D0*
     &    ctw**(-2)*mz**2*s*u*Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s**2*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*u**3*
     &    Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*s**3*u**2*Yfun**(-2) - 12.D
     &    0*ctw**(-2)*mz**2*s**4*u*Yfun**(-2) - 4.D0*ctw**(-2)*mz**2*
     &    s**5*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 8.D0
     &    *ctw**(-2)*mz**4*s*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    Yfun**(-2) - ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 24.
     &    D0*ctw**(-2)*mz**4*s**3*u*Yfun**(-2) + 12.D0*ctw**(-2)*mz**4*
     &    s**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*ctw**(-2)*mz**6*s**2*u*Yfun**(-2) - 16.D0*ctw**(-2)*
     &    mz**6*s**3*Yfun**(-2) - 4.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s**2*Yfun**(-2) - 8.D0*
     &    rwm2**2 - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 16.D0*s**2*u*
     &    Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 8.D0*s**3*
     &    Yfun**(-1)*rwm2 - 8.D0*s**3*u**3*Yfun**(-2) - 12.D0*s**4*u**2
     &    *Yfun**(-2) - 8.D0*s**5*u*Yfun**(-2) - 2.D0*s**6*Yfun**(-2)
     &     - 4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 32.D0*mz**2*s
     &    *u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s**2*Yfun**(-1)*rwm2 + 16.D0
     &    *mz**2*s**2*u**3*Yfun**(-2) + 48.D0*mz**2*s**3*u**2*
     &    Yfun**(-2) + 48.D0*mz**2*s**4*u*Yfun**(-2) + 16.D0*mz**2*s**5
     &    *Yfun**(-2) + 8.D0*mz**4*s4fun**(-1)*rwm2 - 32.D0*mz**4*s*
     &    Yfun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 
     &    8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    u**2*Yfun**(-2) + 4.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 96.
     &    D0*mz**4*s**3*u*Yfun**(-2) - 48.D0*mz**4*s**4*Yfun**(-2) - 16.
     &    D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s**2*u*Yfun**(-2) + 64.D0
     &    *mz**6*s**3*Yfun**(-2) + 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s**2*Yfun**(-2) + 24.D0*ctw**2*rwm2**2 - 8.D0*
     &    ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D0*
     &    ctw**2*s*s2fun + 8.D0*ctw**2*s*u*s2fun*s4fun**(-1) + 24.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 8.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1) + 48.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 6.D0*
     &    ctw**2*s**2*u**4*Yfun**(-2) + 24.D0*ctw**2*s**3*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**3*u**3*Yfun**(-2) + 36.D0*ctw**2*s**4*
     &    u**2*Yfun**(-2) + 24.D0*ctw**2*s**5*u*Yfun**(-2) + 6.D0*
     &    ctw**2*s**6*Yfun**(-2) - 16.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 96.D0*
     &    ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    u**3*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*u**2*Yfun**(-2) - 
     &    144.D0*ctw**2*mz**2*s**4*u*Yfun**(-2) - 48.D0*ctw**2*mz**2*
     &    s**5*Yfun**(-2) - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 96.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*u**2*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*Yfun**(-1)
     &     + 288.D0*ctw**2*mz**4*s**3*u*Yfun**(-2) + 144.D0*ctw**2*
     &    mz**4*s**4*Yfun**(-2) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 192.D0*ctw**2*mz**6*s**2*u*Yfun**(-2) - 192.D0*ctw**2*
     &    mz**6*s**3*Yfun**(-2) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**8*s**2*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    u**2*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**3*Yfun**(-2)*rzm2 + 
     &    ctw**(-2)*s*u**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*
     &    Yfun**(-1)*rwm2 + 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0*s*u**4*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*u1fun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*ctw**2*s2fun*s4fun**(-1)*rzm2 - 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 - 16.D0*ctw**2*u*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0*ctw**2*s*u**4*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*u1fun**(-1)*rwm2
     &     + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*s*u*Yfun**(-1)*rwm2 + ctw**(-2)*s*u**3*Yfun**(-2)*
     &    rzm2 + ctw**(-2)*s*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*s**2*
     &    Yfun**(-1)*rwm2 + 3.D0*ctw**(-2)*s**2*u**2*Yfun**(-2)*rzm2 + 
     &    4.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) + 3.D0*ctw**(-2)*s**3*u*
     &    Yfun**(-2)*rzm2 + 6.D0*ctw**(-2)*s**3*u**2*Yfun**(-2) + 
     &    ctw**(-2)*s**4*Yfun**(-2)*rzm2 + 4.D0*ctw**(-2)*s**4*u*
     &    Yfun**(-2) + ctw**(-2)*s**5*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*Yfun**(-1)*rwm2
     &    *rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s*
     &    u**2*Yfun**(-2)*rzm2 - 8.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2)
     &     - 12.D0*ctw**(-2)*mz**2*s**2*u*Yfun**(-2)*rzm2 - 24.D0*
     &    ctw**(-2)*mz**2*s**2*u**2*Yfun**(-2) - 6.D0*ctw**(-2)*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) + 8.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s*u*Yfun**(-2)*rzm2 + 24.D0
     &    *ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*
     &    Yfun**(-2)*rzm2 + 48.D0*ctw**(-2)*mz**4*s**2*u*Yfun**(-2) + 
     &    24.D0*ctw**(-2)*mz**4*s**3*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*Yfun**(-2)*rzm2 - 32.D0*
     &    ctw**(-2)*mz**6*s*u*Yfun**(-2) - 32.D0*ctw**(-2)*mz**6*s**2*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 16.
     &    D0*ctw**(-2)*mz**8*s*Yfun**(-2) - 8.D0*u*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 
     &    16.D0*s*u*Yfun**(-1)*rwm2 - 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0
     &    *s*u**4*Yfun**(-2) - 8.D0*s**2*Yfun**(-1)*rwm2 - 12.D0*s**2*
     &    u**2*Yfun**(-2)*rzm2 - 16.D0*s**2*u**3*Yfun**(-2) - 12.D0*
     &    s**3*u*Yfun**(-2)*rzm2 - 24.D0*s**3*u**2*Yfun**(-2) - 4.D0*
     &    s**4*Yfun**(-2)*rzm2 - 16.D0*s**4*u*Yfun**(-2) - 4.D0*s**5*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*t1fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 + 24.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 32.D0*
     &    mz**2*s*u**3*Yfun**(-2) + 48.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 96.D0*mz**2*s**2*u**2*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 96.D0*mz**2*s**3*u*Yfun**(-2) + 32.D0*mz**2
     &    *s**4*Yfun**(-2) - 32.D0*mz**4*Yfun**(-1)*rwm2 + 8.D0*mz**4*u
     &    *s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0
     &    *mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s*u*
     &    Yfun**(-2)*rzm2 - 96.D0*mz**4*s*u**2*Yfun**(-2) + 8.D0*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*Yfun**(-2)*
     &    rzm2 - 192.D0*mz**4*s**2*u*Yfun**(-2) - 96.D0*mz**4*s**3*
     &    Yfun**(-2) - 16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*
     &    mz**6*u*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*mz**6*s*Yfun**(-2)*rzm2 + 128.D0*mz**6*s*u
     &    *Yfun**(-2) + 128.D0*mz**6*s**2*Yfun**(-2) + 32.D0*mz**8*
     &    s4fun**(-1)*Yfun**(-1) - 64.D0*mz**8*s*Yfun**(-2) + 16.D0*
     &    ctw**2*s2fun*s4fun**(-1)*rzm2 + 24.D0*ctw**2*u*Yfun**(-1)*
     &    rwm2*rzm2 + 16.D0*ctw**2*u*s2fun*s4fun**(-1) + 24.D0*ctw**2*
     &    u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 
     &    16.D0*ctw**2*s*s2fun*s4fun**(-1) + 48.D0*ctw**2*s*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0
     &    *ctw**2*s*u**4*Yfun**(-2) + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2
     &     + 36.D0*ctw**2*s**2*u**2*Yfun**(-2)*rzm2 + 48.D0*ctw**2*s**2
     &    *u**3*Yfun**(-2) + 36.D0*ctw**2*s**3*u*Yfun**(-2)*rzm2 + 72.D0
     &    *ctw**2*s**3*u**2*Yfun**(-2) + 12.D0*ctw**2*s**4*Yfun**(-2)*
     &    rzm2 + 48.D0*ctw**2*s**4*u*Yfun**(-2) + 12.D0*ctw**2*s**5*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*t1fun**(-1)*rwm2
     &     - 48.D0*ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 32.D0*ctw**2*
     &    mz**2*s2fun*s4fun**(-1) - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 72.D0*ctw**2*
     &    mz**2*s*u**2*Yfun**(-2)*rzm2 - 96.D0*ctw**2*mz**2*s*u**3*
     &    Yfun**(-2) - 144.D0*ctw**2*mz**2*s**2*u*Yfun**(-2)*rzm2 - 288.
     &    D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) - 72.D0*ctw**2*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 288.D0*ctw**2*mz**2*s**3*u*Yfun**(-2)
     &     - 96.D0*ctw**2*mz**2*s**4*Yfun**(-2) + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s*u*
     &    Yfun**(-2)*rzm2 + 288.D0*ctw**2*mz**4*s*u**2*Yfun**(-2) - 24.D
     &    0*ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*
     &    mz**4*s**2*Yfun**(-2)*rzm2 + 576.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) + 288.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 48.D0*
     &    ctw**2*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6
     &    *u*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**6*s*Yfun**(-2)*rzm2 - 384.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) - 384.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 96.D0*ctw**2*mz**8*s4fun**(-1)*Yfun**(-1) + 192.D
     &    0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 + 3.D0/2.D0*ctw**(-2)*
     &    s**2*s2fun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**3*s2fun*s4fun*
     &    Yfun**(-2) + ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - 1.D0/2.D0*ctw**(-2)*mz**4*s**2*s2fun*Yfun**(-2) + 4.D0*s*
     &    s2fun*Yfun**(-1)*rwm2 - 6.D0*s**2*s2fun*Yfun**(-1) + 2.D0*
     &    s**3*s2fun*s4fun*Yfun**(-2) - 4.D0*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*mz**4*s**2*s2fun*Yfun**(-2) - 16.D0*ctw**2*
     &    s*s2fun*s4fun**(-1) - 12.D0*ctw**2*s*s2fun*Yfun**(-1)*rwm2 + 
     &    18.D0*ctw**2*s**2*s2fun*Yfun**(-1) - 6.D0*ctw**2*s**3*s2fun*
     &    s4fun*Yfun**(-2) + 12.D0*ctw**2*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 6.D0*ctw**2*mz**4*s**2*s2fun*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**2*
     &    Yfun**(-1) - ctw**(-2)*s*u**4*Yfun**(-2) - ctw**(-2)*s**2*u*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**3*Yfun**(-1) - 3.D0*ctw**(-2)*s**3*u**2*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*s**4*u*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**5*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*Yfun**(-1) + 12.D0*ctw**(-2)*mz**2*s**2*
     &    u**2*Yfun**(-2) + 12.D0*ctw**(-2)*mz**2*s**3*u*Yfun**(-2) + 4.
     &    D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s*
     &    Yfun**(-1) - 11.D0*ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 23.D0*
     &    ctw**(-2)*mz**4*s**2*u*Yfun**(-2) - 23.D0/2.D0*ctw**(-2)*
     &    mz**4*s**3*Yfun**(-2) + 14.D0*ctw**(-2)*mz**6*s*u*Yfun**(-2)
     &     + 14.D0*ctw**(-2)*mz**6*s**2*Yfun**(-2) - 6.D0*ctw**(-2)*
     &    mz**8*s*Yfun**(-2) + 4.D0*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*
     &    u**2*Yfun**(-1) + 4.D0*s*u**4*Yfun**(-2) + 4.D0*s**2*u*
     &    Yfun**(-1) + 8.D0*s**2*u**3*Yfun**(-2) + 2.D0*s**3*Yfun**(-1)
     &     + 12.D0*s**3*u**2*Yfun**(-2) + 8.D0*s**4*u*Yfun**(-2) + 2.D0
     &    *s**5*Yfun**(-2) - 8.D0*mz**2*s*u*Yfun**(-1) - 16.D0*mz**2*s*
     &    u**3*Yfun**(-2) - 8.D0*mz**2*s**2*Yfun**(-1) - 48.D0*mz**2*
     &    s**2*u**2*Yfun**(-2) - 48.D0*mz**2*s**3*u*Yfun**(-2) - 16.D0*
     &    mz**2*s**4*Yfun**(-2) + 8.D0*mz**4*s*Yfun**(-1) + 44.D0*mz**4
     &    *s*u**2*Yfun**(-2) + 92.D0*mz**4*s**2*u*Yfun**(-2) + 46.D0*
     &    mz**4*s**3*Yfun**(-2) - 56.D0*mz**6*s*u*Yfun**(-2) - 56.D0*
     &    mz**6*s**2*Yfun**(-2) + 24.D0*mz**8*s*Yfun**(-2) - 16.D0*
     &    ctw**2*s - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1) - 12.D0*ctw**2*s*u**4*Yfun**(-2) - 
     &    12.D0*ctw**2*s**2*u*Yfun**(-1) - 24.D0*ctw**2*s**2*u**3*
     &    Yfun**(-2) - 6.D0*ctw**2*s**3*Yfun**(-1) - 36.D0*ctw**2*s**3*
     &    u**2*Yfun**(-2) - 24.D0*ctw**2*s**4*u*Yfun**(-2) - 6.D0*
     &    ctw**2*s**5*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*u*Yfun**(-1) + 
     &    48.D0*ctw**2*mz**2*s*u**3*Yfun**(-2) + 24.D0*ctw**2*mz**2*
     &    s**2*Yfun**(-1) + 144.D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) + 
     &    144.D0*ctw**2*mz**2*s**3*u*Yfun**(-2) + 48.D0*ctw**2*mz**2*
     &    s**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*s*Yfun**(-1) - 132.D0*
     &    ctw**2*mz**4*s*u**2*Yfun**(-2) - 276.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) - 138.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 168.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) + 168.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 72.D0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*b0fmzww * (  - 
     &    4.D0 + ctw**(-2) + ctw**(-2)*s2fun*s4fun**(-1) + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1) - ctw**(-2)*s*s2fun*Yfun**(-1) + 2.D
     &    0*ctw**(-2)*s*u*Yfun**(-1) + ctw**(-2)*s**2*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s4fun**(-1) - 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    ctw**(-2)*mz**4*s*u1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**6*u1fun**(-2)*s4fun**(-1) - 4.D0*s2fun*s4fun**(-1) - 8.D0
     &    *u**2*Yfun**(-1) + 4.D0*s*s2fun*Yfun**(-1) - 8.D0*s*u*
     &    Yfun**(-1) - 4.D0*s**2*Yfun**(-1) + 8.D0*mz**2*s4fun**(-1) + 
     &    16.D0*mz**2*u*Yfun**(-1) + 16.D0*mz**2*s*Yfun**(-1) - 16.D0*
     &    mz**4*Yfun**(-1) - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2 + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1) - 12.D0*
     &    ctw**2*s*s2fun*Yfun**(-1) + 24.D0*ctw**2*s*u*Yfun**(-1) + 12.D
     &    0*ctw**2*s**2*Yfun**(-1) - 24.D0*ctw**2*mz**2*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**2*u*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**4*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) + 
     &    48.D0*ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*b0fusww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 4.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 
     &    4.D0*u**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 8.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 6.D0*
     &    ctw**2 - 12.D0*ctw**2*u**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) - 48.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*b0ftsww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*s*u*Yfun**(-1) - ctw**(-2)*s**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*u**2*Yfun**(-1) + 8.D0*s*u*Yfun**(-1) + 4.D
     &    0*s**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 16.D0*mz**2*u*
     &    Yfun**(-1) - 16.D0*mz**2*s*Yfun**(-1) + 16.D0*mz**4*
     &    Yfun**(-1) + 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 4.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) - 6.D0*ctw**2 - 12.D0*ctw**2*
     &    u**2*Yfun**(-1) - 24.D0*ctw**2*s*u*Yfun**(-1) - 12.D0*ctw**2*
     &    s**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*s4fun**(-1) + 48.D0*
     &    ctw**2*mz**2*u*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*Yfun**(-1)
     &     - 48.D0*ctw**2*mz**4*Yfun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*b0fqsww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) + ctw**(-2)*s*s2fun*Yfun**(-1) + 
     &    4.D0*s2fun*s4fun**(-1) - 4.D0*s*s2fun*Yfun**(-1) - 12.D0*
     &    ctw**2*s2fun*s4fun**(-1) + 12.D0*ctw**2*s*s2fun*Yfun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun * ( 
     &     - 2.D0*ctw**(-2)*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - ctw**(-2)*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 4.D0*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - 24.D0*ctw**2*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**2*s**2*t1fun**(-1)*u1fun**(-1)
     &    *s4fun**(-1) + 24.D0*ctw**2*mz**4*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( 32.D0*ctw**2*s*u*s4fun**(-1)*rwm2 + 8.D0*
     &    ctw**2*s*u*s2fun*s4fun**(-1) + 16.D0*ctw**2*s**2*s4fun**(-1)*
     &    rwm2 + 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) - 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 - 2.D
     &    0*ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**3*s4fun**(-1)*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*s4fun**(-1)
     &    *rwm2 + 8.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) + 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*mz**2*s*u*s4fun**(-1) + 2.D0*mz**2*s**2*
     &    s4fun**(-1) - 4.D0*mz**4*s*s4fun**(-1) - 8.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*ctw**2*s*u*s4fun**(-1)*rwm2 + 8.
     &    D0*ctw**2*s*u*s2fun*s4fun**(-1) - 24.D0*ctw**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*ctw**2*s**2*s4fun**(-1)*
     &    rwm2 + 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) - 8.D0*ctw**2*s**2*
     &    u*s4fun**(-1) - 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 32.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 6.D0*ctw**2*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 - 2.D
     &    0*ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*
     &    ctw**(-2)*s**2*s4fun**(-1)*rwm2 - 6.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 1.D0/2.D0*ctw**(-2)*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**2*u**5*
     &    s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**(-2)*s**3*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 3.D0/2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 5.D0/2.D0*ctw**(-2)*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)*rwm2
     &     - 3.D0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-1) - 5.D0*
     &    ctw**(-2)*s**4*u**3*s4fun**(-1)*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**5*s4fun**(-1)*Yfun**(-1) - 5.D0*ctw**(-2)*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 5.D0/2.D0*ctw**(-2)*s**6*u*
     &    s4fun**(-1)*Yfun**(-2) - 1.D0/2.D0*ctw**(-2)*s**7*s4fun**(-1)
     &    *Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 12.D0
     &    *ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 3.D0*
     &    ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 5.D0*
     &    ctw**(-2)*mz**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 12.D0*
     &    ctw**(-2)*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 20.D0*
     &    ctw**(-2)*mz**2*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 3.D0*
     &    ctw**(-2)*mz**2*s**4*s4fun**(-1)*Yfun**(-1) + 30.D0*ctw**(-2)
     &    *mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 20.D0*ctw**(-2)*
     &    mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 5.D0*ctw**(-2)*mz**2*
     &    s**6*s4fun**(-1)*Yfun**(-2) - 22.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 22.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 39.D0/2.D0*ctw**(-2)*mz**4*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**(-2)*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 117.D0/2.D0*ctw**(-2)*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 117.D0/2.D0*ctw**(-2)*mz**4*
     &    s**4*u*s4fun**(-1)*Yfun**(-2) - 39.D0/2.D0*ctw**(-2)*mz**4*
     &    s**5*s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 37.D0*ctw**(-2)*mz**6*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 74.D0*ctw**(-2)*mz**6*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) + 37.D0*ctw**(-2)*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 34.D0*ctw**(-2)*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 34.D0*ctw**(-2)*mz**8*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**10*s**2*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*s4fun**(-1)*rwm2 + 8.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 8.D0*s**2*s4fun**(-1)*rwm2
     &     + 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) + 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*
     &    s**3*u**2*s4fun**(-1)*Yfun**(-1) + 10.D0*s**3*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*s**4*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 6.D0*s**4*u*s4fun**(-1)*Yfun**(-1) + 20.D0*s**4*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*s**5*s4fun**(-1)*Yfun**(-1) + 
     &    20.D0*s**5*u**2*s4fun**(-1)*Yfun**(-2) + 10.D0*s**6*u*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*s**7*s4fun**(-1)*Yfun**(-2) - 
     &    16.D0*mz**2*s*s4fun**(-1)*rwm2 + 4.D0*mz**2*s*u*s4fun**(-1)
     &     - 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*
     &    mz**2*s**2*s4fun**(-1) - 96.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 20.D0*mz**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 
     &    48.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*mz**2*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) - 80.D0*mz**2*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 120.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    80.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 20.D0*mz**2*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 4.D0*mz**4*s*s4fun**(-1) + 88.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 88.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 78.D0*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 
     &    24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 234.D0*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 234.D0*mz**4*s**4*u*s4fun**(-1)
     &    *Yfun**(-2) + 78.D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) - 48.D0
     &    *mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 148.D0*mz**6*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 296.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) - 148.
     &    D0*mz**6*s**4*s4fun**(-1)*Yfun**(-2) + 136.D0*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 136.D0*mz**8*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 48.D0*mz**10*s**2*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    ctw**2*s*u*s4fun**(-1)*rwm2 + 8.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1) - 24.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     - 8.D0*ctw**2*s**2*s4fun**(-1)*rwm2 + 4.D0*ctw**2*s**2*s2fun
     &    *s4fun**(-1) - 8.D0*ctw**2*s**2*u*s4fun**(-1) - 72.D0*ctw**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**2*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*ctw**2*s**3*s4fun**(-1) - 72.D0*ctw**2*s**3
     &    *u*s4fun**(-1)*Yfun**(-1)*rwm2 - 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 30.D0*ctw**2*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    18.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-1) - 60.D0*ctw**2*
     &    s**4*u**3*s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-1) - 60.D0*ctw**2*s**5*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 30.D0*ctw**2*s**6*u*s4fun**(-1)*Yfun**(-2) - 6.D0
     &    *ctw**2*s**7*s4fun**(-1)*Yfun**(-2) + 16.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**2*s*s2fun*s4fun**(-1) + 
     &    144.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0
     &    *ctw**2*mz**2*s**2*s4fun**(-1) + 288.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 36.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 60.D0*ctw**2*mz**2*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*mz**2*s**3*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*
     &    Yfun**(-1) + 240.D0*ctw**2*mz**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*Yfun**(-1)
     &     + 360.D0*ctw**2*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 240.
     &    D0*ctw**2*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 60.D0*ctw**2*
     &    mz**2*s**6*s4fun**(-1)*Yfun**(-2) - 264.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 264.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 234.D0*ctw**2*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 72.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 702.D0*ctw**2*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 702.D0*ctw**2*mz**4*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) - 234.D0*ctw**2*mz**4*s**5*s4fun**(-1)*Yfun**(-2)
     &     + 144.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 444.D0*ctw**2*
     &    mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 888.D0*ctw**2*mz**6*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) + 444.D0*ctw**2*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 408.D0*ctw**2*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 408.D0*ctw**2*mz**8*s**3*s4fun**(-1)
     &    *Yfun**(-2) + 144.D0*ctw**2*mz**10*s**2*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D
     &    0*ctw**(-2)*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s*u**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - ctw**(-2)*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*u1fun**(-1)*s4fun**(-1)*
     &    rwm2 - 2.D0*ctw**(-2)*mz**4*s*u*u1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + ctw**(-2)*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*s
     &    *u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*rwm2 + 8.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 4.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 
     &    16.D0*ctw**2*s*s4fun**(-1)*rzm2 - 16.D0*ctw**2*s*u*
     &    s4fun**(-1) + 12.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 24.D0*ctw**2*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 12.D0*
     &    ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 12.D0*ctw**2*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s*u*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*
     &    s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 12.D0*ctw**2*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 2.D0*ctw**(-2)*s*u**3*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s*
     &    u**4*s4fun**(-1)*Yfun**(-2)*rzm2 - ctw**(-2)*s*u**5*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*s**2*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 3.D0*ctw**(-2)*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 5.D0*ctw**(-2)*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 + 6.D0
     &    *ctw**(-2)*s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 3.
     &    D0*ctw**(-2)*s**3*u*s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**(-2)*
     &    s**3*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 10.D0*ctw**(-2)*s**3*
     &    u**3*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*s**4*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 - ctw**(-2)*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 10.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    ctw**(-2)*s**5*s4fun**(-1)*Yfun**(-2)*rzm2 - 5.D0*ctw**(-2)*
     &    s**5*u*s4fun**(-1)*Yfun**(-2) - ctw**(-2)*s**6*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    rwm2 + 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     - 12.D0*ctw**(-2)*mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 24.D0*ctw**(-2)*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 40.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 60.D0*ctw**(-2)*mz**2*s**3*u**2
     &    *s4fun**(-1)*Yfun**(-2) + 8.D0*ctw**(-2)*mz**2*s**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*ctw**(-2)*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 10.D0*ctw**(-2)*mz**2*s**5*
     &    s4fun**(-1)*Yfun**(-2) - 4.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 22.D0*ctw**(-2)*mz**4*s*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 23.D0*ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 39.D0*ctw**(-2)*mz**4*s*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 22.D0*ctw**(-2)*mz**4*s**2*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**(-2)*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 46.D0*ctw**(-2)*mz**4*
     &    s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 117.D0*ctw**(-2)*mz**4*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-2) - 23.D0*ctw**(-2)*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 - 117.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) - 39.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 8.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 28.D0*ctw**(-2)*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 74.D0*ctw**(-2)*mz**6*s*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 28.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 148.D0*ctw**(-2)*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 74.D0*ctw**(-2)*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 68.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*
     &    Yfun**(-2) - 68.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2)
     &     + 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 4.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*s**2*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 + 20.
     &    D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 24.D0*s**3*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*s**3*u*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *s**3*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 4.D0*s**4*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**4*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*s**5*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 20.D0*s**5*u*s4fun**(-1)*Yfun**(-2) + 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)
     &    *rwm2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*
     &    mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 
     &    96.D0*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    48.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**2*s**2*
     &    u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 160.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 48.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 96.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     - 240.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 32.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 160.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 40.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 88.D0
     &    *mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 48.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 92.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 156.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 88.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 
     &    184.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 468.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 92.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 468.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) + 156.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 48.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 32.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) - 112.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 296.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     - 112.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 592.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 48.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 + 272.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 272.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*ctw**2*s*s4fun**(-1)*rzm2 - 16.
     &    D0*ctw**2*s*u*s4fun**(-1) - 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 24.D0*ctw**2*s*u**3*t1fun**(-1)*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 12.D0*ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 12.D0*
     &    ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) - 16.D0*ctw**2*s**2*
     &    s4fun**(-1) - 24.D0*ctw**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 72.D0*ctw**2*s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 48.D0*
     &    ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 - 60.D0*ctw**2*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 72.D0*ctw**2*s**3*u*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 - 36.D0*ctw**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) - 72.D0*ctw**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 120.D0*ctw**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*ctw**2*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1) - 
     &    48.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 120.D0*
     &    ctw**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 60.D0*ctw**2*s**5*u*s4fun**(-1)
     &    *Yfun**(-2) - 12.D0*ctw**2*s**6*s4fun**(-1)*Yfun**(-2) + 24.D0
     &    *ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*rwm2 + 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1) + 48.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**2*s*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 120.D0*ctw**2*mz**2*s*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 48.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 288.D0*ctw**2*mz**2*s**2*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 144.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 288.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 480.D0*ctw**2*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 288.D0*ctw**2*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 720.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*ctw**2*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 480.D0*ctw**2*mz**2*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) + 120.D0*ctw**2*mz**2*s**5*s4fun**(-1)*Yfun**(-2)
     &     - 48.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 264.D0*
     &    ctw**2*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    144.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 276.D0*
     &    ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 468.D0*
     &    ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 264.D0*ctw**2*
     &    mz**4*s**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 144.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 552.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 1404.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 276.D0*ctw**2*mz**4*
     &    s**3*s4fun**(-1)*Yfun**(-2)*rzm2 - 1404.D0*ctw**2*mz**4*s**3*
     &    u*s4fun**(-1)*Yfun**(-2) - 468.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 336.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 888.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 336.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 1776.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 
     &    888.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2) - 144.D0*
     &    ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-2)*rzm2 - 816.D0*ctw**2*
     &    mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 816.D0*ctw**2*mz**8*s**2*
     &    s4fun**(-1)*Yfun**(-2) + 288.D0*ctw**2*mz**10*s*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    ctw**(-2)*s**2*u*s2fun*s4fun**(-1)*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**3*s2fun*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s**3*
     &    u*s2fun*Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - ctw**(-2)*mz**2*s**3*s2fun*Yfun**(-2) + 
     &    ctw**(-2)*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2) + 1.D0/2.D
     &    0*ctw**(-2)*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 
     &    ctw**(-2)*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*s**3*s2fun*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    s**3*u*s2fun*Yfun**(-2) - 2.D0*s**4*s2fun*Yfun**(-2) + 8.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*mz**2*s**3*s2fun*
     &    Yfun**(-2) - 4.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2)
     &     - 2.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 4.D0*mz**6*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**2*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**2*s**3*u*s2fun*Yfun**(-2) + 6.D0*
     &    ctw**2*s**4*s2fun*Yfun**(-2) - 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*mz**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*mz**2*s**3*s2fun*
     &    Yfun**(-2) + 12.D0*ctw**2*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 6.D0*ctw**2*mz**4*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**6*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * ( 2.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) + ctw**(-2)*s**2*Yfun**(-1)*rwm2
     &     + ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) + 3.D0*
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 5.D0*ctw**(-2)*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*ctw**(-2)*s**3*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)
     &     + 7.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 3.D0*
     &    ctw**(-2)*s**5*u*s4fun**(-1)*Yfun**(-2) + 1.D0/2.D0*ctw**(-2)
     &    *s**6*s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*
     &    Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)
     &    *Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 32.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 3.D0*ctw**(-2)*mz**2*s**3*s4fun**(-1)*Yfun**(-1)
     &     - 42.D0*ctw**(-2)*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    24.D0*ctw**(-2)*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) - 5.D0*
     &    ctw**(-2)*mz**2*s**5*s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 30.D0*ctw**(-2)*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) + 5.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 81.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 70.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) + 39.D0/2.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 50.D0*ctw**(-2)*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 88.D0*ctw**(-2)*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) - 37.D0*ctw**(-2)*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 40.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*Yfun**(-2)
     &     + 34.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 12.D0*
     &    ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*
     &    Yfun**(-1)*rwm2 - 8.D0*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) - 
     &    8.D0*s*u**3*s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**2*Yfun**(-1)*rwm2 - 4.D0*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 20.D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) - 32.D0*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*s**4*s4fun**(-1)*Yfun**(-1) - 28.D0*s**4*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 12.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*s**6*s4fun**(-1)*Yfun**(-2) + 8.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 + 8.D0*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 40.D0
     &    *mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) + 32.D0*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 128.D0*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 12.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) + 168.D0
     &    *mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 20.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 120.D0*
     &    mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) - 20.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 324.D0*mz**4*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 280.D0*mz**4*s**3*u*s4fun**(-1)*Yfun**(-2) - 78.D
     &    0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 8.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 200.D0*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 352.D0*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 148.
     &    D0*mz**6*s**3*s4fun**(-1)*Yfun**(-2) - 160.D0*mz**8*s*u*
     &    s4fun**(-1)*Yfun**(-2) - 136.D0*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) + 48.D0*mz**10*s*s4fun**(-1)*Yfun**(-2) + 24.D0*
     &    ctw**2*s*u*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) + 12.D
     &    0*ctw**2*s**2*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 60.D0*ctw**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 
     &    24.D0*ctw**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*
     &    s**3*u**3*s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**2*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 84.D0*ctw**2*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 36.D0*ctw**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 6.D0
     &    *ctw**2*s**6*s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*mz**2*s*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 120.D0*ctw**2*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) - 96.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 384.D0*ctw**2*
     &    mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 36.D0*ctw**2*mz**2*
     &    s**3*s4fun**(-1)*Yfun**(-1) - 504.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 288.D0*ctw**2*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 60.D0*ctw**2*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 72.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 
     &    360.D0*ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 60.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 972.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 840.D0*ctw**2*mz**4*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) + 234.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 600.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 1056.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) - 444.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2)
     &     + 480.D0*ctw**2*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 408.D0*
     &    ctw**2*mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*
     &    mz**10*s*s4fun**(-1)*Yfun**(-2) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - 4.D0*ctw**(-2)*u*s4fun**(-1) - 4.D0*ctw**(-2)*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*s*s4fun**(-1) + 
     &    2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s4fun**(-1) + 12.D0*
     &    ctw**(-2)*mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*ctw**(-2)*mz**2*
     &    s*u*s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*s4fun**(-1)
     &     - 16.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1) - ctw**(-2)
     &    *mz**4*s*t1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*
     &    u1fun**(-2)*s4fun**(-1) - 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) + 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 
     &    2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 8.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1) + 16.D0*u*s4fun**(-1) + 16.D0*
     &    u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*s*s4fun**(-1) - 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 24.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s4fun**(-1) - 48.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 64.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) - 24.D0*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 64.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*u*s4fun**(-1) - 
     &    48.D0*ctw**2*u**3*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s*
     &    s4fun**(-1) + 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - 72.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**2*s4fun**(-1) + 144.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**2*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) - 
     &    24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) - 192.D0*ctw**2*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**2*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 144.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1) + 
     &    24.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 96.D0*ctw**2*mz**6*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & b0fusww * ( 2.D0 - 1.D0/2.D0*ctw**(-2) + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) - 
     &    8.D0*u*s4fun**(-1) - 8.D0*u**3*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    s*u**2*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*
     &    mz**4*s*u1fun**(-2)*s4fun**(-1) + 8.D0*mz**6*u1fun**(-2)*
     &    s4fun**(-1) - 6.D0*ctw**2 + 24.D0*ctw**2*u*s4fun**(-1) + 24.D0
     &    *ctw**2*u**3*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - 2.D0 + 1.D0/2.D0*ctw**(-2) + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) + 2.D
     &    0*ctw**(-2)*s*s4fun**(-1) + 5.D0*ctw**(-2)*s*u**2*s4fun**(-1)
     &    *Yfun**(-1) + 4.D0*ctw**(-2)*s**2*u*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) - 10.D0*ctw**(-2)*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)
     &     - 6.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*ctw**(-2)*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) - 
     &    8.D0*ctw**(-2)*mz**6*s4fun**(-1)*Yfun**(-1) - 8.D0*u*
     &    s4fun**(-1) - 8.D0*u**3*s4fun**(-1)*Yfun**(-1) - 8.D0*s*
     &    s4fun**(-1) - 20.D0*s*u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 40.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1) - 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 64.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*s4fun**(-1) - 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*s4fun**(-1)*Yfun**(-1) + 6.D0*
     &    ctw**2 + 24.D0*ctw**2*u*s4fun**(-1) + 24.D0*ctw**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*s*s4fun**(-1) + 60.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**2*s4fun**(-1) - 120.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) + 
     &    192.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 144.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**6*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,1,2) = ewamp(2,1,1,2) + g**4*stw**2*betaZfun*
     & b0fqsww * (  - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      ewamp(2,1,3,1) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - deltafun*Yfun - 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     - ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**2*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*u**2*s4fun**(-1)*deltafun - 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**4*u*s4fun**(-1)*deltafun + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 4.D0*mz**6*s4fun**(-1)*deltafun - 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun - 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.D0
     &    *ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    /2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 5.D0*s**2*u*s4fun**(-1)*deltafun + 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**3*
     &    s4fun**(-1)*deltafun + 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun - 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun + 22.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 20.D0*mz**6*s4fun**(-1)*deltafun - 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun - 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D
     &    0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0
     &    *ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*
     &    rzm2 + 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0
     &    *s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*
     &    deltafun - 8.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D
     &    0*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun - 4.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*mz**2*s**(-1)*deltafun*
     &    rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *mz**2*s**(-1)*u*deltafun - 16.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0
     &    *ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D
     &    0*ctw**2*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 
     &    32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*
     &    mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s**(-1)*
     &    u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 + 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*rzm2 + 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*s**(-1)*u**2*deltafun
     &     + 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 6.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun + 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*s**(-1)*u*deltafun + 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*deltafun
     &     - 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s**(-1)*
     &    deltafun + 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun - 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*c0ccqswww * ( 
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    deltafun - 2.D0*s*deltafun - 4.D0*s*u*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**2*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**2*deltafun + 4.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*ctw**2*s*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*mz**2*s*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*c0mzmzqswww
     &  * ( ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) - 32.D0*ctw**2*u*deltafun - 16.D0
     &    *ctw**2*s*deltafun + 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) + 32.D
     &    0*ctw**2*mz**2*deltafun - 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*deltafun + 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u*s4fun**(-1)*deltafun - 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun - 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*s4fun**(-1)*deltafun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun + 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*b0ftsww * (  - 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*b0fqsww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    24.D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun * ( 
     &     - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( deltafun*Yfun + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun - s**2*u*s4fun**(-1)*deltafun - 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 - 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - s**3*s4fun**(-1)*deltafun + 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun + 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 10.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     - 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun - 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 - 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*u*deltafun - 8.D0*s*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2
     &     + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*
     &    u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun
     &    *rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*mz**2*s
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 + 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun + 16.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 8.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*deltafun - 16.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun - 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*deltafun
     &     + 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * (  - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 
     &    16.D0*ctw**2*s*deltafun - 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & b0fusww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*u*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & b0ftsww * ( ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun - 4.D0*u*s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun + 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,3,1) = ewamp(2,1,3,1) + g**4*stw**2*betaZfun*
     & b0fqsww * (  - ctw**(-2)*s*s4fun**(-1)*deltafun + 4.D0*s*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(2,1,3,3) =
     &  + g**4*stw**2 * (  - 4.D0*ctw**(-2)*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**2*t1fun**(-1)
     &    *u1fun**(-1)*s4fun**(-1)*Yfun )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*
     &    s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*rwm2 - 
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 8.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**4*s4fun**(-1)
     &    *rwm2 - 4.D0*ctw**(-2)*mz**6*s**(-1)*s4fun**(-1)*rwm2 + 32.D0
     &    *s4fun**(-1)*Yfun*rwm2 + 32.D0*rwm2**2 + 2.D0*Yfun + 4.D0*
     &    s2fun*rwm2 + 16.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 + 16.D0
     &    *mz**2*u*s4fun**(-1)*rwm2 + 4.D0*mz**2*s*s4fun**(-1)*rwm2 - 
     &    32.D0*mz**4*s**(-1)*u*s4fun**(-1)*rwm2 - 16.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 16.D0*mz**6*s**(-1)*s4fun**(-1)*rwm2 - 48.D
     &    0*ctw**2*mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 - 48.D0*ctw**2*
     &    mz**2*u*s4fun**(-1)*rwm2 - 12.D0*ctw**2*mz**2*s*s4fun**(-1)*
     &    rwm2 + 96.D0*ctw**2*mz**4*s**(-1)*u*s4fun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**4*s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**6*s**(-1)*
     &    s4fun**(-1)*rwm2 )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*
     &    rwm2**2 + 4.D0*s2fun*rwm2 - 32.D0*u1fun**2*s4fun**(-1)*rwm2
     &     + 16.D0*s*u**2*Yfun**(-1)*rwm2 + 2.D0*s*u**4*s4fun**(-1)*
     &    Yfun**(-1) - 2.D0*s**2*u*s4fun**(-1) - 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) - 2.D0*mz**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 6.D0*ctw**2*mz**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 1.D0/2.
     &    D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*
     &    s**3*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*rwm2**2 + 4.D0*s2fun*rwm2 - 32.
     &    D0*t1fun**2*s4fun**(-1)*rwm2 + 16.D0*s*u**2*Yfun**(-1)*rwm2
     &     + 2.D0*s*u**4*s4fun**(-1)*Yfun**(-1) + 2.D0*s**2*u*
     &    s4fun**(-1) + 32.D0*s**2*u*Yfun**(-1)*rwm2 + 8.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*s**3*s4fun**(-1) + 16.D0*s**3*
     &    Yfun**(-1)*rwm2 + 12.D0*s**3*u**2*s4fun**(-1)*Yfun**(-1) + 8.D
     &    0*s**4*u*s4fun**(-1)*Yfun**(-1) + 2.D0*s**5*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*mz**2*s*s4fun**(-1)*rwm2 - 64.D0*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) - 16.D0*mz**2*
     &    s*u**3*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**2*s**2*s4fun**(-1)
     &     - 64.D0*mz**2*s**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**2*u*
     &    Yfun**(-1) - 50.D0*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 4.
     &    D0*mz**2*s**3*Yfun**(-1) - 52.D0*mz**2*s**3*u*s4fun**(-1)*
     &    Yfun**(-1) - 18.D0*mz**2*s**4*s4fun**(-1)*Yfun**(-1) + 64.D0*
     &    mz**4*s*Yfun**(-1)*rwm2 + 16.D0*mz**4*s*u*Yfun**(-1) + 44.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 52.D0
     &    *mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s*Yfun**(-1)
     &     - 48.D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 56.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 18.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 6.D0*ctw**2*mz**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**2*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*ctw**2*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*c0cmzuswww * ( 
     &     - 8.D0*ctw**(-2)*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 - ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    s4fun**(-1)*rwm2*rzm2 - 4.D0*rzm2 - 32.D0*u1fun*s4fun**(-1)*
     &    rwm2 + 64.D0*u*s4fun**(-1)*rwm2 + 4.D0*u + 4.D0*u*u1fun*s2fun
     &    *Yfun**(-1) - 32.D0*u**2*u1fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 64.D0
     &    *mz**2*u*u1fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 4.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 32.D0*mz**4*u1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    96.D0*ctw**2*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*rwm2
     &     - 12.D0*ctw**2*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 12.D0
     &    *ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*c0cmztswww * ( 
     &     - 8.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 + ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*mz**4*s**2*s4fun**(-1)*Yfun**(-1)
     &     + 4.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    s4fun**(-1)*rwm2*rzm2 - 4.D0*rzm2 - 32.D0*t1fun*s4fun**(-1)*
     &    rwm2 - 64.D0*u*s4fun**(-1)*rwm2 - 4.D0*u - 4.D0*u*t1fun*s2fun
     &    *Yfun**(-1) - 32.D0*u**2*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    64.D0*s*s4fun**(-1)*rwm2 - 4.D0*s - 4.D0*s*t1fun*s2fun*
     &    Yfun**(-1) - 64.D0*s*u*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 32.
     &    D0*s**2*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 128.D0*mz**2*
     &    s4fun**(-1)*rwm2 + 8.D0*mz**2 + 8.D0*mz**2*t1fun*s2fun*
     &    Yfun**(-1) + 192.D0*mz**2*u*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 192.D0*mz**2*s*t1fun*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*
     &    mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 4.D0*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 8.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 
     &    4.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 288.D0*mz**4*t1fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 8.D0*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 16.
     &    D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*ctw**2*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 12.D0*ctw**2*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 24.D0*ctw**2*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**2*mz**2*s**3*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*c0ccqswww * ( 
     &     - 1.D0/2.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 64.D0*s*s4fun**(-1)*rwm2 + 4.D0*s - 16.D0*s*
     &    s2fun*Yfun**(-1)*rwm2 - 2.D0*s*s2fun**2*Yfun**(-1) + 2.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**2*mz**2*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*c0mzmzqswww
     &  * ( ctw**(-2)*mz**2*s*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*
     &    s**2*Yfun**(-1) + 32.D0*rwm2 + 4.D0*s2fun - 16.D0*s*s4fun*
     &    Yfun**(-1)*rwm2 - 2.D0*s*s2fun*s4fun*Yfun**(-1) - 4.D0*mz**2*
     &    s*s4fun**(-1) + 2.D0*mz**2*s**2*Yfun**(-1) + 12.D0*ctw**2*
     &    mz**2*s*s4fun**(-1) - 6.D0*ctw**2*mz**2*s**2*Yfun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 2.D0*
     &    ctw**(-2)*mz**2*u**2*u1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)
     &    *mz**2*s*u*t1fun**(-2)*s4fun**(-1) + 2.D0*ctw**(-2)*mz**2*
     &    s**2*t1fun**(-2)*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 10.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1)
     &     + 2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**2*
     &    u**2*t1fun**(-2)*s4fun**(-1) - 8.D0*mz**2*u**2*u1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**2*s*u*t1fun**(-2)*s4fun**(-1) - 8.D0*
     &    mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 32.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 40.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 8.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*mz**2*u**2*
     &    t1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*mz**2*u**2*u1fun**(-2)
     &    *s4fun**(-1) + 48.D0*ctw**2*mz**2*s*u*t1fun**(-2)*s4fun**(-1)
     &     + 24.D0*ctw**2*mz**2*s**2*t1fun**(-2)*s4fun**(-1) - 96.D0*
     &    ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) - 96.D0*ctw**2*mz**4*s
     &    *t1fun**(-2)*s4fun**(-1) + 120.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*mz**2*u**2*u1fun**(-2)*s4fun**(-1) - 2.D0*
     &    ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 8.D0*mz**2*u**2*
     &    u1fun**(-2)*s4fun**(-1) + 8.D0*mz**6*u1fun**(-2)*s4fun**(-1)
     &     - 24.D0*ctw**2*mz**2*u**2*u1fun**(-2)*s4fun**(-1) - 24.D0*
     &    ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,3,3) = ewamp(2,1,3,3) + g**4*stw**2*b0ftsww * (  - 
     &    2.D0*ctw**(-2)*mz**2*u**2*t1fun**(-2)*s4fun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*u*t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 8.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 8.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 10.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1)
     &     + 8.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 16.D0*mz**2*s*u*
     &    t1fun**(-2)*s4fun**(-1) + 8.D0*mz**2*s**2*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 40.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 24.D0*ctw**2*mz**2*u**2*t1fun**(-2)*s4fun**(-1)
     &     - 48.D0*ctw**2*mz**2*s*u*t1fun**(-2)*s4fun**(-1) - 24.D0*
     &    ctw**2*mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 96.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 96.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) - 120.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      ewamp(2,1,3,2) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + deltafun*Yfun + 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**2*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*u**2*s4fun**(-1)*deltafun + 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**4*u*s4fun**(-1)*deltafun - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun - 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**6*s4fun**(-1)*deltafun + 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun + 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun + 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     + ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 1.D0/2.D0*
     &    ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0/
     &    2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**3*s4fun**(-1)*
     &    deltafun - 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun - 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 5.D0*s**2*u*s4fun**(-1)*deltafun - 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - s**3*
     &    s4fun**(-1)*deltafun - 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun + 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*s4fun**(-1)*deltafun + 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 - 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun - 22.D0*mz**4*s*s4fun**(-1)*deltafun - 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 20.D0*mz**6*s4fun**(-1)*deltafun + 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun + 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun + 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun + 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun + 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 4.D0*s**(-1)*u**2*deltafun + 8.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D
     &    0*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0
     &    *mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*mz**2*
     &    s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s**(-1)*u*deltafun + 16.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)
     &    *deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 32.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 
     &    48.D0*ctw**2*mz**4*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 48.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2
     &     - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 - 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     + 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 + 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 4.D0*s**(-1)*u*deltafun*rzm2 - 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*deltafun
     &     - 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*deltafun*rzm2 - 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 6.D0*u*
     &    deltafun - 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun - 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 - 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*s**(-1)*u*deltafun - 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*deltafun
     &     + 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s**(-1)*
     &    deltafun - 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 - 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     + 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun - 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 1.
     &    D0/2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*u*deltafun + 2.D0*s*deltafun + 4.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*s**2*s2fun**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*deltafun - 4.D0
     &    *mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**6*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*s*u*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 6.D0*ctw**2*s**2*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2
     &    *s*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) + 32.D0*ctw**2*u*deltafun + 16.D0
     &    *ctw**2*s*deltafun - 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) - 32.D
     &    0*ctw**2*mz**2*deltafun + 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - ctw**(-2)*s*s4fun**(-1)*deltafun - 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*deltafun + 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*s4fun**(-1)*deltafun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun - 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     - 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*b0fqsww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*
     &    deltafun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun - 8.D0*u
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun + 8.D0*
     &    mz**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun - 24.D0*ctw**2
     &    *mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun * ( 
     &     - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( deltafun*Yfun + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun - s**2*u*s4fun**(-1)*deltafun - 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 - 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - s**3*s4fun**(-1)*deltafun + 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun + 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 10.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     - 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun - 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 - 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*u*deltafun - 8.D0*s*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2
     &     + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*
     &    u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun
     &    *rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*mz**2*s
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 + 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun + 16.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 8.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*deltafun - 16.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun - 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*deltafun
     &     + 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * (  - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 
     &    16.D0*ctw**2*s*deltafun - 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & b0fusww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*u*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & b0ftsww * ( ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun - 4.D0*u*s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun + 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,3,2) = ewamp(2,1,3,2) + g**4*stw**2*betaZfun*
     & b0fqsww * (  - ctw**(-2)*s*s4fun**(-1)*deltafun + 4.D0*s*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(2,1,2,1) =
     &  + g**4*stw**2 * ( ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 - 
     &    8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D
     &    0*ctw**2*s*s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**4*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*rwm2**2
     &     - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 
     &    4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 - 8.D0*ctw**2*s2fun*s4fun**(-1)*Yfun
     &     + 16.D0*ctw**2*s*rwm2 + 4.D0*ctw**2*s*s2fun - 8.D0*ctw**2*s*
     &    u*s2fun*s4fun**(-1) + 24.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2 + 6.
     &    D0*ctw**2*s**2*u**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*s*
     &    u**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2
     &     + 1.D0/2.D0*ctw**(-2)*s**2*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*
     &    s**3*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**3*u**3*Yfun**(-2) + 
     &    3.D0*ctw**(-2)*s**4*u**2*Yfun**(-2) + 2.D0*ctw**(-2)*s**5*u*
     &    Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**6*Yfun**(-2) - 8.D0*
     &    ctw**(-2)*mz**2*s*u*Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s**2*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*u**3*
     &    Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*s**3*u**2*Yfun**(-2) - 12.D
     &    0*ctw**(-2)*mz**2*s**4*u*Yfun**(-2) - 4.D0*ctw**(-2)*mz**2*
     &    s**5*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 8.D0
     &    *ctw**(-2)*mz**4*s*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    Yfun**(-2) - ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) + 24.
     &    D0*ctw**(-2)*mz**4*s**3*u*Yfun**(-2) + 12.D0*ctw**(-2)*mz**4*
     &    s**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*ctw**(-2)*mz**6*s**2*u*Yfun**(-2) - 16.D0*ctw**(-2)*
     &    mz**6*s**3*Yfun**(-2) - 4.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s**2*Yfun**(-2) - 8.D0*
     &    rwm2**2 - 8.D0*s*u**2*Yfun**(-1)*rwm2 - 16.D0*s**2*u*
     &    Yfun**(-1)*rwm2 - 2.D0*s**2*u**4*Yfun**(-2) - 8.D0*s**3*
     &    Yfun**(-1)*rwm2 - 8.D0*s**3*u**3*Yfun**(-2) - 12.D0*s**4*u**2
     &    *Yfun**(-2) - 8.D0*s**5*u*Yfun**(-2) - 2.D0*s**6*Yfun**(-2)
     &     - 4.D0*mz**2*s4fun**(-1)*Yfun + 2.D0*mz**2*s + 32.D0*mz**2*s
     &    *u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s**2*Yfun**(-1)*rwm2 + 16.D0
     &    *mz**2*s**2*u**3*Yfun**(-2) + 48.D0*mz**2*s**3*u**2*
     &    Yfun**(-2) + 48.D0*mz**2*s**4*u*Yfun**(-2) + 16.D0*mz**2*s**5
     &    *Yfun**(-2) + 8.D0*mz**4*s4fun**(-1)*rwm2 - 32.D0*mz**4*s*
     &    Yfun**(-1)*rwm2 + 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 
     &    8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    u**2*Yfun**(-2) + 4.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 96.
     &    D0*mz**4*s**3*u*Yfun**(-2) - 48.D0*mz**4*s**4*Yfun**(-2) - 16.
     &    D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s**2*u*Yfun**(-2) + 64.D0
     &    *mz**6*s**3*Yfun**(-2) + 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s**2*Yfun**(-2) + 24.D0*ctw**2*rwm2**2 - 8.D0*
     &    ctw**2*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s*rwm2 + 4.D0*
     &    ctw**2*s*s2fun + 8.D0*ctw**2*s*u*s2fun*s4fun**(-1) + 24.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 8.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1) + 48.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 6.D0*
     &    ctw**2*s**2*u**4*Yfun**(-2) + 24.D0*ctw**2*s**3*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**3*u**3*Yfun**(-2) + 36.D0*ctw**2*s**4*
     &    u**2*Yfun**(-2) + 24.D0*ctw**2*s**5*u*Yfun**(-2) + 6.D0*
     &    ctw**2*s**6*Yfun**(-2) - 16.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 96.D0*
     &    ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    u**3*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*u**2*Yfun**(-2) - 
     &    144.D0*ctw**2*mz**2*s**4*u*Yfun**(-2) - 48.D0*ctw**2*mz**2*
     &    s**5*Yfun**(-2) - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 96.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s**2*u**2*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*Yfun**(-1)
     &     + 288.D0*ctw**2*mz**4*s**3*u*Yfun**(-2) + 144.D0*ctw**2*
     &    mz**4*s**4*Yfun**(-2) + 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     - 192.D0*ctw**2*mz**6*s**2*u*Yfun**(-2) - 192.D0*ctw**2*
     &    mz**6*s**3*Yfun**(-2) - 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**8*s**2*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*
     &    u**2*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**3*Yfun**(-2)*rzm2 + 
     &    ctw**(-2)*s*u**4*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*
     &    Yfun**(-1)*rwm2 + 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0*s*u**4*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*u1fun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*ctw**2*s2fun*s4fun**(-1)*rzm2 - 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 - 16.D0*ctw**2*u*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0*ctw**2*s*u**4*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*u1fun**(-1)*rwm2
     &     + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*s*u*Yfun**(-1)*rwm2 + ctw**(-2)*s*u**3*Yfun**(-2)*
     &    rzm2 + ctw**(-2)*s*u**4*Yfun**(-2) + 2.D0*ctw**(-2)*s**2*
     &    Yfun**(-1)*rwm2 + 3.D0*ctw**(-2)*s**2*u**2*Yfun**(-2)*rzm2 + 
     &    4.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) + 3.D0*ctw**(-2)*s**3*u*
     &    Yfun**(-2)*rzm2 + 6.D0*ctw**(-2)*s**3*u**2*Yfun**(-2) + 
     &    ctw**(-2)*s**4*Yfun**(-2)*rzm2 + 4.D0*ctw**(-2)*s**4*u*
     &    Yfun**(-2) + ctw**(-2)*s**5*Yfun**(-2) + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*Yfun**(-1)*rwm2
     &    *rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s*
     &    u**2*Yfun**(-2)*rzm2 - 8.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2)
     &     - 12.D0*ctw**(-2)*mz**2*s**2*u*Yfun**(-2)*rzm2 - 24.D0*
     &    ctw**(-2)*mz**2*s**2*u**2*Yfun**(-2) - 6.D0*ctw**(-2)*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) + 8.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 2.D0*ctw**(-2)*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s*u*Yfun**(-2)*rzm2 + 24.D0
     &    *ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**2*
     &    Yfun**(-2)*rzm2 + 48.D0*ctw**(-2)*mz**4*s**2*u*Yfun**(-2) + 
     &    24.D0*ctw**(-2)*mz**4*s**3*Yfun**(-2) + 4.D0*ctw**(-2)*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*Yfun**(-2)*rzm2 - 32.D0*
     &    ctw**(-2)*mz**6*s*u*Yfun**(-2) - 32.D0*ctw**(-2)*mz**6*s**2*
     &    Yfun**(-2) - 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 16.
     &    D0*ctw**(-2)*mz**8*s*Yfun**(-2) - 8.D0*u*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 
     &    16.D0*s*u*Yfun**(-1)*rwm2 - 4.D0*s*u**3*Yfun**(-2)*rzm2 - 4.D0
     &    *s*u**4*Yfun**(-2) - 8.D0*s**2*Yfun**(-1)*rwm2 - 12.D0*s**2*
     &    u**2*Yfun**(-2)*rzm2 - 16.D0*s**2*u**3*Yfun**(-2) - 12.D0*
     &    s**3*u*Yfun**(-2)*rzm2 - 24.D0*s**3*u**2*Yfun**(-2) - 4.D0*
     &    s**4*Yfun**(-2)*rzm2 - 16.D0*s**4*u*Yfun**(-2) - 4.D0*s**5*
     &    Yfun**(-2) - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 8.D0*mz**2*t1fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 + 24.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 32.D0*
     &    mz**2*s*u**3*Yfun**(-2) + 48.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 96.D0*mz**2*s**2*u**2*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 96.D0*mz**2*s**3*u*Yfun**(-2) + 32.D0*mz**2
     &    *s**4*Yfun**(-2) - 32.D0*mz**4*Yfun**(-1)*rwm2 + 8.D0*mz**4*u
     &    *s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0
     &    *mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s*u*
     &    Yfun**(-2)*rzm2 - 96.D0*mz**4*s*u**2*Yfun**(-2) + 8.D0*mz**4*
     &    s**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**4*s**2*Yfun**(-2)*
     &    rzm2 - 192.D0*mz**4*s**2*u*Yfun**(-2) - 96.D0*mz**4*s**3*
     &    Yfun**(-2) - 16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*
     &    mz**6*u*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*mz**6*s*Yfun**(-2)*rzm2 + 128.D0*mz**6*s*u
     &    *Yfun**(-2) + 128.D0*mz**6*s**2*Yfun**(-2) + 32.D0*mz**8*
     &    s4fun**(-1)*Yfun**(-1) - 64.D0*mz**8*s*Yfun**(-2) + 16.D0*
     &    ctw**2*s2fun*s4fun**(-1)*rzm2 + 24.D0*ctw**2*u*Yfun**(-1)*
     &    rwm2*rzm2 + 16.D0*ctw**2*u*s2fun*s4fun**(-1) + 24.D0*ctw**2*
     &    u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 
     &    16.D0*ctw**2*s*s2fun*s4fun**(-1) + 48.D0*ctw**2*s*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s*u**3*Yfun**(-2)*rzm2 + 12.D0
     &    *ctw**2*s*u**4*Yfun**(-2) + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2
     &     + 36.D0*ctw**2*s**2*u**2*Yfun**(-2)*rzm2 + 48.D0*ctw**2*s**2
     &    *u**3*Yfun**(-2) + 36.D0*ctw**2*s**3*u*Yfun**(-2)*rzm2 + 72.D0
     &    *ctw**2*s**3*u**2*Yfun**(-2) + 12.D0*ctw**2*s**4*Yfun**(-2)*
     &    rzm2 + 48.D0*ctw**2*s**4*u*Yfun**(-2) + 12.D0*ctw**2*s**5*
     &    Yfun**(-2) + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*Yfun*rwm2 - 24.D0*ctw**2*mz**2*t1fun**(-1)*rwm2
     &     - 48.D0*ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 32.D0*ctw**2*
     &    mz**2*s2fun*s4fun**(-1) - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 72.D0*ctw**2*
     &    mz**2*s*u**2*Yfun**(-2)*rzm2 - 96.D0*ctw**2*mz**2*s*u**3*
     &    Yfun**(-2) - 144.D0*ctw**2*mz**2*s**2*u*Yfun**(-2)*rzm2 - 288.
     &    D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) - 72.D0*ctw**2*mz**2*
     &    s**3*Yfun**(-2)*rzm2 - 288.D0*ctw**2*mz**2*s**3*u*Yfun**(-2)
     &     - 96.D0*ctw**2*mz**2*s**4*Yfun**(-2) + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*mz**4*s*u*
     &    Yfun**(-2)*rzm2 + 288.D0*ctw**2*mz**4*s*u**2*Yfun**(-2) - 24.D
     &    0*ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 144.D0*ctw**2*
     &    mz**4*s**2*Yfun**(-2)*rzm2 + 576.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) + 288.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 48.D0*
     &    ctw**2*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6
     &    *u*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) - 96.D0*ctw**2*mz**6*s*Yfun**(-2)*rzm2 - 384.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) - 384.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 96.D0*ctw**2*mz**8*s4fun**(-1)*Yfun**(-1) + 192.D
     &    0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 + 3.D0/2.D0*ctw**(-2)*
     &    s**2*s2fun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**3*s2fun*s4fun*
     &    Yfun**(-2) + ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1)
     &     - 1.D0/2.D0*ctw**(-2)*mz**4*s**2*s2fun*Yfun**(-2) + 4.D0*s*
     &    s2fun*Yfun**(-1)*rwm2 - 6.D0*s**2*s2fun*Yfun**(-1) + 2.D0*
     &    s**3*s2fun*s4fun*Yfun**(-2) - 4.D0*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*mz**4*s**2*s2fun*Yfun**(-2) - 16.D0*ctw**2*
     &    s*s2fun*s4fun**(-1) - 12.D0*ctw**2*s*s2fun*Yfun**(-1)*rwm2 + 
     &    18.D0*ctw**2*s**2*s2fun*Yfun**(-1) - 6.D0*ctw**2*s**3*s2fun*
     &    s4fun*Yfun**(-2) + 12.D0*ctw**2*mz**4*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 6.D0*ctw**2*mz**4*s**2*s2fun*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 - ctw**(-2)*s*u**2*
     &    Yfun**(-1) - ctw**(-2)*s*u**4*Yfun**(-2) - ctw**(-2)*s**2*u*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*s**2*u**3*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**3*Yfun**(-1) - 3.D0*ctw**(-2)*s**3*u**2*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*s**4*u*Yfun**(-2) - 1.D0/2.D0*
     &    ctw**(-2)*s**5*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*u**3*Yfun**(-2) + 2.D0*
     &    ctw**(-2)*mz**2*s**2*Yfun**(-1) + 12.D0*ctw**(-2)*mz**2*s**2*
     &    u**2*Yfun**(-2) + 12.D0*ctw**(-2)*mz**2*s**3*u*Yfun**(-2) + 4.
     &    D0*ctw**(-2)*mz**2*s**4*Yfun**(-2) - 2.D0*ctw**(-2)*mz**4*s*
     &    Yfun**(-1) - 11.D0*ctw**(-2)*mz**4*s*u**2*Yfun**(-2) - 23.D0*
     &    ctw**(-2)*mz**4*s**2*u*Yfun**(-2) - 23.D0/2.D0*ctw**(-2)*
     &    mz**4*s**3*Yfun**(-2) + 14.D0*ctw**(-2)*mz**6*s*u*Yfun**(-2)
     &     + 14.D0*ctw**(-2)*mz**6*s**2*Yfun**(-2) - 6.D0*ctw**(-2)*
     &    mz**8*s*Yfun**(-2) + 4.D0*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*
     &    u**2*Yfun**(-1) + 4.D0*s*u**4*Yfun**(-2) + 4.D0*s**2*u*
     &    Yfun**(-1) + 8.D0*s**2*u**3*Yfun**(-2) + 2.D0*s**3*Yfun**(-1)
     &     + 12.D0*s**3*u**2*Yfun**(-2) + 8.D0*s**4*u*Yfun**(-2) + 2.D0
     &    *s**5*Yfun**(-2) - 8.D0*mz**2*s*u*Yfun**(-1) - 16.D0*mz**2*s*
     &    u**3*Yfun**(-2) - 8.D0*mz**2*s**2*Yfun**(-1) - 48.D0*mz**2*
     &    s**2*u**2*Yfun**(-2) - 48.D0*mz**2*s**3*u*Yfun**(-2) - 16.D0*
     &    mz**2*s**4*Yfun**(-2) + 8.D0*mz**4*s*Yfun**(-1) + 44.D0*mz**4
     &    *s*u**2*Yfun**(-2) + 92.D0*mz**4*s**2*u*Yfun**(-2) + 46.D0*
     &    mz**4*s**3*Yfun**(-2) - 56.D0*mz**6*s*u*Yfun**(-2) - 56.D0*
     &    mz**6*s**2*Yfun**(-2) + 24.D0*mz**8*s*Yfun**(-2) - 16.D0*
     &    ctw**2*s - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 - 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1) - 12.D0*ctw**2*s*u**4*Yfun**(-2) - 
     &    12.D0*ctw**2*s**2*u*Yfun**(-1) - 24.D0*ctw**2*s**2*u**3*
     &    Yfun**(-2) - 6.D0*ctw**2*s**3*Yfun**(-1) - 36.D0*ctw**2*s**3*
     &    u**2*Yfun**(-2) - 24.D0*ctw**2*s**4*u*Yfun**(-2) - 6.D0*
     &    ctw**2*s**5*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*u*Yfun**(-1) + 
     &    48.D0*ctw**2*mz**2*s*u**3*Yfun**(-2) + 24.D0*ctw**2*mz**2*
     &    s**2*Yfun**(-1) + 144.D0*ctw**2*mz**2*s**2*u**2*Yfun**(-2) + 
     &    144.D0*ctw**2*mz**2*s**3*u*Yfun**(-2) + 48.D0*ctw**2*mz**2*
     &    s**4*Yfun**(-2) - 24.D0*ctw**2*mz**4*s*Yfun**(-1) - 132.D0*
     &    ctw**2*mz**4*s*u**2*Yfun**(-2) - 276.D0*ctw**2*mz**4*s**2*u*
     &    Yfun**(-2) - 138.D0*ctw**2*mz**4*s**3*Yfun**(-2) + 168.D0*
     &    ctw**2*mz**6*s*u*Yfun**(-2) + 168.D0*ctw**2*mz**6*s**2*
     &    Yfun**(-2) - 72.D0*ctw**2*mz**8*s*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*b0fmzww * (  - 
     &    4.D0 + ctw**(-2) + ctw**(-2)*s2fun*s4fun**(-1) + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1) - ctw**(-2)*s*s2fun*Yfun**(-1) + 2.D
     &    0*ctw**(-2)*s*u*Yfun**(-1) + ctw**(-2)*s**2*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s4fun**(-1) - 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) - 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    ctw**(-2)*mz**4*s*u1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**6*u1fun**(-2)*s4fun**(-1) - 4.D0*s2fun*s4fun**(-1) - 8.D0
     &    *u**2*Yfun**(-1) + 4.D0*s*s2fun*Yfun**(-1) - 8.D0*s*u*
     &    Yfun**(-1) - 4.D0*s**2*Yfun**(-1) + 8.D0*mz**2*s4fun**(-1) + 
     &    16.D0*mz**2*u*Yfun**(-1) + 16.D0*mz**2*s*Yfun**(-1) - 16.D0*
     &    mz**4*Yfun**(-1) - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2 + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 24.D0*ctw**2*u**2*Yfun**(-1) - 12.D0*
     &    ctw**2*s*s2fun*Yfun**(-1) + 24.D0*ctw**2*s*u*Yfun**(-1) + 12.D
     &    0*ctw**2*s**2*Yfun**(-1) - 24.D0*ctw**2*mz**2*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**2*u*Yfun**(-1) - 48.D0*ctw**2*mz**2*s*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**4*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) + 
     &    48.D0*ctw**2*mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*b0fusww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 4.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 
     &    4.D0*u**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 8.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) - 4.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 6.D0*
     &    ctw**2 - 12.D0*ctw**2*u**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) - 48.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*b0ftsww * ( 2.D0
     &     - 1.D0/2.D0*ctw**(-2) - ctw**(-2)*u**2*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*s*u*Yfun**(-1) - ctw**(-2)*s**2*Yfun**(-1) + 
     &    ctw**(-2)*mz**2*s4fun**(-1) + 4.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**4*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 4.D0*u**2*Yfun**(-1) + 8.D0*s*u*Yfun**(-1) + 4.D
     &    0*s**2*Yfun**(-1) - 4.D0*mz**2*s4fun**(-1) - 16.D0*mz**2*u*
     &    Yfun**(-1) - 16.D0*mz**2*s*Yfun**(-1) + 16.D0*mz**4*
     &    Yfun**(-1) + 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 4.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) - 6.D0*ctw**2 - 12.D0*ctw**2*
     &    u**2*Yfun**(-1) - 24.D0*ctw**2*s*u*Yfun**(-1) - 12.D0*ctw**2*
     &    s**2*Yfun**(-1) + 12.D0*ctw**2*mz**2*s4fun**(-1) + 48.D0*
     &    ctw**2*mz**2*u*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*Yfun**(-1)
     &     - 48.D0*ctw**2*mz**4*Yfun**(-1) - 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**2*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*b0fqsww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) + ctw**(-2)*s*s2fun*Yfun**(-1) + 
     &    4.D0*s2fun*s4fun**(-1) - 4.D0*s*s2fun*Yfun**(-1) - 12.D0*
     &    ctw**2*s2fun*s4fun**(-1) + 12.D0*ctw**2*s*s2fun*Yfun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun * ( 2.D
     &    0*ctw**(-2)*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 
     &    ctw**(-2)*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 2.D
     &    0*ctw**(-2)*mz**4*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 8.D0
     &    *mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 4.D0*mz**2*
     &    s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 8.D0*mz**4*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 24.D0*ctw**2*mz**2*s*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*mz**2*s**2
     &    *t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - 32.D0*ctw**2*s*u*s4fun**(-1)*rwm2 - 8.D0
     &    *ctw**2*s*u*s2fun*s4fun**(-1) - 16.D0*ctw**2*s**2*s4fun**(-1)
     &    *rwm2 - 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) + 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1)*rwm2 + 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**3*s4fun**(-1)*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 1.D0/2.D0*ctw**(-2)*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*s4fun**(-1)
     &    *rwm2 - 8.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*mz**2*s*u*s4fun**(-1) - 2.D0*mz**2*s**2*
     &    s4fun**(-1) + 4.D0*mz**4*s*s4fun**(-1) + 8.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*ctw**2*s*u*s4fun**(-1)*rwm2 - 8.
     &    D0*ctw**2*s*u*s2fun*s4fun**(-1) + 24.D0*ctw**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*ctw**2*s**2*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**2*s**2*s2fun*s4fun**(-1) + 8.D0*ctw**2*s**2*
     &    u*s4fun**(-1) + 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 32.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 8.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1) - 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 6.D0*ctw**2*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)
     &    *s**2*s4fun**(-1)*rwm2 + 6.D0*ctw**(-2)*s**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 1.D0/2.D0*ctw**(-2)*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 6.D0*ctw**(-2)*s**3*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 3.D0/2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 5.D0/2.D0*ctw**(-2)*s**3*u**4*s4fun**(-1)*Yfun**(-2) + 2.D0
     &    *ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 3.D0/2.D0*
     &    ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-1) + 5.D0*ctw**(-2)*s**4
     &    *u**3*s4fun**(-1)*Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**5*
     &    s4fun**(-1)*Yfun**(-1) + 5.D0*ctw**(-2)*s**5*u**2*s4fun**(-1)
     &    *Yfun**(-2) + 5.D0/2.D0*ctw**(-2)*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) + 1.D0/2.D0*ctw**(-2)*s**7*s4fun**(-1)*Yfun**(-2)
     &     - 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 - 12.D0*ctw**(-2)*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*ctw**(-2)*
     &    mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 5.D0*ctw**(-2)*mz**2
     &    *s**2*u**4*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**2*
     &    s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) - 20.D0*ctw**(-2)*mz**2*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 3.D0*ctw**(-2)*mz**2*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 30.D0*ctw**(-2)*mz**2*s**4*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 20.D0*ctw**(-2)*mz**2*s**5*u*
     &    s4fun**(-1)*Yfun**(-2) - 5.D0*ctw**(-2)*mz**2*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 22.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 22.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*ctw**(-2)*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 39.D0/2.D0*ctw**(-2)*mz**4*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**(-2)*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 117.D0/2.D0*ctw**(-2)*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 117.D0/2.D0*ctw**(-2)*mz**4*
     &    s**4*u*s4fun**(-1)*Yfun**(-2) + 39.D0/2.D0*ctw**(-2)*mz**4*
     &    s**5*s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 37.D0*ctw**(-2)*mz**6*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 74.D0*ctw**(-2)*mz**6*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) - 37.D0*ctw**(-2)*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 34.D0*ctw**(-2)*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 34.D0*ctw**(-2)*mz**8*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 12.D0*ctw**(-2)*mz**10*s**2*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*s*u*s4fun**(-1)*rwm2 - 8.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*s**2*s4fun**(-1)*rwm2
     &     - 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*s**2*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*
     &    s**3*u**2*s4fun**(-1)*Yfun**(-1) - 10.D0*s**3*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 8.D0*s**4*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 6.D0*s**4*u*s4fun**(-1)*Yfun**(-1) - 20.D0*s**4*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*s**5*s4fun**(-1)*Yfun**(-1) - 
     &    20.D0*s**5*u**2*s4fun**(-1)*Yfun**(-2) - 10.D0*s**6*u*
     &    s4fun**(-1)*Yfun**(-2) - 2.D0*s**7*s4fun**(-1)*Yfun**(-2) + 
     &    16.D0*mz**2*s*s4fun**(-1)*rwm2 - 4.D0*mz**2*s*u*s4fun**(-1)
     &     + 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 2.D0*
     &    mz**2*s**2*s4fun**(-1) + 96.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 20.D0*mz**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 
     &    48.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*mz**2*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) + 80.D0*mz**2*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 120.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    80.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 20.D0*mz**2*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*mz**4*s*s4fun**(-1) - 88.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 - 88.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 24.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 78.D0*mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 
     &    24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 234.D0*mz**4*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 234.D0*mz**4*s**4*u*s4fun**(-1)
     &    *Yfun**(-2) - 78.D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) + 48.D0
     &    *mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 148.D0*mz**6*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 296.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) + 148.
     &    D0*mz**6*s**4*s4fun**(-1)*Yfun**(-2) - 136.D0*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 136.D0*mz**8*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 48.D0*mz**10*s**2*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    ctw**2*s*u*s4fun**(-1)*rwm2 - 8.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1) + 24.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 8.D0*ctw**2*s**2*s4fun**(-1)*rwm2 - 4.D0*ctw**2*s**2*s2fun
     &    *s4fun**(-1) + 8.D0*ctw**2*s**2*u*s4fun**(-1) + 72.D0*ctw**2*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 6.D0*ctw**2*s**2*u**3
     &    *s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**2*s**2*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*ctw**2*s**3*s4fun**(-1) + 72.D0*ctw**2*s**3
     &    *u*s4fun**(-1)*Yfun**(-1)*rwm2 + 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 30.D0*ctw**2*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    18.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-1) + 60.D0*ctw**2*
     &    s**4*u**3*s4fun**(-1)*Yfun**(-2) + 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-1) + 60.D0*ctw**2*s**5*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 30.D0*ctw**2*s**6*u*s4fun**(-1)*Yfun**(-2) + 6.D0
     &    *ctw**2*s**7*s4fun**(-1)*Yfun**(-2) - 16.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*rwm2 + 8.D0*ctw**2*mz**2*s*s2fun*s4fun**(-1) - 
     &    144.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0
     &    *ctw**2*mz**2*s**2*s4fun**(-1) - 288.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 36.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 60.D0*ctw**2*mz**2*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 144.D0*ctw**2*mz**2*s**3*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*
     &    Yfun**(-1) - 240.D0*ctw**2*mz**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*Yfun**(-1)
     &     - 360.D0*ctw**2*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 240.
     &    D0*ctw**2*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 60.D0*ctw**2*
     &    mz**2*s**6*s4fun**(-1)*Yfun**(-2) + 264.D0*ctw**2*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 264.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 72.D0*ctw**2*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 234.D0*ctw**2*mz**4*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 72.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 702.D0*ctw**2*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 702.D0*ctw**2*mz**4*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) + 234.D0*ctw**2*mz**4*s**5*s4fun**(-1)*Yfun**(-2)
     &     - 144.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1) - 444.D0*ctw**2*
     &    mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 888.D0*ctw**2*mz**6*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) - 444.D0*ctw**2*mz**6*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 408.D0*ctw**2*mz**8*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 408.D0*ctw**2*mz**8*s**3*s4fun**(-1)
     &    *Yfun**(-2) - 144.D0*ctw**2*mz**10*s**2*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2
     &     - 2.D0*ctw**(-2)*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s*
     &    u**4*s4fun**(-1)*Yfun**(-2)*rzm2 + ctw**(-2)*s*u**5*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*u1fun**(-1)*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - ctw**(-2)*mz**4*s*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 8.D0*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 
     &    4.D0*s*u**3*s4fun**(-1)*Yfun**(-1) + 4.D0*s*u**4*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1)*rwm2 - 8.D0*mz**4*s*u*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 4.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*ctw**2*s*s4fun**(-1)*rzm2 + 16.D0*ctw**2*s
     &    *u*s4fun**(-1) - 12.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 24.D0*ctw**2*s*u**3*u1fun**(-1)*s4fun**(-1)*Yfun**(-1)
     &    *rwm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1) - 12.D0*
     &    ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 12.D0*ctw**2*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s*u*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*mz**4*
     &    s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 12.D0*ctw**2*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 2.D
     &    0*ctw**(-2)*s*u**3*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) + ctw**(-2)*s*u**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + ctw**(-2)*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*ctw**(-2)*s**2*u*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 6.D0*ctw**(-2)*s**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 3.D0*ctw**(-2)*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 5.D0*ctw**(-2)*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 
     &    ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 - 6.D0*ctw**(-2)*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 3.D0*
     &    ctw**(-2)*s**3*u*s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**(-2)*s**3
     &    *u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 10.D0*ctw**(-2)*s**3*u**3
     &    *s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*s**4*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + ctw**(-2)*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*ctw**(-2)*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 10.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    ctw**(-2)*s**5*s4fun**(-1)*Yfun**(-2)*rzm2 + 5.D0*ctw**(-2)*
     &    s**5*u*s4fun**(-1)*Yfun**(-2) + ctw**(-2)*s**6*s4fun**(-1)*
     &    Yfun**(-2) - 2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     + 12.D0*ctw**(-2)*mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*ctw**(-2)*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 24.D0*ctw**(-2)*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**(-2)*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 40.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 12.D0*ctw**(-2)*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**(-2)*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 60.D0*ctw**(-2)*mz**2*s**3*u**2
     &    *s4fun**(-1)*Yfun**(-2) - 8.D0*ctw**(-2)*mz**2*s**4*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*ctw**(-2)*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 10.D0*ctw**(-2)*mz**2*s**5*
     &    s4fun**(-1)*Yfun**(-2) + 4.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 22.D0*ctw**(-2)*mz**4*s*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**(-2)*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 23.D0*ctw**(-2)*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 39.D0*ctw**(-2)*mz**4*s*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 22.D0*ctw**(-2)*mz**4*s**2*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**(-2)*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 46.D0*ctw**(-2)*mz**4*
     &    s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 117.D0*ctw**(-2)*mz**4*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-2) + 23.D0*ctw**(-2)*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 + 117.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) + 39.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 28.D0*ctw**(-2)*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 74.D0*ctw**(-2)*mz**6*s*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 28.D0*ctw**(-2)*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 148.D0*ctw**(-2)*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 74.D0*ctw**(-2)*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**(-2)*mz**8*s*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 68.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*
     &    Yfun**(-2) + 68.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2)
     &     - 4.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 4.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 24.D0*s**2*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 - 20.
     &    D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 24.D0*s**3*u*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 12.D0*s**3*u*s4fun**(-1)*Yfun**(-1) - 24.D0
     &    *s**3*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**3*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 4.D0*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**4*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 4.D0*s**5*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 20.D0*s**5*u*s4fun**(-1)*Yfun**(-2) - 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)
     &    *rwm2 + 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    mz**2*s*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 
     &    96.D0*mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    48.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 96.D0*mz**2*s**2*
     &    u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 160.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 48.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 24.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     + 240.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 32.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 160.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 40.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 88.D0
     &    *mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 - 48.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 92.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 156.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 88.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 
     &    184.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 468.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 92.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 468.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) - 156.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 48.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) + 112.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 296.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     + 112.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 592.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 48.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 - 272.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 272.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*ctw**2*s*s4fun**(-1)*rzm2 + 16.
     &    D0*ctw**2*s*u*s4fun**(-1) + 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 24.D0*ctw**2*s*u**3*t1fun**(-1)*s4fun**(-1)
     &    *Yfun**(-1)*rwm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 12.D0*ctw**2*s*u**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 12.D0*
     &    ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) + 16.D0*ctw**2*s**2*
     &    s4fun**(-1) + 24.D0*ctw**2*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2
     &     - 72.D0*ctw**2*s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 + 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) + 48.D0*
     &    ctw**2*s**2*u**3*s4fun**(-1)*Yfun**(-2)*rzm2 + 60.D0*ctw**2*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 72.D0*ctw**2*s**3*u*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 36.D0*ctw**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) + 72.D0*ctw**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 120.D0*ctw**2*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*ctw**2*s**4*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**4*s4fun**(-1)*Yfun**(-1) + 
     &    48.D0*ctw**2*s**4*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 120.D0*
     &    ctw**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 12.D0*ctw**2*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 60.D0*ctw**2*s**5*u*s4fun**(-1)
     &    *Yfun**(-2) + 12.D0*ctw**2*s**6*s4fun**(-1)*Yfun**(-2) - 24.D0
     &    *ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*rwm2 - 32.D0*ctw**2*
     &    mz**2*s*s4fun**(-1) - 48.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 144.D0*ctw**2*mz**2*s*u**2*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*ctw**2*mz**2*s*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 120.D0*ctw**2*mz**2*s*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 48.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 288.D0*ctw**2*mz**2*s**2*u*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 144.D0*ctw**2*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 288.D0*ctw**2*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 480.D0*ctw**2*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*mz**2*s**3*t1fun**(-1)
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 - 72.D0*ctw**2*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 288.D0*ctw**2*mz**2*s**3*u*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 720.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*ctw**2*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 480.D0*ctw**2*mz**2*s**4*u*s4fun**(-1)*
     &    Yfun**(-2) - 120.D0*ctw**2*mz**2*s**5*s4fun**(-1)*Yfun**(-2)
     &     + 48.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 264.D0*
     &    ctw**2*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    144.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 276.D0*
     &    ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 468.D0*
     &    ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) - 264.D0*ctw**2*
     &    mz**4*s**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*rwm2 + 144.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 552.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 1404.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 276.D0*ctw**2*mz**4*
     &    s**3*s4fun**(-1)*Yfun**(-2)*rzm2 + 1404.D0*ctw**2*mz**4*s**3*
     &    u*s4fun**(-1)*Yfun**(-2) + 468.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*mz**6*s*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 336.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 888.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 336.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 1776.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 
     &    888.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2) + 144.D0*
     &    ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-2)*rzm2 + 816.D0*ctw**2*
     &    mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 816.D0*ctw**2*mz**8*s**2*
     &    s4fun**(-1)*Yfun**(-2) - 288.D0*ctw**2*mz**10*s*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &    *rwm2 - ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**2*u*s2fun*s4fun**(-1)*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**3*s2fun*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s**3*
     &    u*s2fun*Yfun**(-2) - 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*mz**2*s**3*s2fun*Yfun**(-2) - 
     &    ctw**(-2)*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2) - 1.D0/2.D
     &    0*ctw**(-2)*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 
     &    ctw**(-2)*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 2.D0*s**3*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*
     &    s**3*u*s2fun*Yfun**(-2) + 2.D0*s**4*s2fun*Yfun**(-2) - 8.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**2*s**3*s2fun*
     &    Yfun**(-2) + 4.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*Yfun**(-2)
     &     + 2.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 4.D0*mz**6*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-2) - 24.D0*ctw**2*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**2*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*ctw**2*s**3*u*s2fun*Yfun**(-2) - 6.D0*
     &    ctw**2*s**4*s2fun*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**2*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**2*s**3*s2fun*
     &    Yfun**(-2) - 12.D0*ctw**2*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 6.D0*ctw**2*mz**4*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 12.D0*ctw**2*mz**6*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * (  - 2.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 - 2.D0*
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*s*u**3*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*s*
     &    u**5*s4fun**(-1)*Yfun**(-2) - ctw**(-2)*s**2*Yfun**(-1)*rwm2
     &     - ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) - 3.D0*
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 5.D0*ctw**(-2)*
     &    s**2*u**4*s4fun**(-1)*Yfun**(-2) - 2.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*ctw**(-2)*s**3*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 1.D0/2.D0*ctw**(-2)*s**4*s4fun**(-1)*Yfun**(-1)
     &     - 7.D0*ctw**(-2)*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 3.D0*
     &    ctw**(-2)*s**5*u*s4fun**(-1)*Yfun**(-2) - 1.D0/2.D0*ctw**(-2)
     &    *s**6*s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*mz**2*s*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)
     &    *Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 10.D0*ctw**(-2)*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*ctw**(-2)*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 3.D0*ctw**(-2)*mz**2*s**3*s4fun**(-1)*Yfun**(-1)
     &     + 42.D0*ctw**(-2)*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    24.D0*ctw**(-2)*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) + 5.D0*
     &    ctw**(-2)*mz**2*s**5*s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**(-2)*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 30.D0*ctw**(-2)*mz**4*s*
     &    u**3*s4fun**(-1)*Yfun**(-2) - 5.D0*ctw**(-2)*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 81.D0*ctw**(-2)*mz**4*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-2) - 70.D0*ctw**(-2)*mz**4*s**3*u*
     &    s4fun**(-1)*Yfun**(-2) - 39.D0/2.D0*ctw**(-2)*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 2.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 50.D0*ctw**(-2)*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 88.D0*ctw**(-2)*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) + 37.D0*ctw**(-2)*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 40.D0*ctw**(-2)*mz**8*s*u*s4fun**(-1)*Yfun**(-2)
     &     - 34.D0*ctw**(-2)*mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 12.D0*
     &    ctw**(-2)*mz**10*s*s4fun**(-1)*Yfun**(-2) + 8.D0*s*u*
     &    Yfun**(-1)*rwm2 + 8.D0*s*u*s2fun**2*s4fun**(-1)*Yfun**(-1) + 
     &    8.D0*s*u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**5*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**2*Yfun**(-1)*rwm2 + 4.D0*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 20.D0*s**2*u**4*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    s**3*u*s4fun**(-1)*Yfun**(-1) + 32.D0*s**3*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*s**4*s4fun**(-1)*Yfun**(-1) + 28.D0*s**4*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 12.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) + 2.D0*s**6*s4fun**(-1)*Yfun**(-2) - 8.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 - 8.D0*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 40.D0
     &    *mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) - 32.D0*mz**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 128.D0*mz**2*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 12.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 168.D0
     &    *mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 20.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 120.D0*
     &    mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) + 20.D0*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 324.D0*mz**4*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 280.D0*mz**4*s**3*u*s4fun**(-1)*Yfun**(-2) + 78.D
     &    0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 8.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) - 200.D0*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 352.D0*mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 148.
     &    D0*mz**6*s**3*s4fun**(-1)*Yfun**(-2) + 160.D0*mz**8*s*u*
     &    s4fun**(-1)*Yfun**(-2) + 136.D0*mz**8*s**2*s4fun**(-1)*
     &    Yfun**(-2) - 48.D0*mz**10*s*s4fun**(-1)*Yfun**(-2) - 24.D0*
     &    ctw**2*s*u*Yfun**(-1)*rwm2 - 24.D0*ctw**2*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*s*u**5*s4fun**(-1)*Yfun**(-2) - 12.D
     &    0*ctw**2*s**2*Yfun**(-1)*rwm2 - 12.D0*ctw**2*s**2*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) - 36.D0*ctw**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 60.D0*ctw**2*s**2*u**4*s4fun**(-1)*Yfun**(-2) - 
     &    24.D0*ctw**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 96.D0*ctw**2*
     &    s**3*u**3*s4fun**(-1)*Yfun**(-2) - 6.D0*ctw**2*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 84.D0*ctw**2*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 36.D0*ctw**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 6.D0
     &    *ctw**2*s**6*s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*mz**2*s*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 120.D0*ctw**2*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2) + 96.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 384.D0*ctw**2*
     &    mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 36.D0*ctw**2*mz**2*
     &    s**3*s4fun**(-1)*Yfun**(-1) + 504.D0*ctw**2*mz**2*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-2) + 288.D0*ctw**2*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 60.D0*ctw**2*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 72.D0*ctw**2*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 
     &    360.D0*ctw**2*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) - 60.D0*
     &    ctw**2*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 972.D0*ctw**2*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 840.D0*ctw**2*mz**4*
     &    s**3*u*s4fun**(-1)*Yfun**(-2) - 234.D0*ctw**2*mz**4*s**4*
     &    s4fun**(-1)*Yfun**(-2) + 24.D0*ctw**2*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 600.D0*ctw**2*mz**6*s*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 1056.D0*ctw**2*mz**6*s**2*u*s4fun**(-1)*
     &    Yfun**(-2) + 444.D0*ctw**2*mz**6*s**3*s4fun**(-1)*Yfun**(-2)
     &     - 480.D0*ctw**2*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 408.D0*
     &    ctw**2*mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 144.D0*ctw**2*
     &    mz**10*s*s4fun**(-1)*Yfun**(-2) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & b0fmzww * ( 4.D0*ctw**(-2)*u*s4fun**(-1) + 4.D0*ctw**(-2)*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*s*s4fun**(-1) - 2.D0*
     &    ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 6.D0*ctw**(-2)*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*s**2*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1) - 12.D0*ctw**(-2)*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*ctw**(-2)*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*s4fun**(-1)
     &     + 16.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1) + ctw**(-2)
     &    *mz**4*s*t1fun**(-2)*s4fun**(-1) + ctw**(-2)*mz**4*s*
     &    u1fun**(-2)*s4fun**(-1) + 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) - 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) - 
     &    2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) - 8.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1) - 16.D0*u*s4fun**(-1) - 16.D0*
     &    u**3*s4fun**(-1)*Yfun**(-1) - 8.D0*s*s4fun**(-1) + 8.D0*s*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 24.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 48.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 64.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 4.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 4.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**6*u1fun**(-2)*s4fun**(-1) + 32.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*u*s4fun**(-1) + 
     &    48.D0*ctw**2*u**3*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*s*
     &    s4fun**(-1) - 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1)
     &     + 72.D0*ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**2*s4fun**(-1) - 144.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**2*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) + 
     &    24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) + 192.D0*ctw**2*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 12.D0*ctw**2*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 144.D0*ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1) - 
     &    24.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) - 96.D0*ctw**2*mz**6*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & b0fusww * (  - 2.D0 + 1.D0/2.D0*ctw**(-2) - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) - 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) - ctw**(-2)*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**6*u1fun**(-2)*s4fun**(-1) + 
     &    8.D0*u*s4fun**(-1) + 8.D0*u**3*s4fun**(-1)*Yfun**(-1) + 4.D0*
     &    s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 4.D0*
     &    mz**4*s*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**6*u1fun**(-2)*
     &    s4fun**(-1) + 6.D0*ctw**2 - 24.D0*ctw**2*u*s4fun**(-1) - 24.D0
     &    *ctw**2*u**3*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*u*u1fun**(-2)*s4fun**(-1) - 
     &    12.D0*ctw**2*mz**4*s*u1fun**(-2)*s4fun**(-1) + 24.D0*ctw**2*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & b0ftsww * ( 2.D0 - 1.D0/2.D0*ctw**(-2) - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1) - 2.D0*ctw**(-2)*u**3*s4fun**(-1)*Yfun**(-1) - 2.D
     &    0*ctw**(-2)*s*s4fun**(-1) - 5.D0*ctw**(-2)*s*u**2*s4fun**(-1)
     &    *Yfun**(-1) - 4.D0*ctw**(-2)*s**2*u*s4fun**(-1)*Yfun**(-1) - 
     &    ctw**(-2)*s**3*s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) + 10.D0*ctw**(-2)*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*ctw**(-2)*mz**2*s*u*s4fun**(-1)*Yfun**(-1)
     &     + 6.D0*ctw**(-2)*mz**2*s**2*s4fun**(-1)*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**4*u*t1fun**(-2)*s4fun**(-1) - 16.D0*ctw**(-2)*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) - 12.D0*ctw**(-2)*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1) + 2.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 
     &    8.D0*ctw**(-2)*mz**6*s4fun**(-1)*Yfun**(-1) + 8.D0*u*
     &    s4fun**(-1) + 8.D0*u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*s*
     &    s4fun**(-1) + 20.D0*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s**2*u*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s4fun**(-1) - 40.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1) + 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*mz**4*s*t1fun**(-2)*s4fun**(-1) + 48.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1) - 6.D0*
     &    ctw**2 - 24.D0*ctw**2*u*s4fun**(-1) - 24.D0*ctw**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s*s4fun**(-1) - 60.D0*
     &    ctw**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*ctw**2*mz**2*s4fun**(-1) + 120.D0*ctw**2*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 192.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 72.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*mz**4*u*t1fun**(-2)*s4fun**(-1) - 
     &    192.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) - 144.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 96.D0*ctw**2*mz**6*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,1) = ewamp(2,1,2,1) + g**4*stw**2*betaZfun*
     & b0fqsww * ( 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 
     &    ctw**(-2)*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 2.D0*ctw**(-2)*
     &    mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *ctw**2*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      ewamp(2,1,2,3) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - deltafun*Yfun - 8.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 16.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - 1.D0/2.D0*ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2
     &     - ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**2*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*u**2*s4fun**(-1)*deltafun - 2.D0*mz**2*
     &    s*u*s4fun**(-1)*deltafun + 8.D0*mz**4*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**4*u*s4fun**(-1)*deltafun + 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 4.D0*mz**4*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 4.D0*mz**6*s4fun**(-1)*deltafun - 16.D0*ctw**2*
     &    u*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*u*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**2*s*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0
     &    *ctw**2*mz**2*s4fun**(-1)*deltafun*Yfun - 16.D0*ctw**2*mz**2*
     &    s*u*s4fun**(-1)*deltafun - 24.D0*ctw**2*mz**4*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.D0
     &    *ctw**(-2)*s*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0
     &    /2.D0*ctw**(-2)*s**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 3.D0/2.D0*ctw**(-2)*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 1.D0/2.D0*ctw**(-2)*s**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*mz**2*s*u**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*
     &    u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0*ctw**(-2)*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 6.D0*ctw**(-2)*
     &    mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + ctw**(-2)*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)
     &    *mz**4*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0
     &    *ctw**(-2)*mz**6*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*u*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**3*s4fun**(-1)*
     &    deltafun + 4.D0*s*s2fun*s4fun**(-1)*deltafun*rwm2 + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun + 2.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 5.D0*s**2*u*s4fun**(-1)*deltafun + 6.D0
     &    *s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + s**3*
     &    s4fun**(-1)*deltafun + 6.D0*s**3*u*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 2.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s2fun*s4fun**(-1)*deltafun*rwm2 - 20.D0*mz**2*
     &    u**2*s4fun**(-1)*deltafun - 26.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 36.D0*mz**4*u*s4fun**(-1)*
     &    deltafun + 22.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 20.D0*mz**6*s4fun**(-1)*deltafun - 16.D
     &    0*mz**6*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6
     &    *s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*ctw**2*u*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun - 6.D0*ctw**2*s*u**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 18.D0
     &    *ctw**2*s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D
     &    0*ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**4*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s2fun*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun + 36.D0*ctw**2*mz**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 80.D0*ctw**2*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 36.D0*ctw**2*mz**2*s**3*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*
     &    deltafun - 72.D0*ctw**2*mz**4*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*ctw**2*mz**4*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 72.D0*ctw**2*mz**4*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 4.D
     &    0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 2.D0
     &    *ctw**(-2)*mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*
     &    rzm2 + 8.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 + 16.D0
     &    *s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*s**(-1)*u**2*
     &    deltafun - 8.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D
     &    0*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun - 4.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*mz**2*s**(-1)*deltafun*
     &    rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *mz**2*s**(-1)*u*deltafun - 16.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*mz**4*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0
     &    *ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*
     &    ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 24.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 - 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 12.D0
     &    *ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D
     &    0*ctw**2*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**2*s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 
     &    32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*ctw**2*
     &    mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**4*s**(-1)*
     &    u**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 - ctw**(-2)*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2
     &    *rzm2 + 3.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 2.D0*ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2
     &     - 2.D0*ctw**(-2)*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*ctw**(-2)*t**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 + 6.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s*u**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - ctw**(-2)*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*
     &    mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*ctw**(-2)*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**(-2)*mz**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 12.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*ctw**(-2)*mz**4
     &    *u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*
     &    ctw**(-2)*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    ctw**(-2)*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**8*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*
     &    s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 4.D0*s**(-1)*u*deltafun*rzm2 + 8.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*deltafun*rwm2 - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*s**(-1)*u**2*deltafun
     &     + 8.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 8.D0*t**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2*rzm2 + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)*deltafun*
     &    rwm2*rzm2 + 2.D0*deltafun*rzm2 + 8.D0*s2fun*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 6.D0*u*
     &    deltafun + 4.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 4.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun + 8.D0*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1)*rzm2 + 12.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 16.D0*mz**2*s**(-1)
     &    *s2fun*s4fun**(-1)*deltafun*rwm2 + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*s**(-1)*u*deltafun + 
     &    32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*deltafun
     &     - 16.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    16.D0*mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.
     &    D0*mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*
     &    mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*
     &    mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s**(-1)*
     &    deltafun + 16.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.
     &    D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 32.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*t**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s**(-1)*t**(-1)
     &    *u**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun
     &    *s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 24.D0*ctw**2*t**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*
     &    rzm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*deltafun*rwm2 + 32.D0*
     &    ctw**2*u*s4fun**(-1)*deltafun*rzm2 + 72.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 12.D0*ctw**2*s*t**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 32.D0*ctw**2*s*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*
     &    u*s4fun**(-1)*deltafun - 24.D0*ctw**2*s*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 36.D0*ctw**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rwm2 - 144.D0*ctw**2*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)
     &    *deltafun*rzm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*deltafun*rwm2
     &     - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*
     &    ctw**2*mz**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 160.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*mz**2*s
     &    *s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*ctw**2*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 144.
     &    D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*ctw**2*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*u*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*mz**6*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*ctw**2*mz**6*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**8*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*c0ccqswww * ( 
     &    ctw**(-2)*s*u*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**2*s*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 2.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*u*
     &    deltafun - 2.D0*s*deltafun - 4.D0*s*u*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**2*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 4.D0*mz**2*deltafun + 4.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*ctw**2*s*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*ctw**2*s**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*ctw**2*mz**2*s*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*mz**2*s*s2fun**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 24.D0*ctw**2*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*ctw**2*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*ctw**2*mz**6*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*c0mzmzqswww
     &  * ( ctw**(-2)*s*u*s2fun*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) - ctw**(-2)*mz**2*s*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s*u*s2fun*deltafun*
     &    Yfun**(-1) - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*mz**2
     &    *s*s2fun*deltafun*Yfun**(-1) - 32.D0*ctw**2*u*deltafun - 16.D0
     &    *ctw**2*s*deltafun + 12.D0*ctw**2*s*u*s2fun*deltafun*
     &    Yfun**(-1) + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) + 32.D
     &    0*ctw**2*mz**2*deltafun - 12.D0*ctw**2*mz**2*s*s2fun*deltafun
     &    *Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun + ctw**(-2)*s*s4fun**(-1)*deltafun + 2.D
     &    0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - ctw**(-2)*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 5.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**(-2)*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u*s4fun**(-1)*deltafun - 4.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*u1fun**(-1)
     &    *s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*deltafun - 8.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*s4fun**(-1)*deltafun + 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 20.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun + 24.D
     &    0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*ctw**2*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)
     &    *deltafun + 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*mz**2*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun - 60.D0*ctw**2*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2
     &    *u*u1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun
     &     + 12.D0*ctw**2*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 12.D0
     &    *ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*b0ftsww * (  - 
     &    ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 2.D0*
     &    ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - ctw**(-2)*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 5.D0*ctw**(-2)*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun + 5.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 6.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 4.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun - 4.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 20.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 20.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 24.D0*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*deltafun - 12.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*deltafun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 60.D0*ctw**2*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun + 60.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1)*deltafun - 72.D0*ctw**2*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*b0fqsww * (  - 
     &    2.D0*ctw**(-2)*u*s4fun**(-1)*deltafun - ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun + 8.D0*u*s4fun**(-1)*deltafun + 4.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    24.D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun * ( 
     &     - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 4.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( deltafun*Yfun + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 + 1.D0/2.D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2
     &    *s4fun**(-1)*deltafun - s**2*u*s4fun**(-1)*deltafun - 2.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*mz**4*s*
     &    s4fun**(-1)*deltafun + 64.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &    *rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*s*u**2*
     &    s4fun**(-1)*deltafun + 6.D0*ctw**2*s**2*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*ctw**2*mz**2*s*u*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*ctw**(-2)*s*u*s4fun**(-1)*deltafun*
     &    rwm2 - 2.D0*ctw**(-2)*s**2*s4fun**(-1)*deltafun*rwm2 - 1.D0/2.
     &    D0*ctw**(-2)*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D0/
     &    2.D0*ctw**(-2)*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 3.D
     &    0/2.D0*ctw**(-2)*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 1.D0
     &    /2.D0*ctw**(-2)*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0*ctw**(-2)*
     &    mz**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**(-2)*mz**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 3.D0
     &    *ctw**(-2)*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*
     &    ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0
     &    *ctw**(-2)*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*
     &    ctw**(-2)*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*s**2*s4fun**(-1)*deltafun*rwm2 - 3.D0*s**2*u*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - s**3*s4fun**(-1)*deltafun + 6.D0*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 2.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    mz**2*s*u*s4fun**(-1)*deltafun + 6.D0*mz**2*s**2*s4fun**(-1)*
     &    deltafun - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*mz**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 10.D0*mz**4*s*s4fun**(-1)*deltafun + 24.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*ctw**2*s*s4fun**(-1)*deltafun*Yfun
     &     - 24.D0*ctw**2*s*u*s4fun**(-1)*deltafun*rwm2 + 16.D0*ctw**2*
     &    s*u**2*s4fun**(-1)*deltafun - 24.D0*ctw**2*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*u*s4fun**(-1)*deltafun - 6.D
     &    0*ctw**2*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun - 18.D0*ctw**2*s**3*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 18.D0*ctw**2*s**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 6.D0*ctw**2*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*mz**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 80.D0*ctw**2*mz**2*s*u*
     &    s4fun**(-1)*deltafun - 80.D0*ctw**2*mz**2*s**2*s4fun**(-1)*
     &    deltafun + 36.D0*ctw**2*mz**2*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 72.D0*ctw**2*mz**2*s**3*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 36.D0*ctw**2*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*ctw**2*mz**4*s*s4fun**(-1)*deltafun - 72.D0
     &    *ctw**2*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 72.D0*
     &    ctw**2*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*
     &    ctw**2*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*s*u*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    ctw**(-2)*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    ctw**(-2)*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 - 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*u*deltafun - 8.D0*s*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 32.D0*ctw**2*u*s4fun**(-1)*deltafun*rzm2
     &     + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 32.D0*ctw**2*
     &    u**2*s4fun**(-1)*deltafun + 24.D0*ctw**2*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**2*s*u**3*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*ctw**2*mz**2*s4fun**(-1)*deltafun
     &    *rzm2 + 32.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 4.D0*ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 4.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - ctw**(-2)*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*u
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**2*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 3.D0*ctw**(-2)*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - ctw**(-2)*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 6.D0*ctw**(-2)*mz**2*s
     &    *u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*ctw**(-2)*mz**2*
     &    s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 12.D0*ctw**(-2)*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*ctw**(-2)
     &    *mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*ctw**(-2)*
     &    mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 12.D0*
     &    ctw**(-2)*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*
     &    ctw**(-2)*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*deltafun*rzm2 + 16.D0*u
     &    *s4fun**(-1)*deltafun*rwm2 - 2.D0*u*deltafun + 16.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun + 8.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 4.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 12.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 12.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*deltafun - 16.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**4*s*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    48.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 32.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*u**2*s4fun**(-1)*deltafun + 32.D0
     &    *ctw**2*s*s4fun**(-1)*deltafun*rzm2 - 48.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 64.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun - 12.D0*ctw**2*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 12.D0*ctw**2*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 32.D0*ctw**2*s**2*s4fun**(-1)*deltafun - 24.D0
     &    *ctw**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*
     &    ctw**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 36.D0*
     &    ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*ctw**2*
     &    s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 160.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun + 
     &    48.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 
     &    160.D0*ctw**2*mz**2*s*s4fun**(-1)*deltafun + 48.D0*ctw**2*
     &    mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 72.D0*ctw**2
     &    *mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*ctw**2*
     &    mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 144.D0*
     &    ctw**2*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 72.D0*
     &    ctw**2*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 192.D0*
     &    ctw**2*mz**4*s4fun**(-1)*deltafun - 48.D0*ctw**2*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 144.D0*ctw**2*mz**4*s*
     &    u*s4fun**(-1)*deltafun*Yfun**(-1) - 144.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*ctw**2*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    ctw**(-2)*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    ctw**(-2)*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0*
     &    ctw**(-2)*mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 2.D0
     &    *ctw**(-2)*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 2.D0*
     &    ctw**(-2)*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s
     &    *s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun - 4.D0*s**2*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 2.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 16.D0*ctw**2*s*deltafun
     &     + 12.D0*ctw**2*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*ctw**2*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 6.D0*
     &    ctw**2*s**4*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*ctw**2*
     &    mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & c0mzmzqswww * (  - ctw**(-2)*s*s2fun*s4fun**(-1)*deltafun + 1.D0/
     &    2.D0*ctw**(-2)*s**2*s2fun*deltafun*Yfun**(-1) + 4.D0*s*s2fun*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*s2fun*deltafun*Yfun**(-1) - 
     &    16.D0*ctw**2*s*deltafun - 12.D0*ctw**2*s*s2fun*s4fun**(-1)*
     &    deltafun + 6.D0*ctw**2*s**2*s2fun*deltafun*Yfun**(-1) )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*deltafun + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & b0fusww * (  - ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*u*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*u*s4fun**(-1)*deltafun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & b0ftsww * ( ctw**(-2)*u*s4fun**(-1)*deltafun + ctw**(-2)*s*
     &    s4fun**(-1)*deltafun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun - 4.D0*u*s4fun**(-1)*deltafun - 4.D0*s*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun + 12.D0*ctw**2*u*
     &    s4fun**(-1)*deltafun + 12.D0*ctw**2*s*s4fun**(-1)*deltafun + 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*mz**2*s4fun**(-1)*deltafun )
      ewamp(2,1,2,3) = ewamp(2,1,2,3) + g**4*stw**2*betaZfun*
     & b0fqsww * (  - ctw**(-2)*s*s4fun**(-1)*deltafun + 4.D0*s*
     &    s4fun**(-1)*deltafun - 12.D0*ctw**2*s*s4fun**(-1)*deltafun )

      ewamp(2,1,2,2) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 8.D0*mz**2*rwm2 + 12.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2 )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 
     &    ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    s4fun**(-1)*Yfun*rwm2 + 24.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 - 32.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s*u**2*
     &    Yfun**(-1) - 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 12.D0
     &    *ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     + 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 + ctw**(-2)*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + ctw**(-2)*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 4.D
     &    0*ctw**(-2)*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 4.D0*ctw**(-2)*mz**8*s*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s4fun**(-1)*Yfun*rwm2 + 24.D0*
     &    rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 + 
     &    48.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 
     &    16.D0*s**4*s4fun**(-1)*Yfun**(-1)*rwm2 + 4.D0*mz**2*
     &    s4fun**(-1)*Yfun - 8.D0*mz**2*rwm2 + 16.D0*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 128.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 4.D0*mz**2*s*u**2*Yfun**(-1) + 16.D0*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 256.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 8.D0*mz**2*s**2*u*Yfun**(-1) - 128.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 4.D0*mz**2*s**3*Yfun**(-1) - 8.D
     &    0*mz**4*s4fun**(-1)*rwm2 - 16.D0*mz**4*s*Yfun**(-1)*rwm2 + 
     &    336.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s*
     &    u*Yfun**(-1) - 4.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 336.
     &    D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**2*
     &    Yfun**(-1) - 8.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 4.D0*
     &    mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 288.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 16.D0*mz**6*s*Yfun**(-1) + 16.D0
     &    *mz**6*s*u*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2*Yfun**(-1)*rwm2
     &     + 24.D0*ctw**2*s**2*u*Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*
     &    Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.
     &    D0*ctw**2*mz**2*s**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**4*s*Yfun**(-1)*rwm2 + 12.D
     &    0*ctw**2*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*
     &    mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 12.D0*ctw**2*mz**4*s**3
     &    *s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 48.D0*ctw**2*mz**6*s**2*s4fun**(-1)*Yfun**(-1)
     &     + 48.D0*ctw**2*mz**8*s*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*
     &    Yfun**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 
     &    2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 2.D0*
     &    ctw**(-2)*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 8.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 8.D0*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2
     &     + 8.D0*u*Yfun**(-1)*rwm2*rzm2 + 32.D0*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 32.D0*u**3
     &    *s4fun**(-1)*Yfun**(-1)*rwm2 + 64.D0*mz**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2*rzm2 + 8.D0*mz**2*u*Yfun**(-1)*rzm2 - 64.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*u**2*
     &    Yfun**(-1) + 32.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 8.
     &    D0*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 32.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 24.D0*ctw**2*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2
     &     + 24.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*ctw**2*u*
     &    Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 24.
     &    D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 24.D0*ctw**2*
     &    mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 - 2.D0*
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*
     &    s2fun*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D
     &    0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*
     &    Yfun**(-1)*rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*Yfun**(-1)*rwm2
     &     - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 2.D0*ctw**(-2)*mz**4*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 2.D0*ctw**(-2)*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 2.D0
     &    *ctw**(-2)*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 4.D0*ctw**(-2)
     &    *mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*ctw**(-2)*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*ctw**(-2)*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*ctw**(-2)*mz**8*s4fun**(-1)*Yfun**(-1) + 8.D
     &    0*s**(-1)*s2fun*s4fun**(-1)*rwm2*rzm2 + 8.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 + 8.D0*s2fun*s4fun**(-1)*rwm2 + 8.D0*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*
     &    rzm2 + 32.D0*u**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 - 8.D0*
     &    u**2*Yfun**(-1)*rwm2 + 32.D0*u**3*s4fun**(-1)*Yfun**(-1)*rwm2
     &     + 8.D0*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 8.D0*s*
     &    Yfun**(-1)*rwm2*rzm2 + 64.D0*s*u*s4fun**(-1)*Yfun**(-1)*rwm2*
     &    rzm2 - 16.D0*s*u*Yfun**(-1)*rwm2 + 96.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**2*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*s**2*Yfun**(-1)*rwm2 + 96.D0*s**2*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*s**3*s4fun**(-1)*Yfun**(-1)*rwm2 - 16.
     &    D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*mz**2*Yfun**(-1)*
     &    rwm2*rzm2 - 192.D0*mz**2*u*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2
     &     - 8.D0*mz**2*u*Yfun**(-1)*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*
     &    rwm2 - 256.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*
     &    mz**2*u**2*Yfun**(-1) - 192.D0*mz**2*s*s4fun**(-1)*Yfun**(-1)
     &    *rwm2*rzm2 - 8.D0*mz**2*s*Yfun**(-1)*rzm2 + 32.D0*mz**2*s*
     &    Yfun**(-1)*rwm2 - 512.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*
     &    rwm2 - 16.D0*mz**2*s*u*Yfun**(-1) - 256.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**2*Yfun**(-1) + 
     &    288.D0*mz**4*s4fun**(-1)*Yfun**(-1)*rwm2*rzm2 + 16.D0*mz**4*
     &    Yfun**(-1)*rzm2 - 32.D0*mz**4*Yfun**(-1)*rwm2 - 8.D0*mz**4*u*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 672.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 + 32.D0*mz**4*u*Yfun**(-1) - 8.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 672.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rwm2 + 32.D0*
     &    mz**4*s*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 576.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rwm2 - 32.D0*mz**6*Yfun**(-1) + 32.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*mz**6*s*s4fun**(-1)*Yfun**(-1)
     &     - 32.D0*mz**8*s4fun**(-1)*Yfun**(-1) - 24.D0*ctw**2*s**(-1)*
     &    s2fun*s4fun**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 24.D0
     &    *ctw**2*u*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*ctw**2*u
     &    *Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*rwm2 - 
     &    24.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 24.D0*
     &    ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 48.D0*ctw**2*mz**2
     &    *s**(-1)*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*
     &    Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*Yfun**(-1)*rwm2
     &     - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 24.D0*ctw**2*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 24.D0
     &    *ctw**2*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*ctw**2*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 24.D0*ctw**2*mz**4*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*ctw**2*mz**6*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 96.D0*ctw**2*mz**6*u*s4fun**(-1)*Yfun**(-1)
     &     - 96.D0*ctw**2*mz**6*s*s4fun**(-1)*Yfun**(-1) + 96.D0*ctw**2
     &    *mz**8*s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*c0ccqswww * ( 
     &     - ctw**(-2)*s*s2fun*Yfun**(-1)*rwm2 - ctw**(-2)*mz**4*s*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s*s4fun**(-1)*rwm2 - 12.D
     &    0*s*s2fun*Yfun**(-1)*rwm2 + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 4.
     &    D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) - 12.D0*ctw**2*s*
     &    s2fun*Yfun**(-1)*rwm2 - 12.D0*ctw**2*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*c0mzmzqswww
     &  * ( 2.D0*ctw**(-2)*rwm2 - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 2.
     &    D0*ctw**(-2)*mz**4*s4fun**(-1) - ctw**(-2)*mz**4*s*Yfun**(-1)
     &     + 24.D0*rwm2 - 12.D0*s*s4fun*Yfun**(-1)*rwm2 - 8.D0*mz**2 + 
     &    4.D0*mz**2*s*s4fun*Yfun**(-1) - 8.D0*mz**4*s4fun**(-1) + 4.D0
     &    *mz**4*s*Yfun**(-1) + 24.D0*ctw**2*rwm2 - 12.D0*ctw**2*s*
     &    s4fun*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**4*s4fun**(-1) - 12.D0
     &    *ctw**2*mz**4*s*Yfun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - ctw**(-2)*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*s*t1fun**(-2)*s4fun**(-1)
     &     - 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*s4fun**(-1) + 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*s*t1fun**(-2)*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**6*t1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*b0fusww * ( 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 8.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 12.D
     &    0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) )
      ewamp(2,1,2,2) = ewamp(2,1,2,2) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) - 2.D0*ctw**(-2)*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*t1fun**(-2)*s4fun**(-1) + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*
     &    mz**4*u*t1fun**(-2)*s4fun**(-1) - 24.D0*ctw**2*mz**4*s*
     &    t1fun**(-2)*s4fun**(-1) + 48.D0*ctw**2*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      ewamp(2,2,1,1) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s*t1fun**(-1)*u1fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 4.D0
     &    *mz**2*s*t1fun**(-1)*u1fun**(-1) + 24.D0*ctw**2*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*denh12p * ( 6.D0
     &    *ctw**(-2)*rwm2 + ctw**(-2)*mh**2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 1.D0/2.D0*ctw**(-2)*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 - 8.D0*s**(-1)*s2fun*s4fun**(-1)*
     &    Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 - 4.D0*mz**2*s**(-1)*Yfun + 6.D0*ctw**2*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 - 8.D0*ctw**2*s**(-1)*s2fun*Yfun + 24.D0
     &    *ctw**2*rwm2**2 - 16.D0*ctw**2*s2fun*rwm2 + 4.D0*ctw**2*s*
     &    s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D
     &    0*ctw**2*mz**4 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*ctw**(-2)*
     &    s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s2fun*
     &    s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0
     &    *s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 4.D0*s**(-2)*u*s2fun*
     &    s4fun**(-1)*Yfun - 8.D0*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*rwm2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 8.D0*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2
     &    *rzm2 + 8.D0*mz**2*s**(-1)*rzm2 + 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 - 8.
     &    D0*mz**2*s**(-1)*u - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*
     &    Yfun*rzm2 + 12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 
     &    24.D0*ctw**2*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2
     &     + 16.D0*ctw**2*s**(-1)*s2fun*rzm2 + 48.D0*ctw**2*s**(-1)*u*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*ctw**2*s**(-1)*u*s2fun + 24.D0
     &    *ctw**2*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*
     &    rwm2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*c0cmztswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - ctw**(-2)*s**(-1)*
     &    s2fun*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 + 2.D
     &    0*ctw**(-2)*mz**2*s**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*rwm2 + 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2
     &     + 4.D0*s**(-2)*u*s2fun*s4fun**(-1)*Yfun - 8.D0*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 8.
     &    D0*s2fun*s4fun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2
     &     + 8.D0*mz**2*s**(-1)*rzm2 - 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 8.
     &    D0*mz**2*s**(-1)*u - 16.D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*
     &    mz**2 + 32.D0*mz**4*s**(-1)*s4fun**(-1)*rwm2 - 16.D0*mz**4*
     &    s**(-1) - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 
     &    12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    s**(-1)*t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - 12.D0*
     &    ctw**2*s**(-1)*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s**(-1)*
     &    s2fun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 
     &    16.D0*ctw**2*s**(-1)*u*s2fun - 24.D0*ctw**2*s2fun*s4fun**(-1)
     &    *rwm2 + 16.D0*ctw**2*s2fun + 24.D0*ctw**2*mz**2*s**(-2)*s2fun
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2
     &     - 32.D0*ctw**2*mz**2*s**(-1)*s2fun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s4fun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*rwm2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 16.D0*mz**2*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*c0ccqswww*
     & denh12p * (  - 12.D0*ctw**(-2)*rwm2**2 + 8.D0*ctw**(-2)*s*rwm2
     &     - 2.D0*ctw**(-2)*mh**2*rwm2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) + 3.D0/2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s2fun*s4fun**(-1) + 2.D0
     &    *u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1) + 2.D0*s*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 4.D0*mz**2
     &    *s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0
     &    *ctw**2*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1) - 6.
     &    D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*
     &    ctw**2*mz**2*s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 48.D
     &    0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) + 18.D0*ctw**2*mz**2
     &    *t1fun**(-1)*s2fun*s4fun**(-1) + 6.D0*ctw**2*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) + 4.D0*s**(-1)*u*s2fun*s4fun**(-1) + 2.D0*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*mz**2*s**(-1)*s2fun
     &    *s4fun**(-1) + 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D
     &    0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0*ctw**2*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*ctw**2*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) + ctw**(-2)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*t1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 3.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*s**(-1)*u*s2fun*s4fun**(-1) - 4.D0*s2fun*
     &    s4fun**(-1) - 2.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 6.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     + 12.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) - 
     &    12.D0*ctw**2*mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun
     &     - 36.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) - 18.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun * ( 
     &     - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 4.D0
     &    *s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 12.D0*ctw**2*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*
     &    Yfun**2 - 2.D0*ctw**(-2)*s4fun**(-1)*Yfun*rwm2 + 2.D0*s**(-1)
     &    *s4fun**(-1)*Yfun**2 + 8.D0*s4fun**(-1)*Yfun*rwm2 + 4.D0*
     &    mz**2*s4fun**(-1)*Yfun - 6.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    Yfun**2 - 24.D0*ctw**2*s4fun**(-1)*Yfun*rwm2 + 8.D0*ctw**2*
     &    Yfun + 16.D0*ctw**2*s*rwm2 - 4.D0*ctw**2*s**2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - 2.D0*mz**2*s*s2fun*s4fun**(-1) + 16.D0*
     &    ctw**2*s*rwm2 - 4.D0*ctw**2*s**2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - 2.D0*mz**2*s*s2fun*s4fun**(-1) + 16.D0*
     &    ctw**2*s*rwm2 - 4.D0*ctw**2*s**2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 4.D0*ctw**(-2)*u*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*rwm2 - 4.D0*s**(-1)*s4fun**(-1)*
     &    Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*Yfun + 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 16.D0*u*s4fun**(-1)*rwm2
     &     + 8.D0*s*s4fun**(-1)*rwm2 - 8.D0*mz**2*s4fun**(-1)*rzm2 - 16.
     &    D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*mz**2*u*s4fun**(-1) + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 - 16.D0*ctw**2*rzm2 - 48.D0*ctw**2*u*s4fun**(-1)*rwm2
     &     + 16.D0*ctw**2*u - 24.D0*ctw**2*s*s4fun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + ctw**(-2)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*u*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 - 4.D0*s**(-1)*
     &    s4fun**(-1)*Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*Yfun + 8.D0
     &    *t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 4.D0*s4fun**(-1)*Yfun - 
     &    16.D0*u*s4fun**(-1)*rwm2 - 8.D0*s*s4fun**(-1)*rwm2 + 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun - 8.D0*mz**2*s4fun**(-1)*rzm2
     &     + 16.D0*mz**2*s4fun**(-1)*rwm2 - 8.D0*mz**2*u*s4fun**(-1) - 
     &    8.D0*mz**2*s*s4fun**(-1) + 16.D0*mz**4*s4fun**(-1) + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 12.D0*ctw**2*s4fun**(-1)*Yfun - 16.D0*ctw**2*rzm2 + 
     &    48.D0*ctw**2*u*s4fun**(-1)*rwm2 - 16.D0*ctw**2*u + 24.D0*
     &    ctw**2*s*s4fun**(-1)*rwm2 - 16.D0*ctw**2*s - 24.D0*ctw**2*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*rwm2 + 32.D0*ctw**2*mz**2 )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 8.D0*mz**2*s*s4fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & b0fmzww * ( ctw**(-2)*s*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*u*u1fun**(-1)
     &    *s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1) - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)
     &    *mz**2*s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*s*u1fun**(-1)*
     &    s4fun**(-1) - 4.D0*s*s4fun**(-1) - 2.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 2.D0*s*u*u1fun**(-1)*s4fun**(-1) - 2.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**2*
     &    s4fun**(-1) + 6.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 2.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*s*s4fun**(-1)
     &     + 6.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1) - 6.D0*ctw**2*s*u*
     &    u1fun**(-1)*s4fun**(-1) + 6.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1) - 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun
     &     - 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 48.D0*
     &    ctw**2*mz**2*s4fun**(-1) - 18.D0*ctw**2*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 6.D0*ctw**2*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & b0fusww * ( ctw**(-2)*u*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*u*
     &    u1fun**(-1)*s4fun**(-1) + ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun + ctw**(-2)*mz**2*s4fun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*mz**2*s*u1fun**(-1)*s4fun**(-1) - 4.D0*u*
     &    s4fun**(-1) - 2.D0*s*u*u1fun**(-1)*s4fun**(-1) - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*mz**2*s4fun**(-1) - 2.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*u*s4fun**(-1)
     &     + 6.D0*ctw**2*s*u*u1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) + 6.D0*ctw**2*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      ewamp(2,2,1,1) = ewamp(2,2,1,1) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - ctw**(-2)*u*s4fun**(-1) - ctw**(-2)*s*s4fun**(-1)
     &     - 1.D0/2.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1) - 1.D0/2.D0
     &    *ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1) + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 3.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) + 3.D0/2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*u*s4fun**(-1) + 4.D0*s*s4fun**(-1) + 2.D0*
     &    s*u*t1fun**(-1)*s4fun**(-1) + 2.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0
     &    *mz**2*s4fun**(-1) - 6.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 
     &    12.D0*ctw**2*u*s4fun**(-1) - 12.D0*ctw**2*s*s4fun**(-1) - 6.D0
     &    *ctw**2*s*u*t1fun**(-1)*s4fun**(-1) - 6.D0*ctw**2*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + 36.D0*ctw**2*mz**2*s4fun**(-1) + 18.D0*
     &    ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      ewamp(2,2,1,3) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun + 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 - 
     &    u**2*deltafun - 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 + mz**4*
     &    deltafun - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u
     &    *s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + u**2*deltafun - 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 + 2.D0*s*u*deltafun - 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 + s**2*deltafun + 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**2*u*deltafun + 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s*deltafun - 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*mz**4*deltafun + 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*ctw**(-2)*
     &    u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)
     &    *mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*
     &    mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*deltafun*
     &    rzm2 - 2.D0*u*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 + 4.D0*mz**2*s**(-1)*u*deltafun
     &     + 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0
     &    *s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun + 2.D0*deltafun*rzm2 + 6.D0*u*deltafun + 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 12.D0*
     &    mz**2*s**(-1)*u*deltafun - 8.D0*mz**2*deltafun - 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**4*s**(-1)*deltafun + 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*u*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*u*deltafun + 8.D0*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    s*deltafun - 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*
     &    mz**2*deltafun - 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 
     &    24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*mz**2
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*
     &    ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.
     &    D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*b0fusww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D
     &    0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun * ( 
     &     - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun + s*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun - mz**4*s*s4fun**(-1)*deltafun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u*s4fun**(-1)*deltafun - 
     &    s**3*s4fun**(-1)*deltafun + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*s**2*s4fun**(-1)*deltafun - 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *u**2*s4fun**(-1)*deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 2.D0*s*u*s4fun**(-1)*deltafun + 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun - 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**2*s4fun**(-1)
     &    *deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 - 6.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s**2*s4fun**(-1)
     &    *deltafun + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 - 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     + 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 4.D0*s*u*s4fun**(-1)*deltafun + 2.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & b0fmzww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & b0fusww * (  - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,2,1,3) = ewamp(2,2,1,3) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(2,2,1,2) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*rwm2**2 - ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*
     &    Yfun**2 - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2
     &     - 4.D0*s**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 4.D0*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 + 12.D0*ctw**2*s**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2
     &     - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 - 24.D0*
     &    ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 8.D0
     &    *rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2
     &    *Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 - 8.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 16.D0*mz**2*s*u*Yfun**(-1)*rwm2 + 16.D0*
     &    mz**2*s**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.
     &    D0*mz**4*s*Yfun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*Yfun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 48.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*rwm2 - 2.D0*ctw**(-2)*u*Yfun**(-1)*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*
     &    rzm2 - 8.D0*s**(-1)*u*rwm2 - 8.D0*rwm2 + 8.D0*u*Yfun**(-1)*
     &    rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*
     &    ctw**2*s**(-1)*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0
     &    *ctw**2*rwm2 - 24.D0*ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*
     &    ctw**2*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 24.D0*ctw**2*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*
     &    rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 4.D
     &    0*ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*Yfun + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*rzm2 + 8.D0*s**(-1)
     &    *u*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*
     &    rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 16.D0*s*u*Yfun**(-1)*
     &    rwm2 - 8.D0*s**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*
     &    rwm2 - 8.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 
     &    32.D0*mz**2*s*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*Yfun**(-1)*rwm2 - 24.D0*ctw**2
     &    *s**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u
     &    *Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 24.D0*ctw**2*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*
     &    Yfun**(-1)*rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 48.D0
     &    *ctw**2*mz**4*s**(-2)*s4fun**(-1)*Yfun + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*c0ccqswww * ( 2.
     &    D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 - ctw**(-2)*s*s2fun*
     &    Yfun**(-1)*rwm2 - 8.D0*s2fun*s4fun**(-1)*rwm2 + 4.D0*s*s2fun*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 12.D0
     &    *ctw**2*s*s2fun*Yfun**(-1)*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*s4fun*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*
     &    Yfun - 3.D0*ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)
     &    *mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*u
     &    *u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 
     &    16.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 36.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**4*
     &    s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun
     &     + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*Yfun )
      ewamp(2,2,1,2) = ewamp(2,2,1,2) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 3.D0
     &    *ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 36.D0*ctw**2*mz**2*t1fun**(-2)
     &    *s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )

      ewamp(2,2,3,1) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun - 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 + 
     &    u**2*deltafun + 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 - mz**4*
     &    deltafun + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*
     &    s*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - u**2*deltafun + 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*s*u*deltafun + 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 - s**2*deltafun - 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**2*u*deltafun - 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s*deltafun + 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*mz**4*deltafun - 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     + 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 + 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - ctw**(-2)*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*ctw**(-2)*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*deltafun*
     &    rzm2 + 2.D0*u*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 - 4.D0*mz**2*s**(-1)*u*deltafun
     &     - 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 3.D0*ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun - 2.D0*deltafun*rzm2 - 6.D0*u*deltafun - 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun
     &     - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 12.D0*
     &    mz**2*s**(-1)*u*deltafun + 8.D0*mz**2*deltafun + 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*mz**4*s**(-1)*deltafun - 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*c0ccqswww * ( 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*u*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*
     &    deltafun + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*b0ftsww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2
     &    *s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun * ( 2.D
     &    0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun - s*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun + mz**4*s*s4fun**(-1)*deltafun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u*s4fun**(-1)*deltafun + 
     &    s**3*s4fun**(-1)*deltafun - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*s**2*s4fun**(-1)*deltafun + 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *u**2*s4fun**(-1)*deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 2.D0*s*u*s4fun**(-1)*deltafun - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun + 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*s4fun**(-1)*deltafun*Yfun + 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**2*s4fun**(-1)
     &    *deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 + 6.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s**2*s4fun**(-1)
     &    *deltafun - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 + 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     - 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 4.D0*s*u*s4fun**(-1)*deltafun - 2.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & b0fusww * ( 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,2,3,1) = ewamp(2,2,3,1) + g**4*stw**2*betaZfun*
     & b0ftsww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun - 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(2,2,3,3) =
     &  + g**4*stw**2 * ( 4.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)
     &    *s4fun**(-1)*Yfun )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*denh12p * (  - 
     &    3.D0/2.D0*ctw**(-2)*s*rwm2*rzm2**(-1) - 1.D0/4.D0*ctw**(-2)*
     &    mh**2*s*rzm2**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - 2.D0*ctw**(-2)*mz**2*s**(-2)*u**2*
     &    s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)
     &    *Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 - 
     &    1.D0/2.D0*ctw**(-2)*mz**2*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)*
     &    mz**2*u*s4fun**(-1)*rwm2 - ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2
     &     + 4.D0*ctw**(-2)*mz**4*s**(-2)*u*s4fun**(-1)*Yfun + 2.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*Yfun + 8.D0*ctw**(-2)*
     &    mz**4*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**4*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**6*s**(-2)*s4fun**(-1)*
     &    Yfun - 4.D0*ctw**(-2)*mz**6*s**(-1)*s4fun**(-1)*rwm2 + 16.D0*
     &    s**(-1)*Yfun*rwm2 + 32.D0*rwm2**2 - 4.D0*s*rwm2 + 8.D0*mz**2*
     &    s**(-2)*u**2*s4fun**(-1)*Yfun - 4.D0*mz**2*s**(-1)*Yfun + 8.D0
     &    *mz**2*s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*mz**2*s**(-1)*u**2*
     &    s4fun**(-1)*rwm2 + 2.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*
     &    rwm2 + 16.D0*mz**2*u*s4fun**(-1)*rwm2 + 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**4*s**(-2)*u*s4fun**(-1)*Yfun - 8.
     &    D0*mz**4*s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**4*s4fun**(-1)*rwm2 + 8.D0*mz**6*
     &    s**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**6*s**(-1)*s4fun**(-1)*
     &    rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*u**2*s4fun**(-1)*Yfun - 24.D
     &    0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*Yfun - 48.D0*ctw**2*
     &    mz**2*s**(-1)*u**2*s4fun**(-1)*rwm2 - 6.D0*ctw**2*mz**2*
     &    s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**2*u*s4fun**(-1)*rwm2 - 12.
     &    D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**4*
     &    s**(-2)*u*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**4*s**(-1)*
     &    s4fun**(-1)*Yfun + 96.D0*ctw**2*mz**4*s**(-1)*u*s4fun**(-1)*
     &    rwm2 + 48.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 - 24.D0*ctw**2*
     &    mz**6*s**(-2)*s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**6*s**(-1)*
     &    s4fun**(-1)*rwm2 )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 32.D0*
     &    rwm2**2 - 4.D0*s*rwm2 - 8.D0*mz**2*rwm2 - 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*mz**2*s*s4fun**(-1)*rwm2 + 32.D0*
     &    rwm2**2 - 4.D0*s*rwm2 - 8.D0*mz**2*rwm2 - 4.D0*mz**2*s*
     &    s4fun**(-1)*rwm2 + 12.D0*ctw**2*mz**2*s*s4fun**(-1)*rwm2 )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*c0cmzuswww * ( 
     &    4.D0*ctw**(-2)*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 - 4.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u**3*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**2*s4fun**(-1) + ctw**(-2)*mz**2*s4fun**(-1)*rzm2
     &     - ctw**(-2)*mz**2*u*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*
     &    s**(-2)*u*s4fun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**4*s**(-2)*
     &    u**2*s4fun**(-1) - 4.D0*ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*
     &    rzm2 + 4.D0*ctw**(-2)*mz**4*s**(-1)*u*s4fun**(-1) - 8.D0*
     &    ctw**(-2)*mz**4*u1fun**(-1)*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)
     &    *mz**6*s**(-2)*s4fun**(-1)*rzm2 - 4.D0*ctw**(-2)*mz**6*
     &    s**(-2)*u*s4fun**(-1) - 32.D0*s**(-1)*rwm2*rzm2 + 32.D0*
     &    s**(-1)*u*rwm2 - 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 
     &    16.D0*mz**2*s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rzm2 - 8.D0*mz**2*
     &    s**(-1)*u + 16.D0*mz**2*s**(-1)*u**2*s4fun**(-1) - 4.D0*mz**2
     &    *s4fun**(-1)*rzm2 + 4.D0*mz**2*u*s4fun**(-1) + 32.D0*mz**4*
     &    s**(-2)*u*s4fun**(-1)*rzm2 - 32.D0*mz**4*s**(-2)*u**2*
     &    s4fun**(-1) + 16.D0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 16.D0*
     &    mz**4*s**(-1)*u*s4fun**(-1) + 32.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**6*s**(-2)*s4fun**(-1)*rzm2 + 16.D
     &    0*mz**6*s**(-2)*u*s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-2)*
     &    u**2*s4fun**(-1)*rzm2 - 48.D0*ctw**2*mz**2*s**(-2)*u**3*
     &    s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*rzm2
     &     - 48.D0*ctw**2*mz**2*s**(-1)*u**2*s4fun**(-1) + 12.D0*ctw**2
     &    *mz**2*s4fun**(-1)*rzm2 - 12.D0*ctw**2*mz**2*u*s4fun**(-1) - 
     &    96.D0*ctw**2*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 + 96.D0*ctw**2*
     &    mz**4*s**(-2)*u**2*s4fun**(-1) - 48.D0*ctw**2*mz**4*s**(-1)*
     &    s4fun**(-1)*rzm2 + 48.D0*ctw**2*mz**4*s**(-1)*u*s4fun**(-1)
     &     - 96.D0*ctw**2*mz**4*u1fun**(-1)*s4fun**(-1)*rwm2 + 48.D0*
     &    ctw**2*mz**6*s**(-2)*s4fun**(-1)*rzm2 - 48.D0*ctw**2*mz**6*
     &    s**(-2)*u*s4fun**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*c0cmztswww * ( 
     &    4.D0*ctw**(-2)*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u**3*s4fun**(-1) + 4.D0*ctw**(-2)*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 + 8.D0*ctw**(-2)*mz**2*
     &    s**(-1)*u**2*s4fun**(-1) + ctw**(-2)*mz**2*s4fun**(-1)*rzm2
     &     + 5.D0*ctw**(-2)*mz**2*u*s4fun**(-1) + ctw**(-2)*mz**2*s*
     &    s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*s**(-2)*u*s4fun**(-1)*rzm2
     &     - 16.D0*ctw**(-2)*mz**4*s**(-2)*u**2*s4fun**(-1) - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 20.D0*ctw**(-2)*
     &    mz**4*s**(-1)*u*s4fun**(-1) - 8.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1)*rwm2 - 6.D0*ctw**(-2)*mz**4*
     &    s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*s**(-2)*s4fun**(-1)*rzm2
     &     + 20.D0*ctw**(-2)*mz**6*s**(-2)*u*s4fun**(-1) + 12.D0*
     &    ctw**(-2)*mz**6*s**(-1)*s4fun**(-1) - 8.D0*ctw**(-2)*mz**8*
     &    s**(-2)*s4fun**(-1) - 32.D0*s**(-1)*rwm2*rzm2 - 32.D0*s**(-1)
     &    *u*rwm2 - 32.D0*rwm2 - 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*
     &    s**(-1)*rzm2 + 64.D0*mz**2*s**(-1)*rwm2 - 16.D0*mz**2*s**(-1)
     &    *u*s4fun**(-1)*rzm2 + 8.D0*mz**2*s**(-1)*u - 32.D0*mz**2*
     &    s**(-1)*u**2*s4fun**(-1) - 4.D0*mz**2*s4fun**(-1)*rzm2 + 8.D0
     &    *mz**2 - 20.D0*mz**2*u*s4fun**(-1) - 4.D0*mz**2*s*s4fun**(-1)
     &     + 32.D0*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 + 64.D0*mz**4*
     &    s**(-2)*u**2*s4fun**(-1) + 16.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    rzm2 - 16.D0*mz**4*s**(-1) + 80.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1) + 32.D0*mz**4*t1fun**(-1)*s4fun**(-1)*rwm2 + 24.D0
     &    *mz**4*s4fun**(-1) - 16.D0*mz**6*s**(-2)*s4fun**(-1)*rzm2 - 
     &    80.D0*mz**6*s**(-2)*u*s4fun**(-1) - 48.D0*mz**6*s**(-1)*
     &    s4fun**(-1) + 32.D0*mz**8*s**(-2)*s4fun**(-1) + 48.D0*ctw**2*
     &    mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 48.D0*ctw**2*mz**2*
     &    s**(-2)*u**3*s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*rzm2 + 96.D0*ctw**2*mz**2*s**(-1)*u**2*
     &    s4fun**(-1) + 12.D0*ctw**2*mz**2*s4fun**(-1)*rzm2 + 60.D0*
     &    ctw**2*mz**2*u*s4fun**(-1) + 12.D0*ctw**2*mz**2*s*s4fun**(-1)
     &     - 96.D0*ctw**2*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 - 192.D0*
     &    ctw**2*mz**4*s**(-2)*u**2*s4fun**(-1) - 48.D0*ctw**2*mz**4*
     &    s**(-1)*s4fun**(-1)*rzm2 - 240.D0*ctw**2*mz**4*s**(-1)*u*
     &    s4fun**(-1) - 96.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*rwm2
     &     - 72.D0*ctw**2*mz**4*s4fun**(-1) + 48.D0*ctw**2*mz**6*
     &    s**(-2)*s4fun**(-1)*rzm2 + 240.D0*ctw**2*mz**6*s**(-2)*u*
     &    s4fun**(-1) + 144.D0*ctw**2*mz**6*s**(-1)*s4fun**(-1) - 96.D0
     &    *ctw**2*mz**8*s**(-2)*s4fun**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*c0ccqswww * ( 
     &     - 8.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 - 4.D0*s + 32.D0*
     &    mz**2*s4fun**(-1)*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*rwm2
     &     )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*c0ccqswww*
     & denh12p * ( 3.D0*ctw**(-2)*s*rwm2**2*rzm2**(-1) - 2.D0*ctw**(-2)
     &    *s**2*rwm2*rzm2**(-1) + 1.D0/2.D0*ctw**(-2)*mh**2*s*rwm2*
     &    rzm2**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*b0fmzww * ( 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) + 6.D0*
     &    ctw**(-2)*mz**2*u*t1fun**(-1)*s4fun**(-1) + 2.D0*ctw**(-2)*
     &    mz**2*u*u1fun**(-1)*s4fun**(-1) + 2.D0*ctw**(-2)*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*ctw**(-2)*mz**4*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1) - 10.D0*ctw**(-2)*mz**4*t1fun**(-1)
     &    *s4fun**(-1) + 2.D0*ctw**(-2)*mz**4*u1fun**(-1)*s4fun**(-1)
     &     + 12.D0*ctw**(-2)*mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) - 4.D
     &    0*ctw**(-2)*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) - 16.D0*
     &    mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 16.D0*mz**2*
     &    s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 24.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1) - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 16.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*
     &    s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun + 64.D0*mz**4*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1) + 40.D0*mz**4*t1fun**(-1)*s4fun**(-1)
     &     - 8.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 48.D0*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) + 16.D0*mz**6*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u**2*t1fun**(-1)*
     &    s4fun**(-1) + 48.D0*ctw**2*mz**2*s**(-1)*u**2*u1fun**(-1)*
     &    s4fun**(-1) + 72.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1) + 
     &    24.D0*ctw**2*mz**2*u*u1fun**(-1)*s4fun**(-1) + 24.D0*ctw**2*
     &    mz**2*s*t1fun**(-1)*s4fun**(-1) + 48.D0*ctw**2*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 192.D0*ctw**2*mz**4*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1) - 120.D0*ctw**2*mz**4*t1fun**(-1)*
     &    s4fun**(-1) + 24.D0*ctw**2*mz**4*u1fun**(-1)*s4fun**(-1) + 
     &    144.D0*ctw**2*mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) - 48.D0*
     &    ctw**2*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*b0fusww * (  - 
     &    4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 2.
     &    D0*ctw**(-2)*mz**2*u*u1fun**(-1)*s4fun**(-1) - 4.D0*ctw**(-2)
     &    *mz**4*s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun - 2.D0*ctw**(-2)*
     &    mz**4*u1fun**(-1)*s4fun**(-1) + 4.D0*ctw**(-2)*mz**6*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s**(-1)*u**2*
     &    u1fun**(-1)*s4fun**(-1) + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) + 16.D0*mz**4*s**(-1)*u1fun**(-2)*s4fun**(-1)*
     &    Yfun + 8.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 16.D0*mz**6*
     &    s**(-1)*u1fun**(-1)*s4fun**(-1) - 48.D0*ctw**2*mz**2*s**(-1)*
     &    u**2*u1fun**(-1)*s4fun**(-1) - 24.D0*ctw**2*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1) - 48.D0*ctw**2*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*mz**4*u1fun**(-1)
     &    *s4fun**(-1) + 48.D0*ctw**2*mz**6*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*b0ftsww * (  - 
     &    4.D0*ctw**(-2)*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 6.
     &    D0*ctw**(-2)*mz**2*u*t1fun**(-1)*s4fun**(-1) - 2.D0*ctw**(-2)
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1) - 4.D0*ctw**(-2)*mz**4*
     &    s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*ctw**(-2)*mz**4*
     &    s**(-1)*u*t1fun**(-1)*s4fun**(-1) + 10.D0*ctw**(-2)*mz**4*
     &    t1fun**(-1)*s4fun**(-1) - 12.D0*ctw**(-2)*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s**(-1)*u**2*
     &    t1fun**(-1)*s4fun**(-1) + 24.D0*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 64.D0*mz**4*
     &    s**(-1)*u*t1fun**(-1)*s4fun**(-1) - 40.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1) + 48.D0*mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) - 
     &    48.D0*ctw**2*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 72.D
     &    0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1) - 24.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*s4fun**(-1) - 48.D0*ctw**2*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 192.D0*ctw**2*mz**4*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1) + 120.D0*ctw**2*mz**4*t1fun**(-1)*
     &    s4fun**(-1) - 144.D0*ctw**2*mz**6*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*betaZfun**2*
     & denh12p * (  - 3.D0/2.D0*ctw**(-2)*s*rwm2*rzm2**(-1) - 1.D0/4.D0
     &    *ctw**(-2)*mh**2*s*rzm2**(-1) )
      ewamp(2,2,3,3) = ewamp(2,2,3,3) + g**4*stw**2*betaZfun**2*
     & c0ccqswww*denh12p * ( 3.D0*ctw**(-2)*s*rwm2**2*rzm2**(-1) - 2.D0
     &    *ctw**(-2)*s**2*rwm2*rzm2**(-1) + 1.D0/2.D0*ctw**(-2)*mh**2*s
     &    *rwm2*rzm2**(-1) )

      ewamp(2,2,3,2) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*s**(-1)*u*deltafun*Yfun - 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 2.D0*mz**2*s**(-1)*deltafun*Yfun + 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 - 
     &    u**2*deltafun - 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 + mz**4*
     &    deltafun - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 + 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*u
     &    *s4fun**(-1)*deltafun*rwm2 + ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + u**2*deltafun - 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 + 2.D0*s*u*deltafun - 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 + s**2*deltafun + 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 - 4.D0*mz**2*u*deltafun + 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s*deltafun - 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 + 3.D0*mz**4*deltafun + 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 + 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     - 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 + 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*ctw**(-2)*
     &    u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)
     &    *mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*
     &    mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*deltafun*
     &    rzm2 - 2.D0*u*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 + 4.D0*mz**2*s**(-1)*u*deltafun
     &     + 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0
     &    *s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun + 2.D0*deltafun*rzm2 + 6.D0*u*deltafun + 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun - 4.D0*mz**2*s**(-1)*deltafun*rzm2 - 12.D0*
     &    mz**2*s**(-1)*u*deltafun - 8.D0*mz**2*deltafun - 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**4*s**(-1)*deltafun + 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 - 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*u*s4fun**(-1)*deltafun*rwm2
     &     - 4.D0*u*deltafun + 8.D0*s*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    s*deltafun - 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 4.D0*
     &    mz**2*deltafun - 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 
     &    24.D0*ctw**2*s*s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*mz**2
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.
     &    D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*
     &    ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.
     &    D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*b0fusww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D
     &    0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun * ( 2.D
     &    0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 + 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 - 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun - s*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 + 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun + mz**4*s*s4fun**(-1)*deltafun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun + 2.D0*s**2*u*s4fun**(-1)*deltafun + 
     &    s**3*s4fun**(-1)*deltafun - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 4.D0*mz**2*s**2*s4fun**(-1)*deltafun + 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *u**2*s4fun**(-1)*deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 2.D0*s*u*s4fun**(-1)*deltafun - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun + 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 + 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun - ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 - 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 - 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 + 4.
     &    D0*s4fun**(-1)*deltafun*Yfun + 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 24.D0*u*s4fun**(-1)*deltafun*rwm2 + 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*u**2*s4fun**(-1)
     &    *deltafun + 2.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 + 6.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D0*s**2*s4fun**(-1)
     &    *deltafun - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 + 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**4*s4fun**(-1)*deltafun
     &     - 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     - 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 4.D0*s*u*s4fun**(-1)*deltafun - 2.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & b0fusww * ( 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,2,3,2) = ewamp(2,2,3,2) + g**4*stw**2*betaZfun*
     & b0ftsww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun - 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(2,2,2,1) =
     &  + g**4*stw**2 * (  - ctw**(-2)*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 4.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun - 12.D0*ctw**2*t1fun**(-1)*u1fun**(-1)*s2fun
     &    *s4fun**(-1)*Yfun )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( ctw**(-2)*s**(-1)*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*rwm2**2 - ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*
     &    Yfun**2 - 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2
     &     - 4.D0*s**(-1)*Yfun*rwm2 - 8.D0*rwm2**2 + 4.D0*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*
     &    rwm2 + 12.D0*ctw**2*s**(-1)*Yfun*rwm2 + 24.D0*ctw**2*rwm2**2
     &     - 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 - 24.D0*
     &    ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 - 8.D0
     &    *rwm2**2 - 4.D0*s*u**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*
     &    s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*ctw**2*s*u**2
     &    *Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + ctw**(-2)*s*u**2*
     &    Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s**2*u*Yfun**(-1)*rwm2 + 
     &    ctw**(-2)*s**3*Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s*u*
     &    Yfun**(-1)*rwm2 - 4.D0*ctw**(-2)*mz**2*s**2*Yfun**(-1)*rwm2
     &     - 2.D0*ctw**(-2)*mz**4*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s*Yfun**(-1)*rwm2 - 8.D0*rwm2**2 - 4.D0*s*u**2*
     &    Yfun**(-1)*rwm2 - 8.D0*s**2*u*Yfun**(-1)*rwm2 - 4.D0*s**3*
     &    Yfun**(-1)*rwm2 + 16.D0*mz**2*s*u*Yfun**(-1)*rwm2 + 16.D0*
     &    mz**2*s**2*Yfun**(-1)*rwm2 + 8.D0*mz**4*s4fun**(-1)*rwm2 - 16.
     &    D0*mz**4*s*Yfun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 + 12.D0*
     &    ctw**2*s*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*u*
     &    Yfun**(-1)*rwm2 + 12.D0*ctw**2*s**3*Yfun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*s*u*Yfun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**2*
     &    Yfun**(-1)*rwm2 - 24.D0*ctw**2*mz**4*s4fun**(-1)*rwm2 + 48.D0
     &    *ctw**2*mz**4*s*Yfun**(-1)*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*c0cmzuswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 + 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*rwm2 - 2.D0*ctw**(-2)*u*Yfun**(-1)*
     &    rwm2*rzm2 + 2.D0*ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*
     &    rzm2 - 8.D0*s**(-1)*u*rwm2 - 8.D0*rwm2 + 8.D0*u*Yfun**(-1)*
     &    rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 + 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 24.D0*
     &    ctw**2*s**(-1)*rwm2*rzm2 + 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0
     &    *ctw**2*rwm2 - 24.D0*ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*
     &    ctw**2*u**2*Yfun**(-1)*rwm2 + 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 24.D0*ctw**2*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*c0cmztswww * ( 
     &     - 2.D0*ctw**(-2)*s**(-1)*rwm2*rzm2 - 2.D0*ctw**(-2)*s**(-1)*
     &    u*rwm2 + 2.D0*ctw**(-2)*u*Yfun**(-1)*rwm2*rzm2 + 2.D0*
     &    ctw**(-2)*u**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*Yfun**(-1)*
     &    rwm2*rzm2 + 4.D0*ctw**(-2)*s*u*Yfun**(-1)*rwm2 + 2.D0*
     &    ctw**(-2)*s**2*Yfun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *s4fun**(-1)*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)*u*
     &    s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*rwm2 + 2.D0*ctw**(-2)*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 4.D0*ctw**(-2)*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 - 8.D0*ctw**(-2)*mz**2*u*
     &    Yfun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**2*s*Yfun**(-1)*rwm2 - 4.D
     &    0*ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*Yfun + 8.D0*ctw**(-2)*
     &    mz**4*Yfun**(-1)*rwm2 + 8.D0*s**(-1)*rwm2*rzm2 + 8.D0*s**(-1)
     &    *u*rwm2 - 8.D0*u*Yfun**(-1)*rwm2*rzm2 - 8.D0*u**2*Yfun**(-1)*
     &    rwm2 - 8.D0*s*Yfun**(-1)*rwm2*rzm2 - 16.D0*s*u*Yfun**(-1)*
     &    rwm2 - 8.D0*s**2*Yfun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun
     &     - 8.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*
     &    rwm2 - 8.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 + 16.D0*
     &    mz**2*Yfun**(-1)*rwm2*rzm2 + 32.D0*mz**2*u*Yfun**(-1)*rwm2 + 
     &    32.D0*mz**2*s*Yfun**(-1)*rwm2 + 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*Yfun**(-1)*rwm2 - 24.D0*ctw**2
     &    *s**(-1)*rwm2*rzm2 - 24.D0*ctw**2*s**(-1)*u*rwm2 + 24.D0*
     &    ctw**2*u*Yfun**(-1)*rwm2*rzm2 + 24.D0*ctw**2*u**2*Yfun**(-1)*
     &    rwm2 + 24.D0*ctw**2*s*Yfun**(-1)*rwm2*rzm2 + 48.D0*ctw**2*s*u
     &    *Yfun**(-1)*rwm2 + 24.D0*ctw**2*s**2*Yfun**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 24.D0*ctw**2*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*rwm2 + 24.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*Yfun**(-1)*rwm2*rzm2 - 96.D0*ctw**2*mz**2*u*
     &    Yfun**(-1)*rwm2 - 96.D0*ctw**2*mz**2*s*Yfun**(-1)*rwm2 - 48.D0
     &    *ctw**2*mz**4*s**(-2)*s4fun**(-1)*Yfun + 96.D0*ctw**2*mz**4*
     &    Yfun**(-1)*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*c0ccqswww * ( 2.
     &    D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 - ctw**(-2)*s*s2fun*
     &    Yfun**(-1)*rwm2 - 8.D0*s2fun*s4fun**(-1)*rwm2 + 4.D0*s*s2fun*
     &    Yfun**(-1)*rwm2 + 24.D0*ctw**2*s2fun*s4fun**(-1)*rwm2 - 12.D0
     &    *ctw**2*s*s2fun*Yfun**(-1)*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*c0mzmzqswww
     &  * (  - ctw**(-2)*s*s4fun*Yfun**(-1)*rwm2 + 4.D0*s*s4fun*
     &    Yfun**(-1)*rwm2 - 12.D0*ctw**2*s*s4fun*Yfun**(-1)*rwm2 )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*b0fmzww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun
     &     + 2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*
     &    Yfun - 3.D0*ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 
     &    ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)
     &    *mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*u
     &    *u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 
     &    16.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*
     &    ctw**2*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 36.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**4*
     &    s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun
     &     + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*Yfun )
      ewamp(2,2,2,1) = ewamp(2,2,2,1) + g**4*stw**2*b0ftsww * ( 2.D0
     &    *ctw**(-2)*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun + 3.D0
     &    *ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 4.D0*
     &    ctw**(-2)*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*ctw**2*mz**2*s**(-1)*u*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 36.D0*ctw**2*mz**2*t1fun**(-2)
     &    *s4fun**(-1)*Yfun - 48.D0*ctw**2*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )

      ewamp(2,2,2,3) =
     &  + g**4*stw**2 * (  - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D
     &    0*ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 2.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*ctw**(-2)*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + 2.D0*s**(-1)*u*deltafun*Yfun + 4.D0*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + deltafun*Yfun - 4.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 8.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 2.D0*mz**2*s**(-1)*deltafun*Yfun - 12.D0
     &    *ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 24.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 + deltafun*Yfun - 4.D0*u**2*s4fun**(-1)*deltafun*rwm2 + 
     &    u**2*deltafun + 4.D0*mz**4*s4fun**(-1)*deltafun*rwm2 - mz**4*
     &    deltafun + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0
     &    *ctw**2*u**2*s4fun**(-1)*deltafun*rwm2 - 12.D0*ctw**2*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*
     & d0ccmzmzqstswwww * (  - ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2
     &     - ctw**(-2)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*
     &    s*u*s4fun**(-1)*deltafun*rwm2 - ctw**(-2)*s**2*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*ctw**(-2)*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*ctw**(-2)*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 3.D0
     &    *ctw**(-2)*mz**4*s4fun**(-1)*deltafun*rwm2 + 4.D0*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - deltafun*Yfun + 4.D0*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - u**2*deltafun + 8.D0*s*u*s4fun**(-1)*deltafun
     &    *rwm2 - 2.D0*s*u*deltafun + 4.D0*s**2*s4fun**(-1)*deltafun*
     &    rwm2 - s**2*deltafun - 16.D0*mz**2*u*s4fun**(-1)*deltafun*
     &    rwm2 + 4.D0*mz**2*u*deltafun - 16.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s*deltafun + 12.D0*mz**4*
     &    s4fun**(-1)*deltafun*rwm2 - 3.D0*mz**4*deltafun - 12.D0*
     &    ctw**2*s4fun**(-1)*deltafun*Yfun*rwm2 - 12.D0*ctw**2*u**2*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*s*u*s4fun**(-1)*
     &    deltafun*rwm2 - 12.D0*ctw**2*s**2*s4fun**(-1)*deltafun*rwm2
     &     + 48.D0*ctw**2*mz**2*u*s4fun**(-1)*deltafun*rwm2 + 48.D0*
     &    ctw**2*mz**2*s*s4fun**(-1)*deltafun*rwm2 - 36.D0*ctw**2*mz**4
     &    *s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*c0cmzuswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - ctw**(-2)*s**(-1)*u
     &    *s4fun**(-1)*deltafun*Yfun - 2.D0*ctw**(-2)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*ctw**(-2)*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    s**(-1)*u*deltafun*rzm2 + 4.D0*s**(-1)*u**2*deltafun + 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*deltafun*
     &    rzm2 + 2.D0*u*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*
     &    rzm2 - 8.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    mz**2*s**(-1)*deltafun*rzm2 - 4.D0*mz**2*s**(-1)*u*deltafun
     &     - 8.D0*mz**4*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*
     &    ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*c0cmztswww * ( 
     &    2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0
     &    *ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + ctw**(-2)
     &    *s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 3.D0*ctw**(-2)*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*ctw**(-2)*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*mz**2*
     &    s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0*ctw**(-2)*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *s**(-1)*u*deltafun*rzm2 - 4.D0*s**(-1)*u**2*deltafun - 8.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 4.D0*s4fun**(-1)
     &    *deltafun*Yfun - 2.D0*deltafun*rzm2 - 6.D0*u*deltafun - 8.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s*deltafun
     &     - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 8.D0*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*mz**2*s**(-2)*
     &    u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)
     &    *deltafun*Yfun + 4.D0*mz**2*s**(-1)*deltafun*rzm2 + 12.D0*
     &    mz**2*s**(-1)*u*deltafun + 8.D0*mz**2*deltafun + 32.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 16.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*mz**4*s**(-1)*deltafun - 24.D
     &    0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*rzm2 + 36.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 12.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun + 24.D0*ctw**2*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u*s4fun**(-1)
     &    *deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*c0ccqswww * ( 4.
     &    D0*ctw**(-2)*u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*deltafun*rwm2 - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 16.D0*u*s4fun**(-1)*deltafun*rwm2 + 4.D0*u*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun*rwm2 + 2.D0*s*deltafun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun*rwm2 - 4.D0*mz**2*
     &    deltafun + 48.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*
     &    ctw**2*s*s4fun**(-1)*deltafun*rwm2 - 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*b0fmzww * ( 2.D0
     &    *ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*b0fusww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 4.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2*
     &    s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*b0ftsww * (  - 
     &    2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.
     &    D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 24.D0*ctw**2
     &    *s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun * ( 
     &     - 2.D0*ctw**(-2)*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*u*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 4.D0*s*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 24.D0*
     &    ctw**2*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    12.D0*ctw**2*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 24.D0*ctw**2*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * (  - ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun
     &    *Yfun**2 - 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    rwm2 - 2.D0*ctw**(-2)*s4fun**(-1)*deltafun*Yfun*rwm2 + 
     &    ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 4.D0
     &    *s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2 + 2.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 8.D0*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D
     &    0*u*s4fun**(-1)*deltafun*Yfun + s*s4fun**(-1)*deltafun*Yfun
     &     - 4.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 - 2.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun**2 - 6.D0*ctw**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 48.D0*ctw**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 - 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 + 12.D0*ctw**2*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( s*s4fun**(-1)*deltafun*Yfun + s*u**2*
     &    s4fun**(-1)*deltafun - mz**4*s*s4fun**(-1)*deltafun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * (  - s*s4fun**(-1)*deltafun*Yfun - s*u**2*
     &    s4fun**(-1)*deltafun - 2.D0*s**2*u*s4fun**(-1)*deltafun - 
     &    s**3*s4fun**(-1)*deltafun + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 4.D0*mz**2*s**2*s4fun**(-1)*deltafun - 3.D0*mz**4*
     &    s*s4fun**(-1)*deltafun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & c0cmzuswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 - 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 - 2.D0*
     &    ctw**(-2)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 - 2.D0*ctw**(-2)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 2.D0*ctw**(-2)*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 2.D0*ctw**(-2)*mz**2*s**(-2)
     &    *u*s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rwm2 + 2.D0*ctw**(-2)*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     + 4.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 8.D0*u1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rwm2 + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *u**2*s4fun**(-1)*deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 2.D0*s*u*s4fun**(-1)*deltafun + 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 4.D0*
     &    mz**2*u*s4fun**(-1)*deltafun - 8.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*ctw**2*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 - 12.D0*ctw**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*ctw**2*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*rwm2 - 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*deltafun*rwm2*rzm2 - 24.
     &    D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*u**2*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*
     &    s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*ctw**2*mz**2*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & c0cmztswww * ( 2.D0*ctw**(-2)*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun*rzm2 + 2.D0*ctw**(-2)*s**(-2)*u**2*s4fun**(-1)*deltafun*
     &    Yfun + ctw**(-2)*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 4.D0
     &    *ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 3.D0*
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 4.D0*
     &    ctw**(-2)*s**(-1)*u**2*s4fun**(-1)*deltafun*rwm2 + 2.D0*
     &    ctw**(-2)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*rwm2 + 2.D0*
     &    ctw**(-2)*s4fun**(-1)*deltafun*rwm2*rzm2 + ctw**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 6.D0*ctw**(-2)*u*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*u**2*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*deltafun*rwm2 + 
     &    4.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 2.D
     &    0*ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0
     &    *ctw**(-2)*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 6.D0
     &    *ctw**(-2)*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.D0
     &    *ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun - 12.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*ctw**(-2)*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    ctw**(-2)*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 4.D0
     &    *ctw**(-2)*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    ctw**(-2)*mz**4*s**(-1)*s4fun**(-1)*deltafun*rwm2 + 6.D0*
     &    ctw**(-2)*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 8.D0*
     &    s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-2)*u**2*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*s**(-1)*s4fun**(-1)*deltafun
     &    *Yfun*rzm2 - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2
     &     - 12.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 - 8.D0*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*Yfun*rwm2 - 8.D0*s4fun**(-1)*deltafun*rwm2*rzm2 - 4.
     &    D0*s4fun**(-1)*deltafun*Yfun - 4.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 - 24.D0*u*s4fun**(-1)*deltafun*rwm2 - 8.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 4.D0*u**2*s4fun**(-1)
     &    *deltafun - 2.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    s4fun**(-1)*deltafun*rwm2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*rwm2 - 6.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 - 2.D0*s**2*s4fun**(-1)
     &    *deltafun + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 24.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*rwm2*rzm2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun + 48.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 + 4.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 32.D0*mz**2*s4fun**(-1)*deltafun*rwm2 + 32.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 12.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 16.D0*
     &    mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*s**(-1)
     &    *s4fun**(-1)*deltafun*rwm2 - 24.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 8.D0*mz**4*s4fun**(-1)*deltafun
     &     + 24.D0*ctw**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 24.
     &    D0*ctw**2*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*rwm2*rzm2 + 36.D0*ctw**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun + 48.D0*ctw**2*s**(-1)*
     &    u**2*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rwm2 + 24.D0*ctw**2*s4fun**(-1)*
     &    deltafun*rwm2*rzm2 + 12.D0*ctw**2*s4fun**(-1)*deltafun*Yfun
     &     + 72.D0*ctw**2*u*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s*
     &    s4fun**(-1)*deltafun*rwm2 + 48.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 + 24.D0*ctw**2*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*rwm2 - 24.D0*ctw**2*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 72.D0*ctw**2*mz**2*s**(-2)*u
     &    *s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 144.D0*ctw**2*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rwm2 - 96.D0*ctw**2*mz**2*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 - 96.D0*ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 48.D0*ctw**2*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 + 72.D0*ctw**2*mz**4*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*rwm2 )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & c0ccqswww * ( 4.D0*s*u*s4fun**(-1)*deltafun + 2.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*mz**2*s*s4fun**(-1)*deltafun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & b0fmzww * ( 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 2.D0*ctw**(-2)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 4.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*s**(-1)*
     &    u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 24.D0*ctw**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 48.D0*ctw**2*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 12.D0*ctw**2*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & b0fusww * (  - 2.D0*ctw**(-2)*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )
      ewamp(2,2,2,3) = ewamp(2,2,2,3) + g**4*stw**2*betaZfun*
     & b0ftsww * (  - 2.D0*ctw**(-2)*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 2.D0*ctw**(-2)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*ctw**(-2)*mz**2*s**(-1)*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 24.D0*ctw**2*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *Yfun + 48.D0*ctw**2*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 12.D0*ctw**2*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun )

      ewamp(2,2,2,2) =
     &  + g**4*stw**2 * ( 2.D0*ctw**(-2)*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s*t1fun**(-1)*u1fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 4.D0
     &    *mz**2*s*t1fun**(-1)*u1fun**(-1) + 24.D0*ctw**2*mz**2*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*denh12p * ( 6.D0
     &    *ctw**(-2)*rwm2 + ctw**(-2)*mh**2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*
     & d0cmzcmztsuswwww * ( 1.D0/2.D0*ctw**(-2)*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun**2 + 2.D0*ctw**(-2)*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 + 2.D0*ctw**(-2)*rwm2**2 - 2.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 2.D0*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 - 8.D0*s**(-1)*s2fun*s4fun**(-1)*
     &    Yfun*rwm2 - 8.D0*rwm2**2 + 8.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*rwm2 - 4.D0*mz**2*s**(-1)*Yfun + 6.D0*ctw**2*s**(-2)*
     &    s2fun*s4fun**(-1)*Yfun**2 + 24.D0*ctw**2*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*rwm2 - 8.D0*ctw**2*s**(-1)*s2fun*Yfun + 24.D0
     &    *ctw**2*rwm2**2 - 16.D0*ctw**2*s2fun*rwm2 + 4.D0*ctw**2*s*
     &    s2fun - 24.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*Yfun*rwm2 - 8.D
     &    0*ctw**2*mz**4 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*
     & d0ccmzmzqsuswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*
     & d0ccmzmzqstswwww * ( 2.D0*ctw**(-2)*rwm2**2 + 2.D0*ctw**(-2)*
     &    mz**4*s4fun**(-1)*rwm2 - 8.D0*rwm2**2 + 2.D0*mz**2*s - 8.D0*
     &    mz**4*s4fun**(-1)*rwm2 + 24.D0*ctw**2*rwm2**2 - 16.D0*ctw**2*
     &    s2fun*rwm2 + 4.D0*ctw**2*s*s2fun + 24.D0*ctw**2*mz**4*
     &    s4fun**(-1)*rwm2 - 8.D0*ctw**2*mz**4 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*c0cmzuswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*ctw**(-2)*
     &    s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s2fun*
     &    s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    rwm2 - 4.D0*ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0
     &    *s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 4.D0*s**(-2)*u*s2fun*
     &    s4fun**(-1)*Yfun - 8.D0*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*rwm2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 - 8.D0*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2
     &    *rzm2 + 8.D0*mz**2*s**(-1)*rzm2 + 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 + 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 - 8.
     &    D0*mz**2*s**(-1)*u - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*
     &    Yfun*rzm2 + 12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 
     &    24.D0*ctw**2*s**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2
     &     + 16.D0*ctw**2*s**(-1)*s2fun*rzm2 + 48.D0*ctw**2*s**(-1)*u*
     &    s2fun*s4fun**(-1)*rwm2 - 16.D0*ctw**2*s**(-1)*u*s2fun + 24.D0
     &    *ctw**2*s2fun*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s**(-1)*
     &    s4fun**(-1)*rwm2*rzm2 - 48.D0*ctw**2*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s**(-1)*u*s4fun**(-1)*
     &    rwm2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*c0cmztswww * ( 
     &     - ctw**(-2)*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - ctw**(-2)*
     &    s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - ctw**(-2)*s**(-1)*
     &    s2fun*s4fun**(-1)*Yfun - 4.D0*ctw**(-2)*s**(-1)*u*s2fun*
     &    s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s2fun*s4fun**(-1)*rwm2 + 2.D
     &    0*ctw**(-2)*mz**2*s**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2 + 4.D0*
     &    ctw**(-2)*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 4.D0*ctw**(-2)*
     &    mz**2*s4fun**(-1)*rwm2 - 8.D0*ctw**(-2)*mz**4*s**(-1)*
     &    s4fun**(-1)*rwm2 + 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2
     &     + 4.D0*s**(-2)*u*s2fun*s4fun**(-1)*Yfun - 8.D0*s**(-1)*
     &    t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 + 4.D0*s**(-1)*s2fun*
     &    s4fun**(-1)*Yfun + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 8.
     &    D0*s2fun*s4fun**(-1)*rwm2 - 8.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 16.D0*mz**2*s**(-1)*s4fun**(-1)*rwm2*rzm2
     &     + 8.D0*mz**2*s**(-1)*rzm2 - 16.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*rwm2 - 16.D0*mz**2*s**(-1)*u*s4fun**(-1)*rwm2 + 8.
     &    D0*mz**2*s**(-1)*u - 16.D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*
     &    mz**2 + 32.D0*mz**4*s**(-1)*s4fun**(-1)*rwm2 - 16.D0*mz**4*
     &    s**(-1) - 12.D0*ctw**2*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 
     &    12.D0*ctw**2*s**(-2)*u*s2fun*s4fun**(-1)*Yfun + 24.D0*ctw**2*
     &    s**(-1)*t1fun**(-1)*s2fun*s4fun**(-1)*Yfun*rwm2 - 12.D0*
     &    ctw**2*s**(-1)*s2fun*s4fun**(-1)*Yfun + 16.D0*ctw**2*s**(-1)*
     &    s2fun*rzm2 - 48.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1)*rwm2 + 
     &    16.D0*ctw**2*s**(-1)*u*s2fun - 24.D0*ctw**2*s2fun*s4fun**(-1)
     &    *rwm2 + 16.D0*ctw**2*s2fun + 24.D0*ctw**2*mz**2*s**(-2)*s2fun
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s**(-1)*s4fun**(-1)*
     &    rwm2*rzm2 + 48.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1)*rwm2
     &     - 32.D0*ctw**2*mz**2*s**(-1)*s2fun + 48.D0*ctw**2*mz**2*
     &    s**(-1)*u*s4fun**(-1)*rwm2 + 48.D0*ctw**2*mz**2*s4fun**(-1)*
     &    rwm2 - 96.D0*ctw**2*mz**4*s**(-1)*s4fun**(-1)*rwm2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*c0ccqswww * ( 
     &     - 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 16.D0*mz**2*
     &    s4fun**(-1)*rwm2 - 48.D0*ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*c0ccqswww*
     & denh12p * (  - 12.D0*ctw**(-2)*rwm2**2 + 8.D0*ctw**(-2)*s*rwm2
     &     - 2.D0*ctw**(-2)*mh**2*rwm2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*b0fmzww * (  - 
     &    ctw**(-2)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + ctw**(-2)*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*ctw**(-2)*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) + 3.D0/2.D0*ctw**(-2)*mz**2*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s2fun*s4fun**(-1) + 2.D0
     &    *u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1) + 2.D0*s*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 4.D0*mz**2
     &    *s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0
     &    *ctw**2*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*u*u1fun**(-1)*s2fun*s4fun**(-1) - 6.
     &    D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*
     &    ctw**2*mz**2*s**(-1)*u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 48.D
     &    0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) + 18.D0*ctw**2*mz**2
     &    *t1fun**(-1)*s2fun*s4fun**(-1) + 6.D0*ctw**2*mz**2*
     &    u1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*b0fusww * (  - 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) + 4.D0*s**(-1)*u*s2fun*s4fun**(-1) + 2.D0*u
     &    *u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 4.D0*mz**2*s**(-1)*s2fun
     &    *s4fun**(-1) + 2.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) - 12.D
     &    0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) - 6.D0*ctw**2*u*
     &    u1fun**(-1)*s2fun*s4fun**(-1) - 12.D0*ctw**2*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1) - 6.D0*ctw**2*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*b0ftsww * ( 
     &    ctw**(-2)*s**(-1)*u*s2fun*s4fun**(-1) + ctw**(-2)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - ctw**(-2)*mz**2*s**(-1)*t1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 3.D0*ctw**(-2)*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*s**(-1)*u*s2fun*s4fun**(-1) - 4.D0*s2fun*
     &    s4fun**(-1) - 2.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 2.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 12.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 6.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     + 12.D0*ctw**2*s**(-1)*u*s2fun*s4fun**(-1) + 12.D0*ctw**2*
     &    s2fun*s4fun**(-1) + 6.D0*ctw**2*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) + 6.D0*ctw**2*s*t1fun**(-1)*s2fun*s4fun**(-1) - 
     &    12.D0*ctw**2*mz**2*s**(-1)*t1fun**(-2)*s2fun*s4fun**(-1)*Yfun
     &     - 36.D0*ctw**2*mz**2*s**(-1)*s2fun*s4fun**(-1) - 18.D0*
     &    ctw**2*mz**2*t1fun**(-1)*s2fun*s4fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun * ( 
     &    ctw**(-2)*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun - 4.D0*s
     &    *t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 12.D0*ctw**2*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & d0cmzcmztsuswwww * ( 1.D0/2.D0*ctw**(-2)*s**(-1)*s4fun**(-1)*
     &    Yfun**2 + 2.D0*ctw**(-2)*s4fun**(-1)*Yfun*rwm2 - 2.D0*s**(-1)
     &    *s4fun**(-1)*Yfun**2 - 8.D0*s4fun**(-1)*Yfun*rwm2 - 4.D0*
     &    mz**2*s4fun**(-1)*Yfun + 6.D0*ctw**2*s**(-1)*s4fun**(-1)*
     &    Yfun**2 + 24.D0*ctw**2*s4fun**(-1)*Yfun*rwm2 - 8.D0*ctw**2*
     &    Yfun - 16.D0*ctw**2*s*rwm2 + 4.D0*ctw**2*s**2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqsuswwww * ( 2.D0*mz**2*s*s2fun*s4fun**(-1) - 16.D0*
     &    ctw**2*s*rwm2 + 4.D0*ctw**2*s**2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & d0ccmzmzqstswwww * ( 2.D0*mz**2*s*s2fun*s4fun**(-1) - 16.D0*
     &    ctw**2*s*rwm2 + 4.D0*ctw**2*s**2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & c0cmzuswww * (  - ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 4.D0*ctw**(-2)*u*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*s*s4fun**(-1)*rwm2 - 4.D0*
     &    ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 4.D0*s**(-1)*s4fun**(-1)*
     &    Yfun*rzm2 - 4.D0*s**(-1)*u*s4fun**(-1)*Yfun - 8.D0*
     &    u1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - 16.D0*u*s4fun**(-1)*rwm2
     &     - 8.D0*s*s4fun**(-1)*rwm2 + 8.D0*mz**2*s4fun**(-1)*rzm2 + 16.
     &    D0*mz**2*s4fun**(-1)*rwm2 - 8.D0*mz**2*u*s4fun**(-1) - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun + 24.D0*ctw**2*u1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 + 16.D0*ctw**2*rzm2 + 48.D0*ctw**2*u*s4fun**(-1)*rwm2
     &     - 16.D0*ctw**2*u + 24.D0*ctw**2*s*s4fun**(-1)*rwm2 - 48.D0*
     &    ctw**2*mz**2*s4fun**(-1)*rwm2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & c0cmztswww * (  - ctw**(-2)*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 
     &    ctw**(-2)*s**(-1)*u*s4fun**(-1)*Yfun + 2.D0*ctw**(-2)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 - ctw**(-2)*s4fun**(-1)*
     &    Yfun - 4.D0*ctw**(-2)*u*s4fun**(-1)*rwm2 - 2.D0*ctw**(-2)*s*
     &    s4fun**(-1)*rwm2 + 2.D0*ctw**(-2)*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s4fun**(-1)*rwm2 + 4.D0*s**(-1)*
     &    s4fun**(-1)*Yfun*rzm2 + 4.D0*s**(-1)*u*s4fun**(-1)*Yfun - 8.D0
     &    *t1fun**(-1)*s4fun**(-1)*Yfun*rwm2 + 4.D0*s4fun**(-1)*Yfun + 
     &    16.D0*u*s4fun**(-1)*rwm2 + 8.D0*s*s4fun**(-1)*rwm2 - 8.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2*s4fun**(-1)*rzm2
     &     - 16.D0*mz**2*s4fun**(-1)*rwm2 + 8.D0*mz**2*u*s4fun**(-1) + 
     &    8.D0*mz**2*s*s4fun**(-1) - 16.D0*mz**4*s4fun**(-1) - 12.D0*
     &    ctw**2*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 12.D0*ctw**2*s**(-1)*u
     &    *s4fun**(-1)*Yfun + 24.D0*ctw**2*t1fun**(-1)*s4fun**(-1)*Yfun
     &    *rwm2 - 12.D0*ctw**2*s4fun**(-1)*Yfun + 16.D0*ctw**2*rzm2 - 
     &    48.D0*ctw**2*u*s4fun**(-1)*rwm2 + 16.D0*ctw**2*u - 24.D0*
     &    ctw**2*s*s4fun**(-1)*rwm2 + 16.D0*ctw**2*s + 24.D0*ctw**2*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*
     &    s4fun**(-1)*rwm2 - 32.D0*ctw**2*mz**2 )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & c0ccqswww * (  - 8.D0*mz**2*s*s4fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & b0fmzww * (  - ctw**(-2)*s*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*u
     &    *t1fun**(-1)*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*s*u*
     &    u1fun**(-1)*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s**2*
     &    t1fun**(-1)*s4fun**(-1) + ctw**(-2)*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun + ctw**(-2)*mz**2*u1fun**(-2)*s4fun**(-1)*
     &    Yfun + 4.D0*ctw**(-2)*mz**2*s4fun**(-1) + 3.D0/2.D0*ctw**(-2)
     &    *mz**2*s*t1fun**(-1)*s4fun**(-1) + 1.D0/2.D0*ctw**(-2)*mz**2*
     &    s*u1fun**(-1)*s4fun**(-1) + 4.D0*s*s4fun**(-1) + 2.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1) - 2.D0*s*u*u1fun**(-1)*s4fun**(-1) + 
     &    2.D0*s**2*t1fun**(-1)*s4fun**(-1) - 4.D0*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun - 4.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 
     &    16.D0*mz**2*s4fun**(-1) - 6.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 2.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) - 12.D0*
     &    ctw**2*s*s4fun**(-1) - 6.D0*ctw**2*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 6.D0*ctw**2*s*u*u1fun**(-1)*s4fun**(-1) - 6.D0*
     &    ctw**2*s**2*t1fun**(-1)*s4fun**(-1) + 12.D0*ctw**2*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 12.D0*ctw**2*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*Yfun + 48.D0*ctw**2*mz**2*s4fun**(-1) + 18.D0*
     &    ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1) + 6.D0*ctw**2*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & b0fusww * (  - ctw**(-2)*u*s4fun**(-1) - 1.D0/2.D0*ctw**(-2)*s*u
     &    *u1fun**(-1)*s4fun**(-1) - ctw**(-2)*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - ctw**(-2)*mz**2*s4fun**(-1) - 1.D0/2.D0*
     &    ctw**(-2)*mz**2*s*u1fun**(-1)*s4fun**(-1) + 4.D0*u*
     &    s4fun**(-1) + 2.D0*s*u*u1fun**(-1)*s4fun**(-1) + 4.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 4.D0*mz**2*s4fun**(-1) + 2.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) - 12.D0*ctw**2*u*s4fun**(-1)
     &     - 6.D0*ctw**2*s*u*u1fun**(-1)*s4fun**(-1) - 12.D0*ctw**2*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 12.D0*ctw**2*mz**2*
     &    s4fun**(-1) - 6.D0*ctw**2*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      ewamp(2,2,2,2) = ewamp(2,2,2,2) + g**4*stw**2*betaZfun*
     & b0ftsww * ( ctw**(-2)*u*s4fun**(-1) + ctw**(-2)*s*s4fun**(-1) + 
     &    1.D0/2.D0*ctw**(-2)*s*u*t1fun**(-1)*s4fun**(-1) + 1.D0/2.D0*
     &    ctw**(-2)*s**2*t1fun**(-1)*s4fun**(-1) - ctw**(-2)*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun - 3.D0*ctw**(-2)*mz**2*
     &    s4fun**(-1) - 3.D0/2.D0*ctw**(-2)*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*u*s4fun**(-1) - 4.D0*s*s4fun**(-1) - 2.D0*
     &    s*u*t1fun**(-1)*s4fun**(-1) - 2.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 12.D0
     &    *mz**2*s4fun**(-1) + 6.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 
     &    12.D0*ctw**2*u*s4fun**(-1) + 12.D0*ctw**2*s*s4fun**(-1) + 6.D0
     &    *ctw**2*s*u*t1fun**(-1)*s4fun**(-1) + 6.D0*ctw**2*s**2*
     &    t1fun**(-1)*s4fun**(-1) - 12.D0*ctw**2*mz**2*t1fun**(-2)*
     &    s4fun**(-1)*Yfun - 36.D0*ctw**2*mz**2*s4fun**(-1) - 18.D0*
     &    ctw**2*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      endif

c$$$      ewamp(2,1,1,1) = ewamp(1,1,1,1)
c$$$      ewamp(2,1,1,2) = ewamp(1,1,1,2)
c$$$      ewamp(2,1,1,3) = -ewamp(1,1,1,3)
c$$$      ewamp(2,1,2,1) = ewamp(1,1,2,1)
c$$$      ewamp(2,1,2,2) = ewamp(1,1,2,2)
c$$$      ewamp(2,1,2,3) = -ewamp(1,1,2,3)
c$$$      ewamp(2,2,1,1) = ewamp(1,2,1,1)
c$$$      ewamp(2,2,1,2) = ewamp(1,2,1,2)
c$$$      ewamp(2,2,1,3) = -ewamp(1,2,1,3)
c$$$      ewamp(2,2,2,1) = ewamp(1,2,2,1)
c$$$      ewamp(2,2,2,2) = ewamp(1,2,2,2)
c$$$      ewamp(2,2,2,3) = -ewamp(1,2,2,3)
*}}}
***************************
* QED helicity amplitudes *
***************************
* {{{

c$$$      qedamp(1,1,1,1) = 0d0
c$$$      qedamp(1,1,1,2) = 0d0
c$$$      qedamp(1,1,1,3) = 0d0
c$$$      qedamp(1,1,2,1) = 0d0
c$$$      qedamp(1,1,2,2) = 0d0
c$$$      qedamp(1,1,2,3) = 0d0
c$$$      qedamp(1,2,1,1) = 0d0
c$$$      qedamp(1,2,1,2) = 0d0
c$$$      qedamp(1,2,1,3) = 0d0
c$$$      qedamp(1,2,2,1) = 0d0
c$$$      qedamp(1,2,2,2) = 0d0
c$$$      qedamp(1,2,2,3) = 0d0

      Nc = 3
      if (iqed.eq.1) then
      do i=1,9 ! type count of fermion loop particles
*like electron
      if (i.eq.1) then
      mgen2 = relm2
      mgen2i = elm2
      af  = ael
      vf  = vel
      qff = qel
      cqfactor = qff**2*alpha**2/(2*stw*ctw)**2
      endif
      if (i.eq.2) then
      mgen2 = rmom2
      mgen2i = mom2
      af  = amo
      vf  = vmo
      qff = qmo
      cqfactor = qff**2*alpha**2/(2*stw*ctw)**2
      endif
      if (i.eq.3) then
      mgen2 = rtam2
      mgen2i = tam2
      af  = ata
      vf  = vta
      qff = qta
      cqfactor = qta**2*alpha**2/(2*stw*ctw)**2
      endif
*like up-quark (only up and charm)
      if (i.eq.4) then
      mgen2 = rupm2
      mgen2i = upm2
      af  = aup
      vf  = vup
      qff = qup
      cqfactor = Nc*qup**2*alpha**2/(2*stw*ctw)**2
      endif
      if (i.eq.5) then
      mgen2 = rchm2
      mgen2i = chm2
      af  = ach
      vf  = vch
      qff = qch
      cqfactor = Nc*qff**2*alpha**2/(2*stw*ctw)**2
      endif
*like down-quark (only down and strange)
      if (i.eq.6) then
      mgen2 = rdnm2
      mgen2i = dnm2
      af  = adn
      vf  = vdn
      qff = qdn
      cqfactor = Nc*qdn**2*alpha**2/(2*stw*ctw)**2
      endif
      if (i.eq.7) then
      mgen2 = rstm2
      mgen2i = stm2
      af  = ast
      vf  = vst
      qff = qst
      cqfactor = Nc*qff**2*alpha**2/(2*stw*ctw)**2
      endif
*like bottom-quark (other mass)
      if (i.eq.8) then
      mgen2 = rbtm2
      mgen2i = btm2
      af  = abt
      vf  = vbt
      qff = qbt
      cqfactor = Nc*qff**2*alpha**2/(2*stw*ctw)**2
      endif
*like top-quark (other mass)
      if (i.eq.9) then
      mgen2 = rtpm2
      mgen2i = tpm2
      af  = atp
      vf  = vtp
      qff = qtp
      cqfactor = Nc*qff**2*alpha**2/(2*stw*ctw)**2
      endif

c$$$      Nc = 3
c$$$      if (iqed.eq.1) then
c$$$      do i=1,5 ! type count of fermion loop particles
c$$$*like electron
c$$$      if (i.eq.1) then
c$$$      mgen2 = relm2
c$$$      mgen2i = elm2
c$$$      af = ael
c$$$      vf = vel
c$$$      cqfactor = 3d0*qel**2*alpha**2/(2*stw*ctw)**2
c$$$      endif
c$$$*like up-quark (only up and charm)
c$$$      if (i.eq.2) then
c$$$      mgen2 = rupm2
c$$$      mgen2i = upm2
c$$$      af = aup
c$$$      vf = vup
c$$$      cqfactor = 2d0*Nc*qup**2*alpha**2/(2*stw*ctw)**2
c$$$      endif
c$$$*like down-quark (only down and strange)
c$$$      if (i.eq.3) then
c$$$      mgen2 = rdnm2
c$$$      mgen2i = dnm2
c$$$      af = adn
c$$$      vf = vdn
c$$$      cqfactor = 2d0*Nc*qdn**2*alpha**2/(2*stw*ctw)**2
c$$$      endif
c$$$*like bottom-quark (other mass)
c$$$      if (i.eq.4) then
c$$$      mgen2 = rbtm2
c$$$      mgen2i = btm2
c$$$      af = abt
c$$$      vf = vbt
c$$$      cqfactor = Nc*qbt**2*alpha**2/(2*stw*ctw)**2
c$$$      endif
c$$$*like top-quark (other mass)
c$$$      if (i.eq.5) then
c$$$      mgen2 = rtpm2
c$$$      mgen2i = tpm2
c$$$      af = atp
c$$$      vf = vtp
c$$$      cqfactor = Nc*qtp**2*alpha**2/(2*stw*ctw)**2
c$$$      endif

      b0fmzff          = b0f(-rzm2,thmu2,mgen2i,mgen2i)
      b0fqsff          = b0f(-s,thmu2,mgen2i,mgen2i)
      b0ftsff          = b0f(-t,thmu2,mgen2i,mgen2i)
      b0fusff          = b0f(-u,thmu2,mgen2i,mgen2i)
      c0ccqsfff        = -c0(0d0,0d0,s,mgen2,mgen2,mgen2)
      c0cmztsfff       = -c0(0d0,rzm2,t,mgen2,mgen2,mgen2)
      c0cmzusfff       = -c0(0d0,rzm2,u,mgen2,mgen2,mgen2)
      c0mzmzqsfff      = -c0(rzm2,rzm2,s,mgen2,mgen2,mgen2)
      d0ccmzmzqstsffff = d0(0d0,0d0,rzm2,rzm2,s,t,mgen2,mgen2,mgen2,mgen2)
      d0ccmzmzqsusffff = d0(0d0,0d0,rzm2,rzm2,s,u,mgen2,mgen2,mgen2,mgen2)
      d0cmzcmztsusffff = d0(0d0,rzm2,0d0,rzm2,t,u,mgen2,mgen2,mgen2,mgen2)

      qedamp(1,1,1,1) =
     &  + cqfactor*denh12p * (  - 16.D0*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + cqfactor*c0ccqsfff*
     & denh12p * ( 32.D0*mgen2**2 - 8.D0*s*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     - 16.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*mgen2 + 2.D0*s**(-1)*
     &    s2fun*Yfun - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 4.D0*s**(-1)*s2fun*rzm2 - 32.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.
     &    D0*s2fun*s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.
     &    D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 + 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun
     &     - 4.D0*s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s**(-1)*u*s2fun + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 8.D0*mz**2*
     &    s**(-1)*s2fun - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*s4fun**(-1)*mgen2 + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 - 16.D0
     &    *s4fun**(-1)*Yfun*mgen2 + 2.D0*Yfun + 4.D0*s*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 4.D0*s*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 4.D0*s*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 - 4.D0*rzm2 - 32.D0*u*s4fun**(-1)*mgen2 + 4.D0*u - 16.D0
     &    *s*s4fun**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*Yfun - 4.D0*rzm2 + 32.D0*u*
     &    s4fun**(-1)*mgen2 - 4.D0*u + 16.D0*s*s4fun**(-1)*mgen2 - 4.D0
     &    *s - 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*s*s4fun**(-1) + 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*s4fun**(-1) + 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**2*s4fun**(-1) + 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1) - 8.D0*s*s4fun**(-1) - 4.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1) - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     + 16.D0*s**(-1)*Yfun*mgen2 - 16.D0*s**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*mgen2 + 2.D0*s**(-1)*s2fun*Yfun + 16.D0*mgen2**2 + 4.D0
     &    *s2fun*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 - 4.D0*
     &    s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*mgen2 - 32.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*
     &    mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 + 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*Yfun - 4.D0*s**(-1)*s2fun*rzm2 - 32.
     &    D0*s**(-1)*u*mgen2 + 32.D0*s**(-1)*u*s2fun*s4fun**(-1)*mgen2
     &     - 4.D0*s**(-1)*u*s2fun - 32.D0*mgen2 + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     + 64.D0*mz**2*s**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s**(-1)*s2fun - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     + 64.D0*mz**4*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 + 2.D0*
     &    Yfun + 4.D0*s*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 4.D0*s*mgen2 - 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 4.D0*s*mgen2 - 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 - 32.D0*s4fun**(-1)*rzm2*mgen2 - 4.D0*rzm2 + 4.D0*u - 
     &    16.D0*s*s4fun**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 - 32.D0*s4fun**(-1)*rzm2*mgen2 + 8.D0*s4fun**(-1)*Yfun
     &     - 4.D0*rzm2 - 4.D0*u - 16.D0*s*s4fun**(-1)*mgen2 - 4.D0*s - 
     &    16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 32.D0*s*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*s*s4fun**(-1) + 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*s4fun**(-1) + 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**2*s4fun**(-1) + 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,1,1) = qedamp(1,1,1,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1) - 8.D0*s*s4fun**(-1) - 4.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1) - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      qedamp(1,1,1,3) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun
     &    *mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s**(-1)*u*
     &    deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 + 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 + 8.D0*mz**2
     &    *s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*t1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*
     &    mgen2 - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 - 24.D
     &    0*mz**4*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*deltafun*mgen2 + 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 - 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*deltafun*mgen2 - 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 + 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 + 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 + 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2 - 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 4.D0*
     &    s**3*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 - 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 
     &    8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2
     &    *s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,1,3) = qedamp(1,1,1,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(1,1,1,2) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(1,1,1,2) = qedamp(1,1,1,2) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )

      qedamp(1,1,3,1) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun
     &    *Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2
     &     - 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 - 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun - 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*s4fun**(-1)
     &    *deltafun*mgen2 + 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 96.D0
     &    *mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s**(-1)*u*deltafun*
     &    Yfun*rzm2**(-1)*mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 + 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *t1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0
     &    *s**2*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*u*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 + 24.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*deltafun*mgen2 - 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 + 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*deltafun*mgen2 + 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 - 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2
     &     + 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 4.D0*s**3*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*
     &    mz**2*s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,1) = qedamp(1,1,3,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(1,1,3,3) =
     &  + cqfactor*denh12p * ( 16.D0/(1.D0 - betaZfun**2)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + cqfactor*c0ccqsfff*
     & denh12p * (  - 32.D0/(1.D0 - betaZfun**2)*mgen2**2 + 8.D0/(1.D0
     &     - betaZfun**2)*s*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + betaZfun**2*cqfactor*
     & denh12p * ( 16.D0/(1.D0 - betaZfun**2)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + betaZfun**2*cqfactor*
     & c0ccqsfff*denh12p * (  - 32.D0/(1.D0 - betaZfun**2)*mgen2**2 + 8.
     &    D0/(1.D0 - betaZfun**2)*s*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor * (  - 32.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*Yfun
     &     - 4.D0*mz**2*s**(-1)*Yfun + 16.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*Yfun + 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*mgen2
     &     + 4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*mgen2 + 32.D0*
     &    mz**2*u*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*Yfun - 64.D0*mz**4*s**(-1)*u*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s4fun**(-1)*mgen2 + 16.D0*mz**6*s**(-2)*
     &    s4fun**(-1)*Yfun + 32.D0*mz**6*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 32.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 32.D0*mz**2*
     &    s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*rzm2 - 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 - 8.D0*mz**2*s**(-1)*u + 32.D
     &    0*mz**2*s**(-1)*u**2*s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*
     &    rzm2 + 8.D0*mz**2*u*s4fun**(-1) + 64.D0*mz**4*s**(-2)*u*
     &    s4fun**(-1)*rzm2 - 64.D0*mz**4*s**(-2)*u**2*s4fun**(-1) + 32.D
     &    0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 32.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1) + 64.D0*mz**4*u1fun**(-1)*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**6*s**(-2)*s4fun**(-1)*rzm2 + 32.D0*mz**6*s**(-2)*u*
     &    s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 32.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 - 32.D0*mz**2*
     &    s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*rzm2 - 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 + 8.D0*mz**2*s**(-1)*u - 64.D
     &    0*mz**2*s**(-1)*u**2*s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*
     &    rzm2 + 8.D0*mz**2 - 40.D0*mz**2*u*s4fun**(-1) - 8.D0*mz**2*s*
     &    s4fun**(-1) + 64.D0*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 + 128.D0
     &    *mz**4*s**(-2)*u**2*s4fun**(-1) + 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*rzm2 - 16.D0*mz**4*s**(-1) + 160.D0*mz**4*s**(-1)
     &    *u*s4fun**(-1) + 64.D0*mz**4*t1fun**(-1)*s4fun**(-1)*mgen2 + 
     &    48.D0*mz**4*s4fun**(-1) - 32.D0*mz**6*s**(-2)*s4fun**(-1)*
     &    rzm2 - 160.D0*mz**6*s**(-2)*u*s4fun**(-1) - 96.D0*mz**6*
     &    s**(-1)*s4fun**(-1) + 64.D0*mz**8*s**(-2)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*c0ccqsfff
     &  * ( 64.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 32.D0
     &    *mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 48.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1) - 16.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 32.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**4*
     &    s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun + 128.D0*mz**4*s**(-1)*u
     &    *t1fun**(-1)*s4fun**(-1) + 80.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 96.D0*
     &    mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) + 32.D0*mz**6*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*b0fusff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**2*u*u1fun**(-1)*s4fun**(-1) + 32.D0*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1) - 32.D0*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + vf**2*cqfactor*b0ftsff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) + 48.D0*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) + 32.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 128.D0*mz**4*s**(-1)*u*t1fun**(-1)*s4fun**(-1) - 80.D0
     &    *mz**4*t1fun**(-1)*s4fun**(-1) + 96.D0*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor * (  - 32.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*Yfun*mgen2 - 16.D0*s2fun*
     &    rzm2**(-1)*mgen2**2 + 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*
     &    Yfun - 4.D0*mz**2*s**(-1)*Yfun + 16.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*Yfun + 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*mgen2
     &     + 4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*mgen2 + 32.D0*
     &    mz**2*u*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*Yfun - 64.D0*mz**4*s**(-1)*u*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s4fun**(-1)*mgen2 + 16.D0*mz**6*s**(-2)*
     &    s4fun**(-1)*Yfun + 32.D0*mz**6*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 - 8.D0*
     &    mz**2*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 - 8.D0*
     &    mz**2*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 32.D0*s**(-1)*rzm2*mgen2 + 32.D0*s**(-1)*u*mgen2 - 32.D0
     &    *mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 32.D0*mz**2*s**(-2)*
     &    u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*rzm2 - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*rzm2 - 8.D0*mz**2*s**(-1)*u + 32.D0*
     &    mz**2*s**(-1)*u**2*s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*rzm2
     &     + 8.D0*mz**2*u*s4fun**(-1) + 64.D0*mz**4*s**(-2)*u*
     &    s4fun**(-1)*rzm2 - 64.D0*mz**4*s**(-2)*u**2*s4fun**(-1) + 32.D
     &    0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 32.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1) + 64.D0*mz**4*u1fun**(-1)*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**6*s**(-2)*s4fun**(-1)*rzm2 + 32.D0*mz**6*s**(-2)*u*
     &    s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 32.D0*s**(-1)*rzm2*mgen2 - 32.D0*s**(-1)*u*mgen2 - 32.D0
     &    *mgen2 - 32.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 - 32.D0*
     &    mz**2*s**(-2)*u**3*s4fun**(-1) + 64.D0*mz**2*s**(-1)*mgen2 + 
     &    8.D0*mz**2*s**(-1)*rzm2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    rzm2 + 8.D0*mz**2*s**(-1)*u - 64.D0*mz**2*s**(-1)*u**2*
     &    s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*rzm2 + 8.D0*mz**2 - 40.D0
     &    *mz**2*u*s4fun**(-1) - 8.D0*mz**2*s*s4fun**(-1) + 64.D0*mz**4
     &    *s**(-2)*u*s4fun**(-1)*rzm2 + 128.D0*mz**4*s**(-2)*u**2*
     &    s4fun**(-1) + 32.D0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 16.D0*
     &    mz**4*s**(-1) + 160.D0*mz**4*s**(-1)*u*s4fun**(-1) + 64.D0*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*mgen2 + 48.D0*mz**4*s4fun**(-1)
     &     - 32.D0*mz**6*s**(-2)*s4fun**(-1)*rzm2 - 160.D0*mz**6*
     &    s**(-2)*u*s4fun**(-1) - 96.D0*mz**6*s**(-1)*s4fun**(-1) + 64.D
     &    0*mz**8*s**(-2)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*c0ccqsfff
     &  * ( 16.D0*s*rzm2**(-1)*mgen2 + 64.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*b0fmzff
     &  * (  - 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 32.D0
     &    *mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 48.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1) - 16.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 32.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**4*
     &    s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun + 128.D0*mz**4*s**(-1)*u
     &    *t1fun**(-1)*s4fun**(-1) + 80.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 96.D0*
     &    mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) + 32.D0*mz**6*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*b0fusff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**2*u*u1fun**(-1)*s4fun**(-1) + 32.D0*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1) - 32.D0*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,3,3) = qedamp(1,1,3,3) + af**2*cqfactor*b0ftsff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) + 48.D0*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) + 32.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 128.D0*mz**4*s**(-1)*u*t1fun**(-1)*s4fun**(-1) - 80.D0
     &    *mz**4*t1fun**(-1)*s4fun**(-1) + 96.D0*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) )

      qedamp(1,1,3,2) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 96.D0
     &    *mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s**(-1)*u*
     &    deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 + 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 + 8.D0*mz**2
     &    *s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*t1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*
     &    mgen2 - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 - 24.D
     &    0*mz**4*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*deltafun*mgen2 + 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 - 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*deltafun*mgen2 - 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 + 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 + 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 + 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2
     &     + 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 4.D0*s**3*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*
     &    mz**2*s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,3,2) = qedamp(1,1,3,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(1,1,2,1) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(1,1,2,1) = qedamp(1,1,2,1) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )

      qedamp(1,1,2,3) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun
     &    *Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2
     &     - 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 - 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun - 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*s4fun**(-1)
     &    *deltafun*mgen2 + 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun
     &    *mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s**(-1)*u*deltafun*
     &    Yfun*rzm2**(-1)*mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 + 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *t1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0
     &    *s**2*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*u*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 + 24.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*deltafun*mgen2 - 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 + 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*deltafun*mgen2 + 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 - 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2 - 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 4.D0*
     &    s**3*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 - 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 
     &    8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2
     &    *s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,1,2,3) = qedamp(1,1,2,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(1,1,2,2) =
     &  + cqfactor*denh12p * (  - 16.D0*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + cqfactor*c0ccqsfff*
     & denh12p * ( 32.D0*mgen2**2 - 8.D0*s*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     - 16.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*mgen2 + 2.D0*s**(-1)*
     &    s2fun*Yfun - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 4.D0*s**(-1)*s2fun*rzm2 - 32.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.
     &    D0*s2fun*s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.
     &    D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 + 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun
     &     - 4.D0*s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s**(-1)*u*s2fun + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 8.D0*mz**2*
     &    s**(-1)*s2fun - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*s4fun**(-1)*mgen2 + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 + 16.D0*
     &    s4fun**(-1)*Yfun*mgen2 - 2.D0*Yfun - 4.D0*s*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 4.D0*s*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 4.D0*s*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 + 4.D0*rzm2 + 32.D0*u*s4fun**(-1)*mgen2 - 4.D0*u
     &     + 16.D0*s*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*Yfun + 4.D0*rzm2 - 32.D0*u*
     &    s4fun**(-1)*mgen2 + 4.D0*u - 16.D0*s*s4fun**(-1)*mgen2 + 4.D0
     &    *s + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*s*s4fun**(-1) - 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*s4fun**(-1) - 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s4fun**(-1) - 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1) + 8.D0*s*s4fun**(-1) + 4.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     + 16.D0*s**(-1)*Yfun*mgen2 - 16.D0*s**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*mgen2 + 2.D0*s**(-1)*s2fun*Yfun + 16.D0*mgen2**2 + 4.D0
     &    *s2fun*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 - 4.D0*
     &    s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*mgen2 - 32.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*
     &    mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 + 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*Yfun - 4.D0*s**(-1)*s2fun*rzm2 - 32.
     &    D0*s**(-1)*u*mgen2 + 32.D0*s**(-1)*u*s2fun*s4fun**(-1)*mgen2
     &     - 4.D0*s**(-1)*u*s2fun - 32.D0*mgen2 + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     + 64.D0*mz**2*s**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s**(-1)*s2fun - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     + 64.D0*mz**4*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 - 2.D0*
     &    Yfun - 4.D0*s*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 4.D0*s*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 4.D0*s*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 + 32.D0*s4fun**(-1)*rzm2*mgen2 + 4.D0*rzm2 - 4.D0*
     &    u + 16.D0*s*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 + 32.D0*s4fun**(-1)*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    Yfun + 4.D0*rzm2 + 4.D0*u + 16.D0*s*s4fun**(-1)*mgen2 + 4.D0*
     &    s + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 32.D0*s*s4fun**(-1)*mgen2 )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*s*s4fun**(-1) - 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*s4fun**(-1) - 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s4fun**(-1) - 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,1,2,2) = qedamp(1,1,2,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1) + 8.D0*s*s4fun**(-1) + 4.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      qedamp(1,2,1,1) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 - 16.D0*mgen2**2 - 8.D0*mz**2*mgen2 )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) + 
     &    4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*mgen2 + 8.D0*s*s4fun*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 + 16.D0*s4fun**(-1)*Yfun*mgen2 + 16.D0*mgen2**2 - 8.D0*
     &    mz**2*mgen2 )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*s**2*u*Yfun**(-1)*mgen2 + 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 - 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 + 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 - 16.D0*u*Yfun**(-1)*rzm2*mgen2 + 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 + 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 + 32.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 - 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) + 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)
     &     + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 16.D0*mgen2 - 8.D0*s*s4fun*Yfun**(-1)*mgen2 - 8.D
     &    0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,1) = qedamp(1,2,1,1) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )

      qedamp(1,2,1,3) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 8.D0*mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *s4fun**(-1)*deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 16.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    u*s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun - 12.D0*s**3*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 20.D0
     &    *mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)
     &    *s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*
     &    s4fun**(-1)*deltafun - 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s**2*s4fun**(-1)*deltafun - 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*
     &    deltafun + 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     + 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 12.D0*s*u*deltafun*rzm2**(-1)*
     &    mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*
     &    s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    24.D0*mz**2*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s*
     &    deltafun*rzm2**(-1)*mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 24.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 20.D0*mz**2*s**2*s4fun**(-1)*deltafun + 48.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*deltafun*
     &    rzm2**(-1)*mgen2 + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0
     &    *mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*
     &    deltafun*mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 8.D0*deltafun*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*deltafun
     &    *mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**(-1)
     &    *u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*deltafun*
     &    mgen2 - 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 + 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 8.D0*s*deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 - 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*s4fun**(-1)*deltafun
     &     - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 - 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun
     &     - 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*s*deltafun + 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,1,3) = qedamp(1,2,1,3) + af**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(1,2,1,2) =
     &  + vf**2*cqfactor * (  - 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor
     &  * (  - 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 8.D
     &    0*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**4*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 4.D0*s**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 4.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 48.D0*s**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*mgen2 - 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) - 4.
     &    D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 2.D0*s**3*s4fun**(-1)
     &     - 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) - 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**5*s4fun**(-1)*Yfun**(-1) - 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**7*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 + 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**2*s**2*s4fun**(-1) + 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) - 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 - 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) - 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) - 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) - 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 4.D0*s**2*
     &    s4fun**(-1) - 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) - 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**6*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1) + 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     + 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     + 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) - 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) + 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     + 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 - 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0
     &    *s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*s**3*u*s2fun*Yfun**(-2) + 4.D0*s**4*s2fun*
     &    Yfun**(-2) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     + 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) + 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 16.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*s**2*Yfun**(-1)
     &    *mgen2 + 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     + 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) + 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*Yfun**(-1)*mgen2 - 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) - 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) + 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) - 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) - 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 32.D0*u*s4fun**(-1) - 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*s4fun**(-1) + 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 
     &    32.D0*mz**2*s4fun**(-1) + 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) + 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) + 40.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1) - 80.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) - 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + vf**2*betaZfun*cqfactor*
     & b0fqsff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor * (  - 8.D0
     &    *t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*Yfun*mgen2 - 16.D0*
     &    mgen2**2 + 4.D0*s*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor
     &  * (  - 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 8.D
     &    0*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**4*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 4.D0*s**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 4.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 48.D0*s**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*mgen2 - 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) - 4.
     &    D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 2.D0*s**3*s4fun**(-1)
     &     - 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) - 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**5*s4fun**(-1)*Yfun**(-1) - 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**7*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 + 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**2*s**2*s4fun**(-1) + 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) - 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 - 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) - 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) - 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) - 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 4.D0*s**2*
     &    s4fun**(-1) - 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) - 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**6*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1) + 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     + 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     + 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) - 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) + 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     + 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 - 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0
     &    *s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*s**3*u*s2fun*Yfun**(-2) + 4.D0*s**4*s2fun*
     &    Yfun**(-2) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     + 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) + 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 16.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*s**2*Yfun**(-1)
     &    *mgen2 + 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     + 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) + 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*Yfun**(-1)*mgen2 - 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) - 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) + 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) - 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) - 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 32.D0*u*s4fun**(-1) - 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*s4fun**(-1) + 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 
     &    32.D0*mz**2*s4fun**(-1) + 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) + 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) + 40.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1) - 80.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) - 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(1,2,1,2) = qedamp(1,2,1,2) + af**2*betaZfun*cqfactor*
     & b0fqsff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      qedamp(1,2,3,1) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 8.D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     + 16.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *s4fun**(-1)*deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 16.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    u*s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun - 12.D0*s**3*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 20.D0
     &    *mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)
     &    *s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*
     &    s4fun**(-1)*deltafun - 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s**2*s4fun**(-1)*deltafun - 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*
     &    deltafun + 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 12.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 4.D0*s
     &    *u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 - 8.
     &    D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s
     &    *u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**4*deltafun*rzm2**(-1)*mgen2 - 24.D0
     &    *mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*deltafun*mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 8.D0*deltafun*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*deltafun*
     &    mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 32.
     &    D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**(-1)*
     &    u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*deltafun*
     &    mgen2 + 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 - 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*s*deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 + 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*s4fun**(-1)*deltafun
     &     + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*c0ccqsfff
     &  * ( 16.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 - 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun
     &     - 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*s*deltafun + 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,1) = qedamp(1,2,3,1) + af**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(1,2,3,3) =
     &  + vf**2*cqfactor * ( 32.D0*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*mgen2
     &     - 8.D0*mz**2*mgen2 + 32.D0*mz**2*u*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 - 64.D0*mz**4*s**(-1)*u*s4fun**(-1)
     &    *mgen2 - 32.D0*mz**4*s4fun**(-1)*mgen2 + 32.D0*mz**6*s**(-1)*
     &    s4fun**(-1)*mgen2 )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 8.D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0*mz**2
     &    *s**3*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 96.D0
     &    *mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 104.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 80.D0
     &    *mz**8*s*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*c0cmzusfff
     &  * ( 64.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 - 16.
     &    D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 16.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*c0cmztsfff
     &  * ( 64.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 - 16.
     &    D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*mz**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)
     &    *Yfun**(-1) + 64.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D
     &    0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**4*s*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 192.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 80.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 208.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 208.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 160.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*mz**2*s*s4fun**(-1) + 4.D0*mz**2*s*s2fun*
     &    Yfun**(-1) + 4.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*mz**2 - 8.D0*mz**2*s*s4fun**(-1) + 4.D0*
     &    mz**2*s*s4fun*Yfun**(-1) + 4.D0*mz**2*s**2*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) - 16.D0*mz**2*
     &    u**2*u1fun**(-2)*s4fun**(-1) - 32.D0*mz**2*s*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 64.D0
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 80.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*u**2*u1fun**(-2)*s4fun**(-1) + 16.D0*mz**6*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + vf**2*cqfactor*b0ftsff
     &  * ( 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**2*s*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**2*s**2*t1fun**(-2)*
     &    s4fun**(-1) - 64.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 80.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor * ( 32.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 32.D0*s4fun**(-1)*Yfun*mgen2 - 16.D0*s2fun*
     &    rzm2**(-1)*mgen2**2 + 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 32.D0*mz**2*u*s4fun**(-1)*mgen2 + 
     &    8.D0*mz**2*s*s4fun**(-1)*mgen2 - 64.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1)*mgen2 - 32.D0*mz**4*s4fun**(-1)*mgen2 + 32.D0*
     &    mz**6*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 + 8.D0*s
     &    *u**3*Yfun**(-1)*rzm2**(-1)*mgen2 - 8.D0*mz**2*mgen2 - 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     + 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 - 8.D0*s
     &    *u**3*Yfun**(-1)*rzm2**(-1)*mgen2 - 24.D0*s**2*u**2*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 24.D0*s**3*u*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 8.D0*s**4*Yfun**(-1)*rzm2**(-1)*mgen2 - 8.D
     &    0*mz**2*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 - 8.D0*mz**2*s
     &    *u*Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**2*s**2*Yfun**(-1)*mgen2 + 96.D0*mz**2*s**2*u*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**3*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 24.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**2*s**4*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*
     &    Yfun**(-1)*mgen2 - 96.D0*mz**4*s*u*Yfun**(-1)*rzm2**(-1)*
     &    mgen2 + 48.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) - 96.D0*
     &    mz**4*s**2*Yfun**(-1)*rzm2**(-1)*mgen2 + 96.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-1)*rzm2**(-1)*mgen2 - 104.D
     &    0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 80.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*u**3*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 + 64.D0*mz**2*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun*mgen2 - 16.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 
     &    16.D0*mz**2*u*Yfun**(-1)*mgen2 - 16.D0*mz**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*rzm2 + 16.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 
     &    16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*u**3*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 48.D0*s*u**2*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 16.D0*s**2*Yfun**(-1)*mgen2 - 
     &    48.D0*s**2*u*Yfun**(-1)*rzm2**(-1)*mgen2 - 16.D0*s**3*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 + 64.D0*mz**2*s**(-1)*t1fun**(-1)
     &    *s4fun**(-1)*Yfun*mgen2 - 16.D0*mz**2*Yfun**(-1)*rzm2*mgen2
     &     + 48.D0*mz**2*u*Yfun**(-1)*mgen2 - 16.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*u**2*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 16.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) + 
     &    48.D0*mz**2*s*Yfun**(-1)*mgen2 - 32.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 192.D0*mz**2*s*u*Yfun**(-1)*rzm2**(-1)*
     &    mgen2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*s**2*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 48.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 32.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 64.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 192.D0*mz**4*u*Yfun**(-1)*rzm2**(-1)*mgen2 + 96.D0*
     &    mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1)*rzm2 - 192.D0*mz**4*s*Yfun**(-1)*rzm2**(-1)*mgen2
     &     + 192.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 96.D0*mz**4*s**2
     &    *s4fun**(-1)*Yfun**(-1) - 80.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 128.D0*mz**6*Yfun**(-1)*rzm2**(-1)*mgen2 - 208.D0*
     &    mz**6*u*s4fun**(-1)*Yfun**(-1) - 208.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 160.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s*rzm2**(-1)*mgen2 + 8.D0*s*s2fun**2*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 16.D0*mz**2*s*s4fun**(-1) + 4.D0*mz**2*s*
     &    s2fun*Yfun**(-1) + 4.D0*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2 + 8.D0*s*s2fun*
     &    s4fun*Yfun**(-1)*rzm2**(-1)*mgen2 - 8.D0*mz**2 - 8.D0*mz**2*s
     &    *s4fun**(-1) + 4.D0*mz**2*s*s4fun*Yfun**(-1) + 4.D0*mz**2*
     &    s**2*Yfun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) - 16.D0*mz**2*
     &    u**2*u1fun**(-2)*s4fun**(-1) - 32.D0*mz**2*s*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 64.D0
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 80.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*u**2*u1fun**(-2)*s4fun**(-1) + 16.D0*mz**6*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,3,3) = qedamp(1,2,3,3) + af**2*cqfactor*b0ftsff
     &  * ( 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**2*s*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**2*s**2*t1fun**(-2)*
     &    s4fun**(-1) - 64.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 80.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      qedamp(1,2,3,2) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 8.D0*mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *s4fun**(-1)*deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 16.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    u*s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun - 12.D0*s**3*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 20.D0
     &    *mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)
     &    *s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*
     &    s4fun**(-1)*deltafun - 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s**2*s4fun**(-1)*deltafun - 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*
     &    deltafun + 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     + 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 12.D0*s*u*deltafun*rzm2**(-1)*
     &    mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*
     &    s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    24.D0*mz**2*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s*
     &    deltafun*rzm2**(-1)*mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 24.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 20.D0*mz**2*s**2*s4fun**(-1)*deltafun + 48.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*deltafun*
     &    rzm2**(-1)*mgen2 + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0
     &    *mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*
     &    deltafun*mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 8.D0*deltafun*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*deltafun
     &    *mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**(-1)
     &    *u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*deltafun*
     &    mgen2 - 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 + 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 8.D0*s*deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 - 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*s4fun**(-1)*deltafun
     &     - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 - 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun
     &     - 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*s*deltafun + 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,3,2) = qedamp(1,2,3,2) + af**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(1,2,2,1) =
     &  + vf**2*cqfactor * (  - 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor
     &  * ( 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 8.D0*
     &    mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 16.D0*mz**4*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 4.D0*s**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 4.D0*s**2*s4fun**(-1)*mgen2 + 
     &    2.D0*s**2*u*s4fun**(-1) + 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**2*s4fun**(-1)*mgen2
     &     + 2.D0*s**2*u*s4fun**(-1) + 48.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) + 4.D
     &    0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 2.D0*s**3*s4fun**(-1) + 
     &    48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) + 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**5*s4fun**(-1)*Yfun**(-1) + 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**7*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 - 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**2*s**2*s4fun**(-1) - 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) + 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 + 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) + 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) + 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) + 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 4.D0*s**2*
     &    s4fun**(-1) + 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) + 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**6*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) - 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     - 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     - 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) + 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) - 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     - 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 + 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**2*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s**3*u*s2fun*Yfun**(-2) - 4.D0*s**4*s2fun*
     &    Yfun**(-2) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) - 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s*u*
     &    s2fun**2*s4fun**(-1)*Yfun**(-1) - 16.D0*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*s**2*
     &    Yfun**(-1)*mgen2 - 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     - 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) - 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*Yfun**(-1)*mgen2 + 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) + 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) + 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) - 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) + 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) + 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 32.D0*u*s4fun**(-1) + 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) - 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1) - 
     &    32.D0*mz**2*s4fun**(-1) - 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*s*s4fun**(-1) - 40.D0*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*s4fun**(-1)
     &     + 80.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 128.D0*mz**2*s*u
     &    *s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + vf**2*betaZfun*cqfactor*
     & b0fqsff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor * (  - 8.D0
     &    *t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*Yfun*mgen2 - 16.D0*
     &    mgen2**2 + 4.D0*s*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor
     &  * ( 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 8.D0*
     &    mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 16.D0*mz**4*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 4.D0*s**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 4.D0*s**2*s4fun**(-1)*mgen2 + 
     &    2.D0*s**2*u*s4fun**(-1) + 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**2*s4fun**(-1)*mgen2
     &     + 2.D0*s**2*u*s4fun**(-1) + 48.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) + 4.D
     &    0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 2.D0*s**3*s4fun**(-1) + 
     &    48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) + 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**5*s4fun**(-1)*Yfun**(-1) + 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**7*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 - 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**2*s**2*s4fun**(-1) - 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) + 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 + 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) + 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) + 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) + 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 4.D0*s**2*
     &    s4fun**(-1) + 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) + 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**6*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) - 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     - 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     - 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) + 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) - 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     - 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 + 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**2*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s**3*u*s2fun*Yfun**(-2) - 4.D0*s**4*s2fun*
     &    Yfun**(-2) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) - 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s*u*
     &    s2fun**2*s4fun**(-1)*Yfun**(-1) - 16.D0*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*s**2*
     &    Yfun**(-1)*mgen2 - 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     - 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) - 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*Yfun**(-1)*mgen2 + 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) + 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) + 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) - 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) + 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) + 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 32.D0*u*s4fun**(-1) + 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) - 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1) - 
     &    32.D0*mz**2*s4fun**(-1) - 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*s*s4fun**(-1) - 40.D0*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*s4fun**(-1)
     &     + 80.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 128.D0*mz**2*s*u
     &    *s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(1,2,2,1) = qedamp(1,2,2,1) + af**2*betaZfun*cqfactor*
     & b0fqsff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      qedamp(1,2,2,3) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 8.D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     + 16.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *s4fun**(-1)*deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*s4fun**(-1)*
     &    deltafun - 16.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    u*s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 4.D0*s**3*s4fun**(-1)*deltafun - 12.D0*s**3*u**2
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**5*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 20.D0
     &    *mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)
     &    *s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*
     &    u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*
     &    s4fun**(-1)*deltafun - 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s**2*s4fun**(-1)*deltafun - 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*
     &    deltafun + 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 12.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 4.D0*s
     &    *u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 - 8.
     &    D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s
     &    *u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**4*deltafun*rzm2**(-1)*mgen2 - 24.D0
     &    *mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*deltafun*mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 8.D0*deltafun*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*deltafun*
     &    mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 32.
     &    D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**(-1)*
     &    u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*deltafun*
     &    mgen2 + 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 - 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*s*deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 + 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*s4fun**(-1)*deltafun
     &     + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*c0ccqsfff
     &  * ( 16.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D
     &    0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 - 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun + 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun + 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*u**2*s4fun**(-1)*deltafun - 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun
     &     - 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*s*deltafun + 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 4.D0*s*deltafun - 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*
     &    deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(1,2,2,3) = qedamp(1,2,2,3) + af**2*betaZfun*cqfactor*
     & b0fqsff * (  - 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(1,2,2,2) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 - 16.D0*mgen2**2 - 8.D0*mz**2*mgen2 )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) + 
     &    4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*mgen2 + 8.D0*s*s4fun*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 + 16.D0*s4fun**(-1)*Yfun*mgen2 + 16.D0*mgen2**2 - 8.D0*
     &    mz**2*mgen2 )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*s**2*u*Yfun**(-1)*mgen2 + 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 - 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 + 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 - 16.D0*u*Yfun**(-1)*rzm2*mgen2 + 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 + 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 + 32.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 - 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) + 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)
     &     + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*
     & c0mzmzqsfff * ( 16.D0*mgen2 - 8.D0*s*s4fun*Yfun**(-1)*mgen2 - 8.D
     &    0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(1,2,2,2) = qedamp(1,2,2,2) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )

      qedamp(2,1,1,1) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 - 16.D0*mgen2**2 - 8.D0*mz**2*mgen2 )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) + 
     &    4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*mgen2 + 8.D0*s*s4fun*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 + 16.D0*s4fun**(-1)*Yfun*mgen2 + 16.D0*mgen2**2 - 8.D0*
     &    mz**2*mgen2 )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*s**2*u*Yfun**(-1)*mgen2 + 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 - 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 + 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 - 16.D0*u*Yfun**(-1)*rzm2*mgen2 + 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 + 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 + 32.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 - 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) + 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)
     &     + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 16.D0*mgen2 - 8.D0*s*s4fun*Yfun**(-1)*mgen2 - 8.D
     &    0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,1) = qedamp(2,1,1,1) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )

      qedamp(2,1,1,3) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 8.D0*mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *s4fun**(-1)*deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 16.D0*s**2*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*s**2*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0
     &    *mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 48.D0*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*
     &    s4fun**(-1)*deltafun + 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s**2*s4fun**(-1)*deltafun + 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*
     &    deltafun - 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + vf**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     + 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 12.D0*s*u*deltafun*rzm2**(-1)*
     &    mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*
     &    s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    24.D0*mz**2*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s*
     &    deltafun*rzm2**(-1)*mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 24.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 20.D0*mz**2*s**2*s4fun**(-1)*deltafun + 48.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*deltafun*
     &    rzm2**(-1)*mgen2 + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0
     &    *mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*
     &    deltafun*mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 8.D0*deltafun*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*deltafun
     &    *mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**(-1)
     &    *u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*deltafun*
     &    mgen2 - 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 + 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 8.D0*s*deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 - 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*s4fun**(-1)*deltafun
     &     - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)
     &    *mgen2 )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun + 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun
     &     + 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*s*deltafun - 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,1,3) = qedamp(2,1,1,3) + af**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(2,1,1,2) =
     &  + vf**2*cqfactor * (  - 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor
     &  * ( 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 8.D0*
     &    mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 16.D0*mz**4*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 4.D0*s**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 4.D0*s**2*s4fun**(-1)*mgen2 + 
     &    2.D0*s**2*u*s4fun**(-1) + 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**2*s4fun**(-1)*mgen2
     &     + 2.D0*s**2*u*s4fun**(-1) + 48.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) + 4.D
     &    0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 2.D0*s**3*s4fun**(-1) + 
     &    48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) + 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**5*s4fun**(-1)*Yfun**(-1) + 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**7*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 - 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**2*s**2*s4fun**(-1) - 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) + 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 + 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) + 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) + 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) + 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 4.D0*s**2*
     &    s4fun**(-1) + 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) + 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**6*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) - 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     - 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     - 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) + 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) - 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     - 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 + 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**2*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s**3*u*s2fun*Yfun**(-2) - 4.D0*s**4*s2fun*
     &    Yfun**(-2) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) - 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s*u*
     &    s2fun**2*s4fun**(-1)*Yfun**(-1) - 16.D0*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*s**2*
     &    Yfun**(-1)*mgen2 - 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     - 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) - 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*Yfun**(-1)*mgen2 + 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) + 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) + 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) - 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) + 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) + 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 32.D0*u*s4fun**(-1) + 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) - 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1) - 
     &    32.D0*mz**2*s4fun**(-1) - 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*s*s4fun**(-1) - 40.D0*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*s4fun**(-1)
     &     + 80.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 128.D0*mz**2*s*u
     &    *s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + vf**2*betaZfun*cqfactor*
     & b0fqsff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor * (  - 8.D0
     &    *t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*Yfun*mgen2 - 16.D0*
     &    mgen2**2 + 4.D0*s*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor
     &  * ( 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 8.D0*
     &    mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 16.D0*mz**4*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 4.D0*s**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 4.D0*s**2*s4fun**(-1)*mgen2 + 
     &    2.D0*s**2*u*s4fun**(-1) + 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 16.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**2*s4fun**(-1)*mgen2
     &     + 2.D0*s**2*u*s4fun**(-1) + 48.D0*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) + 4.D
     &    0*s**2*u**5*s4fun**(-1)*Yfun**(-2) + 2.D0*s**3*s4fun**(-1) + 
     &    48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*s**3*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 + 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) + 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**5*s4fun**(-1)*Yfun**(-1) + 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) + 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*s**7*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 - 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 4.D0*mz**2*s**2*s4fun**(-1) - 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) - 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) - 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) + 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 + 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) + 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) + 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) + 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 4.D0*s*s4fun**(-1)*rzm2 + 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 4.D0*s**2*
     &    s4fun**(-1) + 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) + 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 + 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) + 8.D0*s**6*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) - 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     - 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) - 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     - 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) - 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 - 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) - 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) + 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 - 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) - 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 + 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) + 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) - 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 - 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     - 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 - 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) - 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) + 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 + 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 
     &    8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0*s**2*u*
     &    s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s**3*u*s2fun*Yfun**(-2) - 4.D0*s**4*s2fun*
     &    Yfun**(-2) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) - 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) + 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s*u*
     &    s2fun**2*s4fun**(-1)*Yfun**(-1) - 16.D0*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*s**2*
     &    Yfun**(-1)*mgen2 - 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     - 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) - 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) - 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s*Yfun**(-1)*mgen2 + 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) + 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) + 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) + 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) - 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) + 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) + 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) + 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) - 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 32.D0*u*s4fun**(-1) + 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) - 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*Yfun**(-1) - 
     &    32.D0*mz**2*s4fun**(-1) - 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) - 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2
     &    *u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 4.D0 - 16.D0*u*s4fun**(-1) - 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*s*s4fun**(-1) - 40.D0*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*s4fun**(-1)
     &     + 80.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 128.D0*mz**2*s*u
     &    *s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(2,1,1,2) = qedamp(2,1,1,2) + af**2*betaZfun*cqfactor*
     & b0fqsff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) + 8.D0*s**2*
     &    s2fun*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      qedamp(2,1,3,1) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 8.D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     + 16.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *s4fun**(-1)*deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 16.D0*s**2*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*s**2*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0
     &    *mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 48.D0*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*
     &    s4fun**(-1)*deltafun + 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s**2*s4fun**(-1)*deltafun + 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*
     &    deltafun - 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + vf**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 12.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 4.D0*s
     &    *u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 - 8.
     &    D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s
     &    *u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**4*deltafun*rzm2**(-1)*mgen2 - 24.D0
     &    *mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*deltafun*mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 8.D0*deltafun*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*deltafun*
     &    mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 32.
     &    D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**(-1)*
     &    u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*deltafun*
     &    mgen2 + 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 - 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*s*deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 + 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*s4fun**(-1)*deltafun
     &     + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*c0ccqsfff
     &  * ( 16.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)
     &    *mgen2 )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun + 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun
     &     + 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*s*deltafun - 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,1) = qedamp(2,1,3,1) + af**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(2,1,3,3) =
     &  + vf**2*cqfactor * ( 32.D0*mz**2*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*mgen2
     &     - 8.D0*mz**2*mgen2 + 32.D0*mz**2*u*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 - 64.D0*mz**4*s**(-1)*u*s4fun**(-1)
     &    *mgen2 - 32.D0*mz**4*s4fun**(-1)*mgen2 + 32.D0*mz**6*s**(-1)*
     &    s4fun**(-1)*mgen2 )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 8.D0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 24.D0*mz**2*s**2*u**2*s4fun**(-1)*Yfun**(-1) - 24.D0*mz**2
     &    *s**3*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 96.D0
     &    *mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*u*s4fun**(-1)*
     &    Yfun**(-1) - 104.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-1) + 80.D0
     &    *mz**8*s*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*c0cmzusfff
     &  * ( 64.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 - 16.
     &    D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 16.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*c0cmztsfff
     &  * ( 64.D0*mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 - 16.
     &    D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 16.D0*mz**2*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 48.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)
     &    *Yfun**(-1) + 64.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D
     &    0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**4*s*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 192.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 80.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 208.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 208.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 160.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*mz**2*s*s4fun**(-1) + 4.D0*mz**2*s*s2fun*
     &    Yfun**(-1) + 4.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*mz**2 - 8.D0*mz**2*s*s4fun**(-1) + 4.D0*
     &    mz**2*s*s4fun*Yfun**(-1) + 4.D0*mz**2*s**2*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) - 16.D0*mz**2*
     &    u**2*u1fun**(-2)*s4fun**(-1) - 32.D0*mz**2*s*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 64.D0
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 80.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*u**2*u1fun**(-2)*s4fun**(-1) + 16.D0*mz**6*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + vf**2*cqfactor*b0ftsff
     &  * ( 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**2*s*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**2*s**2*t1fun**(-2)*
     &    s4fun**(-1) - 64.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 80.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor * ( 32.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 32.D0*s4fun**(-1)*Yfun*mgen2 - 16.D0*s2fun*
     &    rzm2**(-1)*mgen2**2 + 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 32.D0*mz**2*u*s4fun**(-1)*mgen2 + 
     &    8.D0*mz**2*s*s4fun**(-1)*mgen2 - 64.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1)*mgen2 - 32.D0*mz**4*s4fun**(-1)*mgen2 + 32.D0*
     &    mz**6*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 + 8.D0*s
     &    *u**3*Yfun**(-1)*rzm2**(-1)*mgen2 - 8.D0*mz**2*mgen2 - 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     + 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 - 8.D0*s
     &    *u**3*Yfun**(-1)*rzm2**(-1)*mgen2 - 24.D0*s**2*u**2*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 24.D0*s**3*u*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 8.D0*s**4*Yfun**(-1)*rzm2**(-1)*mgen2 - 8.D
     &    0*mz**2*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 - 8.D0*mz**2*s
     &    *u*Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 8.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**2*s**2*Yfun**(-1)*mgen2 + 96.D0*mz**2*s**2*u*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**3*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 24.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     - 8.D0*mz**2*s**4*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*
     &    Yfun**(-1)*mgen2 - 96.D0*mz**4*s*u*Yfun**(-1)*rzm2**(-1)*
     &    mgen2 + 48.D0*mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) - 96.D0*
     &    mz**4*s**2*Yfun**(-1)*rzm2**(-1)*mgen2 + 96.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-1)*rzm2**(-1)*mgen2 - 104.D
     &    0*mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 80.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*u**3*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 + 64.D0*mz**2*s**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*Yfun*mgen2 - 16.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 
     &    16.D0*mz**2*u*Yfun**(-1)*mgen2 - 16.D0*mz**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*rzm2 + 16.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 
     &    16.D0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*u**3*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 48.D0*s*u**2*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 16.D0*s**2*Yfun**(-1)*mgen2 - 
     &    48.D0*s**2*u*Yfun**(-1)*rzm2**(-1)*mgen2 - 16.D0*s**3*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 + 64.D0*mz**2*s**(-1)*t1fun**(-1)
     &    *s4fun**(-1)*Yfun*mgen2 - 16.D0*mz**2*Yfun**(-1)*rzm2*mgen2
     &     + 48.D0*mz**2*u*Yfun**(-1)*mgen2 - 16.D0*mz**2*u**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*u**2*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 16.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) + 
     &    48.D0*mz**2*s*Yfun**(-1)*mgen2 - 32.D0*mz**2*s*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 192.D0*mz**2*s*u*Yfun**(-1)*rzm2**(-1)*
     &    mgen2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**2*s**2*
     &    Yfun**(-1)*rzm2**(-1)*mgen2 - 48.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 32.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 64.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 - 192.D0*mz**4*u*Yfun**(-1)*rzm2**(-1)*mgen2 + 96.D0*
     &    mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**4*s*s4fun**(-1)
     &    *Yfun**(-1)*rzm2 - 192.D0*mz**4*s*Yfun**(-1)*rzm2**(-1)*mgen2
     &     + 192.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) + 96.D0*mz**4*s**2
     &    *s4fun**(-1)*Yfun**(-1) - 80.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 + 128.D0*mz**6*Yfun**(-1)*rzm2**(-1)*mgen2 - 208.D0*
     &    mz**6*u*s4fun**(-1)*Yfun**(-1) - 208.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 160.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s*rzm2**(-1)*mgen2 + 8.D0*s*s2fun**2*Yfun**(-1)*
     &    rzm2**(-1)*mgen2 - 16.D0*mz**2*s*s4fun**(-1) + 4.D0*mz**2*s*
     &    s2fun*Yfun**(-1) + 4.D0*mz**2*s**2*s2fun*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2 + 8.D0*s*s2fun*
     &    s4fun*Yfun**(-1)*rzm2**(-1)*mgen2 - 8.D0*mz**2 - 8.D0*mz**2*s
     &    *s4fun**(-1) + 4.D0*mz**2*s*s4fun*Yfun**(-1) + 4.D0*mz**2*
     &    s**2*Yfun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) - 16.D0*mz**2*
     &    u**2*u1fun**(-2)*s4fun**(-1) - 32.D0*mz**2*s*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**2*s**2*t1fun**(-2)*s4fun**(-1) + 64.D0
     &    *mz**4*u*t1fun**(-2)*s4fun**(-1) + 64.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 80.D0*mz**6*t1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*u**2*u1fun**(-2)*s4fun**(-1) + 16.D0*mz**6*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,3,3) = qedamp(2,1,3,3) + af**2*cqfactor*b0ftsff
     &  * ( 16.D0*mz**2*u**2*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**2*s*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**2*s**2*t1fun**(-2)*
     &    s4fun**(-1) - 64.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) - 64.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 80.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )

      qedamp(2,1,3,2) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun + 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 - 8.D0*mz**2*u*s4fun**(-1)*deltafun + 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *s4fun**(-1)*deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*s4fun**(-1)*
     &    deltafun - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*s4fun**(-1)*
     &    deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 16.D0*s**2*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*s**2*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0
     &    *mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 48.D0*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*
     &    s4fun**(-1)*deltafun + 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s**2*s4fun**(-1)*deltafun + 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*
     &    deltafun - 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + vf**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 - 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2
     &     + 8.D0*u**2*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 + 12.D0*s*u*deltafun*rzm2**(-1)*
     &    mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*u*s4fun**(-1)*deltafun - 12.D0*
     &    s**2*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**3*
     &    s4fun**(-1)*deltafun - 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 - 
     &    24.D0*mz**2*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s*
     &    deltafun*rzm2**(-1)*mgen2 - 20.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 24.D0*mz**2*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 20.D0*mz**2*s**2*s4fun**(-1)*deltafun + 48.D0*
     &    mz**2*s**2*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 24.D0*
     &    mz**2*s**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*
     &    mz**4*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*deltafun*
     &    rzm2**(-1)*mgen2 + 24.D0*mz**4*s*s4fun**(-1)*deltafun - 48.D0
     &    *mz**4*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4
     &    *s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0
     &    *s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*
     &    deltafun*mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*
     &    mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 8.D0*deltafun*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2 - 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2
     &     - 16.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 + 24.D0*s**(-1)
     &    *u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*deltafun
     &    *mgen2 - 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**(-1)
     &    *u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*deltafun*
     &    mgen2 - 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 + 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     + 24.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*
     &    deltafun - 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     + 8.D0*s*deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun - 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.
     &    D0*s**2*s4fun**(-1)*deltafun - 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 - 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 - 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 - 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun + 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 40.D0*mz**2*s*s4fun**(-1)*deltafun + 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*s4fun**(-1)*deltafun
     &     - 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**2*
     &    s4fun**(-1)*deltafun + 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     + 8.D0*mz**2*s*s4fun**(-1)*deltafun - 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*
     & c0mzmzqsfff * (  - 8.D0*u*deltafun - 4.D0*s*deltafun + 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) + 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*deltafun - 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun + 8.D0*s*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*mz**2*s4fun**(-1)*deltafun - 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun + 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun - 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*cqfactor*b0fqsff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)
     &    *mgen2 )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun + 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun
     &     + 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*s*deltafun - 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,3,2) = qedamp(2,1,3,2) + af**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(2,1,2,1) =
     &  + vf**2*cqfactor * (  - 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 16.D0*
     &    mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor
     &  * (  - 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 8.D
     &    0*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**4*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s*u*s4fun**(-1)*mgen2 + 4.D0*s**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 4.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 48.D0*s**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*mgen2 - 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) - 4.
     &    D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 2.D0*s**3*s4fun**(-1)
     &     - 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) - 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**5*s4fun**(-1)*Yfun**(-1) - 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**7*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 + 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**2*s**2*s4fun**(-1) + 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) - 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 - 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) - 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) - 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) - 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 4.D0*s**2*
     &    s4fun**(-1) - 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) - 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**6*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1) + 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     + 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     + 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) - 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) + 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     + 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 - 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0
     &    *s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*s**3*u*s2fun*Yfun**(-2) + 4.D0*s**4*s2fun*
     &    Yfun**(-2) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     + 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) + 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 16.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*s**2*Yfun**(-1)
     &    *mgen2 + 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     + 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) + 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*Yfun**(-1)*mgen2 - 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) - 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) + 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) - 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) - 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 32.D0*u*s4fun**(-1) - 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*s4fun**(-1) + 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 
     &    32.D0*mz**2*s4fun**(-1) + 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) + 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) + 40.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1) - 80.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) - 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + vf**2*betaZfun*cqfactor*
     & b0fqsff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor * (  - 8.D0
     &    *t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*Yfun*mgen2 - 16.D0*
     &    mgen2**2 + 4.D0*s*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 + 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 4.D0*
     &    s**2*u**4*Yfun**(-2) + 16.D0*mz**4*s4fun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 4.D0*s*mgen2 - 2.D0*s*u
     &    *s2fun*s4fun**(-1) - 16.D0*s*u**2*Yfun**(-1)*mgen2 - 2.D0*
     &    s**2*s2fun*s4fun**(-1) - 32.D0*s**2*u*Yfun**(-1)*mgen2 - 4.D0
     &    *s**2*u**4*Yfun**(-2) - 16.D0*s**3*Yfun**(-1)*mgen2 - 16.D0*
     &    s**3*u**3*Yfun**(-2) - 24.D0*s**4*u**2*Yfun**(-2) - 16.D0*
     &    s**5*u*Yfun**(-2) - 4.D0*s**6*Yfun**(-2) + 4.D0*mz**2*s*s2fun
     &    *s4fun**(-1) + 64.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 64.D0*mz**2
     &    *s**2*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*u**3*Yfun**(-2) + 
     &    96.D0*mz**2*s**3*u**2*Yfun**(-2) + 96.D0*mz**2*s**4*u*
     &    Yfun**(-2) + 32.D0*mz**2*s**5*Yfun**(-2) + 16.D0*mz**4*
     &    s4fun**(-1)*mgen2 - 64.D0*mz**4*s*Yfun**(-1)*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*u**2*Yfun**(-2) + 8.
     &    D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 192.D0*mz**4*s**3*u*
     &    Yfun**(-2) - 96.D0*mz**4*s**4*Yfun**(-2) - 32.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 128.D0*mz**6*s**2*u*Yfun**(-2) + 128.D0*mz**6*
     &    s**3*Yfun**(-2) + 32.D0*mz**8*s*s4fun**(-1)*Yfun**(-1) - 64.D0
     &    *mz**8*s**2*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*c0cmzusfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 + 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 + 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 + 8.D0*s*u**3*Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2)
     &     - 32.D0*mz**2*s**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 
     &    16.D0*mz**2*u1fun**(-1)*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*c0cmztsfff
     &  * (  - 4.D0*s2fun*s4fun**(-1)*rzm2 - 16.D0*u*Yfun**(-1)*rzm2*
     &    mgen2 - 4.D0*u*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s*Yfun**(-1)*rzm2*mgen2 - 4.D0*s*s2fun*
     &    s4fun**(-1) - 32.D0*s*u*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    Yfun**(-2)*rzm2 - 8.D0*s*u**4*Yfun**(-2) - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 24.D0*s**2*u**2*Yfun**(-2)*rzm2 - 32.D0*
     &    s**2*u**3*Yfun**(-2) - 24.D0*s**3*u*Yfun**(-2)*rzm2 - 48.D0*
     &    s**3*u**2*Yfun**(-2) - 8.D0*s**4*Yfun**(-2)*rzm2 - 32.D0*s**4
     &    *u*Yfun**(-2) - 8.D0*s**5*Yfun**(-2) - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*Yfun*mgen2 + 16.D0*mz**2*t1fun**(-1)*
     &    mgen2 + 32.D0*mz**2*Yfun**(-1)*rzm2*mgen2 + 8.D0*mz**2*s2fun*
     &    s4fun**(-1) + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0*mz**2*s*
     &    Yfun**(-1)*mgen2 + 48.D0*mz**2*s*u**2*Yfun**(-2)*rzm2 + 64.D0
     &    *mz**2*s*u**3*Yfun**(-2) + 96.D0*mz**2*s**2*u*Yfun**(-2)*rzm2
     &     + 192.D0*mz**2*s**2*u**2*Yfun**(-2) + 48.D0*mz**2*s**3*
     &    Yfun**(-2)*rzm2 + 192.D0*mz**2*s**3*u*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*Yfun**(-2) - 64.D0*mz**4*Yfun**(-1)*mgen2 + 16.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 32.D0*mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*
     &    s*u*Yfun**(-2)*rzm2 - 192.D0*mz**4*s*u**2*Yfun**(-2) + 16.D0*
     &    mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 96.D0*mz**4*s**2*
     &    Yfun**(-2)*rzm2 - 384.D0*mz**4*s**2*u*Yfun**(-2) - 192.D0*
     &    mz**4*s**3*Yfun**(-2) - 32.D0*mz**6*s4fun**(-1)*Yfun**(-1)*
     &    rzm2 - 64.D0*mz**6*u*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1) + 64.D0*mz**6*s*Yfun**(-2)*rzm2 + 256.D
     &    0*mz**6*s*u*Yfun**(-2) + 256.D0*mz**6*s**2*Yfun**(-2) + 64.D0
     &    *mz**8*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**8*s*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*c0ccqsfff
     &  * ( 4.D0*s*s2fun*s4fun**(-1) + 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 
     &    12.D0*s**2*s2fun*Yfun**(-1) + 4.D0*s**3*s2fun*s4fun*
     &    Yfun**(-2) - 8.D0*mz**4*s*s2fun*s4fun**(-1)*Yfun**(-1) + 4.D0
     &    *mz**4*s**2*s2fun*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s + 8.D0*s*s4fun*Yfun**(-1)*mgen2 + 8.D0*s*
     &    u**2*Yfun**(-1) + 8.D0*s*u**4*Yfun**(-2) + 8.D0*s**2*u*
     &    Yfun**(-1) + 16.D0*s**2*u**3*Yfun**(-2) + 4.D0*s**3*
     &    Yfun**(-1) + 24.D0*s**3*u**2*Yfun**(-2) + 16.D0*s**4*u*
     &    Yfun**(-2) + 4.D0*s**5*Yfun**(-2) - 16.D0*mz**2*s*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*u**3*Yfun**(-2) - 16.D0*mz**2*s**2
     &    *Yfun**(-1) - 96.D0*mz**2*s**2*u**2*Yfun**(-2) - 96.D0*mz**2*
     &    s**3*u*Yfun**(-2) - 32.D0*mz**2*s**4*Yfun**(-2) + 16.D0*mz**4
     &    *s*Yfun**(-1) + 88.D0*mz**4*s*u**2*Yfun**(-2) + 184.D0*mz**4*
     &    s**2*u*Yfun**(-2) + 92.D0*mz**4*s**3*Yfun**(-2) - 112.D0*
     &    mz**6*s*u*Yfun**(-2) - 112.D0*mz**6*s**2*Yfun**(-2) + 48.D0*
     &    mz**8*s*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*b0fmzff
     &  * (  - 8.D0 - 8.D0*s2fun*s4fun**(-1) - 16.D0*u**2*Yfun**(-1) + 
     &    8.D0*s*s2fun*Yfun**(-1) - 16.D0*s*u*Yfun**(-1) - 8.D0*s**2*
     &    Yfun**(-1) + 16.D0*mz**2*s4fun**(-1) + 32.D0*mz**2*u*
     &    Yfun**(-1) + 32.D0*mz**2*s*Yfun**(-1) - 32.D0*mz**4*
     &    Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 16.D0*
     &    mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 32.D0*
     &    mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*b0fusff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 16.D0
     &    *mz**4*u*u1fun**(-2)*s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) + 32.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*b0ftsff
     &  * ( 4.D0 + 8.D0*u**2*Yfun**(-1) + 16.D0*s*u*Yfun**(-1) + 8.D0*
     &    s**2*Yfun**(-1) - 8.D0*mz**2*s4fun**(-1) - 32.D0*mz**2*u*
     &    Yfun**(-1) - 32.D0*mz**2*s*Yfun**(-1) + 32.D0*mz**4*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 8.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*cqfactor*b0fqsff
     &  * ( 8.D0*s2fun*s4fun**(-1) - 8.D0*s*s2fun*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor
     &  * (  - 16.D0*mz**2*s*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) - 8.D
     &    0*mz**2*s**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**4*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 4.D0*s**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 4.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 4.D0*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 4.D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 8.D0*
     &    mz**2*s*s4fun**(-1)*mgen2 + 16.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**4*s**2*u**3*s4fun**(-1)*
     &    Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*u*s4fun**(-1)*mgen2 - 16.D0*s*
     &    u**3*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**2*s4fun**(-1)*
     &    mgen2 - 2.D0*s**2*u*s4fun**(-1) - 48.D0*s**2*u**2*s4fun**(-1)
     &    *Yfun**(-1)*mgen2 - 4.D0*s**2*u**3*s4fun**(-1)*Yfun**(-1) - 4.
     &    D0*s**2*u**5*s4fun**(-1)*Yfun**(-2) - 2.D0*s**3*s4fun**(-1)
     &     - 48.D0*s**3*u*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*s**3*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 20.D0*s**3*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*s**4*s4fun**(-1)*Yfun**(-1)*mgen2 - 12.D0*
     &    s**4*u*s4fun**(-1)*Yfun**(-1) - 40.D0*s**4*u**3*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**5*s4fun**(-1)*Yfun**(-1) - 40.D0*s**5*
     &    u**2*s4fun**(-1)*Yfun**(-2) - 20.D0*s**6*u*s4fun**(-1)*
     &    Yfun**(-2) - 4.D0*s**7*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s
     &    *s4fun**(-1)*mgen2 + 96.D0*mz**2*s*u**2*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 + 4.D0*mz**2*s**2*s4fun**(-1) + 192.D0*mz**2
     &    *s**2*u*s4fun**(-1)*Yfun**(-1)*mgen2 + 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**2*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) + 96.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1)*mgen2 + 
     &    48.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) + 160.D0*mz**2*s**3
     &    *u**3*s4fun**(-1)*Yfun**(-2) + 24.D0*mz**2*s**4*s4fun**(-1)*
     &    Yfun**(-1) + 240.D0*mz**2*s**4*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    160.D0*mz**2*s**5*u*s4fun**(-1)*Yfun**(-2) + 40.D0*mz**2*s**6
     &    *s4fun**(-1)*Yfun**(-2) - 176.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 176.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1)*
     &    mgen2 - 48.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-1) - 156.D0*
     &    mz**4*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 48.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 468.D0*mz**4*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 468.D0*mz**4*s**4*u*s4fun**(-1)*Yfun**(-2) - 156.
     &    D0*mz**4*s**5*s4fun**(-1)*Yfun**(-2) + 96.D0*mz**6*s*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 32.D0*mz**6*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 296.D0*mz**6*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 
     &    592.D0*mz**6*s**3*u*s4fun**(-1)*Yfun**(-2) + 296.D0*mz**6*
     &    s**4*s4fun**(-1)*Yfun**(-2) - 272.D0*mz**8*s**2*u*s4fun**(-1)
     &    *Yfun**(-2) - 272.D0*mz**8*s**3*s4fun**(-1)*Yfun**(-2) + 96.D0
     &    *mz**10*s**2*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1) + 
     &    8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    u1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*
     &    u1fun**(-1)*s4fun**(-1)*mgen2 - 16.D0*mz**4*s*u*u1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*mz**4*s*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 + 8.D0*mz**4*s*u**3*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 4.D0*s*s4fun**(-1)*rzm2 - 4.D0*s*u*s4fun**(-1)
     &     - 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 + 16.D0*s*u**3*
     &    t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s*u**3*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s*u**4*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 8.D0*s*u**5*s4fun**(-1)*Yfun**(-2) - 4.D0*s**2*
     &    s4fun**(-1) - 16.D0*s**2*u*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0
     &    *s**2*u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**2*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1)*rzm2 + 48.D0*
     &    s**3*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 24.D0*s**3*
     &    u*s4fun**(-1)*Yfun**(-1) - 48.D0*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2)*rzm2 - 80.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 16.
     &    D0*s**4*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*s**4*u*s4fun**(-1)*Yfun**(-2)*
     &    rzm2 - 80.D0*s**4*u**2*s4fun**(-1)*Yfun**(-2) - 8.D0*s**5*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 40.D0*s**5*u*s4fun**(-1)*
     &    Yfun**(-2) - 8.D0*s**6*s4fun**(-1)*Yfun**(-2) + 16.D0*mz**2*s
     &    *t1fun**(-1)*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1) + 
     &    32.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 96.D0*mz**2*s*
     &    u**2*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s
     &    *u**2*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*u**3*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 80.D0*mz**2*s*u**4*s4fun**(-1)*Yfun**(-2)
     &     + 32.D0*mz**2*s**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 192.D0*
     &    mz**2*s**2*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) + 192.D0*mz**2*s**2*u**2
     &    *s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**2*u**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**2*s**3*t1fun**(-1)*
     &    s4fun**(-1)*Yfun**(-1)*mgen2 + 48.D0*mz**2*s**3*s4fun**(-1)*
     &    Yfun**(-1) + 192.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-2)*rzm2
     &     + 480.D0*mz**2*s**3*u**2*s4fun**(-1)*Yfun**(-2) + 64.D0*
     &    mz**2*s**4*s4fun**(-1)*Yfun**(-2)*rzm2 + 320.D0*mz**2*s**4*u*
     &    s4fun**(-1)*Yfun**(-2) + 80.D0*mz**2*s**5*s4fun**(-1)*
     &    Yfun**(-2) - 32.D0*mz**4*s*s4fun**(-1)*Yfun**(-1)*rzm2 + 176.D
     &    0*mz**4*s*u*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 - 96.D0*
     &    mz**4*s*u*s4fun**(-1)*Yfun**(-1) - 184.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 312.D0*mz**4*s*u**3*s4fun**(-1)
     &    *Yfun**(-2) + 176.D0*mz**4*s**2*t1fun**(-1)*s4fun**(-1)*
     &    Yfun**(-1)*mgen2 - 96.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 
     &    368.D0*mz**4*s**2*u*s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*
     &    mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) - 184.D0*mz**4*s**3*
     &    s4fun**(-1)*Yfun**(-2)*rzm2 - 936.D0*mz**4*s**3*u*s4fun**(-1)
     &    *Yfun**(-2) - 312.D0*mz**4*s**4*s4fun**(-1)*Yfun**(-2) - 96.D0
     &    *mz**6*s*t1fun**(-1)*s4fun**(-1)*Yfun**(-1)*mgen2 + 64.D0*
     &    mz**6*s*s4fun**(-1)*Yfun**(-1) + 224.D0*mz**6*s*u*s4fun**(-1)
     &    *Yfun**(-2)*rzm2 + 592.D0*mz**6*s*u**2*s4fun**(-1)*Yfun**(-2)
     &     + 224.D0*mz**6*s**2*s4fun**(-1)*Yfun**(-2)*rzm2 + 1184.D0*
     &    mz**6*s**2*u*s4fun**(-1)*Yfun**(-2) + 592.D0*mz**6*s**3*
     &    s4fun**(-1)*Yfun**(-2) - 96.D0*mz**8*s*s4fun**(-1)*Yfun**(-2)
     &    *rzm2 - 544.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) - 544.D0*
     &    mz**8*s**2*s4fun**(-1)*Yfun**(-2) + 192.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 + 8.D0
     &    *s**2*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2 - 8.D0*s**2*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 4.D0*s**3*s2fun*s4fun**(-1)*
     &    Yfun**(-1) + 8.D0*s**3*u*s2fun*Yfun**(-2) + 4.D0*s**4*s2fun*
     &    Yfun**(-2) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1)*mgen2
     &     + 8.D0*mz**2*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*
     &    s**3*s2fun*Yfun**(-2) + 8.D0*mz**4*s**2*u*s2fun*s4fun**(-1)*
     &    Yfun**(-2) + 4.D0*mz**4*s**3*s2fun*s4fun**(-1)*Yfun**(-2) - 8.
     &    D0*mz**6*s**2*s2fun*s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 16.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s*u*s2fun**2*
     &    s4fun**(-1)*Yfun**(-1) + 16.D0*s*u**3*s4fun**(-1)*Yfun**(-1)
     &     + 16.D0*s*u**5*s4fun**(-1)*Yfun**(-2) + 8.D0*s**2*Yfun**(-1)
     &    *mgen2 + 8.D0*s**2*s2fun**2*s4fun**(-1)*Yfun**(-1) + 24.D0*
     &    s**2*u**2*s4fun**(-1)*Yfun**(-1) + 40.D0*s**2*u**4*
     &    s4fun**(-1)*Yfun**(-2) + 16.D0*s**3*u*s4fun**(-1)*Yfun**(-1)
     &     + 64.D0*s**3*u**3*s4fun**(-1)*Yfun**(-2) + 4.D0*s**4*
     &    s4fun**(-1)*Yfun**(-1) + 56.D0*s**4*u**2*s4fun**(-1)*
     &    Yfun**(-2) + 24.D0*s**5*u*s4fun**(-1)*Yfun**(-2) + 4.D0*s**6*
     &    s4fun**(-1)*Yfun**(-2) - 16.D0*mz**2*s*Yfun**(-1)*mgen2 - 16.D
     &    0*mz**2*s*s2fun**2*s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s*
     &    u**2*s4fun**(-1)*Yfun**(-1) - 80.D0*mz**2*s*u**4*s4fun**(-1)*
     &    Yfun**(-2) - 64.D0*mz**2*s**2*u*s4fun**(-1)*Yfun**(-1) - 256.D
     &    0*mz**2*s**2*u**3*s4fun**(-1)*Yfun**(-2) - 24.D0*mz**2*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 336.D0*mz**2*s**3*u**2*s4fun**(-1)*
     &    Yfun**(-2) - 192.D0*mz**2*s**4*u*s4fun**(-1)*Yfun**(-2) - 40.D
     &    0*mz**2*s**5*s4fun**(-1)*Yfun**(-2) + 48.D0*mz**4*s*u*
     &    s4fun**(-1)*Yfun**(-1) + 240.D0*mz**4*s*u**3*s4fun**(-1)*
     &    Yfun**(-2) + 40.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) + 648.D0
     &    *mz**4*s**2*u**2*s4fun**(-1)*Yfun**(-2) + 560.D0*mz**4*s**3*u
     &    *s4fun**(-1)*Yfun**(-2) + 156.D0*mz**4*s**4*s4fun**(-1)*
     &    Yfun**(-2) - 16.D0*mz**6*s*s4fun**(-1)*Yfun**(-1) - 400.D0*
     &    mz**6*s*u**2*s4fun**(-1)*Yfun**(-2) - 704.D0*mz**6*s**2*u*
     &    s4fun**(-1)*Yfun**(-2) - 296.D0*mz**6*s**3*s4fun**(-1)*
     &    Yfun**(-2) + 320.D0*mz**8*s*u*s4fun**(-1)*Yfun**(-2) + 272.D0
     &    *mz**8*s**2*s4fun**(-1)*Yfun**(-2) - 96.D0*mz**10*s*
     &    s4fun**(-1)*Yfun**(-2) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 32.D0*u*s4fun**(-1) - 32.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*s*s4fun**(-1) + 16.D0*s*u*s2fun*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     + 8.D0*s**2*s2fun*s4fun**(-1)*Yfun**(-1) - 32.D0*s**2*u*
     &    s4fun**(-1)*Yfun**(-1) - 8.D0*s**3*s4fun**(-1)*Yfun**(-1) + 
     &    32.D0*mz**2*s4fun**(-1) + 96.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**2*s*s2fun*s4fun**(-1)*Yfun**(-1) + 128.
     &    D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1) + 48.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 8.D0*mz**4*s*u1fun**(-2)*s4fun**(-1) - 96.D0*
     &    mz**4*s*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) + 64.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*
     &    s4fun**(-1)*Yfun**(-1) + 8.D0*s*u**2*s4fun**(-1)*Yfun**(-1)
     &     - 16.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) + 8.D0*mz**4*s*u1fun**(-2)*
     &    s4fun**(-1) - 16.D0*mz**6*u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 4.D0 + 16.D0*u*s4fun**(-1) + 16.D0*u**3*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*s*s4fun**(-1) + 40.D0*s*u**2*s4fun**(-1)*
     &    Yfun**(-1) + 32.D0*s**2*u*s4fun**(-1)*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*s4fun**(-1) - 80.D0*
     &    mz**2*u**2*s4fun**(-1)*Yfun**(-1) - 128.D0*mz**2*s*u*
     &    s4fun**(-1)*Yfun**(-1) - 48.D0*mz**2*s**2*s4fun**(-1)*
     &    Yfun**(-1) + 16.D0*mz**4*u*t1fun**(-2)*s4fun**(-1) + 128.D0*
     &    mz**4*u*s4fun**(-1)*Yfun**(-1) + 8.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) + 96.D0*mz**4*s*s4fun**(-1)*Yfun**(-1) - 16.D0*
     &    mz**6*t1fun**(-2)*s4fun**(-1) - 64.D0*mz**6*s4fun**(-1)*
     &    Yfun**(-1) )
      qedamp(2,1,2,1) = qedamp(2,1,2,1) + af**2*betaZfun*cqfactor*
     & b0fqsff * (  - 16.D0*s*u*s2fun*s4fun**(-1)*Yfun**(-1) - 8.D0*
     &    s**2*s2fun*s4fun**(-1)*Yfun**(-1) + 16.D0*mz**2*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )

      qedamp(2,1,2,3) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*s2fun*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) - 8.D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*
     &    u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 
     &    20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2
     &     - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*deltafun*mgen2 + 32.D0*s**(-1)*u**2*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*rzm2*mgen2
     &     + 16.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*
     &    deltafun*rzm2 + 8.D0*u**2*s4fun**(-1)*deltafun - 8.D0*u**2*
     &    s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*u**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**(-1)*u**(-1)
     &    *s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*
     &    rzm2 + 8.D0*mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**4*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4
     &    *s**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s**(-1)*u**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u**2*s4fun**(-1)*deltafun + 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s*t**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 - 16.D0*s*u
     &    *s4fun**(-1)*deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 8.D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun
     &    *s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*
     &    s4fun**(-1)*deltafun*rzm2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 
     &    96.D0*mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**4*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 4.D0*mz**2*s*u*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u**2*
     &    s4fun**(-1)*deltafun + 16.D0*s**2*s4fun**(-1)*deltafun*mgen2
     &     - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*s**2*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 
     &    12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**5*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0
     &    *mz**2*s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*
     &    deltafun + 48.D0*mz**4*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 48.D0*mz**4*s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*
     &    mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 16.D0*s*u
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*
     &    s4fun**(-1)*deltafun + 8.D0*s*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 8.D0*s*u**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s**2*s4fun**(-1)*deltafun + 16.D0*s**2*u*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 + 24.D0*s**2*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*s**3*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 24.D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 40.D0
     &    *mz**2*u*s4fun**(-1)*deltafun - 32.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 40.D0*mz**2*s*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*u*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 - 48.D0*mz**2*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 
     &    32.D0*mz**2*s**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0
     &    *mz**2*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*
     &    s**3*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*
     &    s4fun**(-1)*deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 96.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 64.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*
     &    deltafun - 8.D0*s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**3*u*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**4*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + vf**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 16.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 - 4.D0*s*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*
     &    s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*u**2*s4fun**(-1)*deltafun + 4.D0*s*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*mz**2*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*mz**4*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    mz**4*s*u**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*u**2*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun*mgen2 - 12.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 4.D0*s
     &    *u**2*s4fun**(-1)*deltafun + 4.D0*s*u**3*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 - 8.
     &    D0*s**2*u*s4fun**(-1)*deltafun + 12.D0*s**2*u**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*s4fun**(-1)*
     &    deltafun + 12.D0*s**3*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)
     &     + 4.D0*s**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*
     &    mz**2*s2fun*s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**2*u*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun - 24.D0*mz**2*s
     &    *u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 48.D0*mz**2*s**2*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 24.D0*mz**2*s**3*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**4*deltafun*rzm2**(-1)*mgen2 - 24.D0
     &    *mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4*s*u*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**2*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*mz**4*s**3*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 32.D0*mz**6*s*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 32.D0*mz**6*s*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s**2*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 32.D0*mz**8*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-1)*s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.
     &    D0*s**(-1)*u*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*deltafun*mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0
     &    *s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 - 8.D0*deltafun*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2 + 8.D0*u*
     &    deltafun*rzm2**(-1)*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun - 
     &    8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*
     &    u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*
     &    s**(-1)*u**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*mz**2*
     &    s**(-1)*deltafun*mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*s**(-1)*u*deltafun*rzm2**(-1)*
     &    mgen2 - 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 8.D0*mz**2*u*
     &    s4fun**(-1)*deltafun - 32.D0*mz**4*s**(-1)*u**(-1)*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 16.D0*mz**4*u**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*t**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.
     &    D0*s**(-1)*t**(-1)*u**2*s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D
     &    0*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s**(-1)*
     &    s2fun*s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s**(-1)*u*deltafun*
     &    mgen2 + 16.D0*s**(-1)*u*s2fun*s4fun**(-1)*deltafun*mgen2 - 32.
     &    D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**(-1)*
     &    u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t**(-1)*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 16.D0*t**(-1)*u*s4fun**(-1)*deltafun*
     &    rzm2*mgen2 + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*deltafun*
     &    mgen2 + 16.D0*s2fun*s4fun**(-1)*deltafun*mgen2 - 48.D0*u*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*rzm2
     &     - 24.D0*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*u**2*s4fun**(-1)*
     &    deltafun + 8.D0*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 8.D0*u**3*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0
     &    *s*t**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2
     &     - 8.D0*s*deltafun*rzm2**(-1)*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun + 16.D0*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*
     &    rzm2 + 24.D0*s*u**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 8.
     &    D0*s**2*s4fun**(-1)*deltafun + 8.D0*s**2*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1)*rzm2 + 24.D0*s**2*u*s2fun*s4fun**(-1)*
     &    deltafun*Yfun**(-1) + 8.D0*s**3*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 48.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2*
     &    mgen2 + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 32.D0*mz**2*
     &    s**(-1)*s2fun*s4fun**(-1)*deltafun*mgen2 + 96.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u*
     &    deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 + 32.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*u*s2fun*s4fun**(-1)*deltafun*
     &    Yfun**(-1)*rzm2 - 48.D0*mz**2*u**2*s2fun*s4fun**(-1)*deltafun
     &    *Yfun**(-1) + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*
     &    mz**2*s*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*
     &    mz**2*s*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2
     &    *s**2*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 96.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*s4fun**(-1)*deltafun
     &     + 32.D0*mz**4*s2fun*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 
     &    16.D0*mz**4*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*
     &    mz**4*u*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1) - 16.D0*mz**4*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 32.D0*mz**6*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     - 64.D0*mz**6*s2fun*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*
     &    mz**6*u*s4fun**(-1)*deltafun*Yfun**(-1) + 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**8*s4fun**(-1)*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*c0ccqsfff
     &  * ( 16.D0*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s*u*s4fun**(-1)*deltafun - 8.D0*s*u*
     &    s2fun**2*s4fun**(-1)*deltafun*Yfun**(-1) + 4.D0*s**2*
     &    s4fun**(-1)*deltafun - 4.D0*s**2*s2fun**2*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*mz**2*s*s4fun**(-1)*deltafun + 8.D0*mz**2*s*s2fun**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*mz**4*s*u*s4fun**(-1)
     &    *deltafun*Yfun**(-1) + 8.D0*mz**4*s**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 16.D0*mz**6*s*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*u*deltafun + 4.D0*s*deltafun - 8.D0*s*u*
     &    s2fun*deltafun*Yfun**(-1) - 4.D0*s**2*s2fun*deltafun*
     &    Yfun**(-1) - 8.D0*mz**2*deltafun + 8.D0*mz**2*s*s2fun*
     &    deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*u*s4fun**(-1)*deltafun - 8.D0*u**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*
     &    deltafun - 8.D0*s*s4fun**(-1)*deltafun - 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun - 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*s4fun**(-1)*deltafun + 40.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun + 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*u*
     &    u1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*b0ftsff
     &  * ( 8.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun + 16.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun + 8.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 40.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun - 40.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun + 
     &    48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*cqfactor*b0fqsff
     &  * ( 16.D0*u*s4fun**(-1)*deltafun + 8.D0*s*s4fun**(-1)*deltafun
     &     - 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)
     &    *mgen2 )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun - 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 
     &    16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*u**2*s4fun**(-1)*deltafun + 16.D0*
     &    s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0*s**2*deltafun*
     &    rzm2**(-1)*mgen2 - 8.D0*s**2*u*s4fun**(-1)*deltafun + 4.D0*
     &    s**2*u**3*s4fun**(-1)*deltafun*Yfun**(-1) - 4.D0*s**3*
     &    s4fun**(-1)*deltafun + 12.D0*s**3*u**2*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 12.D0*s**4*u*s4fun**(-1)*deltafun*Yfun**(-1) + 4.
     &    D0*s**5*s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**2*s*deltafun*rzm2**(-1)
     &    *mgen2 + 20.D0*mz**2*s*u*s4fun**(-1)*deltafun + 20.D0*mz**2*
     &    s**2*s4fun**(-1)*deltafun - 24.D0*mz**2*s**2*u**2*s4fun**(-1)
     &    *deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 24.D0*mz**2*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) - 24.D0*mz**4*s*s4fun**(-1)*deltafun + 48.D0*mz**4
     &    *s**2*u*s4fun**(-1)*deltafun*Yfun**(-1) + 48.D0*mz**4*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**6*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 - 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 - 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun - 16.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u**2*s4fun**(-1)*deltafun
     &    *Yfun**(-1)*rzm2 - 8.D0*s*u**3*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 8.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 8.D0*
     &    mz**2*u*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 32.D0*s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*
     &    deltafun*mgen2 + 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 8.D0*u*
     &    s4fun**(-1)*deltafun*rzm2 + 8.D0*u*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*u**2*s4fun**(-1)*deltafun + 32.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s*s4fun**(-1)*deltafun*rzm2 + 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*s*u*t1fun**(-1)*s4fun**(-1)
     &    *deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun + 8.D0*s*
     &    u**2*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 8.D0*s*u**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun
     &     + 16.D0*s**2*u*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*
     &    s**2*u**2*s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 + 24.D0*s**3*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) + 8.D0*s**4*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 64.D0*mz**2*s4fun**(-1)*deltafun*mgen2
     &     + 24.D0*mz**2*s4fun**(-1)*deltafun*rzm2 - 16.D0*mz**2*
     &    deltafun*rzm2**(-1)*mgen2 + 40.D0*mz**2*u*s4fun**(-1)*
     &    deltafun - 32.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 40.D0*mz**2*s*s4fun**(-1)*deltafun - 32.D0*mz**2*s*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 48.D0*mz**2*s*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 32.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1)*rzm2 - 96.D0*mz**2*s**2*u*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**2*s**3*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 48.D0*mz**4*s4fun**(-1)*
     &    deltafun + 32.D0*mz**4*s*s4fun**(-1)*deltafun*Yfun**(-1)*rzm2
     &     + 96.D0*mz**4*s*u*s4fun**(-1)*deltafun*Yfun**(-1) + 96.D0*
     &    mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) - 64.D0*mz**6*s*
     &    s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*
     &    deltafun*rzm2**(-1)*mgen2 + 4.D0*s*deltafun - 8.D0*s**2*u**2*
     &    s4fun**(-1)*deltafun*Yfun**(-1) - 8.D0*s**3*u*s4fun**(-1)*
     &    deltafun*Yfun**(-1) - 4.D0*s**4*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**2*u*s4fun**(-1)*deltafun*
     &    Yfun**(-1) + 16.D0*mz**2*s**3*s4fun**(-1)*deltafun*Yfun**(-1)
     &     - 16.D0*mz**4*s**2*s4fun**(-1)*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & c0mzmzqsfff * ( 4.D0*s*deltafun + 8.D0*s*s2fun*s4fun**(-1)*
     &    deltafun - 4.D0*s**2*s2fun*deltafun*Yfun**(-1) )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.
     &    D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*
     &    s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1)*deltafun - 8.D0*mz**2*u1fun**(-2)
     &    *s4fun**(-1)*deltafun*Yfun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1)*deltafun - 8.D0*s*s4fun**(-1)*
     &    deltafun - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s4fun**(-1)*deltafun )
      qedamp(2,1,2,3) = qedamp(2,1,2,3) + af**2*betaZfun*cqfactor*
     & b0fqsff * ( 8.D0*s*s4fun**(-1)*deltafun )

      qedamp(2,1,2,2) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 - 16.D0*mgen2**2 - 8.D0*mz**2*mgen2 )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) + 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) - 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1) + 
     &    4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*
     & c0mzmzqsfff * (  - 16.D0*mgen2 + 8.D0*s*s4fun*Yfun**(-1)*mgen2
     &     - 8.D0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*
     &    mgen2 + 16.D0*s4fun**(-1)*Yfun*mgen2 + 16.D0*mgen2**2 - 8.D0*
     &    mz**2*mgen2 )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 + 4.D0*mz**2*s*u**3*s4fun**(-1)*
     &    Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*mgen2 + 4.D0*mz**6*s*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*s**2*u*Yfun**(-1)*mgen2 + 8.D0*s**3*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*mgen2 - 32.D0*mz**2*s*u*Yfun**(-1)*mgen2
     &     - 4.D0*mz**2*s*u**3*s4fun**(-1)*Yfun**(-1) - 32.D0*mz**2*
     &    s**2*Yfun**(-1)*mgen2 - 12.D0*mz**2*s**2*u**2*s4fun**(-1)*
     &    Yfun**(-1) - 12.D0*mz**2*s**3*u*s4fun**(-1)*Yfun**(-1) - 4.D0
     &    *mz**2*s**4*s4fun**(-1)*Yfun**(-1) - 16.D0*mz**4*s4fun**(-1)*
     &    mgen2 + 32.D0*mz**4*s*Yfun**(-1)*mgen2 + 24.D0*mz**4*s*u**2*
     &    s4fun**(-1)*Yfun**(-1) + 48.D0*mz**4*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) + 24.D0*mz**4*s**3*s4fun**(-1)*Yfun**(-1) - 52.D0*
     &    mz**6*s*u*s4fun**(-1)*Yfun**(-1) - 52.D0*mz**6*s**2*
     &    s4fun**(-1)*Yfun**(-1) + 40.D0*mz**8*s*s4fun**(-1)*Yfun**(-1)
     &     )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 - 16.D0*u*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 - 16.D0*u*Yfun**(-1)*rzm2*mgen2 + 16.D0*
     &    u**2*Yfun**(-1)*mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*
     &    Yfun**(-1)*rzm2 + 8.D0*mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 8.D
     &    0*mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 + 8.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*s2fun*s4fun**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*
     &    u*s2fun*s4fun**(-1)*mgen2 + 16.D0*s2fun*s4fun**(-1)*mgen2 + 
     &    16.D0*u*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 + 16.D0*u**2*Yfun**(-1)*mgen2 + 16.D0*s
     &    *t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 16.D0*s*Yfun**(-1)*
     &    rzm2*mgen2 + 32.D0*s*u*Yfun**(-1)*mgen2 + 16.D0*s**2*
     &    Yfun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*
     &    mgen2 - 32.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*Yfun**(-1)*rzm2*mgen2 - 64.D0*mz**2*u*Yfun**(-1)*
     &    mgen2 - 8.D0*mz**2*u**2*s4fun**(-1)*Yfun**(-1)*rzm2 - 8.D0*
     &    mz**2*u**3*s4fun**(-1)*Yfun**(-1) - 64.D0*mz**2*s*Yfun**(-1)*
     &    mgen2 - 16.D0*mz**2*s*u*s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*
     &    mz**2*s*u**2*s4fun**(-1)*Yfun**(-1) - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*Yfun**(-1)*rzm2 - 24.D0*mz**2*s**2*u*s4fun**(-1)*
     &    Yfun**(-1) - 8.D0*mz**2*s**3*s4fun**(-1)*Yfun**(-1) + 64.D0*
     &    mz**4*Yfun**(-1)*mgen2 + 32.D0*mz**4*u*s4fun**(-1)*Yfun**(-1)
     &    *rzm2 + 48.D0*mz**4*u**2*s4fun**(-1)*Yfun**(-1) + 32.D0*mz**4
     &    *s*s4fun**(-1)*Yfun**(-1)*rzm2 + 96.D0*mz**4*s*u*s4fun**(-1)*
     &    Yfun**(-1) + 48.D0*mz**4*s**2*s4fun**(-1)*Yfun**(-1) - 40.D0*
     &    mz**6*s4fun**(-1)*Yfun**(-1)*rzm2 - 104.D0*mz**6*u*
     &    s4fun**(-1)*Yfun**(-1) - 104.D0*mz**6*s*s4fun**(-1)*
     &    Yfun**(-1) + 80.D0*mz**8*s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*c0ccqsfff
     &  * (  - 8.D0*s*s2fun*Yfun**(-1)*mgen2 - 8.D0*mz**2*s*s4fun**(-1)
     &     + 4.D0*mz**2*s*s2fun*Yfun**(-1) + 8.D0*mz**4*s*s2fun*
     &    s4fun**(-1)*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*
     & c0mzmzqsfff * ( 16.D0*mgen2 - 8.D0*s*s4fun*Yfun**(-1)*mgen2 - 8.D
     &    0*mz**2 + 4.D0*mz**2*s*s4fun*Yfun**(-1) - 16.D0*mz**4*
     &    s4fun**(-1) + 8.D0*mz**4*s*Yfun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*t1fun**(-2)*
     &    s4fun**(-1) + 16.D0*mz**4*u*u1fun**(-2)*s4fun**(-1) - 16.D0*
     &    mz**4*s*t1fun**(-2)*s4fun**(-1) + 32.D0*mz**6*t1fun**(-2)*
     &    s4fun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*b0fusff
     &  * (  - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*mz**4*u*
     &    u1fun**(-2)*s4fun**(-1) )
      qedamp(2,1,2,2) = qedamp(2,1,2,2) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u*
     &    t1fun**(-2)*s4fun**(-1) + 16.D0*mz**4*s*t1fun**(-2)*
     &    s4fun**(-1) - 32.D0*mz**6*t1fun**(-2)*s4fun**(-1) )

      qedamp(2,2,1,1) =
     &  + cqfactor*denh12p * (  - 16.D0*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + cqfactor*c0ccqsfff*
     & denh12p * ( 32.D0*mgen2**2 - 8.D0*s*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     - 16.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*mgen2 + 2.D0*s**(-1)*
     &    s2fun*Yfun - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 4.D0*s**(-1)*s2fun*rzm2 - 32.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.
     &    D0*s2fun*s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.
     &    D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 + 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun
     &     - 4.D0*s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s**(-1)*u*s2fun + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 8.D0*mz**2*
     &    s**(-1)*s2fun - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*s4fun**(-1)*mgen2 + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 + 16.D0*
     &    s4fun**(-1)*Yfun*mgen2 - 2.D0*Yfun - 4.D0*s*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 4.D0*s*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 4.D0*s*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 + 4.D0*rzm2 + 32.D0*u*s4fun**(-1)*mgen2 - 4.D0*u
     &     + 16.D0*s*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 - 8.D0*s4fun**(-1)*Yfun + 4.D0*rzm2 - 32.D0*u*
     &    s4fun**(-1)*mgen2 + 4.D0*u - 16.D0*s*s4fun**(-1)*mgen2 + 4.D0
     &    *s + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*s*s4fun**(-1) - 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*s4fun**(-1) - 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s4fun**(-1) - 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1) + 8.D0*s*s4fun**(-1) + 4.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     + 16.D0*s**(-1)*Yfun*mgen2 - 16.D0*s**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*mgen2 + 2.D0*s**(-1)*s2fun*Yfun + 16.D0*mgen2**2 + 4.D0
     &    *s2fun*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 - 4.D0*
     &    s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*mgen2 - 32.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*
     &    mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 + 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*Yfun - 4.D0*s**(-1)*s2fun*rzm2 - 32.
     &    D0*s**(-1)*u*mgen2 + 32.D0*s**(-1)*u*s2fun*s4fun**(-1)*mgen2
     &     - 4.D0*s**(-1)*u*s2fun - 32.D0*mgen2 + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     + 64.D0*mz**2*s**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s**(-1)*s2fun - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     + 64.D0*mz**4*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor
     &  * ( 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 - 2.D0*
     &    Yfun - 4.D0*s*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 4.D0*s*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 4.D0*s*mgen2 + 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 + 32.D0*s4fun**(-1)*rzm2*mgen2 + 4.D0*rzm2 - 4.D0*
     &    u + 16.D0*s*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*
     &    s**(-1)*u*s4fun**(-1)*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*
     &    Yfun*mgen2 + 32.D0*s4fun**(-1)*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    Yfun + 4.D0*rzm2 + 4.D0*u + 16.D0*s*s4fun**(-1)*mgen2 + 4.D0*
     &    s + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 - 8.D0*mz**2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 32.D0*s*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 8.D0*s*s4fun**(-1) - 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*s4fun**(-1) - 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) + 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 8.D0*u*s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*s4fun**(-1) - 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,1,1) = qedamp(2,2,1,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 8.D0*u*s4fun**(-1) + 8.D0*s*s4fun**(-1) + 4.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1) + 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      qedamp(2,2,1,3) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 96.D0
     &    *mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s**(-1)*u*
     &    deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 + 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 + 8.D0*mz**2
     &    *s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*t1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*
     &    mgen2 - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 - 24.D
     &    0*mz**4*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*deltafun*mgen2 + 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 - 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*deltafun*mgen2 - 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 + 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 + 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 + 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2
     &     + 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 4.D0*s**3*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*
     &    mz**2*s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,1,3) = qedamp(2,2,1,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(2,2,1,2) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(2,2,1,2) = qedamp(2,2,1,2) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )

      qedamp(2,2,3,1) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun
     &    *Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2
     &     - 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 - 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun - 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*s4fun**(-1)
     &    *deltafun*mgen2 + 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun
     &    *mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s**(-1)*u*deltafun*
     &    Yfun*rzm2**(-1)*mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 + 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *t1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0
     &    *s**2*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*u*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 + 24.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*deltafun*mgen2 - 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 + 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*deltafun*mgen2 + 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 - 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2 - 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 4.D0*
     &    s**3*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 - 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 
     &    8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2
     &    *s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,1) = qedamp(2,2,3,1) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(2,2,3,3) =
     &  + cqfactor*denh12p * ( 16.D0/(1.D0 - betaZfun**2)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + cqfactor*c0ccqsfff*
     & denh12p * (  - 32.D0/(1.D0 - betaZfun**2)*mgen2**2 + 8.D0/(1.D0
     &     - betaZfun**2)*s*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + betaZfun**2*cqfactor*
     & denh12p * ( 16.D0/(1.D0 - betaZfun**2)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + betaZfun**2*cqfactor*
     & c0ccqsfff*denh12p * (  - 32.D0/(1.D0 - betaZfun**2)*mgen2**2 + 8.
     &    D0/(1.D0 - betaZfun**2)*s*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor * (  - 32.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*Yfun
     &     - 4.D0*mz**2*s**(-1)*Yfun + 16.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*Yfun + 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*mgen2
     &     + 4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*mgen2 + 32.D0*
     &    mz**2*u*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*Yfun - 64.D0*mz**4*s**(-1)*u*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s4fun**(-1)*mgen2 + 16.D0*mz**6*s**(-2)*
     &    s4fun**(-1)*Yfun + 32.D0*mz**6*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*mz**2*mgen2 - 8.D0*mz**2*s*
     &    s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 32.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 32.D0*mz**2*
     &    s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*rzm2 - 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 - 8.D0*mz**2*s**(-1)*u + 32.D
     &    0*mz**2*s**(-1)*u**2*s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*
     &    rzm2 + 8.D0*mz**2*u*s4fun**(-1) + 64.D0*mz**4*s**(-2)*u*
     &    s4fun**(-1)*rzm2 - 64.D0*mz**4*s**(-2)*u**2*s4fun**(-1) + 32.D
     &    0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 32.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1) + 64.D0*mz**4*u1fun**(-1)*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**6*s**(-2)*s4fun**(-1)*rzm2 + 32.D0*mz**6*s**(-2)*u*
     &    s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 32.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 - 32.D0*mz**2*
     &    s**(-2)*u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*rzm2 - 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*rzm2 + 8.D0*mz**2*s**(-1)*u - 64.D
     &    0*mz**2*s**(-1)*u**2*s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*
     &    rzm2 + 8.D0*mz**2 - 40.D0*mz**2*u*s4fun**(-1) - 8.D0*mz**2*s*
     &    s4fun**(-1) + 64.D0*mz**4*s**(-2)*u*s4fun**(-1)*rzm2 + 128.D0
     &    *mz**4*s**(-2)*u**2*s4fun**(-1) + 32.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*rzm2 - 16.D0*mz**4*s**(-1) + 160.D0*mz**4*s**(-1)
     &    *u*s4fun**(-1) + 64.D0*mz**4*t1fun**(-1)*s4fun**(-1)*mgen2 + 
     &    48.D0*mz**4*s4fun**(-1) - 32.D0*mz**6*s**(-2)*s4fun**(-1)*
     &    rzm2 - 160.D0*mz**6*s**(-2)*u*s4fun**(-1) - 96.D0*mz**6*
     &    s**(-1)*s4fun**(-1) + 64.D0*mz**8*s**(-2)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*c0ccqsfff
     &  * ( 64.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 32.D0
     &    *mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 48.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1) - 16.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 32.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**4*
     &    s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun + 128.D0*mz**4*s**(-1)*u
     &    *t1fun**(-1)*s4fun**(-1) + 80.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 96.D0*
     &    mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) + 32.D0*mz**6*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*b0fusff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**2*u*u1fun**(-1)*s4fun**(-1) + 32.D0*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1) - 32.D0*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + vf**2*cqfactor*b0ftsff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) + 48.D0*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) + 32.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 128.D0*mz**4*s**(-1)*u*t1fun**(-1)*s4fun**(-1) - 80.D0
     &    *mz**4*t1fun**(-1)*s4fun**(-1) + 96.D0*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor * (  - 32.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 16.D0*s**(-1)*Yfun*mgen2 - 16.D0*s2fun*
     &    rzm2**(-1)*mgen2**2 + 16.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*
     &    Yfun - 4.D0*mz**2*s**(-1)*Yfun + 16.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*Yfun + 32.D0*mz**2*s**(-1)*u**2*s4fun**(-1)*mgen2
     &     + 4.D0*mz**2*s4fun**(-1)*Yfun - 8.D0*mz**2*mgen2 + 32.D0*
     &    mz**2*u*s4fun**(-1)*mgen2 + 8.D0*mz**2*s*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**4*s**(-1)*
     &    s4fun**(-1)*Yfun - 64.D0*mz**4*s**(-1)*u*s4fun**(-1)*mgen2 - 
     &    32.D0*mz**4*s4fun**(-1)*mgen2 + 16.D0*mz**6*s**(-2)*
     &    s4fun**(-1)*Yfun + 32.D0*mz**6*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 - 8.D0*
     &    mz**2*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*s2fun*rzm2**(-1)*mgen2**2 - 8.D0*
     &    mz**2*mgen2 - 8.D0*mz**2*s*s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 32.D0*s**(-1)*rzm2*mgen2 + 32.D0*s**(-1)*u*mgen2 - 32.D0
     &    *mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 + 32.D0*mz**2*s**(-2)*
     &    u**3*s4fun**(-1) + 8.D0*mz**2*s**(-1)*rzm2 - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*rzm2 - 8.D0*mz**2*s**(-1)*u + 32.D0*
     &    mz**2*s**(-1)*u**2*s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*rzm2
     &     + 8.D0*mz**2*u*s4fun**(-1) + 64.D0*mz**4*s**(-2)*u*
     &    s4fun**(-1)*rzm2 - 64.D0*mz**4*s**(-2)*u**2*s4fun**(-1) + 32.D
     &    0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 32.D0*mz**4*s**(-1)*u*
     &    s4fun**(-1) + 64.D0*mz**4*u1fun**(-1)*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**6*s**(-2)*s4fun**(-1)*rzm2 + 32.D0*mz**6*s**(-2)*u*
     &    s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 32.D0*s**(-1)*rzm2*mgen2 - 32.D0*s**(-1)*u*mgen2 - 32.D0
     &    *mgen2 - 32.D0*mz**2*s**(-2)*u**2*s4fun**(-1)*rzm2 - 32.D0*
     &    mz**2*s**(-2)*u**3*s4fun**(-1) + 64.D0*mz**2*s**(-1)*mgen2 + 
     &    8.D0*mz**2*s**(-1)*rzm2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*
     &    rzm2 + 8.D0*mz**2*s**(-1)*u - 64.D0*mz**2*s**(-1)*u**2*
     &    s4fun**(-1) - 8.D0*mz**2*s4fun**(-1)*rzm2 + 8.D0*mz**2 - 40.D0
     &    *mz**2*u*s4fun**(-1) - 8.D0*mz**2*s*s4fun**(-1) + 64.D0*mz**4
     &    *s**(-2)*u*s4fun**(-1)*rzm2 + 128.D0*mz**4*s**(-2)*u**2*
     &    s4fun**(-1) + 32.D0*mz**4*s**(-1)*s4fun**(-1)*rzm2 - 16.D0*
     &    mz**4*s**(-1) + 160.D0*mz**4*s**(-1)*u*s4fun**(-1) + 64.D0*
     &    mz**4*t1fun**(-1)*s4fun**(-1)*mgen2 + 48.D0*mz**4*s4fun**(-1)
     &     - 32.D0*mz**6*s**(-2)*s4fun**(-1)*rzm2 - 160.D0*mz**6*
     &    s**(-2)*u*s4fun**(-1) - 96.D0*mz**6*s**(-1)*s4fun**(-1) + 64.D
     &    0*mz**8*s**(-2)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*c0ccqsfff
     &  * ( 16.D0*s*rzm2**(-1)*mgen2 + 64.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*b0fmzff
     &  * (  - 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) - 32.D0
     &    *mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) - 48.D0*mz**2*u*
     &    t1fun**(-1)*s4fun**(-1) - 16.D0*mz**2*u*u1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 32.D0*
     &    mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**4*
     &    s**(-1)*u1fun**(-2)*s4fun**(-1)*Yfun + 128.D0*mz**4*s**(-1)*u
     &    *t1fun**(-1)*s4fun**(-1) + 80.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1) - 16.D0*mz**4*u1fun**(-1)*s4fun**(-1) - 96.D0*
     &    mz**6*s**(-1)*t1fun**(-1)*s4fun**(-1) + 32.D0*mz**6*s**(-1)*
     &    u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*b0fusff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*u1fun**(-1)*s4fun**(-1) + 16.D0*
     &    mz**2*u*u1fun**(-1)*s4fun**(-1) + 32.D0*mz**4*s**(-1)*
     &    u1fun**(-2)*s4fun**(-1)*Yfun + 16.D0*mz**4*u1fun**(-1)*
     &    s4fun**(-1) - 32.D0*mz**6*s**(-1)*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,3,3) = qedamp(2,2,3,3) + af**2*cqfactor*b0ftsff
     &  * ( 32.D0*mz**2*s**(-1)*u**2*t1fun**(-1)*s4fun**(-1) + 48.D0*
     &    mz**2*u*t1fun**(-1)*s4fun**(-1) + 16.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1) + 32.D0*mz**4*s**(-1)*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 128.D0*mz**4*s**(-1)*u*t1fun**(-1)*s4fun**(-1) - 80.D0
     &    *mz**4*t1fun**(-1)*s4fun**(-1) + 96.D0*mz**6*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1) )

      qedamp(2,2,3,2) =
     &  + vf**2*cqfactor * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 - 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 - 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 48.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)
     &    *s4fun**(-1)*deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*
     &    s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*s4fun**(-1)*deltafun
     &    *mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*
     &    s**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor * (  - 16.D
     &    0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 16.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s**(-1)*u*
     &    deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun*mgen2 + 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 + 8.D0*mz**2
     &    *s**(-2)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*t1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 - 4.D0*s*u*
     &    deltafun*rzm2**(-1)*mgen2 - 8.D0*s**2*s4fun**(-1)*deltafun*
     &    mgen2 - 4.D0*s**2*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 - 24.D
     &    0*mz**4*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*deltafun*mgen2 + 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 - 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*deltafun*mgen2 - 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 + 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 + 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 - 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     - 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 + 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 + 16.D0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*b0fusff
     &  * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 
     &    8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor
     &  * (  - 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*
     &    deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*
     &    Yfun**2 - 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 - 32.D0*
     &    s**(-1)*u*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*s4fun**(-1)
     &    *deltafun*Yfun*mgen2 + 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 + 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2 - 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * (  - 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2
     &     - 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 4.D0*
     &    s**3*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 8.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 - 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 + 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 
     &    16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun + 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 32.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 24.D0*s**(-1)*u*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 32.D0*u*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 + 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun - 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 - 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 - 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 
     &    8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**2
     &    *s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & b0fusff * (  - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,3,2) = qedamp(2,2,3,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(2,2,2,1) =
     &  + vf**2*cqfactor * ( 8.D0*t1fun**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + vf**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor * ( 8.D0*
     &    t1fun**(-1)*u1fun**(-1)*s2fun*s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 8.D0*s**(-1)*Yfun*mgen2 - 16.D0*mgen2**2
     &     + 8.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun**2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 - 8.D0*s*u**2*Yfun**(-1)*
     &    mgen2 - 16.D0*s**2*u*Yfun**(-1)*mgen2 - 8.D0*s**3*Yfun**(-1)*
     &    mgen2 + 32.D0*mz**2*s*u*Yfun**(-1)*mgen2 + 32.D0*mz**2*s**2*
     &    Yfun**(-1)*mgen2 + 16.D0*mz**4*s4fun**(-1)*mgen2 - 32.D0*
     &    mz**4*s*Yfun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*c0cmzusfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 - 16.D0*s**(-1)*u*mgen2 - 16.D0*
     &    mgen2 + 16.D0*u*Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)
     &    *mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*rzm2 + 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*u1fun**(-1)*
     &    s2fun*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*c0cmztsfff
     &  * ( 16.D0*s**(-1)*rzm2*mgen2 + 16.D0*s**(-1)*u*mgen2 - 16.D0*u*
     &    Yfun**(-1)*rzm2*mgen2 - 16.D0*u**2*Yfun**(-1)*mgen2 - 16.D0*s
     &    *Yfun**(-1)*rzm2*mgen2 - 32.D0*s*u*Yfun**(-1)*mgen2 - 16.D0*
     &    s**2*Yfun**(-1)*mgen2 - 16.D0*mz**2*s**(-2)*s4fun**(-1)*Yfun*
     &    rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*Yfun - 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*mgen2 - 16.D0*
     &    mz**2*t1fun**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*mz**2*
     &    Yfun**(-1)*rzm2*mgen2 + 64.D0*mz**2*u*Yfun**(-1)*mgen2 + 64.D0
     &    *mz**2*s*Yfun**(-1)*mgen2 + 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    Yfun - 64.D0*mz**4*Yfun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*c0ccqsfff
     &  * (  - 16.D0*s2fun*s4fun**(-1)*mgen2 + 8.D0*s*s2fun*Yfun**(-1)*
     &    mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*
     & c0mzmzqsfff * ( 8.D0*s*s4fun*Yfun**(-1)*mgen2 )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*b0fmzff
     &  * ( 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 16.D0*
     &    mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*mz**2*u1fun**(-2)*
     &    s4fun**(-1)*Yfun - 32.D0*mz**4*s**(-1)*t1fun**(-2)*
     &    s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*mz**2*s**(-1)*u*u1fun**(-2)*s4fun**(-1)*Yfun - 8.D0*
     &    mz**2*u1fun**(-2)*s4fun**(-1)*Yfun )
      qedamp(2,2,2,1) = qedamp(2,2,2,1) + af**2*cqfactor*b0ftsff
     &  * (  - 16.D0*mz**2*s**(-1)*u*t1fun**(-2)*s4fun**(-1)*Yfun - 24.D
     &    0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 32.D0*mz**4*s**(-1)*
     &    t1fun**(-2)*s4fun**(-1)*Yfun )

      qedamp(2,2,2,3) =
     &  + vf**2*cqfactor * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun + 8.D0*s*t1fun**(-1)*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun - 16.D0*mz**2*t1fun**(-1)*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*s4fun**(-1)*deltafun
     &    *Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun**2
     &     - 16.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u**2*s4fun**(-1)*deltafun*mgen2 + 8.D0*mz**4*s4fun**(-1)*
     &    deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *u**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*s*u*s4fun**(-1)*
     &    deltafun*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 - 32.D0
     &    *mz**2*u*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*mgen2 + 24.D0*mz**4*s4fun**(-1)*deltafun
     &    *mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 
     &    16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 - 8.D0*s4fun**(-1)*deltafun*Yfun - 16.D0*u**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*s4fun**(-1)
     &    *deltafun*mgen2 + 32.D0*mz**2*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun
     &    *Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*
     &    mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 16.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*Yfun*rzm2 - 
     &    16.D0*mz**2*s**(-2)*u*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2
     &    *s**(-1)*s4fun**(-1)*deltafun*rzm2*mgen2 - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 48.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 16.D0*s*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun + 96.D0
     &    *mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*s*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**4*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + vf**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor * ( 16.D0*
     &    u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*s*
     &    t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0*
     &    mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 16.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s**(-1)*u*deltafun*
     &    Yfun*rzm2**(-1)*mgen2 + 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2
     &     - 4.D0*deltafun*Yfun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun**2 - 16.D0*mz**2*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*mz**2*s**(-1)*deltafun
     &    *Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 - 8.
     &    D0*u1fun*deltafun*mgen2 - 8.D0*u**2*s4fun**(-1)*deltafun*
     &    mgen2 + 4.D0*s*u*deltafun*rzm2**(-1)*mgen2 + 8.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0
     &    *t1fun*deltafun*mgen2 + 8.D0*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*s*u*s4fun**(-1)*deltafun*mgen2 + 4.D0*s*u*deltafun*
     &    rzm2**(-1)*mgen2 + 8.D0*s**2*s4fun**(-1)*deltafun*mgen2 + 4.D0
     &    *s**2*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*u*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*mz**2*s*s4fun**(-1)*deltafun*mgen2 - 8.
     &    D0*mz**2*s*deltafun*rzm2**(-1)*mgen2 + 24.D0*mz**4*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*c0cmzusfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 + 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 - 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 + 16.D0*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 + 8.D0*deltafun*mgen2 - 8.D0*
     &    u*deltafun*rzm2**(-1)*mgen2 + 16.D0*u**2*u1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun - 16.D0*mz**2*s**(-1)*deltafun*mgen2 + 16.D0*
     &    mz**2*s**(-1)*u*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*c0cmztsfff
     &  * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2 - 16.D0*
     &    s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*s**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 - 24.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 16.D0*s**(-1)*u*deltafun*mgen2 + 16.D0*
     &    s**(-1)*u**2*deltafun*rzm2**(-1)*mgen2 - 16.D0*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*Yfun*mgen2 - 8.D0*s4fun**(-1)*deltafun*
     &    Yfun + 8.D0*deltafun*mgen2 + 24.D0*u*deltafun*rzm2**(-1)*
     &    mgen2 - 16.D0*u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.
     &    D0*s*deltafun*rzm2**(-1)*mgen2 - 32.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 - 16.D0*s**2*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*s**(-1)*deltafun*mgen2 - 48.D0*mz**2*s**(-1)*u
     &    *deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**2*deltafun*rzm2**(-1)*
     &    mgen2 + 64.D0*mz**2*u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2
     &     + 64.D0*mz**2*s*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D
     &    0*mz**4*s**(-2)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**4*
     &    s**(-1)*deltafun*rzm2**(-1)*mgen2 - 48.D0*mz**4*t1fun**(-1)*
     &    s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*c0ccqsfff
     &  * (  - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*u*deltafun*
     &    rzm2**(-1)*mgen2 - 16.D0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*
     &    s*deltafun*rzm2**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*mz**2*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*b0fmzff
     &  * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 
     &    16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 16.D0
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*b0fusff
     &  * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0
     &    *mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*cqfactor*b0ftsff
     &  * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 16.D
     &    0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*mz**2*s**(-1)
     &    *t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor
     &  * ( 16.D0*u*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     + 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * ( 8.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun**2
     &     + 4.D0*s**(-1)*s4fun**(-1)*deltafun*Yfun**2 + 32.D0*s**(-1)*
     &    u*s4fun**(-1)*deltafun*Yfun*mgen2 + 16.D0*s4fun**(-1)*
     &    deltafun*Yfun*mgen2 - 8.D0*u*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 4.D0*s*s4fun**(-1)*deltafun*Yfun*
     &    rzm2**(-1)*mgen2 - 8.D0*mz**2*s**(-2)*s4fun**(-1)*deltafun*
     &    Yfun**2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2
     &     + 8.D0*mz**2*s4fun**(-1)*deltafun*Yfun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * (  - 8.D0*s*u1fun*s4fun**(-1)*deltafun*mgen2
     &     + 4.D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 8.D0*s*t1fun*s4fun**(-1)*deltafun*mgen2 + 4.
     &    D0*s**2*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 4.D0*s**3*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*mz**2*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     + 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 8.D0*s**(-1)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*mgen2
     &     + 16.D0*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 16.D0*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 + 32.D0*u*s4fun**(-1)*
     &    deltafun*mgen2 + 16.D0*u**2*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*u**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 8.D
     &    0*s*s4fun**(-1)*deltafun*mgen2 - 8.D0*s*u*s4fun**(-1)*
     &    deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*s4fun**(-1)*
     &    deltafun*Yfun*rzm2 - 16.D0*mz**2*s**(-2)*u*s4fun**(-1)*
     &    deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*deltafun*rzm2
     &    *mgen2 - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*mgen2 - 
     &    16.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 16.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D0*mz**4*
     &    u1fun**(-1)*s4fun**(-1)*deltafun*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * (  - 16.D0*s**(-2)*u*s4fun**(-1)*deltafun*Yfun*rzm2
     &     - 16.D0*s**(-2)*u**2*s4fun**(-1)*deltafun*Yfun - 8.D0*
     &    s**(-1)*s4fun**(-1)*deltafun*Yfun*rzm2 - 32.D0*s**(-1)*u*
     &    s4fun**(-1)*deltafun*rzm2*mgen2 - 24.D0*s**(-1)*u*s4fun**(-1)
     &    *deltafun*Yfun - 32.D0*s**(-1)*u**2*s4fun**(-1)*deltafun*
     &    mgen2 - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun*mgen2 - 
     &    16.D0*s4fun**(-1)*deltafun*rzm2*mgen2 - 8.D0*s4fun**(-1)*
     &    deltafun*Yfun - 32.D0*u*s4fun**(-1)*deltafun*mgen2 - 16.D0*
     &    u**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 16.D0*u**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 8.D0*s*s4fun**(-1)*
     &    deltafun*mgen2 - 32.D0*s*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    mgen2 + 24.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 16.D
     &    0*s**2*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 + 8.D0*s**2*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*mz**2*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun*rzm2 + 48.D0*mz**2*s**(-2)*u*
     &    s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    deltafun*Yfun + 96.D0*mz**2*s**(-1)*u*s4fun**(-1)*deltafun*
     &    mgen2 + 48.D0*mz**2*s4fun**(-1)*deltafun*mgen2 + 64.D0*mz**2*
     &    u*t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 48.D0*mz**2*u*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 64.D0*mz**2*s*
     &    t1fun**(-1)*s4fun**(-1)*deltafun*mgen2 - 32.D0*mz**2*s*
     &    s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 - 32.D0*mz**4*s**(-2)*
     &    s4fun**(-1)*deltafun*Yfun - 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    deltafun*mgen2 - 48.D0*mz**4*t1fun**(-1)*s4fun**(-1)*deltafun
     &    *mgen2 + 32.D0*mz**4*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * (  - 16.D0*s*u*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2
     &     - 8.D0*s**2*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 + 16.D0*
     &    mz**2*s*s4fun**(-1)*deltafun*rzm2**(-1)*mgen2 )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & b0fmzff * (  - 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*Yfun
     &     - 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 32.D0*mz**2*
     &    s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 8.D0*mz**2*
     &    t1fun**(-2)*s4fun**(-1)*deltafun*Yfun + 8.D0*mz**2*
     &    u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 16.D0*s**(-1)*u*u1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*deltafun*Yfun )
      qedamp(2,2,2,3) = qedamp(2,2,2,3) + af**2*betaZfun*cqfactor*
     & b0ftsff * ( 16.D0*s**(-1)*u*t1fun**(-1)*s4fun**(-1)*deltafun*
     &    Yfun + 16.D0*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun - 32.D0*
     &    mz**2*s**(-1)*t1fun**(-1)*s4fun**(-1)*deltafun*Yfun + 8.D0*
     &    mz**2*t1fun**(-2)*s4fun**(-1)*deltafun*Yfun )

      qedamp(2,2,2,2) =
     &  + cqfactor*denh12p * (  - 16.D0*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + cqfactor*c0ccqsfff*
     & denh12p * ( 32.D0*mgen2**2 - 8.D0*s*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     - 16.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun*mgen2 + 2.D0*s**(-1)*
     &    s2fun*Yfun - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 + 16.D0*mz**2*
     &    s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*
     & d0ccmzmzqsusffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*
     & d0ccmzmzqstsffff * (  - 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 16.D0
     &    *mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 4.D0*s**(-1)*s2fun*rzm2 - 32.D0*
     &    s**(-1)*u*s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.
     &    D0*s2fun*s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*
     &    rzm2*mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.
     &    D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 + 8.D0*s**(-1)*s2fun*s4fun**(-1)*Yfun
     &     - 4.D0*s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s**(-1)*u*s2fun + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     - 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 8.D0*mz**2*
     &    s**(-1)*s2fun - 32.D0*mz**2*s**(-1)*u*s4fun**(-1)*mgen2 - 32.D
     &    0*mz**2*s4fun**(-1)*mgen2 + 64.D0*mz**4*s**(-1)*s4fun**(-1)*
     &    mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 - 16.D0
     &    *s4fun**(-1)*Yfun*mgen2 + 2.D0*Yfun + 4.D0*s*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 4.D0*s*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 4.D0*s*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 - 4.D0*rzm2 - 32.D0*u*s4fun**(-1)*mgen2 + 4.D0*u - 16.D0
     &    *s*s4fun**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 + 8.D0*s4fun**(-1)*Yfun - 4.D0*rzm2 + 32.D0*u*
     &    s4fun**(-1)*mgen2 - 4.D0*u + 16.D0*s*s4fun**(-1)*mgen2 - 4.D0
     &    *s - 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*s*s4fun**(-1) + 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*s4fun**(-1) + 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**2*s4fun**(-1) + 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + vf**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1) - 8.D0*s*s4fun**(-1) - 4.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1) - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor * (  - 16.D
     &    0*mz**2*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun + 8.D0*mz**2
     &    *s*t1fun**(-1)*u1fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun**2
     &     + 16.D0*s**(-1)*Yfun*mgen2 - 16.D0*s**(-1)*s2fun*s4fun**(-1)
     &    *Yfun*mgen2 + 2.D0*s**(-1)*s2fun*Yfun + 16.D0*mgen2**2 + 4.D0
     &    *s2fun*mgen2 + 16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*
     & d0ccmzmzqsusffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*
     & d0ccmzmzqstsffff * ( 16.D0*mgen2**2 + 4.D0*s2fun*mgen2 - 8.D0*s*
     &    mgen2 - 16.D0*mz**4*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*c0cmzusfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*u1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 - 4.D0*
     &    s**(-1)*s2fun*rzm2 + 32.D0*s**(-1)*u*mgen2 - 32.D0*s**(-1)*u*
     &    s2fun*s4fun**(-1)*mgen2 + 4.D0*s**(-1)*u*s2fun - 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*
     &    mgen2 + 32.D0*mz**2*s**(-1)*s2fun*s4fun**(-1)*mgen2 + 32.D0*
     &    mz**2*s**(-1)*u*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*c0cmztsfff
     &  * ( 8.D0*s**(-2)*s2fun*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-2)*u*
     &    s2fun*s4fun**(-1)*Yfun - 16.D0*s**(-1)*t1fun**(-1)*s2fun*
     &    s4fun**(-1)*Yfun*mgen2 - 32.D0*s**(-1)*rzm2*mgen2 + 8.D0*
     &    s**(-1)*s2fun*s4fun**(-1)*Yfun - 4.D0*s**(-1)*s2fun*rzm2 - 32.
     &    D0*s**(-1)*u*mgen2 + 32.D0*s**(-1)*u*s2fun*s4fun**(-1)*mgen2
     &     - 4.D0*s**(-1)*u*s2fun - 32.D0*mgen2 + 16.D0*s2fun*
     &    s4fun**(-1)*mgen2 - 4.D0*s2fun - 16.D0*mz**2*s**(-2)*s2fun*
     &    s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*s4fun**(-1)*rzm2*mgen2
     &     + 64.D0*mz**2*s**(-1)*mgen2 - 32.D0*mz**2*s**(-1)*s2fun*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2*s**(-1)*s2fun - 32.D0*mz**2*
     &    s**(-1)*u*s4fun**(-1)*mgen2 - 32.D0*mz**2*s4fun**(-1)*mgen2
     &     + 64.D0*mz**4*s**(-1)*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*c0ccqsfff
     &  * ( 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*b0fmzff
     &  * ( 8.D0*s2fun*s4fun**(-1) + 4.D0*u*t1fun**(-1)*s2fun*
     &    s4fun**(-1) - 4.D0*u*u1fun**(-1)*s2fun*s4fun**(-1) + 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) - 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 8.D0*mz**2*s**(-1)*
     &    u1fun**(-2)*s2fun*s4fun**(-1)*Yfun - 32.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) - 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     - 4.D0*mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*b0fusff
     &  * ( 8.D0*s**(-1)*u*s2fun*s4fun**(-1) + 4.D0*u*u1fun**(-1)*s2fun
     &    *s4fun**(-1) + 8.D0*mz**2*s**(-1)*u1fun**(-2)*s2fun*
     &    s4fun**(-1)*Yfun + 8.D0*mz**2*s**(-1)*s2fun*s4fun**(-1) + 4.D0
     &    *mz**2*u1fun**(-1)*s2fun*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*cqfactor*b0ftsff
     &  * (  - 8.D0*s**(-1)*u*s2fun*s4fun**(-1) - 8.D0*s2fun*
     &    s4fun**(-1) - 4.D0*u*t1fun**(-1)*s2fun*s4fun**(-1) - 4.D0*s*
     &    t1fun**(-1)*s2fun*s4fun**(-1) + 8.D0*mz**2*s**(-1)*
     &    t1fun**(-2)*s2fun*s4fun**(-1)*Yfun + 24.D0*mz**2*s**(-1)*
     &    s2fun*s4fun**(-1) + 12.D0*mz**2*t1fun**(-1)*s2fun*s4fun**(-1)
     &     )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor
     &  * (  - 8.D0*s*t1fun**(-1)*u1fun**(-1)*s4fun**(-1)*Yfun )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & d0cmzcmztsusffff * (  - 4.D0*s**(-1)*s4fun**(-1)*Yfun**2 + 2.D0*
     &    Yfun + 4.D0*s*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqsusffff * ( 4.D0*s*mgen2 - 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & d0ccmzmzqstsffff * ( 4.D0*s*mgen2 - 8.D0*s*s2fun*s4fun**(-1)*
     &    mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & c0cmzusfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 - 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*u1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 - 32.D0*s4fun**(-1)*rzm2*mgen2 - 4.D0*rzm2 + 4.D0*u - 
     &    16.D0*s*s4fun**(-1)*mgen2 + 32.D0*mz**2*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & c0cmztsfff * ( 8.D0*s**(-1)*s4fun**(-1)*Yfun*rzm2 + 8.D0*s**(-1)
     &    *u*s4fun**(-1)*Yfun - 16.D0*t1fun**(-1)*s4fun**(-1)*Yfun*
     &    mgen2 - 32.D0*s4fun**(-1)*rzm2*mgen2 + 8.D0*s4fun**(-1)*Yfun
     &     - 4.D0*rzm2 - 4.D0*u - 16.D0*s*s4fun**(-1)*mgen2 - 4.D0*s - 
     &    16.D0*mz**2*s**(-1)*s4fun**(-1)*Yfun + 32.D0*mz**2*
     &    s4fun**(-1)*mgen2 + 8.D0*mz**2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & c0ccqsfff * ( 32.D0*s*s4fun**(-1)*mgen2 )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & b0fmzff * ( 8.D0*s*s4fun**(-1) + 4.D0*s*u*t1fun**(-1)*
     &    s4fun**(-1) - 4.D0*s*u*u1fun**(-1)*s4fun**(-1) + 4.D0*s**2*
     &    t1fun**(-1)*s4fun**(-1) - 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*
     &    Yfun - 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun - 32.D0*mz**2*
     &    s4fun**(-1) - 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) - 4.D0*
     &    mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & b0fusff * ( 8.D0*u*s4fun**(-1) + 4.D0*s*u*u1fun**(-1)*
     &    s4fun**(-1) + 8.D0*mz**2*u1fun**(-2)*s4fun**(-1)*Yfun + 8.D0*
     &    mz**2*s4fun**(-1) + 4.D0*mz**2*s*u1fun**(-1)*s4fun**(-1) )
      qedamp(2,2,2,2) = qedamp(2,2,2,2) + af**2*betaZfun*cqfactor*
     & b0ftsff * (  - 8.D0*u*s4fun**(-1) - 8.D0*s*s4fun**(-1) - 4.D0*s*
     &    u*t1fun**(-1)*s4fun**(-1) - 4.D0*s**2*t1fun**(-1)*s4fun**(-1)
     &     + 8.D0*mz**2*t1fun**(-2)*s4fun**(-1)*Yfun + 24.D0*mz**2*
     &    s4fun**(-1) + 12.D0*mz**2*s*t1fun**(-1)*s4fun**(-1) )

      do iph1=1,2
         do iph2=1,2
            do iz1=1,3
               do iz2=1,3
                  qedsum(iph1,iph2,iz1,iz2) = 
     &                 qedsum(iph1,iph2,iz1,iz2) + qedamp(iph1,iph2,iz1,iz2)
               enddo
            enddo
         enddo
      enddo

      enddo
      endif

      do i = 1,2
        do j = 1,2
          do k = 1,3
            do l = 1,3
              if ((iew.eq.0).and.(iqed.eq.0)) then
                amp(i,j,k,l) = 0d0
              endif
              if ((iew.eq.1).and.(iqed.eq.0)) then
                amp(i,j,k,l) = ewcoeff*ewamp(i,j,k,l)
              endif
              if ((iew.eq.0).and.(iqed.eq.1)) then
                amp(i,j,k,l) = qedcoeff*qedsum(i,j,k,l)
              endif
              if ((iew.eq.1).and.(iqed.eq.1)) then
                amp(i,j,k,l) = ewcoeff*ewamp(i,j,k,l)+qedcoeff*qedsum(i,j,k,l)
              endif
            enddo
          enddo
        enddo
      enddo

      return
      end
